// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.Windows.Forms;
using DGNetwork.Gui;
using DGNetwork.Services;
using ICSharpCode.Core.AddIns;
using ICSharpCode.Core.AddIns.Codons;

namespace DGNetwork.AddIns.AutoUpdate
{
	public class AutoCheck : AbstractCommand
	{
		public override void Run()
		{
			Service.Timer.Tick += new EventHandler(TimerTick);
		}
		
		private void TimerTick(object sender, EventArgs e)
		{
			UpdateInterval interval = (UpdateInterval)Service.Property.GetProperty("AutoUpdate.CheckInterval", UpdateInterval.Daily);
			if (interval == UpdateInterval.Never)
				return;
			DateTime lastCheck = (DateTime)Service.Property.GetProperty("AutoUpdate.LastAutoCheck", DateTime.MinValue);
			DateTime now = DateTime.Now;
			
			if (lastCheck > now) // limit value of lastCheck to prevent not checking for updates for
				lastCheck = now; // a long time when the system clock was set back.
			
			if (interval == UpdateInterval.Weekly) {
				if (lastCheck.AddDays(7) > now)
					return;
			} else { // Daily check
				if (lastCheck.AddDays(1) > now)
					return;
			}
			Service.Property.SetProperty("AutoUpdate.LastAutoCheck", DateTime.Now);
			new MethodInvoker(CheckForUpdates).BeginInvoke(null, null);
		}
		
		private void CheckForUpdates()
		{
			try {
				InformationDownloader.ClearCache();
				ArrayList updateInfos = new ArrayList();
				foreach (AddIn addin in AddInTreeSingleton.AddInTree.AddIns) {
					if (addin.UpdateUrl != null && addin.UpdateUrl.Length > 0)
						updateInfos.Add(new UpdateInfo(addin));
				}
				foreach (UpdateInfo info in updateInfos) {
					info.DownloadInformation();
					if (info.ErrorText == null && info.AvailableVersion > info.CurrentVersion) {
						Service.Window.BeginInvoke(new MethodInvoker(UpdatesFound));
						break;
					}
				}
			} catch (Exception ex) {
				Service.Message.ShowError(ex);
			}
		}
		
		private void UpdatesFound()
		{
			Service.TNA.MainIcon.ShowBalloon(BalloonIcon.Info, "${res:AutoUpdate.UpdateFoundText}", "${res:AutoUpdate.UpdateFoundTitle}",
			                                 45000, new EventHandler(BalloonClick), new EventHandler(BalloonTimeout));
		}
		
		private void BalloonClick(object sender, EventArgs e)
		{
			UpdateService.Instance.StartUpdate();
		}
		
		private void BalloonTimeout(object sender, EventArgs e)
		{
			// how can we assure that the user reads the message?
		}
	}
}
