// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.IO;
using System.Net;
using System.Xml;
using DGNetwork.Services;

namespace DGNetwork.AddIns.AutoUpdate
{
	/// <summary>
	/// Class that downloads update information files.
	/// </summary>
	public class InformationDownloader
	{
		public static void ClearCache()
		{
			hash.Clear();
		}
		
		public static XmlElement GetInformation(string url)
		{
			XmlDocument doc;
			int pos = url.IndexOf('#');
			if (pos < 0) {
				return DownloadXml(url).DocumentElement;
			}
			string anchor = url.Substring(pos + 1);
			string downloadUrl = url.Substring(0, pos);
			doc = GetXml(downloadUrl);
			if (doc == null)
				return null;
			if (doc.DocumentElement.Name == "error")
				return doc.DocumentElement;
			if (doc.DocumentElement.Name != "updateInfoCombination")
				return MakeError(doc, "Unknown root element: <" + doc.DocumentElement.Name + ">");
			foreach (XmlElement updateInfo in doc.DocumentElement) {
				if (updateInfo.GetAttribute("name") == anchor)
					return updateInfo;
			}
			return MakeError(doc, "#" + anchor + " not found!");
		}
		
		static XmlElement MakeError(XmlDocument doc, string text)
		{
			XmlElement error = doc.CreateElement("error");
			error.InnerText = text;
			return error;
		}
		
		static Hashtable hash = new Hashtable();
		
		static XmlDocument GetXml(string url)
		{
			object obj;
			bool download;
			lock (hash) {
				if (hash.Contains(url)) {
					download = false;
					obj = hash[url];
				} else {
					download = true;
					obj = new object();
					hash.Add(url, obj);
				}
			}
			if (obj is XmlDocument)
				return (XmlDocument)obj;
			lock (obj) {
				if (download) {
					XmlDocument doc = DownloadXml(url);
					hash[url] = doc;
					return doc;
				}
			}
			return hash[url] as XmlDocument;
		}
		
		static XmlDocument DownloadXml(string url)
		{
			WebRequest request = WebRequest.Create(url);
			request.Timeout = 10000;
			if (request is HttpWebRequest) {
				HttpWebRequest wr = (HttpWebRequest)request;
				wr.UserAgent = Service.Parse("${AppIdentification}");
			}
			using (WebResponse res = request.GetResponse()) {
				using (Stream s = res.GetResponseStream()) {
					XmlDocument xmlDocument = new XmlDocument();
					xmlDocument.Load(s);
					return xmlDocument;
				}
			}
		}
	}
}
