// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Xml;

namespace DGNetwork.AddIns.AutoUpdate
{
	public class UpdateArchive
	{
		int filesize;
		byte[] checksum;
		string url;
		string extractTo;
		UpdateInfo info;
		
		public UpdateArchive(UpdateInfo info, XmlElement element)
		{
			this.info = info;
			checksum = Convert.FromBase64String(element.GetAttribute("checksum"));
			filesize = XmlConvert.ToInt32(element.GetAttribute("filesize"));
			extractTo = element.GetAttribute("extractTo");
			url = element.InnerText;
		}
		
		/// <summary>
		/// Gets the updateinfo of this archive.
		/// </summary>
		public UpdateInfo Info {
			get {
				return info;
			}
		}
		
		/// <summary>
		/// Gets/Sets the sha1 checksum of the download.
		/// </summary>
		public byte[] Checksum {
			get {
				return checksum;
			}
			set {
				checksum = value;
			}
		}
		
		public int FileSize {
			get {
				return filesize;
			}
			set {
				filesize = value;
			}
		}
		
		public string Url {
			get {
				return url;
			}
			set {
				url = value;
			}
		}
		
		public string ExtractTo {
			get {
				return extractTo;
			}
			set {
				extractTo = value;
			}
		}
		
		public string GetTempFileName()
		{
			return System.IO.Path.Combine(DGNetwork.Helper.GetTempPath(), "update" + DGNetwork.Helper.ToHex(this.Checksum) + ".zip");
		}
	}
}
