// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.Drawing;
using System.IO;
using System.Net;
using System.Reflection;
using System.Security.Cryptography;
using System.Threading;
using System.Windows.Forms;
using ICSharpCode.Core;
using ICSharpCode.Core.AddIns;
using ICSharpCode.Core.Services;
using DGNetwork.Services;

namespace DGNetwork.AddIns.AutoUpdate
{
	public class UpdateForm : System.Windows.Forms.Form
	{
		private System.Windows.Forms.Label errorLabel;
		private System.Windows.Forms.ProgressBar progressBar;
		private System.Windows.Forms.ListView listView;
		private System.Windows.Forms.ColumnHeader columnHeader2;
		private System.Windows.Forms.ColumnHeader columnHeader3;
		private System.Windows.Forms.Button abortButton;
		private System.Windows.Forms.Button startButton;
		private System.Windows.Forms.ColumnHeader columnHeader1;
		
		public UpdateForm()
		{
			InformationDownloader.ClearCache();
			//
			// The InitializeComponent() call is required for Windows Forms designer support.
			//
			InitializeComponent();
			
			this.Icon = Service.Resource.GetIcon("Icons.Update");
			foreach (Control ctl in this.Controls) {
				ctl.Text = Service.Parse(ctl.Text);
			}
			startButton.Enabled = false;
			abortButton.Location = startButton.Location;
			string checking = Service.Parse("${res:AutoUpdate.Checking}");
			foreach (AddIn addin in AddInTreeSingleton.AddInTree.AddIns) {
				if (addin.UpdateUrl == null || addin.UpdateUrl.Length == 0)
					continue;
				ListViewItem item = new ListViewItem(new string[] { addin.Name, addin.Version, checking });
				item.Tag = new UpdateInfo(addin);
				listView.Items.Add(item);
			}
		}
		
		#region Windows Forms Designer generated code
		/// <summary>
		/// This method is required for Windows Forms designer support.
		/// Do not change the method contents inside the source code editor. The Forms designer might
		/// not be able to load this method if it was changed manually.
		/// </summary>
		private void InitializeComponent() {
			this.columnHeader1 = new System.Windows.Forms.ColumnHeader();
			this.startButton = new System.Windows.Forms.Button();
			this.abortButton = new System.Windows.Forms.Button();
			this.columnHeader3 = new System.Windows.Forms.ColumnHeader();
			this.columnHeader2 = new System.Windows.Forms.ColumnHeader();
			this.listView = new System.Windows.Forms.ListView();
			this.progressBar = new System.Windows.Forms.ProgressBar();
			this.errorLabel = new System.Windows.Forms.Label();
			this.SuspendLayout();
			// 
			// columnHeader1
			// 
			this.columnHeader1.Text = "Component";
			this.columnHeader1.Width = 230;
			// 
			// startButton
			// 
			this.startButton.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.startButton.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.startButton.Location = new System.Drawing.Point(416, 192);
			this.startButton.Name = "startButton";
			this.startButton.Size = new System.Drawing.Size(104, 23);
			this.startButton.TabIndex = 2;
			this.startButton.Text = "&${res:AutoUpdate.StartButton}";
			this.startButton.Click += new System.EventHandler(this.StartButtonClick);
			// 
			// abortButton
			// 
			this.abortButton.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.abortButton.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.abortButton.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.abortButton.Location = new System.Drawing.Point(176, 192);
			this.abortButton.Name = "abortButton";
			this.abortButton.Size = new System.Drawing.Size(104, 23);
			this.abortButton.TabIndex = 3;
			this.abortButton.Text = "${res:Global.Cancel}";
			this.abortButton.Visible = false;
			this.abortButton.Click += new System.EventHandler(this.AbortButtonClick);
			// 
			// columnHeader3
			// 
			this.columnHeader3.Text = "Status";
			this.columnHeader3.Width = 200;
			// 
			// columnHeader2
			// 
			this.columnHeader2.Text = "Version";
			// 
			// listView
			// 
			this.listView.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
			                                                              | System.Windows.Forms.AnchorStyles.Left)
			                                                             | System.Windows.Forms.AnchorStyles.Right)));
			this.listView.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
			                               	this.columnHeader1,
			                               	this.columnHeader2,
			                               	this.columnHeader3});
			this.listView.FullRowSelect = true;
			this.listView.GridLines = true;
			this.listView.Location = new System.Drawing.Point(8, 8);
			this.listView.Name = "listView";
			this.listView.Size = new System.Drawing.Size(512, 136);
			this.listView.TabIndex = 0;
			this.listView.View = System.Windows.Forms.View.Details;
			// 
			// progressBar
			// 
			this.progressBar.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)
			                                                                | System.Windows.Forms.AnchorStyles.Right)));
			this.progressBar.Location = new System.Drawing.Point(8, 152);
			this.progressBar.Name = "progressBar";
			this.progressBar.Size = new System.Drawing.Size(512, 16);
			this.progressBar.TabIndex = 1;
			// 
			// errorLabel
			// 
			this.errorLabel.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)
			                                                               | System.Windows.Forms.AnchorStyles.Right)));
			this.errorLabel.Location = new System.Drawing.Point(8, 176);
			this.errorLabel.Name = "errorLabel";
			this.errorLabel.Size = new System.Drawing.Size(512, 40);
			this.errorLabel.TabIndex = 4;
			this.errorLabel.Text = "${res:AutoUpdate.InitialStatusText}";
			// 
			// UpdateForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.CancelButton = this.abortButton;
			this.ClientSize = new System.Drawing.Size(528, 222);
			this.Controls.Add(this.abortButton);
			this.Controls.Add(this.startButton);
			this.Controls.Add(this.progressBar);
			this.Controls.Add(this.listView);
			this.Controls.Add(this.errorLabel);
			this.DockPadding.All = 8;
			this.Name = "UpdateForm";
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
			this.Text = "DG Network Update";
			this.Closing += new System.ComponentModel.CancelEventHandler(this.UpdateFormClosing);
			this.Load += new System.EventHandler(this.UpdateFormLoad);
			this.ResumeLayout(false);
		}
		#endregion
		
		void UpdateFormLoad(object sender, EventArgs e)
		{
			threadCount = 2;
			new MethodInvoker(StartThread).BeginInvoke(null, null);
			new MethodInvoker(StartThread).BeginInvoke(null, null);
		}
		
		int threadCount = 0;
		
		object lockObject = new object();
		int listIndex = 0;
		
		void StartThread()
		{
			try {
				while (true) {
					ListViewItem nextJob;
					lock (lockObject) {
						if (listIndex >= listView.Items.Count)
							break;
						nextJob = listView.Items[listIndex];
						listIndex += 1;
					}
					((UpdateInfo)nextJob.Tag).DownloadInformation();
					BeginInvoke(new void_ListViewItem(UpdateDisplay), new object[] { nextJob });
				}
			} catch (Exception ex) {
				Service.Message.ShowError(ex);
			} finally {
				lock (lockObject) {
					threadCount -= 1;
					if (threadCount == 0)
						BeginInvoke(new MethodInvoker(ThreadFinished));
				}
			}
		}
		
		private delegate void void_ListViewItem(ListViewItem item);
		
		void UpdateDisplay(ListViewItem item)
		{
			UpdateInfo info = (UpdateInfo)item.Tag;
			string text;
			if (info.ErrorText != null) {
				text = Service.Parse(info.ErrorText);
			} else if (info.AvailableVersion == info.CurrentVersion) {
				text = "${res:AutoUpdate.CurrentIsInstalled}";
			} else if (info.AvailableVersion < info.CurrentVersion) {
				text = "${res:AutoUpdate.NewerThanOnline}";
			} else {
				text = "${res:AutoUpdate.VersionAvailable}";
			}
			text = Service.Parse(text, "version", info.AvailableVersion.ToString());
			item.SubItems[2].Text = text;
		}
		
		void ThreadFinished()
		{
			bool ok = true;
			foreach (ListViewItem item in listView.Items) {
				UpdateInfo info = (UpdateInfo)item.Tag;
				if (info.ErrorText != null)
					continue;
				if (info.AvailableVersion > info.CurrentVersion)
					ok = false;
			}
			if (ok) {
				startButton.Text = Service.Parse("${res:Global.Close}");
			}
			startButton.Enabled = true;
		}
		
		void StartButtonClick(object sender, EventArgs e)
		{
			int totalBytes = 0;
			foreach (ListViewItem item in listView.Items) {
				UpdateInfo info = (UpdateInfo)item.Tag;
				if (info.ErrorText != null)
					continue;
				if (info.AvailableVersion > info.CurrentVersion) {
					foreach (UpdateArchive archive in info.Archives) {
						totalBytes += archive.FileSize;
					}
				}
			}
			if (totalBytes == 0) {
				Close();
				return;
			}
			if (!UpdateService.Instance.IsUpdateAllowed())
				return;
			startButton.Visible = false;
			abortButton.Visible = true;
			progressBar.Value = 0;
			progressBar.Maximum = totalBytes;
			cancelUpdate = false;
			updateThread = new Thread(new ThreadStart(UpdateThread));
			updateThread.Name = "UpdateThread";
			updateThread.Start();
		}
		
		volatile bool cancelUpdate;
		Thread updateThread;
		int bytesDownloaded;
		
		void UpdateThread()
		{
			try {
				bytesDownloaded = 0;
				ArrayList archives = new ArrayList();
				foreach (ListViewItem item in listView.Items) {
					UpdateInfo info = (UpdateInfo)item.Tag;
					if (info.ErrorText != null)
						continue;
					if (info.AvailableVersion > info.CurrentVersion) {
						SetText(item, "Downloading...");
						try {
							foreach (UpdateArchive archive in info.Archives) {
								archives.Add(archive);
								if (DownloadArchive(archive)) {
									if (cancelUpdate)
										return;
									if (!CheckArchive(archive)) {
										SetText(item, "Error");
										return;
									}
								} else {
									SetText(item, "Error");
									return;
								}
								if (cancelUpdate)
									return;
							}
						} finally {
							if (cancelUpdate)
								SetText(item, "${res:AutoUpdate.Aborted}");
							else
								SetText(item, "${res:Global.Finished}");
						}
					}
				}
				UpdateService.Instance.StartUpdateHelperApplication(archives);
				cancelUpdate = true;
				Application.Exit();
			} catch (ThreadAbortException) {
				// ignore
			} catch (Exception ex) {
				Service.Message.ShowError(ex);
			} finally {
				if (!cancelUpdate)
					BeginInvoke(new MethodInvoker(UpdateAborted));
			}
		}
		
		bool DownloadArchive(UpdateArchive archive)
		{
			string saveName = archive.GetTempFileName();
			if (File.Exists(saveName)) {
				bytesDownloaded += archive.FileSize;
				BeginInvoke(new MethodInvoker(SetProgress));
				return true;
			}
			SetText(errorLabel, string.Format("Downloading {0} ({1} KB)", Path.GetFileName(archive.Url), (archive.FileSize / 1024) + 1));
			try {
				WebRequest request = WebRequest.Create(archive.Url);
				request.Timeout = 5000;
				if (request is HttpWebRequest) {
					HttpWebRequest wr = (HttpWebRequest)request;
					wr.UserAgent = "DG Network Update";
					wr.Referer = Path.GetDirectoryName(archive.Url + "/");
				}
				if (cancelUpdate)
					return false;
				using (WebResponse res = request.GetResponse()) {
					if (cancelUpdate)
						return false;
					using (Stream s = res.GetResponseStream()) {
						if (cancelUpdate)
							return false;
						bool downloadOK = false;
						try {
							using (FileStream fs = new FileStream(saveName, FileMode.Create, FileAccess.Write)) {
								int c;
								byte[] buffer = new byte[4096];
								do {
									c = s.Read(buffer, 0, buffer.Length);
									if (cancelUpdate)
										return false;
									if (c > 0) {
										fs.Write(buffer, 0, c);
										int oldBytesDownloaded = bytesDownloaded;
										bytesDownloaded += c;
										if (bytesDownloaded / 1024 != oldBytesDownloaded / 1024) {
											BeginInvoke(new MethodInvoker(SetProgress));
										}
									}
								} while (c > 0);
							}
							downloadOK = true;
						} finally {
							if (!downloadOK) {
								try {
									File.Delete(saveName);
								} catch { }
							}
						}
					}
				}
				return true;
			} catch (WebException ex) {
				SetText(errorLabel, ex.Message);
				return false;
			}
		}
		
		bool CheckArchive(UpdateArchive archive)
		{
			string saveName = archive.GetTempFileName();
			using (SHA1CryptoServiceProvider sha1 = new SHA1CryptoServiceProvider()) {
				byte[] hash;
				using (FileStream fs = new FileStream(saveName, FileMode.Open, FileAccess.Read)) {
					hash = sha1.ComputeHash(fs);
				}
				if (!ArrayEqual(archive.Checksum, hash)) {
					SetText(errorLabel, "Checksum error in " + Path.GetFileName(archive.Url) + ", please restart update");
					try {
						File.Delete(saveName);
					} catch {
						Service.Message.ShowError("Cannot delete invalid file " + saveName);
					}
					return false;
				}
				return true;
			}
		}
		
		bool ArrayEqual(byte[] a, byte[] b)
		{
			if (a == b)
				return true;
			if (a == null || b == null)
				return false;
			if (a.Length != b.Length)
				return false;
			for (int i = 0; i < a.Length; ++i) {
				if (a[i] != b[i])
					return false;
			}
			return true;
		}
		
		void SetProgress()
		{
			progressBar.Value = bytesDownloaded;
		}
		
		private delegate void void_ListViewItem_string(ListViewItem item, string text);
		
		void SetText(ListViewItem item, string text)
		{
			BeginInvoke(new void_ListViewItem_string(SetTextInvoked), new object[] {item, text});
		}
		
		void SetTextInvoked(ListViewItem item, string text)
		{
			item.SubItems[2].Text = Service.Parse(text);
		}
		
		private delegate void void_Control_string(Control item, string text);
		
		void SetText(Control item, string text)
		{
			BeginInvoke(new void_Control_string(SetTextInvoked), new object[] {item, text});
		}
		
		void SetTextInvoked(Control item, string text)
		{
			item.Text = Service.Parse(text);
		}
		
		void AbortButtonClick(object sender, EventArgs e)
		{
			cancelUpdate = true;
			abortButton.Visible = false;
			updateThread.Join();
			startButton.Visible = true;
			errorLabel.Text = Service.Parse("${res:AutoUpdate.Aborted}");
		}
		
		void UpdateAborted()
		{
			abortButton.Visible = false;
			startButton.Visible = true;
		}
		
		void UpdateFormClosing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			if (abortButton.Visible)
				abortButton.PerformClick();
		}
	}
}
