// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.IO;
using System.Net;
using System.Xml;
using ICSharpCode.Core;
using ICSharpCode.Core.AddIns;
using DGNetwork.Services;

namespace DGNetwork.AddIns.AutoUpdate
{
	public class UpdateInfo
	{
		AddIn addin;
		
		public AddIn Addin {
			get {
				return addin;
			}
		}
		
		public UpdateInfo(AddIn addin)
		{
			this.addin = addin;
			this.currentVersion = VersionNumber.Parse(addin.Version);
		}
		
		VersionNumber currentVersion;
		
		public VersionNumber CurrentVersion {
			get {
				return currentVersion;
			}
		}
		
		VersionNumber availableVersion;
		
		public VersionNumber AvailableVersion {
			get {
				return availableVersion;
			}
		}
		
		ArrayList archives = new ArrayList();
		
		public ArrayList Archives {
			get {
				return archives;
			}
		}
		
		string errorText = null;
		
		public string ErrorText {
			get {
				return errorText;
			}
			set {
				errorText = value;
			}
		}
		
		public void DownloadInformation()
		{
			XmlElement xml;
			try {
				xml = InformationDownloader.GetInformation(addin.UpdateUrl);
			} catch (XmlException ex) {
				errorText = ex.Message;
				return;
			} catch (WebException ex) {
				errorText = ex.Message;
				return;
			}
			if (xml == null) {
				errorText = "Could not get information";
				return;
			}
			ProcessInformation(xml);
		}
		
		void ProcessInformation(XmlElement xml)
		{
			switch (xml.Name) {
				case "error":
					errorText = xml.InnerText;
					break;
				case "updateInfo":
					availableVersion = VersionNumber.Parse(xml.GetAttribute("version"));
					errorText = null;
					foreach (XmlNode node in xml) {
						if (node is XmlElement) {
							XmlElement e = (XmlElement)node;
							switch (e.Name) {
								case "archive":
									UpdateArchive ar = new UpdateArchive(this, e);
									if (ar.ExtractTo.StartsWith("~")) {
										string addindir = Path.GetDirectoryName(addin.FileName);
										if (!addindir.EndsWith("/") && !addindir.EndsWith("\\"))
											addindir += "/";
										if (ar.ExtractTo == "~")
											ar.ExtractTo = addindir;
										else
											ar.ExtractTo = addindir + ar.ExtractTo.Substring(2);
									}
									if (ar.ExtractTo.Length > 1 && ar.ExtractTo.EndsWith("/"))
										ar.ExtractTo = ar.ExtractTo.Substring(0, ar.ExtractTo.Length - 1);
									archives.Add(ar);
									break;
							}
						}
					}
					break;
				default:
					errorText = "unknown element: " + xml.OuterXml;
					break;
			}
		}
	}
}
