// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Drawing;
using System.Reflection;
using System.Resources;
using System.Windows.Forms;
using AxMOZILLACONTROLLib;
using DGNetwork.Services;

namespace DGNetwork.AddIns.Browser
{
	public class MozillaBrowser : UserControl
	{
		AxMozillaBrowser web;
		public MozillaDisplay Display;
		
		public MozillaBrowser()
		{
			try {
				ResourceManager resources = new ResourceManager("Mozilla", Assembly.GetExecutingAssembly());
				web = new AxMozillaBrowser();
				((System.ComponentModel.ISupportInitialize)(web)).BeginInit();
				web.Enabled = true;
				web.OcxState = (AxHost.State)resources.GetObject("browser.OcxState");
				web.Dock = DockStyle.Fill;
				this.Controls.Add(web);
				((System.ComponentModel.ISupportInitialize)(web)).EndInit();
				web.BeforeNavigate2 += new DWebBrowserEvents2_BeforeNavigate2EventHandler(BeforeNavigate2);
			} catch {
				MakeErrorLabel();
			}
		}
		
		private void BeforeNavigate2(object sender, DWebBrowserEvents2_BeforeNavigate2Event e)
		{
			try {
				string url = (string)e.uRL;
				if (url != correctURL) {
					e.cancel = true;
					Display.HasNavigatedTo(url);
				}
			} catch (Exception ex) {
				Service.Message.ShowError(ex);
			}
		}
		
		const string downloadURL = "http://www.iol.ie/~locka/mozilla/mozilla.htm";
		
		private void MakeErrorLabel()
		{
			if (web != null)
				web.Dispose();
			web = null;
			this.Controls.Clear();
			LinkLabel lbl = new LinkLabel();
			lbl.LinkClicked += new LinkLabelLinkClickedEventHandler(LinkClicked);
			string txt = Service.Parse("${res:MozillaBrowser.LoadError}");
			lbl.Text = txt.Replace("%URL%", downloadURL);
			int linkStart = txt.IndexOf("%URL%");
			if (linkStart >= 0) {
				lbl.LinkArea = new LinkArea(linkStart, downloadURL.Length);
			}
			lbl.Dock = DockStyle.Fill;
			lbl.TextAlign = ContentAlignment.MiddleCenter;
			this.Controls.Add(lbl);
		}
		
		private void LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
		{
			Service.Browser.OpenWindow(downloadURL);
		}
		
		string correctURL;
		
		public void Navigate(string url)
		{
			correctURL = url;
			if (web == null)
				return;
			object a = Type.Missing;
			try {
				web.Navigate(url, ref a, ref a, ref a, ref a);
			} catch {
				MakeErrorLabel();
			}
		}
	}
}
