// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Xml;
using ICSharpCode.Core.Properties;
using DGNetwork.Contacts;
using DGNetwork.Chat;
using DGNetwork.DGN;
using DGNetwork.Services;

namespace DGNetwork.AddIns.Discovery
{
	public class DiscoveryContact : DefaultContact, IXmlConvertable
	{
		int discoveryID;
		
		public int DiscoveryID {
			get {
				return discoveryID;
			}
		}
		
		public DiscoveryContact(DiscoveryContact contact) : base(contact)
		{
			this.discoveryID = contact.discoveryID;
			Init();
		}
		
		public DiscoveryContact(DiscoverySearchResult result)
		{
			this.Name = result.Name;
			this.discoveryID = result.DiscoveryID;
			Init();
		}
		
		private void Init()
		{
			this.ContextMenuTreePath = "/Context/Discovery/Contact";
			Service.DGN.RegisterForStatusExchange(Properties.Network, discoveryID);
			UpdateIcon();
		}
		
		private void UpdateIcon()
		{
			int distance = Properties.Network.GetDistance(discoveryID);
			if (distance >= 0) {
				this.Icon = "Icons.Status." + Service.DGN.GetStatus(Properties.Network, discoveryID);
				if (privateChatRoom != null)
					privateChatRoom.CanSend = true;
			} else {
				this.Icon = "Icons.Status.Offline";
				if (privateChatRoom != null)
					privateChatRoom.CanSend = false;
			}
		}
		
		public DiscoveryContact()
		{
			// needed to restore from xml
		}
		
		public object FromXmlElement(XmlElement element)
		{
			Service.Contact.LoadCommonProperties(element, this);
			discoveryID = XmlConvert.ToInt32(element.GetAttribute("discoveryID"));
			Init();
			return this;
		}
		
		public XmlElement ToXmlElement(XmlDocument doc)
		{
			XmlElement element = Service.Contact.GetElementWithCommonProperties(doc, this);
			element.SetAttribute("discoveryID", XmlConvert.ToString(discoveryID));
			return element;
		}
		
		public override void Update()
		{
			UpdateIcon();
			base.Update();
		}
		
		public override void DoubleClick()
		{
			Service.Chat.OpenChat(this.PrivateChatRoom);
			// System.Windows.Forms.Form frm = Service.Chat.CreateChatForm();
			// Service.Window.ManageWindowSize("ContactDoubleClick", frm);
		}
		
		PrivateChatRoom privateChatRoom;
		
		public PrivateChatRoom PrivateChatRoom {
			get {
				if (privateChatRoom == null) {
					privateChatRoom = this.User.PrivateChatRoom;
					privateChatRoom.CanSend = this.Icon != "Icons.Status.Offline";
				}
				return privateChatRoom;
			}
		}
		
		DGNetworkUser user;
		
		public DGNetworkUser User {
			get {
				if (user == null) {
					user = Service.DGN.GetDGNetworkUser(Properties.Network, discoveryID);
					user.Name = this.Name;
				}
				return user;
			}
		}
		
		protected override void OnNameChanged(EventArgs e)
		{
			if (user != null)
				user.Name = this.Name;
			base.OnNameChanged(e);
		}
		
		public override string ToString()
		{
			return String.Format("[{0}: id = {1} Name = {2}]", GetType().Name, discoveryID, this.Name);
		}
		
		public override System.Drawing.FontStyle GetFontStyle(IChatRoom currentRoom)
		{
			if (currentRoom != null && currentRoom == privateChatRoom)
				return System.Drawing.FontStyle.Bold | System.Drawing.FontStyle.Underline;
			else
				return base.GetFontStyle(currentRoom);
		}
	}
}
