// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Drawing;
using System.Windows.Forms;

using DGNetwork.Gui;
using DGNetwork.Gui.Dialogs;
using DGNetwork.Services;
using DGNetwork.Settings;
using Grunwald.Gui.Settings;
using Grunwald.Gui.Settings.Editors;

namespace DGNetwork.AddIns.Discovery
{
	public class CustomSetting : SingleControlEditor
	{
		public CustomSetting(ISetting setting) : base(setting, CreateControl())
		{
		}
		
		private static Control CreateControl()
		{
			XmlControl xml = new XmlControl("DiscoverySettingPanel", new CustomSettingSink());
			return xml.Control;
		}
		
		private class CustomSettingSink
		{
			private Label usernameLabel = null;
			
			private void ControlLoad(object sender, EventArgs e)
			{
				usernameLabel.Text = DGNetwork.AddIns.Discovery.Properties.Username;
			}
			
			private void ChangeAccountClick(object sender, EventArgs e)
			{
				WizardDialog dlg = new WizardDialog(new Wizard(true));
				using (XmlForm f = new XmlForm("WizardDialog", dlg)) {
					f.Form.ShowDialog();
				}
				usernameLabel.Text = DGNetwork.AddIns.Discovery.Properties.Username;
			}
			
			private void EditProfileClick(object sender, EventArgs e)
			{
				Service.Browser.OpenWindow(URLs.EditProfile);
			}
		}
		
		protected override void LoadEditValue() { }
		
		/// <summary>
		/// Creates the controls needed for editing the <see cref="ISetting"/> and
		/// adds them to the specified parent control.
		/// </summary>
		public override void CreateControls(Rectangle bounds, EditorProperties properties)
		{
			base.CreateControls(bounds, properties);
		}
		
		public override LabelMode LabelMode {
			get {
				return LabelMode.NoLabel;
			}
		}
	}
}
