// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Windows.Forms;
using DGNetwork.Chat;
using DGNetwork.Contacts;
using DGNetwork.Services;
using DGNetwork.DGN;
using DGNetwork.DGN.MUC;

namespace DGNetwork.AddIns.Discovery
{
	public class DiscoveryDataObjectConverter : IDataObjectConverter
	{
		public object GetObject(IDataObject d, object dropTarget)
		{
			return null;
		}
		
		public DragDropEffects GetEffects(IDataObject d, object dropTarget)
		{
			return DragDropEffects.None;
		}
		
		DiscoveryContact GetContact(object o)
		{
			ContactWrapperTag tag = o as ContactWrapperTag;
			return tag == null ? null : tag.Contact as DiscoveryContact;
		}
		
		IChatRoom GetChatRoom(object target)
		{
			IEditBox box = target as IEditBox;
			if (box != null)
				return box.Room;
			RoomUserView userView = target as RoomUserView;
			if (userView != null)
				return userView.Room;
			return null;
		}
		
		bool IsValidTargetChatRoom(object target)
		{
			IChatRoom room = GetChatRoom(target);
			if (room is MUCRoom)
				return true;
			PrivateChatRoom privateRoom = room as PrivateChatRoom;
			if (privateRoom == null)
				return false;
			return privateRoom.User is DGNetworkUser;
		}
		
		public DragDropEffects GetEffects(object o, object dropTarget)
		{
			if (IsValidTargetChatRoom(dropTarget) && GetContact(o) != null)
				return DragDropEffects.Link;
			else
				return DragDropEffects.None;
		}
		
		public string GetText(object o, object dropTarget)
		{
			DiscoveryContact c = GetContact(o);
			if (IsValidTargetChatRoom(dropTarget) && c != null) {
				IChatRoom room = GetChatRoom(dropTarget);
				if (room is MUCRoom) {
					((MUCRoom)room).Invite(c.User);
				} else {
					MUCRoom mRoom = MUCService.CreateRoom(c.User, (DGNetworkUser)((PrivateChatRoom)room).User);
					if (mRoom != null)
						Service.Chat.OpenChat(mRoom, true);
				}
				return "";
			} else {
				return null;
			}
		}
	}
}
