// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Text;
using Grunwald.P2PLib.DGNetwork;
using DGNetwork.DGN;
using DGNetwork.Services;

namespace DGNetwork.AddIns.Discovery
{
	/// <summary>
	/// Description of DiscoveryServer.
	/// </summary>
	public class DiscoveryServer : DgnIPServer
	{
		string origURL;
		
		internal DiscoveryServer(Network network) : base(URLs.IPServer, Service.P2P.Server)
		{
			this.RequestHeaders.Add("User-Agent", Service.Parse("${AppIdentification}"));
			this.Encoding = Encoding.GetEncoding("ISO8859-1");
			
			origURL = base.Url;
			LoginChanged(null, null);
			this.UserID = Properties.UserID;
			this.ReceivingUser = Service.DGN.ReceivingUser;
			
			network.Initialized += new EventHandler(NetworkInitialized);
			Properties.LoginChanged += new EventHandler(LoginChanged);
		}
		
		private void NetworkInitialized(object sender, EventArgs e)
		{
			if (this.UserID != 0) {
				this.UpdateUserAddress(((Network)sender).Unwrap());
			}
		}
		
		private void LoginChanged(object sender, EventArgs e)
		{
			this.Url = origURL + "?username=" + Helper.EscapePart(Properties.Username, this.Encoding);
			this.Password = Properties.Password;
		}
		
		protected override void OnUserIDChanged(EventArgs e)
		{
			Properties.UserID = this.UserID;
			base.OnUserIDChanged(e);
		}
	}
}
