// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Drawing;
using System.Net;
using System.Text;
using System.Xml;
using System.Windows.Forms;
using DGNetwork.Services;
using DGNetwork.Gui;
using ICSharpCode.Core.Services;

namespace DGNetwork.AddIns.Discovery
{
	public class DiscoveryWizard
	{
		internal Wizard wizard;
		internal RadioButton newAccountButton = null;
		internal RadioButton existingAccountButton = null;
		private Control loginGroup = null;
		private TextBox usernameTextBox = null;
		private TextBox passwordTextBox = null;
		
		private void RadioClick(object sender, EventArgs e)
		{
			wizard.Container.NextEnabled = true;
			loginGroup.Enabled = existingAccountButton.Checked;
		}
		
		Control oldControl;
		
		public void Next()
		{
			if (newAccountButton.Checked) {
				RegisterDialogSink sink = new RegisterDialogSink();
				sink.wizard = this.wizard;
				XmlControl ctl = new XmlControl("DiscoveryRegisterWizard", sink);
				//${res:Discovery.RegisterTitle}
				wizard.Container.SetControl(ctl.Control);
				wizard.Container.BackEnabled = true;
				wizard.Container.NextText = "${res:Discovery.SkipRegistration}";
				wizard.Page = 1;
			} else {
				if (usernameTextBox.Text == "") {
					Service.Message.ShowMessage("${res:Discovery.NeedUsername}");
					return;
				}
				if (passwordTextBox.Text == "") {
					Service.Message.ShowMessage("${res:Discovery.NeedPassword}");
					return;
				}
				Label lbl = new Label();
				lbl.Dock = DockStyle.Fill;
				lbl.TextAlign = ContentAlignment.MiddleCenter;
				lbl.Text = Service.Parse("${res:Discovery.LoggingIn}");
				oldControl = wizard.Container.Control;
				wizard.Container.NextEnabled = false;
				wizard.Container.Control = lbl;
				
				Encoding iso8859 = Encoding.GetEncoding("ISO8859-1");
				StringBuilder b = new StringBuilder(URLs.MainDiscovery);
				b.Append("?mode=checkaccount&username=");
				b.Append(Helper.EscapePart(usernameTextBox.Text, iso8859));
				b.Append("&password=");
				b.Append(Helper.EscapePart(passwordTextBox.Text, iso8859));
				url = b.ToString();
				new MethodInvoker(TryLogin).BeginInvoke(null, null);
			}
		}
		
		string url;
		int userid;
		
		private void TryLogin()
		{
			try {
				// TODO: Use HttpRequest with smaller timeout for getting the xml
				//     (that's code that should be reused!)
				XmlDocument doc = new XmlDocument();
				doc.Load(url);
				XmlElement e = doc.DocumentElement;
				if (e.Name == "error") {
					wizard.Container.Invoke(new void_string(RestoreWizard), e.InnerText);
					return;
				}
				if (e.Name != "discovery") {
					Service.Message.ShowError("unknown root tag " + e.Name);
					wizard.Container.Invoke(new MethodInvoker(RestoreWizard));
					return;
				}
				userid = XmlConvert.ToInt32(e.GetAttribute("userid"));
				wizard.Container.Invoke(new MethodInvoker(LoginSuccess));
			} catch (WebException ex) {
				Service.Message.ShowError(ex.Message);
				wizard.Container.Invoke(new MethodInvoker(RestoreWizard));
			} catch (Exception ex) {
				Service.Message.ShowError(ex);
				wizard.Container.Invoke(new MethodInvoker(RestoreWizard));
			}
		}
		
		private void LoginSuccess()
		{
			try {
				Properties.Username = usernameTextBox.Text;
				Properties.UserID = userid;
				Properties.Password = passwordTextBox.Text;
				Service.Message.ShowMessageFormatted("${res:Discovery.LoginSuccessful}", new string [] {Properties.Username});
				wizard.Container.NextWizard();
			} catch (Exception ex) {
				Service.Message.ShowError(ex);
			}
		}
		
		private delegate void void_string(string a1);
		
		private void RestoreWizard(string error)
		{
			Service.Message.ShowMessage(error);
			RestoreWizard();
		}
		
		private void RestoreWizard()
		{
			wizard.Container.SetControl(oldControl);
			wizard.Container.NextEnabled = true;
			passwordTextBox.Focus();
			passwordTextBox.SelectAll();
		}
		
		private void PasswordLostClick(object sender, EventArgs e)
		{
			Service.Browser.OpenWindow(URLs.PasswordLost);
		}
	}
}
