// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.IO;
using System.Xml;

namespace ICQTest
{
	class Generator
	{
		const string Header = @"// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file ""LICENSE.txt"".

// 
//      WARNING !
// This file has been autogenerated from a xml file. Do not change it, your changes
// will be overwritten!
//


using System;
using System.Collections;
using System.IO;

namespace Grunwald.NOscar
{";
		const string Footer = "}";
		
		static ArrayList fixedTypes = new ArrayList();
		
		public static int Main(string[] args)
		{
			if (args.Length != 2) {
				Console.WriteLine("Usage: BuildSnacs inputXml outputCSharp");
				return 2;
			}
			try {
				XmlDocument doc = new XmlDocument();
				doc.Load(args[0]);
				ArrayList elements = new ArrayList();
				foreach (XmlNode node in doc.DocumentElement) {
					if (node is XmlElement) {
						elements.Add(node);
					}
				}
				
				using (StreamWriter writer = new StreamWriter(args[1])) {
					writer.WriteLine(Header);
					
					// Generate SnacFactory:
					writer.WriteLine("\t#region SnacFactory");
					writer.WriteLine("\tpublic sealed class SnacFactory");
					writer.WriteLine("\t{");
					writer.WriteLine("\t\tpublic Snac CreateSnac(byte[] data, int index, int count)");
					writer.WriteLine("\t\t{");
					writer.WriteLine("\t\t\tushort family = NetworkBitConverter.ToUInt16(data, index);");
					writer.WriteLine("\t\t\tushort subType = NetworkBitConverter.ToUInt16(data, index + 2);");
					foreach (XmlElement element in elements) {
						if (element.Name.EndsWith("Struct"))
							continue;
						writer.WriteLine("\t\t\tif (family == {0}.SnacFamily && subType == {0}.SnacType)", element.GetAttribute("name"));
						writer.WriteLine("\t\t\t\treturn new {0}(data, index, count);", element.GetAttribute("name"));
					}
					writer.WriteLine("\t\t\treturn new DataSnac(data, index, count);");
					writer.WriteLine("\t\t}");
					writer.WriteLine("\t}");
					writer.WriteLine("\t#endregion");
					
					ArrayList subElements = new ArrayList();
					foreach (XmlElement element in elements) {
						string name = element.GetAttribute("name");
						if (element.Name == "CustomSnac")
							continue;
						string bitConverter = "NetworkBitConverter";
                        
						subElements.Clear();
						foreach (XmlNode subNode in element) {
							if (subNode is XmlElement)
								subElements.Add(subNode);
						}
						
						writer.WriteLine("\t");
						writer.WriteLine("\t#region {0}", name);
						if (element.Name == "CustomStruct") {
							writer.WriteLine("\tpublic sealed class {0} : IByteConvertible", name);
						} else if (element.Name == "LEStruct") {
							writer.WriteLine("\tpublic sealed class {0} : IByteConvertible", name);
							bitConverter = "LEBitConverter";
						} else if (element.Name == "DataSnac") {
							writer.WriteLine("\tpublic sealed class {0} : Snac", name);
						} else {
							writer.WriteLine("\tpublic sealed class {0} : TlvBasedSnac", name);
						}
						writer.WriteLine("\t{");
						bool isFixedSize = false;
						if (element.Name.EndsWith("Struct")) {
							isFixedSize = WriteFixSize(writer, subElements, name);
						} else {
							writer.WriteLine("\t\t/// <summary>The family id normally used for the {0}.</summary>", name);
							writer.WriteLine("\t\tpublic static ushort SnacFamily = {0};", element.GetAttribute("family"));
							writer.WriteLine("\t\t/// <summary>The SNAC subtype normally used for the {0}.</summary>", name);
							writer.WriteLine("\t\tpublic static ushort SnacType = {0};", element.GetAttribute("subType"));
							writer.WriteLine("\t\t");
						}
						writer.WriteLine("\t\t/// <summary>Creates a new {0}.</summary>", name);
						foreach (XmlElement subElement in subElements) {
							if (subElement.HasAttribute("default"))
								continue;
							if (subElement.InnerText != null && subElement.InnerText.Length > 0)
								writer.WriteLine("\t\t/// <param name=\"{0}\">{1}</param>", GetParameterName(subElement), subElement.InnerText.Trim());
						}
						writer.Write("\t\tpublic {0}(", name);
						int num = 0;
						foreach (XmlElement subElement in subElements) {
							if (subElement.HasAttribute("default"))
								continue;
							if (num > 0)
								writer.Write(", ");
							num += 1;
							if (subElement.HasAttribute("enum"))
								writer.Write(subElement.GetAttribute("enum"));
							else
								writer.Write(GetType(subElement));
							writer.Write(' ');
							writer.Write(GetParameterName(subElement));
						}
						writer.Write(")");
						if (element.Name.EndsWith("Struct"))
							writer.WriteLine();
						else
							writer.WriteLine(" : base(SnacFamily, SnacType)");
						writer.WriteLine("\t\t{");
						foreach (XmlElement subElement in subElements) {
							if (subElement.HasAttribute("default")) {
								if (subElement.Name == "String")
									writer.WriteLine("\t\t\tthis.{0} = \"{1}\";", subElement.GetAttribute("name"), subElement.GetAttribute("default"));
								else
									writer.WriteLine("\t\t\tthis.{0} = {1};", subElement.GetAttribute("name"), subElement.GetAttribute("default"));
							} else {
								writer.WriteLine("\t\t\tthis.{0} = {1};", subElement.GetAttribute("name"), GetParameterName(subElement));
							}
						}
						writer.WriteLine("\t\t}");
						writer.WriteLine("\t\t");
						writer.WriteLine("\t\t/// <summary>Loads the {0} from the byte array.</summary>", name);
						writer.Write("\t\tpublic {0}(byte[] data, int index, int count)", name);
						if (element.Name.EndsWith("Struct"))
							writer.WriteLine();
						else
							writer.WriteLine(" : base(data, index, count)");
						writer.WriteLine("\t\t{");
						if (element.Name != "TlvSnac" && subElements.Count > 0) {
							// read information
							writer.Write("\t\t\tint pos = index");
							if (element.Name.EndsWith("Struct"))
								writer.WriteLine(";");
							else
								writer.WriteLine(" + base.GetByteLength();");
							foreach (XmlElement subElement in subElements) {
								string subName = "this." + GetParameterName(subElement);
								switch (subElement.Name) {
									case "Byte":
										writer.WriteLine("\t\t\t{0} = data[pos++];", subName);
										break;
									case "Short":
										writer.WriteLine("\t\t\t{0} = {1}.ToUInt16(data, pos);", subName, bitConverter);
										writer.WriteLine("\t\t\tpos += 2;");
										break;
									case "Int":
										writer.WriteLine("\t\t\t{0} = {1}.ToUInt32(data, pos);", subName, bitConverter);
										writer.WriteLine("\t\t\tpos += 4;");
										break;
									case "Long":
										writer.WriteLine("\t\t\t{0} = {1}.ToUInt64(data, pos);", subName, bitConverter);
										writer.WriteLine("\t\t\tpos += 8;");
										break;
									case "UnixDate":
										writer.WriteLine("\t\t\t{0} = new DateTime(1970, 1, 1).AddSeconds({1}.ToUInt32(data, pos)).ToLocalTime();", subName, bitConverter);
										writer.WriteLine("\t\t\tpos += 4;");
										break;
									case "String":
										writer.WriteLine("\t\t\t{0} = Tlv.Encoding.GetString(data, pos + 1, data[pos]);", subName);
										writer.WriteLine("\t\t\tpos += 1 + data[pos];", subName);
										break;
									case "ZString":
										writer.WriteLine("\t\t\t{0} = MirabilisTlv.ReadString(data, ref pos);", subName);
										break;
									case "Array":
										bool needFillArray = true;
										if (subElement.HasAttribute("length")) {
											if (subElement.GetAttribute("length") == "defined") {
												writer.WriteLine("\t\t\t{1} = new {0}[{2}.ToUInt16(data, pos)];", GetTypeInternal(subElement.GetAttribute("of")), subName, bitConverter);
												writer.WriteLine("\t\t\tpos += 2;");
											} else {
												writer.WriteLine("\t\t\t{1} = new {0}[{2}];", GetTypeInternal(subElement.GetAttribute("of")), subName, subElement.GetAttribute("length"));
											}
										} else {
											string fixedSize = GetFixedSizeInternal(subElement.GetAttribute("of"));
											if (fixedSize != null) {
												writer.WriteLine("\t\t\t{2} = new {0}[(count + index - pos) / {1}];", GetTypeInternal(subElement.GetAttribute("of")), fixedSize, subName);
											} else {
												writer.WriteLine("\t\t\tArrayList _arr = new ArrayList();");
												writer.WriteLine("\t\t\twhile (pos < index + count) {");
												writer.WriteLine("\t\t\t\t_arr.Add(new {0}(data, pos, count + index - pos));", subElement.GetAttribute("of"));
												writer.WriteLine("\t\t\t\tpos += (({0})_arr[_arr.Count - 1]).GetByteLength();", subElement.GetAttribute("of"));
												writer.WriteLine("\t\t\t}");
												writer.WriteLine("\t\t\t{0} = ({1}[])_arr.ToArray(typeof({1}));", subName, subElement.GetAttribute("of"));
												needFillArray = false;
											}
										}
										if (needFillArray) {
											writer.Write("\t\t\tfor (int i = 0; i < {0}.Length; ++i)", subName);
											writer.WriteLine(" {");
											switch (subElement.GetAttribute("of")) {
												case "Byte":
													writer.WriteLine("\t\t\t\t{0}[i] = data[pos++];", subName);
													break;
												case "Short":
													writer.WriteLine("\t\t\t\t{0}[i] = {1}.ToUInt16(data, pos);", subName, bitConverter);
													writer.WriteLine("\t\t\t\tpos += 2;");
													break;
												case "Int":
													writer.WriteLine("\t\t\t\t{0}[i] = {1}.ToUInt32(data, pos);", subName, bitConverter);
													writer.WriteLine("\t\t\t\tpos += 4;");
													break;
												case "Long":
													writer.WriteLine("\t\t\t\t{0}[i] = {1}.ToUInt64(data, pos);", subName, bitConverter);
													writer.WriteLine("\t\t\t\tpos += 8;");
													break;
												default:
													writer.WriteLine("\t\t\t\t{0}[i] = new {1}(data, pos, count + index - pos);", subName, subElement.GetAttribute("of"));
													writer.WriteLine("\t\t\t\tpos += {0}[i].GetByteLength();", subName);
													break;
											}
											writer.WriteLine("\t\t\t}");
										}
										break;
									default:
										writer.WriteLine("\t\t\t{0} = new {1}(data, pos, count + index - pos);", subName, subElement.Name);
										writer.WriteLine("\t\t\tpos += {0}.GetByteLength();", subName);
										break;
								}
							}
						}
						writer.WriteLine("\t\t}");
						
						// Implement IByteConvertible
						if (element.Name != "TlvSnac" && subElements.Count > 0) {
							writer.WriteLine("\t\t");
							writer.WriteLine("\t\t/// <summary>Gets the size of this {0} in bytes.</summary>", name);
							writer.Write("\t\tpublic ");
							if (element.Name == "DataSnac")
								writer.Write("override ");
							writer.WriteLine("int GetByteLength()");
							writer.WriteLine("\t\t{");
							if (isFixedSize) {
								writer.Write("\t\t\treturn FixedSize");
							} else {
								bool hasDynamicArray = false;
								foreach (XmlElement subElement in subElements) {
									if (GetFixedSize(subElement) == null) {
										if (subElement.Name == "Array" && GetFixedSizeInternal(subElement.GetAttribute("of")) == null)
											hasDynamicArray = true;
									}
								}
								if (hasDynamicArray)
									writer.Write("\t\t\tint size = ");
								else
									writer.Write("\t\t\treturn ");
								int num2 = 0;
								if (element.Name == "DataSnac") {
									num2 = 1;
									writer.Write("base.GetByteLength()");
								}
								foreach (XmlElement subElement in subElements) {
									if (num2 > 0)
										writer.Write(" + ");
									num2 += 1;
									string size = GetFixedSize(subElement);
									if (size == null) {
										if (subElement.Name == "Array") {
											if (subElement.GetAttribute("length") == "defined")
												writer.Write("2");
											else
												writer.Write("0");
											size = GetFixedSizeInternal(subElement.GetAttribute("of"));
											if (size != null) {
												writer.Write(" + {0}.Length * {1}", GetParameterName(subElement), size);
											}
										} else if (subElement.Name == "String") {
											writer.Write("1 + Tlv.Encoding.GetByteCount(" + GetParameterName(subElement) + ")");
										} else if (subElement.Name == "ZString") {
											writer.Write("3 + Tlv.Encoding.GetByteCount(" + GetParameterName(subElement) + ")");
										} else {
											writer.Write(GetParameterName(subElement) + ".GetByteLength()");
										}
									} else {
										writer.Write(size);
									}
								}
								if (hasDynamicArray) {
									writer.WriteLine(";");
									foreach (XmlElement subElement in subElements) {
										if (GetFixedSize(subElement) == null) {
											if (subElement.Name == "Array" && GetFixedSizeInternal(subElement.GetAttribute("of")) == null) {
												writer.Write("\t\t\tforeach ({0} a in {1})", GetTypeInternal(subElement.GetAttribute("of")), GetParameterName(subElement));
												writer.WriteLine(" {");
												writer.WriteLine("\t\t\t\tsize += a.GetByteLength();");
												writer.WriteLine("\t\t\t}");
											}
										}
									}
									writer.Write("\t\t\treturn size");
								}
							}
							writer.WriteLine(";");
							writer.WriteLine("\t\t}");
							// end of get size in bytes
							
							writer.WriteLine("\t\t");
							writer.WriteLine("\t\t/// <summary>Writes the byte representation of the {0} into the specified stream.</summary>", name);
							writer.Write("\t\tpublic ");
							if (element.Name == "DataSnac")
								writer.Write("override ");
							writer.WriteLine("void WriteBytes(Stream stream)");
							writer.WriteLine("\t\t{");
							if (element.Name == "DataSnac")
								writer.WriteLine("\t\t\tbase.WriteBytes(stream);");
							
							bool firstString = true;
							foreach (XmlElement subElement in subElements) {
								string subName = GetParameterName(subElement);
								switch (subElement.Name) {
									case "Byte":
										writer.WriteLine("\t\t\tstream.WriteByte({0});", subName);
										break;
									case "Short":
										writer.WriteLine("\t\t\tstream.Write({1}.GetBytes({0}), 0, 2);", subName, bitConverter);
										break;
									case "Int":
										writer.WriteLine("\t\t\tstream.Write({1}.GetBytes({0}), 0, 4);", subName, bitConverter);
										break;
									case "Long":
										writer.WriteLine("\t\t\tstream.Write({1}.GetBytes({0}), 0, 8);", subName, bitConverter);
										break;
									case "UnixDate":
										writer.WriteLine("\t\t\tstream.Write({1}.GetBytes((int){0}.ToUniversalTime().Subtract(new DateTime(1970, 1, 1)).TotalSeconds), 0, 4);", subName, bitConverter);
										break;
									case "String":
										writer.WriteLine("\t\t\t{1}_text = Tlv.Encoding.GetBytes({0});", subName, firstString ? "byte[] " : "");
										firstString = false;
										writer.WriteLine("\t\t\tstream.WriteByte((byte)_text.Length);");
										writer.WriteLine("\t\t\tstream.Write(_text, 0, _text.Length);");
										break;
									case "ZString":
										writer.WriteLine("\t\t\t{1}_text = MirabilisTlv.EncodeString({0});", subName, firstString ? "byte[] " : "");
										firstString = false;
										writer.WriteLine("\t\t\tstream.Write(_text, 0, _text.Length);");
										break;
									case "Array":
										if (subElement.GetAttribute("length") == "defined") {
											writer.WriteLine("\t\t\tstream.Write({1}.GetBytes((ushort){0}.Length), 0, 2);", subName, bitConverter);
										}
										writer.Write("\t\t\tforeach ({0} a in {1})", GetTypeInternal(subElement.GetAttribute("of")), GetParameterName(subElement));
										writer.WriteLine(" {");
										switch (subElement.GetAttribute("of")) {
											case "Byte":
												writer.WriteLine("\t\t\t\tstream.WriteByte(a);");
												break;
											case "Short":
												writer.WriteLine("\t\t\t\tstream.Write({0}.GetBytes(a), 0, 2);", bitConverter);
												break;
											case "Int":
												writer.WriteLine("\t\t\t\tstream.Write({0}.GetBytes(a), 0, 4);", bitConverter);
												break;
											case "Long":
												writer.WriteLine("\t\t\t\tstream.Write({0}.GetBytes(a), 0, 8);", bitConverter);
												break;
											default:
												writer.WriteLine("\t\t\t\ta.WriteBytes(stream);", subName);
												break;
										}
										writer.WriteLine("\t\t\t}");
										break;
									default:
										writer.WriteLine("\t\t\t{0}.WriteBytes(stream);", subName);
										break;
								}
							}
							
							writer.WriteLine("\t\t}");
						}
						
						// Create getter/Setter
						foreach (XmlElement subElement in subElements) {
							writer.WriteLine("\t\t");
							if (element.Name != "TlvSnac") {
								writer.WriteLine("\t\t{0} {1};", GetType(subElement), GetParameterName(subElement));
								writer.WriteLine("\t\t");
							}
							
							if (subElement.InnerText != null && subElement.InnerText.Length > 0) {
								writer.WriteLine("\t\t/// <summary>{0}</summary>", subElement.InnerText.Trim());
							}
							if (subElement.HasAttribute("enum")) {
								writer.Write("\t\tpublic {0} {1} ", subElement.GetAttribute("enum"), subElement.GetAttribute("name"));
								writer.WriteLine("{");
								writer.WriteLine("\t\t\tget {");
								if (element.Name == "TlvSnac")
									writer.WriteLine("\t\t\t\treturn ({2})Chain.{0}({1});", GetGetter(subElement), subElement.GetAttribute("type"), subElement.GetAttribute("enum"));
								else
									writer.WriteLine("\t\t\t\treturn ({1}){0};", GetParameterName(subElement), subElement.GetAttribute("enum"));
							} else {
								writer.Write("\t\tpublic {0} {1} ", GetType(subElement), subElement.GetAttribute("name"));
								writer.WriteLine("{");
								writer.WriteLine("\t\t\tget {");
								if (element.Name == "TlvSnac")
									writer.WriteLine("\t\t\t\treturn Chain.{0}({1});", GetGetter(subElement), subElement.GetAttribute("type"));
								else
									writer.WriteLine("\t\t\t\treturn {0};", GetParameterName(subElement));
							}
							writer.WriteLine("\t\t\t}");
							writer.WriteLine("\t\t\tset {");
							if (IsTypeNullable(subElement)) {
								writer.WriteLine("\t\t\t\tif (value == null)");
								writer.WriteLine("\t\t\t\t\tthrow new ArgumentNullException(\"{0}\");", GetParameterName(subElement));
								if (subElement.Name == "Array" && subElement.HasAttribute("length") && subElement.GetAttribute("length") != "defined") {
									writer.WriteLine("\t\t\t\tif (value.Length != {0})", subElement.GetAttribute("length"));
									writer.WriteLine("\t\t\t\t\tthrow new ArgumentException(\"The array {0} must have exactly {1} elements.\", \"{2}\");", subElement.GetAttribute("name"), subElement.GetAttribute("length"), GetParameterName(subElement));
								}
							}
							if (subElement.HasAttribute("enum")) {
								if (element.Name == "TlvSnac")
									writer.WriteLine("\t\t\t\tChain.{0}({1}, ({2})value);", GetSetter(subElement), subElement.GetAttribute("type"), GetType(subElement));
								else
									writer.WriteLine("\t\t\t\t{0} = ({1})value;", GetParameterName(subElement), GetType(subElement));
							} else {
								if (element.Name == "TlvSnac")
									writer.WriteLine("\t\t\t\tChain.{0}({1}, value);", GetSetter(subElement), subElement.GetAttribute("type"));
								else
									writer.WriteLine("\t\t\t\t{0} = value;", GetParameterName(subElement));
							}
							writer.WriteLine("\t\t\t}");
							writer.WriteLine("\t\t}");
						}
						writer.WriteLine("\t}");
						writer.WriteLine("\t#endregion");
					}
					
					writer.WriteLine(Footer);
				}
				return 0;
			} catch (Exception ex) {
				Console.WriteLine(ex);
				return 1;
			}
		}
		
		static bool WriteFixSize(TextWriter writer, ArrayList subElements, string name)
		{
			foreach (XmlElement subElement in subElements) {
				if (GetFixedSize(subElement) == null)
					return false;
			}
			writer.WriteLine("\t\t/// <summary>The {0} has always this size in bytes.</summary>", name);
			writer.Write("\t\tpublic const int FixedSize = ");
			int num = 0;
			foreach (XmlElement subElement in subElements) {
				if (num > 0)
					writer.Write(" + ");
				num += 1;
				writer.Write(GetFixedSize(subElement));
			}
			writer.WriteLine(";");
			writer.WriteLine("\t\t");
			fixedTypes.Add(name);
			return true;
		}
		
		static string GetFixedSize(XmlElement element)
		{
			if (element.Name == "Array") {
				if (element.HasAttribute("length")) {
					if (element.GetAttribute("length") != "defined")
						return element.GetAttribute("length") + " * " + GetFixedSizeInternal(element.GetAttribute("of"));
				}
			}
			return GetFixedSizeInternal(element.Name);
		}
		
		static string GetFixedSizeInternal(string elementName)
		{
			switch (elementName) {
				case "Byte":
					return "1";
				case "Short":
					return "2";
				case "Int":
				case "UnixDate":
					return "4";
				case "Long":
					return "8";
				default:
					if (fixedTypes.Contains(elementName))
						return elementName + ".FixedSize";
					else
						return null;
			}
		}
		
		static string GetType(XmlElement element)
		{
			if (element.Name == "Array")
				return GetTypeInternal(element.GetAttribute("of")) + "[]";
			else
				return GetTypeInternal(element.Name);
		}
		
		static bool IsTypeNullable(XmlElement element)
		{
			switch (element.Name) {
				case "Short":
				case "Int":
				case "Byte":
				case "UnixDate":
				case "Long":
					return false;
				default:
					return true;
			}
		}
		
		static string GetTypeInternal(string name)
		{
			switch (name) {
				case "Data":
					return "byte[]";
				case "String":
				case "ZString":
					return "string";
				case "Short":
					return "ushort";
				case "Int":
					return "uint";
				case "Byte":
					return "byte";
				case "UnixDate":
					return "DateTime";
				case "Long":
					return "ulong";
				default:
					return name;
			}
		}
		
		static string GetParameterName(XmlElement element)
		{
			string name = element.GetAttribute("name");
			return name.Substring(0, 1).ToLower() + name.Substring(1);
		}
		
		static string GetGetter(XmlElement element)
		{
			return "Get" + element.Name;
		}
		
		static string GetSetter(XmlElement element)
		{
			return "Set" + element.Name;
		}
	}
}
