// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.IO;
using System.Security.Cryptography;

namespace Grunwald.NOscar
{
	/// <summary>
	/// This sequence is used to authenticate with the login server.
	/// </summary>
	public sealed class AuthenticationSequence : SnacSequence
	{
		public AuthenticationSequence(SnacConnection connection) : base(connection) {}
		
		string screenName, password;
		
		public void Start(string screenName, string password)
		{
			this.screenName = screenName;
			this.password = password;
			SendSnac(new KeyRequestSnac(screenName), new SnacEventHandler(Step1));
		}
		
		void Step1(object sender, SnacEventArgs e)
		{
			if (e.Snac is AuthResponseSnac) {
				OnCompleted(e);
				return;
			}
			KeyResponseSnac response = e.Snac as KeyResponseSnac;
			if (response == null) {
				UnknownResponse(e);
				return;
			}
			byte[] passwordBytes;
			using (MemoryStream ms = new MemoryStream()) {
				ms.Write(response.Key, 0, response.Key.Length);
				byte[] passBytes = Tlv.Encoding.GetBytes(password);
				ms.Write(passBytes, 0, passBytes.Length);
				byte[] aim = Tlv.Encoding.GetBytes("AOL Instant Messenger (SM)");
				ms.Write(aim, 0, aim.Length);
				ms.Position = 0;
				using (MD5CryptoServiceProvider md5 = new MD5CryptoServiceProvider()) {
					passwordBytes = md5.ComputeHash(ms);
				}
			}
			AuthRequestSnac requestSnac = new AuthRequestSnac(screenName, passwordBytes);
			SendSnac(requestSnac, new SnacEventHandler(Step2));
		}
		
		void Step2(object sender, SnacEventArgs e)
		{
			if (e.Snac is AuthResponseSnac)
				OnCompleted(e);
			else
				UnknownResponse(e);
		}
		
		void UnknownResponse(SnacEventArgs e)
		{
			OnError(e);
		}
	}
}
