// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Text;

namespace Grunwald.NOscar
{
	/// <summary>
	/// Contains the information about a connection.
	/// </summary>
	public sealed class ConnectionData
	{
		FamilyVersion[] versions;
		
		/// <summary>Gets/Sets the family versions supported by the server.</summary>
		public FamilyVersion[] ServerVersions {
			get {
				return versions;
			}
			set {
				versions = value;
			}
		}
		
		RateClassInfo[] rateInfoBlocks;
		
		/// <summary>Gets/Sets the rate information blocks.</summary>
		public RateClassInfo[] RateInfoBlocks {
			get {
				return rateInfoBlocks;
			}
			set {
				rateInfoBlocks = value;
			}
		}
		
		RateClassCommand[] rateCommandsBlocks;
		
		/// <summary>Gets/Sets the rate commands blocks.</summary>
		public RateClassCommand[] RateCommandsBlocks {
			get {
				return rateCommandsBlocks;
			}
			set {
				rateCommandsBlocks = value;
			}
		}
		
		IcbmParameterInfo icbmParameterInfo;
		
		/// <summary>Gets/Sets the ICBM parameter info.</summary>
		public IcbmParameterInfo IcbmParameterInfo {
			get {
				return icbmParameterInfo;
			}
			set {
				icbmParameterInfo = value;
			}
		}
		
		int maxLocationInfoLength;
		
		/// <summary>Gets/Sets the maximal length of the location information.</summary>
		public int MaxLocationInfoLength {
			get {
				return maxLocationInfoLength;
			}
			set {
				maxLocationInfoLength = value;
			}
		}
		
		SsiItem[] ssiItems;
		
		/// <summary>Server stored information items associated with this connection.</summary>
		public SsiItem[] SsiItems {
			get {
				return ssiItems;
			}
			set {
				ssiItems = value;
			}
		}
	}
}
