// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;

namespace Grunwald.NOscar
{
	/// <summary>
	/// Enumeration containing known flap channel numbers.
	/// </summary>
	public enum FlapChannel : byte
	{
		Unknown = 0,
		/// <summary>
		/// The login channel is only used when initiating a FLAP connection.
		/// </summary>
		Login = 1,
		/// <summary>
		/// The SNAC channel is used for all FLAPs that represent SNACs.
		/// SNACs are used for most of the communication.
		/// </summary>
		Snac = 2,
		/// <summary>
		/// Packets sent over channel 3 are used to indicate errors at the FLAP level of processing.
		/// </summary>
		Error = 3,
		/// <summary>
		/// Packets sent over channel 4 are used to indicate that the connection will be closed.
		/// </summary>
		ConnectionClose = 4,
		/// <summary>
		/// Packets sent over channel 5 can be safely ignore, they are only used to keep the connection alive.
		/// </summary>
		KeepAlive = 5
	}
	
	public enum ConnectionState
	{
		Connecting,
		Connected,
		Negotiating,
		Active,
		Dead
	}
	
	/// <summary>
	/// Enumeration containing the known reasons for authentication failure.
	/// </summary>
	public enum AuthResponseErrorCode : ushort
	{
		Success = 0,
		InvalidNickOrPassword   = 0x0001,
		ServiceUnavailable      = 0x0002,
		UnknownError            = 0x0003,
		IncorrectNickOrPassword = 0x0004,
		MismatchNickOrPassword  = 0x0005,
		/// <summary>Used when the authorizer has received bad input.</summary>
		InternalClientError     = 0x0006,
		InvalidAccount = 0x0007,
		DeletedAccount = 0x0008,
		ExpiredAccount = 0x0009,
		NoAccessToDatabase    = 0x000A,
		NoAccessToResolver    = 0x000B,
		InvalidDatabaseFields = 0x000C,
		BadDatabaseStatus     = 0x000D,
		BadResolverStatus     = 0x000E,
		InternalError         = 0x000F,
		ServiceTemporarilyOffline = 0x0010,
		SuspendedAccount = 0x0011,
		DatabaseSendError    = 0x0012,
		DatabaseLinkError    = 0x0013,
		ReservationMapError  = 0x0014,
		ReservationLinkError = 0x0015,
		TooManyUsersFromThisIP = 0x0016,
		TooManyUsersFromThisIPReservation = 0x0017,
		RateLimitExceededReservation = 0x0018,
		UserTooHeavilyWarned = 0x0019,
		ReservationTimeout = 0x001A,
		UpdateRequired    = 0x001B,
		UpdateRecommended = 0x001C,
		RateLimitExceeded = 0x001D,
		CannotRegister = 0x001E,
		InvalidSecurID = 0x0020,
		AccountSuspendedBecauseOfAge = 0x0022
	}
	
	[Flags]
	public enum UserClass : uint
	{
		/// <summary>User is away.</summary>
		Away = 0x0020,
		/// <summary>User has not responded to signup confirmation email</summary>
		AccountInactive = 0x0001,
		/// <summary>User is an AOL administrator</summary>
		AolAdministrator = 0x0002,
		/// <summary>User is using AIM, but not AOL member</summary>
		NotAolMember = 0x0010,
		/// <summary>User on a mobile device.</summary>
		Mobile = 0x0080,
		/// <summary> User is an ActiveBuddy robot.</summary>
		ActiveBuddyRobot = 0x0400
	}
	
	[Flags]
	public enum IcqStatus : uint
	{
		None = 0,
		Away = 1,
		DoNotDisturb = 2,
		NotAvailable = 4,
		Occupied    = 0x0010,
		FreeForChat = 0x0020,
		Invisible   = 0x0100
	}
}
