// Copyright (C) 2004-2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Runtime.Serialization;

namespace Grunwald.NOscar
{
	#region NOscarException
	/// <summary>
	/// Base exception for all custom exceptions thrown by NOscar.
	/// </summary>
	[Serializable]
	public class NOscarException : ApplicationException
	{
		/// <summary>
		/// Creates a new NOscarException.
		/// </summary>
		public NOscarException()
			: base()
		{
		}
		
		/// <summary>
		/// Creates a new NOscarException.
		/// </summary>
		public NOscarException(string message)
			: base(message)
		{
		}
		
		/// <summary>
		/// Creates a new NOscarException.
		/// </summary>
		public NOscarException(string message, Exception innerException)
			: base(message, innerException)
		{
		}
		
		/// <summary>
		/// Creates a new NOscarException.
		/// </summary>
		protected NOscarException(SerializationInfo info, StreamingContext context)
			: base(info, context)
		{
		}
	}
	#endregion
	
	#region InvalidFlapException
	/// <summary>
	/// The InvalidFlapException is used when a FLAP cannot be read because it contains invalid data.
	/// </summary>
	[Serializable]
	public class InvalidFlapException : NOscarException
	{
		/// <summary>
		/// Creates a new InvalidFlapException.
		/// </summary>
		public InvalidFlapException()
			: base()
		{
		}
		
		/// <summary>
		/// Creates a new InvalidFlapException.
		/// </summary>
		public InvalidFlapException(string message)
			: base(message)
		{
		}
		
		/// <summary>
		/// Creates a new InvalidFlapException.
		/// </summary>
		public InvalidFlapException(string message, Exception innerException)
			: base(message, innerException)
		{
		}
		
		/// <summary>
		/// Creates a new InvalidFlapException.
		/// </summary>
		protected InvalidFlapException(SerializationInfo info, StreamingContext context)
			: base(info, context)
		{
		}
	}
	#endregion
}
