// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;

namespace Grunwald.NOscar
{
	/// <summary>
	/// High level connection to an Oscar server that does automatic initialization.
	/// </summary>
	public sealed class OscarConnection : SnacConnection
	{
		public OscarConnection(FlapConnection flapConnection) : base(flapConnection)
		{
			
		}
		
		/// <summary>
		/// Is called when the SnacConnection has finished negotiation and higher level
		/// connections may start their own negotiation.
		/// If not overridden, this sets the state to active.
		/// </summary>
		protected override void FinishedNegotiation()
		{
			AutoInitializationSequence s = new AutoInitializationSequence(this);
			s.Error += new EventHandler(InitError);
			s.Completed += new EventHandler(InitComplete);
			s.Start();
		}
		
		void InitError(object sender, EventArgs e)
		{
			Close();
		}
		
		void InitComplete(object sender, EventArgs e)
		{
			Send(new ActivateSsiSnac());
			FamilyToolVersion[] tools = {
				new FamilyToolVersion(0x0001, 0x0004, 0x0010, 0x0801),
				new FamilyToolVersion(0x0002, 0x0001, 0x0110, 0x0801),
				new FamilyToolVersion(0x0003, 0x0001, 0x0110, 0x0801),
				new FamilyToolVersion(0x0004, 0x0001, 0x0110, 0x0801),
				new FamilyToolVersion(0x0006, 0x0001, 0x0110, 0x0801),
				new FamilyToolVersion(0x0008, 0x0001, 0x0104, 0x0001),
				new FamilyToolVersion(0x0009, 0x0001, 0x0110, 0x0801),
				new FamilyToolVersion(0x000a, 0x0001, 0x0110, 0x0801),
				new FamilyToolVersion(0x000b, 0x0001, 0x0104, 0x0001),
				new FamilyToolVersion(0x0013, 0x0003, 0x0110, 0x0801),
			};
			Send(new ClientReadySnac(tools));
			base.FinishedNegotiation();
		}
	}
}
