// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;

namespace Grunwald.NOscar
{
	public abstract class SnacSequence
	{
		SnacConnection connection;
		
		/// <summary>
		/// Gets the connection this SnacSequence was created with.
		/// </summary>
		public SnacConnection Connection {
			get {
				return connection;
			}
		}
		
		public SnacSequence(SnacConnection connection)
		{
			if (connection == null)
				throw new ArgumentNullException("connection");
			this.connection = connection;
		}
		
		uint requestId = 0;
		SnacEventHandler callback;
		
		protected void SendSnac(Snac snac, SnacEventHandler callback)
		{
			if (requestId != 0)
				throw new InvalidOperationException("Cannot send SNAC before answer to old SNAC has been received!");
			requestId = snac.RequestID;
			this.callback = callback;
			connection.SnacReceived += new SnacEventHandler(SnacReceived);
			connection.Send(snac);
		}
		
		void SnacReceived(object sender, SnacEventArgs e)
		{
			if (e.Snac.RequestID != requestId)
				return;
			connection.SnacReceived -= new SnacEventHandler(SnacReceived);
			requestId = 0;
			callback(this, e);
		}
		
		public event EventHandler Error;
		
		protected void OnError(EventArgs e)
		{
			if (Error != null)
				Error(this, e);
		}
		
		public event SnacEventHandler Completed;
		
		protected void OnCompleted(SnacEventArgs e)
		{
			if (Completed != null)
				Completed(this, e);
		}
	}
	
	public class SendSnacSequence : SnacSequence
	{
		public static void SendSnac(SnacConnection connection, Snac snac, EventHandler timeoutCallback, SnacEventHandler successCallback)
		{
			SendSnacSequence s = new SendSnacSequence(connection);
			s.Completed += successCallback;
			s.Error += timeoutCallback;
			s.Start(snac);
		}
		
		public SendSnacSequence(SnacConnection connection) : base(connection) {}
		
		public void Start(Snac snac)
		{
			SendSnac(snac, new SnacEventHandler(OnCallback));
		}
		
		void OnCallback(object sender, SnacEventArgs e)
		{
			OnCompleted(e);
		}
	}
}
