// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.IO;

namespace Grunwald.NOscar
{
	/// <summary>
	/// An item that can be stored as <b>s</b>erver <b>s</b>ide <b>i</b>nformation.
	/// </summary>
	public class SsiItem : IByteConvertible
	{
		string name;
		ushort groupID;
		ushort buddyID;
		ushort type;
		TlvChain data;
		
		public SsiItem(string name, ushort groupID, ushort buddyID, ushort type)
		{
			this.BuddyID = buddyID;
			this.GroupID = groupID;
			this.Name = name;
			this.Type = type;
			data = new TlvChain();
		}
		
		/// <summary>Loads the SsiItem from the byte array.</summary>
		public SsiItem(byte[] data, int index, int count)
		{
			int pos = index;
			ushort len = NetworkBitConverter.ToUInt16(data, pos);
			pos += 2;
			name = Tlv.Encoding.GetString(data, pos, len);
			pos += len;
			groupID = NetworkBitConverter.ToUInt16(data, pos);
			pos += 2;
			buddyID = NetworkBitConverter.ToUInt16(data, pos);
			pos += 2;
			type = NetworkBitConverter.ToUInt16(data, pos);
			pos += 2;
			len = NetworkBitConverter.ToUInt16(data, pos);
			this.data = new TlvChain(data, pos + 2, len);
		}
		
		public string Name {
			get {
				return name;
			}
			set {
				if (name == null)
					throw new ArgumentNullException("name");
				name = value;
			}
		}
		
		public ushort GroupID {
			get {
				return groupID;
			}
			set {
				groupID = value;
			}
		}
		
		public ushort BuddyID {
			get {
				return buddyID;
			}
			set {
				buddyID = value;
			}
		}
		
		public ushort Type {
			get {
				return type;
			}
			set {
				type = value;
			}
		}
		
		public TlvChain Data {
			get {
				return data;
			}
		}
		
		/// <summary>
		/// Gets the total length this object would need in the byte array.
		/// </summary>
		public int GetByteLength()
		{
			return 2 + Tlv.Encoding.GetByteCount(name) + 8 + data.GetByteLength();
		}
		
		/// <summary>
		/// Writes the byte representation of this object into the specified stream.
		/// </summary>
		public void WriteBytes(Stream stream)
		{
			byte[] bname = Tlv.Encoding.GetBytes(name);
			stream.Write(NetworkBitConverter.GetBytes((ushort)bname.Length), 0, 2);
			stream.Write(bname, 0, bname.Length);
			stream.Write(NetworkBitConverter.GetBytes(groupID), 0, 2);
			stream.Write(NetworkBitConverter.GetBytes(buddyID), 0, 2);
			stream.Write(NetworkBitConverter.GetBytes(type), 0, 2);
			stream.Write(NetworkBitConverter.GetBytes((ushort)data.GetByteLength()), 0, 2);
			data.WriteBytes(stream);
		}
	}
}
