// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.IO;

namespace Grunwald.NOscar
{
	public sealed class UserInfo : IByteConvertible
	{
		TlvChain chain;
		
		/// <summary>Creates a new UserInfo block.</summary>
		public UserInfo(string screenname)
		{
			this.Screenname = screenname;
			chain = new TlvChain();
		}
		
		/// <summary>Loads the UserInfo from the byte array.</summary>
		public UserInfo(byte[] data, int index, int count)
		{
			int pos = index;
			screenname = Tlv.Encoding.GetString(data, pos + 1, data[pos]);
			pos += 1 + data[pos];
			warningLevel = NetworkBitConverter.ToUInt16(data, pos);
			pos += 2;
			chain = TlvChain.FromData(data, pos + 2, NetworkBitConverter.ToUInt16(data, pos));
		}
		
		/// <summary>Gets the size of this UserInfo in bytes.</summary>
		public int GetByteLength()
		{
			return 1 + Tlv.Encoding.GetByteCount(screenname) + 4 + chain.GetByteLength();
		}
		
		/// <summary>Writes the byte representation of the UserInfo into the specified stream.</summary>
		public void WriteBytes(Stream stream)
		{
			byte[] _text = Tlv.Encoding.GetBytes(screenname);
			stream.WriteByte((byte)_text.Length);
			stream.Write(_text, 0, _text.Length);
			stream.Write(NetworkBitConverter.GetBytes(warningLevel), 0, 2);
			stream.Write(NetworkBitConverter.GetBytes((ushort)chain.Count), 0, 2);
			chain.WriteBytes(stream);
		}
		
		string screenname;
		
		public string Screenname {
			get {
				return screenname;
			}
			set {
				if (value == null)
					throw new ArgumentNullException("screenname");
				screenname = value;
			}
		}
		
		ushort warningLevel;
		
		public ushort WarningLevel {
			get {
				return warningLevel;
			}
			set {
				warningLevel = value;
			}
		}
		
		public UserClass UserFlags {
			get {
				return (UserClass)chain.GetInt(0x0001);
			}
			set {
				chain.SetInt(0x0001, (uint)value);
			}
		}
		
		/// <summary>
		/// The number of minutes for which the user has been idle.
		/// </summary>
		public ushort IdleMins {
			get {
				return chain.GetShort(0x0004);
			}
			set {
				chain.SetShort(0x0004, value);
			}
		}
		
		public IcqStatus IcqStatus {
			get {
				return (IcqStatus)chain.GetInt(0x0006);
			}
			set {
				chain.SetInt(0x0006, (uint)value);
			}
		}
	}
}
