// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using Grunwald.Gui.Settings;
using Grunwald.Gui.Settings.Editors;

using DGNetwork.Services;
using DGNetwork.Settings;

namespace DGNetwork.AddIns.ICQ
{
	public class IcqNetworkPage : AbstractNetworkPage, IDisposable
	{
		/// <summary>Creates a new instance of the class</summary>
		public IcqNetworkPage()
		{
			this.Title = "ICQ";
		}
		
		/// <summary>Creates a new instance of the class copying the values from the specified class.</summary>
		public IcqNetworkPage(IcqNetworkPage original) : base(original)
		{
			// fields with the [Setting]-attribute are copies automatically.
		}
		
		[Setting("Server", "Login Server")]
		string server = "login.icq.com";
		
		[Setting("IcqNr", "ICQ#")]
		int icqNr = 0;
		
		[Setting("Password", "Password")]
		string password = "";
		
		protected override void OnSettingChanged(SettingChangedEventArgs e)
		{
			base.OnSettingChanged(e);
			if (network != null) {
				DestroyNetwork();
				CreateNetwork();
			}
		}
		
		IcqNetwork network;
		
		public IcqNetwork Network {
			get {
				return network;
			}
		}
		
		public override void CreateNetwork()
		{
			if (network == null)
				network = new IcqNetwork(server, icqNr, password, this);
		}
		
		public override void DestroyNetwork()
		{
			if (network != null) {
				network.Dispose();
				network = null;
			}
		}
		
		public void Dispose()
		{
			DestroyNetwork();
		}
	}
	
	public class NewNetworkButtonEditor : ButtonEditor
	{
		public NewNetworkButtonEditor(ISetting setting) : base(setting)
		{
			this.ButtonClick += new EventHandler(ButtonClicked);
		}
		
		private void ButtonClicked(object sender, EventArgs e)
		{
			ISettingPage page = Service.Setting.GetPage("NetworkPage");
			ISettingPage newPage = new IcqNetworkPage();
			page.AddPage(newPage);
			Service.Setting.EditSetting(newPage);
		}
	}
}
