// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using DGNetwork.Chat;
using DGNetwork.Services;
using Grunwald.NOscar;

namespace DGNetwork.AddIns.ICQ
{
	public class IcqUser : AbstractUser
	{
		#region User Cache
		static Hashtable cache = new Hashtable();
		
		public static IcqUser GetUser(IcqNetworkPage page, int uid)
		{
			return GetUser(page, uid, string.Format("#{0}", uid));
		}
		
		public static IcqUser GetUser(IcqNetworkPage page, int uid, string name)
		{
			ulong c = GetCode(page, uid);
			lock (cache) {
				IcqUser u = (IcqUser)cache[uid];
				if (u == null) {
					u = new IcqUser(page, uid, name);
					cache[uid] = u;
				}
				return u;
			}
		}
		
		private static ulong GetCode(IcqNetworkPage n, int id)
		{
			unchecked {
				return ((ulong)id << 32) + (ulong)n.GetHashCode();
			}
		}
		#endregion
		
		IcqNetworkPage page;
		int uid;
		
		public IcqNetworkPage Page {
			get {
				return page;
			}
		}
		public int Uid {
			get {
				return uid;
			}
		}
		
		public IcqUser(IcqNetworkPage page, int uid, string name)
		{
			this.page = page;
			this.uid = uid;
			this.Name = name;
		}
		
		static Random rnd = new Random();
		
		public override IMessageSendResult SendMessage(Message message)
		{
			IcqNetwork n = page.Network;
			if (n == null)
				return null;
			AutomaticConnection c = n.Connection;
			if (c == null || c.State != ConnectionState.Active)
				return null;
			TlvChain data = new TlvChain();
			MessageBlock block = new MessageBlock(message.Text);
			data.SetData(0x0002, block.ToByteArray());
			data.SetFlag(0x0003); // acknowledge flag
			OutgoingIcbmSnac snac = new OutgoingIcbmSnac(1, uid.ToString(), data);
			rnd.NextBytes(snac.IcbmID); // fill IcbmID with random data
			IcqSendResult result = new IcqSendResult(message);
			SendSnacSequence.SendSnac(c.GetConnection(snac.Family), snac, new EventHandler(result.Fail), new SnacEventHandler(result.Response));
			return result;
		}
	}
	
	public class IcqSendResult : DefaultMessageSendResult
	{
		public IcqSendResult(Message message) : base(message) {}
		
		public void Fail(object sender, EventArgs e)
		{
			this.SetResult(MessageSendStatus.SendTimeout);
		}
		
		public void Response(object sender, SnacEventArgs e)
		{
			if (e.Snac is ImAckSnac)
				this.SetResult(MessageSendStatus.Success);
			else
				this.SetResult(MessageSendStatus.SendError);
		}
	}
}
