// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.Threading;
using DGNetwork.Chat;
using DGNetwork.Gui;
using DGNetwork.Services;
using Meebey.SmartIrc4net;

namespace DGNetwork.AddIns.IRC
{
	public class Connection : IDisposable
	{
		IrcClient ircClient;
		string[] serverlist;
		int port;
		bool isDisposed;
		
		public Connection(string server, int port)
		{
			this.serverlist = server.Split(';');
			this.port = port;
			ircClient = new IrcClient();
		}
		
		public IrcClient IrcClient {
			get {
				return ircClient;
			}
		}
		
		public void Connect()
		{
			if (isDisposed)
				throw new ObjectDisposedException("Connection");
			ircClient.ActiveChannelSyncing = true;
			ircClient.SendDelay = 200;
			ircClient.OnError += new ErrorEventHandler(IrcError);
			ircClient.OnDisconnecting += new EventHandler(IrcDisconnecting);
			Thread thread = new Thread(new ThreadStart(Run));
			thread.Name = "IRC Thread";
			thread.IsBackground = true;
			thread.Start();
		}
		
		bool oldAway = false;
		
		private void Run()
		{
			try {
				try {
					ircClient.Connect(serverlist, port);
				} catch (CouldNotConnectException) {
					// TODO: tell that we could not connect
					throw;
				}
				ircClient.Login(Service.Chat.Nickname, "DG Network User");
				ircClient.AutoReconnect = true;
				ircClient.AutoRejoin = true;
				ircClient.AutoRetry = true;
				Service.Chat.StatusChanged += new StatusChangedEventHandler(StatusChanged);
				StatusChanged(null, null);
				while (!isDisposed) {
					ircClient.ListenOnce();
				}
			} catch (Exception ex) {
				Service.Message.ShowError(ex);
			} finally {
				try {
					ircClient.Disconnect();
				} catch { }
			}
		}
		
		private void StatusChanged(object sender, StatusChangedEventArgs e)
		{
			bool away = Service.Chat.IsAway;
			if (away == oldAway) return;
			if (away)
				ircClient.RfcAway("I'm away.");
			else
				ircClient.RfcAway(); // reset away message
			oldAway = away;
		}
		
		public void Dispose()
		{
			if (isDisposed)
				return;
			isDisposed = true;
			try {
				ircClient.RfcQuit();
			} catch { }
			if (Disposed != null)
				Disposed(this, EventArgs.Empty);
		}
		
		private void IrcDisconnecting(object sender, EventArgs e)
		{
			Dispose();
		}
		
		public event EventHandler Disposed;
		
		public bool IsDisposed {
			get {
				return isDisposed;
			}
		}
		
		// this method handles when we receive "ERROR" from the IRC server
		private void IrcError(object sender, ErrorEventArgs e)
		{
			Service.Message.ShowError(e.ErrorMessage);
		}
	}
}
