// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Windows.Forms;
using DGNetwork.AddIns.Codons;
using DGNetwork.Contacts;
using DGNetwork.Gui;
using DGNetwork.Gui.Dialogs;
using DGNetwork.Services;
using ICSharpCode.Core.Services;

namespace DGNetwork.AddIns.IRC
{
	public class AddIRCServerCommand : AbstractMenuCommand
	{
		public override void Run()
		{
			NewServerDialog dlg = new NewServerDialog();
			if (dlg.ShowDialog() == DialogResult.OK) {
				string server = dlg.Server.ToLower();
				foreach (IContact contact in Service.Contact.RecursedContacts) {
					if (contact is IrcServerContact) {
						if (server == ((IrcServerContact)contact).Server.ToLower()) {
							Service.Message.ShowMessage("${res:IRC.ServerAlreadyOnContactList}");
							return;
						}
					}
				}
				IrcServerContact c = new IrcServerContact(dlg.Server, dlg.Port);
				c.Name = dlg.Title;
				((IContactList)this.Owner).Contacts.Add(c);
			}
		}
	}
	
	public class ConnectCommand : AbstractMenuCommand
	{
		public override void Run()
		{
			((IrcServerContact)this.Owner).Connect();
		}
	}
	
	public class DisconnectCommand : AbstractMenuCommand
	{
		public override void Run()
		{
			((IrcServerContact)this.Owner).Disconnect();
		}
	}
	
	public class AddChannelCommand : AbstractMenuCommand
	{
		public override void Run()
		{
			((IrcServerContact)this.Owner).AddChannel();
		}
	}
}
