// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.Drawing;
using System.IO;
using System.Text.RegularExpressions;
using System.Reflection;
using System.Windows.Forms;
using DGNetwork.Services;

namespace DGNetwork.AddIns.IRC
{
	public class NewServerDialog : System.Windows.Forms.Form
	{
		private System.Windows.Forms.Label label3;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.TextBox titleTextBox;
		private System.Windows.Forms.TextBox serverTextBox;
		private System.Windows.Forms.TextBox portTextBox;
		private System.Windows.Forms.Button cancelButton;
		private System.Windows.Forms.ComboBox comboBox;
		private System.Windows.Forms.Button okButton;
		
		public NewServerDialog()
		{
			//
			// The InitializeComponent() call is required for Windows Forms designer support.
			//
			InitializeComponent();
			
			this.DialogResult = DialogResult.Cancel;
			Translate(this);
			Translate(okButton);
			Translate(cancelButton);
			
			comboBox.Items.Add(new ServerEntry(Service.Parse("${res:IRC.NewServerDialog.UserDefined}"), "", ""));
			using (Stream s = Assembly.GetExecutingAssembly().GetManifestResourceStream("IRC_servers.ini")) {
				using (StreamReader r = new StreamReader(s)) {
					string line;
					Regex regex = new Regex(@"^n\d+=([\w-\. ]+): ?(?:Random )?([\w,() ]*?) ?(?:(?:s|S)erver)?SERVER:([\w\.-]+):([\d-,]+)GROUP:(.*)$");
					int num = 0;
					while ((line = r.ReadLine()) != null) {
						num += 1;
						line = line.Trim();
						if (line.Length == 0 || line.StartsWith("#"))
							continue;
						Match m = regex.Match(line);
						if (!m.Success) {
							MessageBox.Show("line " + num + ": " + line);
						} else {
							string name = m.Groups[1].Value;
							if (m.Groups[2].Value.Length > 0) {
								name += ": " + m.Groups[2].Value;
							}
							comboBox.Items.Add(new ServerEntry(name, m.Groups[3].Value, m.Groups[4].Value));
						}
					}
				}
			}
			comboBox.SelectedIndex = 0;
		}
		
		private void Translate(Control ctl)
		{
			ctl.Text = Service.Parse("${res:" + ctl.Text + "}");
		}
		
		#region Windows Forms Designer generated code
		/// <summary>
		/// This method is required for Windows Forms designer support.
		/// Do not change the method contents inside the source code editor. The Forms designer might
		/// not be able to load this method if it was changed manually.
		/// </summary>
		private void InitializeComponent() {
			this.okButton = new System.Windows.Forms.Button();
			this.comboBox = new System.Windows.Forms.ComboBox();
			this.cancelButton = new System.Windows.Forms.Button();
			this.portTextBox = new System.Windows.Forms.TextBox();
			this.serverTextBox = new System.Windows.Forms.TextBox();
			this.titleTextBox = new System.Windows.Forms.TextBox();
			this.label1 = new System.Windows.Forms.Label();
			this.label2 = new System.Windows.Forms.Label();
			this.label3 = new System.Windows.Forms.Label();
			this.SuspendLayout();
			// 
			// okButton
			// 
			this.okButton.Enabled = false;
			this.okButton.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.okButton.Location = new System.Drawing.Point(112, 120);
			this.okButton.Name = "okButton";
			this.okButton.TabIndex = 7;
			this.okButton.Text = "Global.OK";
			this.okButton.Click += new System.EventHandler(this.OkButtonClick);
			// 
			// comboBox
			// 
			this.comboBox.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.comboBox.Location = new System.Drawing.Point(24, 8);
			this.comboBox.MaxDropDownItems = 16;
			this.comboBox.Name = "comboBox";
			this.comboBox.Size = new System.Drawing.Size(224, 21);
			this.comboBox.TabIndex = 0;
			this.comboBox.SelectedIndexChanged += new System.EventHandler(this.ComboBoxSelectedIndexChanged);
			// 
			// cancelButton
			// 
			this.cancelButton.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.cancelButton.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.cancelButton.Location = new System.Drawing.Point(192, 120);
			this.cancelButton.Name = "cancelButton";
			this.cancelButton.TabIndex = 8;
			this.cancelButton.Text = "Global.Cancel";
			this.cancelButton.Click += new System.EventHandler(this.CancelButtonClick);
			// 
			// portTextBox
			// 
			this.portTextBox.Location = new System.Drawing.Point(80, 88);
			this.portTextBox.Name = "portTextBox";
			this.portTextBox.Size = new System.Drawing.Size(176, 20);
			this.portTextBox.TabIndex = 6;
			this.portTextBox.Text = "";
			this.portTextBox.TextChanged += new System.EventHandler(this.TextBoxTextChanged);
			// 
			// serverTextBox
			// 
			this.serverTextBox.Location = new System.Drawing.Point(80, 64);
			this.serverTextBox.Name = "serverTextBox";
			this.serverTextBox.Size = new System.Drawing.Size(176, 20);
			this.serverTextBox.TabIndex = 4;
			this.serverTextBox.Text = "";
			this.serverTextBox.TextChanged += new System.EventHandler(this.TextBoxTextChanged);
			// 
			// titleTextBox
			// 
			this.titleTextBox.Location = new System.Drawing.Point(80, 40);
			this.titleTextBox.Name = "titleTextBox";
			this.titleTextBox.Size = new System.Drawing.Size(176, 20);
			this.titleTextBox.TabIndex = 2;
			this.titleTextBox.Text = "";
			this.titleTextBox.TextChanged += new System.EventHandler(this.TextBoxTextChanged);
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(16, 64);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(64, 16);
			this.label1.TabIndex = 3;
			this.label1.Text = "Server:";
			this.label1.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
			// 
			// label2
			// 
			this.label2.Location = new System.Drawing.Point(16, 88);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(64, 16);
			this.label2.TabIndex = 5;
			this.label2.Text = "Port:";
			this.label2.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
			// 
			// label3
			// 
			this.label3.Location = new System.Drawing.Point(16, 40);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(64, 16);
			this.label3.TabIndex = 1;
			this.label3.Text = "Title:";
			this.label3.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
			// 
			// NewServerDialog
			// 
			this.AcceptButton = this.okButton;
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.CancelButton = this.cancelButton;
			this.ClientSize = new System.Drawing.Size(274, 152);
			this.Controls.Add(this.titleTextBox);
			this.Controls.Add(this.label3);
			this.Controls.Add(this.okButton);
			this.Controls.Add(this.cancelButton);
			this.Controls.Add(this.portTextBox);
			this.Controls.Add(this.label2);
			this.Controls.Add(this.serverTextBox);
			this.Controls.Add(this.label1);
			this.Controls.Add(this.comboBox);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "NewServerDialog";
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent;
			this.Text = "IRC.NewServerDialog.Title";
			this.ResumeLayout(false);
		}
		#endregion
		
		private class ServerEntry
		{
			public string Title, Server, Port;
			
			public ServerEntry(string title, string server, string port)
			{
				this.Port = port;
				this.Server = server;
				this.Title = title;
			}
			
			public override string ToString()
			{
				return Title;
			}
		}
		
		bool eventActive = false;
		
		private void ComboBoxSelectedIndexChanged(object sender, EventArgs e)
		{
			if (eventActive)
				return;
			eventActive = true;
			ServerEntry entry = (ServerEntry)comboBox.SelectedItem;
			if (entry.Server == "")
				titleTextBox.Text = "";
			else
				titleTextBox.Text = entry.Title;
			serverTextBox.Text = entry.Server;
			portTextBox.Text = entry.Port;
			eventActive = false;
		}
		
		private void CancelButtonClick(object sender, EventArgs e)
		{
			Close();
		}
		
		private void OkButtonClick(object sender, EventArgs e)
		{
			this.DialogResult = DialogResult.OK;
			Close();
		}
		
		public string Title {
			get {
				if (titleTextBox.Text.Length == 0)
					return serverTextBox.Text;
				else
					return titleTextBox.Text;
			}
		}
		
		public string Server {
			get {
				return serverTextBox.Text;
			}
		}
		
		public string Port {
			get {
				return portTextBox.Text;
			}
		}
		
		private void TextBoxTextChanged(object sender, EventArgs e)
		{
			if (!eventActive) {
				eventActive = true;
				comboBox.SelectedIndex = 0;
				eventActive = false;
			}
			okButton.Enabled = Regex.IsMatch(serverTextBox.Text, @"([\w\d]+\.)*[\w\d]+")
				&& Regex.IsMatch(portTextBox.Text, @"\d+(-\d+)?");
		}
	}
}
