// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using DGNetwork.Chat;
using DGNetwork.Services;
using Meebey.SmartIrc4net;

namespace DGNetwork.AddIns.IRC
{
	public class ServerChatRoom : AbstractChatRoom
	{
		ArrayList users = new ArrayList();
		Connection connection;
		IrcClient irc;
		
		public ServerChatRoom()
		{
			this.CanSend = false; // cannot send because SendMessage is not implemented
		}
		
		#region Register/Deregister events
		public Connection Connection {
			get {
				return connection;
			}
			set {
				connection = value;
				if (irc != null)
					Deregister();
				if (connection == null) {
					irc = null;
				} else {
					connection.Disposed += new EventHandler(ConnectionDisposed);
					irc = connection.IrcClient;
					irc.OnRawMessage += new IrcEventHandler(OnRawMessage);
					irc.OnInvite += new InviteEventHandler(OnInvite);
					irc.OnQueryAction += new ActionEventHandler(OnAction);
					irc.OnQueryMessage += new IrcEventHandler(OnMessage);
					irc.OnQueryNotice += new IrcEventHandler(OnNotice);
					irc.OnMotd += new MotdEventHandler(OnMotd);
				}
			}
		}
		
		private void Deregister()
		{
			irc.OnRawMessage -= new IrcEventHandler(OnRawMessage);
			irc.OnInvite -= new InviteEventHandler(OnInvite);
			irc.OnQueryAction -= new ActionEventHandler(OnAction);
			irc.OnQueryMessage -= new IrcEventHandler(OnMessage);
			irc.OnQueryNotice -= new IrcEventHandler(OnNotice);
			irc.OnMotd -= new MotdEventHandler(OnMotd);
		}
		
		private void ConnectionDisposed(object sender, EventArgs e)
		{
			this.Connection = null;
			ReceiveMessage("Connection was closed.", false);
		}
		#endregion
		
		#region Event handler
		private void OnRawMessage(object sender, IrcEventArgs e)
		{
			//ReceiveMessageAndNotify(e.Data.RawMessage);
		}
		
		private void OnAction(object sender, ActionEventArgs e)
		{
			ReceiveMessageAndNotify("ACTION:" + e.ActionMessage);
		}
		
		private void OnMessage(object sender, IrcEventArgs e)
		{
			ReceiveMessageAndNotify("MSG:" + e.Data.Message);
		}
		
		private void OnNotice(object sender, IrcEventArgs e)
		{
			ReceiveMessageAndNotify("NOTICE:" + e.Data.Message);
		}
		
		private void OnInvite(object sender, InviteEventArgs e)
		{
			ReceiveMessageAndNotify(e.Who + " invites you to " + e.Channel);
		}
		
		private void OnMotd(object sender, MotdEventArgs e)
		{
			ReceiveMessageAndNotify(e.MotdMessage);
		}
		#endregion
		
		public void ReceiveMessageAndNotify(string text)
		{
			ReceiveMessage(text, true);
		}
		
		public void ReceiveMessage(string text, bool notify)
		{
			Message msg = new Message(false);
			msg.Text = text;
			msg.Nickname = this.Title;
			msg.NotifyUser = notify;
			this.ReceiveMessage(msg);
		}
		
		public override ArrayList Users {
			get {
				return users;
			}
		}
		
		public override IMessageSendResult SendMessage(Message message)
		{
			return null;
		}
	}
}
