// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using Grunwald.Gui.Settings;
using Grunwald.Gui.Settings.Editors;

using DGNetwork.Settings;
using DGNetwork.Services;
using ICSharpCode.Core.Services;

namespace DGNetwork.AddIns.IRC
{
	/// <summary>
	/// A setting page representing a dg network.
	/// </summary>
	public class SettingPage : AbstractNetworkPage, IDisposable
	{
		string server = "irc.fu-berlin.de";
		
		[Setting("server", "Server", "Enter the server(s) you want to use (seperated by ;)")]
		public string Server {
			get {
				return server;
			}
			set {
				if (server == value)
					return;
				server = value;
				Reconnect();
			}
		}
		
		int port = 6667;
		
		[Setting("port", "Port")]
		public int Port {
			get {
				return port;
			}
			set {
				if (port == value)
					return;
				port = value;
				Reconnect();
			}
		}
		
		public SettingPage()
		{
			this.Title = "${res:IRC.SettingPage.Title}";
		}
		
		public SettingPage(SettingPage original) : base(original)
		{
		}
		
		private void Reconnect()
		{
			if (conn != null) {
				DestroyNetwork();
				CreateNetwork();
			}
		}
		
		Connection conn;
		
		public override void CreateNetwork()
		{
			if (conn != null)
				throw new InvalidOperationException();
			conn = new Connection(server, port);
			conn.Connect();
			// create connection
		}
		
		public override void DestroyNetwork()
		{
			if (conn == null)
				throw new InvalidOperationException();
			conn.Dispose();
			conn = null;
		}
		
		public void Dispose()
		{
			if (conn != null)
				DestroyNetwork();
		}
	}
	
	public class NewIRCNetworkButtonEditor : ButtonEditor
	{
		public NewIRCNetworkButtonEditor(ISetting setting) : base(setting)
		{
			this.ButtonClick += new EventHandler(ButtonClicked);
		}
		
		private void ButtonClicked(object sender, EventArgs e)
		{
			ISettingPage page = Service.Setting.GetPage("NetworkPage");
			ISettingPage newPage = new SettingPage();
			page.AddPage(newPage);
			Service.Setting.EditSetting(newPage);
		}
	}
}
