// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.Windows.Forms;

using ICSharpCode.Core.AddIns;
using ICSharpCode.Core.AddIns.Codons;
using ICSharpCode.Core.AddIns.Conditions;
using DGNetwork.Services;

namespace DGNetwork.AddIns.Codons
{
	[CodonName("Include")]
	public class IncludeCodon : AbstractCodon
	{
		[XmlMemberAttribute("path", IsRequired = true)]
		string path = null;
		
		public string Path {
			get {
				return path;
			}
			set {
				path = value;
			}
		}
		
		/// <summary>
		/// Creates an item with the specified sub items. And the current
		/// Condition status for this item.
		/// </summary>
		public override object BuildItem(object owner, ArrayList subItems, ConditionCollection conditions)
		{
			IAddInTreeNode node;
			try {
				node = AddInTreeSingleton.AddInTree.GetTreeNode(path);
			} catch (TreePathNotFoundException) {
				Service.Message.ShowErrorFormatted("${res:Global.TreePathNotFound}", path);
				return null;
			}
			return node.Codon.BuildItem(owner, node.BuildChildItems(owner), node.ConditionCollection);
		}
	}
}
