// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.IO;
using System.Reflection;

using Grunwald.P2PLib.Server.Http;

using ICSharpCode.Core.AddIns;
using ICSharpCode.Core.AddIns.Codons;
using ICSharpCode.Core.AddIns.Conditions;
using ICSharpCode.Core.Properties;
using ICSharpCode.Core.Services;
using DGNetwork.Gui;
using DGNetwork.Services;

namespace DGNetwork.AddIns.Codons
{
	public abstract class ShareCodon : AbstractCodon
	{
		[XmlMemberAttribute("casesensitive", IsRequired = false)]
		bool casesensitive = false;
		
		[XmlMemberAttribute("aliasisregex", IsRequired = false)]
		bool aliasisregex = false;
		
		[XmlMemberAttribute("alias", IsRequired = true)]
		string alias = null;
		
		[XmlMemberAttribute("server", IsRequired = false)]
		string server = Service.StringParser.Parse("DG Network ${exe:ProductVersion}${AppBuildName}");
		
		public bool AliasIsRegex {
			get { return aliasisregex; }
		}
		
		public bool CaseSensitive {
			get { return casesensitive; }
		}
		
		public string Alias {
			get { return alias; }
		}
		
		public string Server {
			get { return server; }
		}
	}
	
	[CodonName("ShareFolder")]
	public class ShareFolderCodon : ShareCodon
	{
		[XmlMemberAttribute("folder", IsRequired = true)]
		string folder = null;
		
		[XmlMemberAttribute("browsable", IsRequired = false)]
		bool browsable = false;
		
		public string Folder {
			get { return folder; }
		}
		
		public bool Browsable {
			get { return browsable; }
		}
		
		/// <summary>
		/// Creates an item with the specified sub items. And the current
		/// Condition status for this item.
		/// </summary>
		public override object BuildItem(object owner, ArrayList subItems, ConditionCollection conditions)
		{
			if (subItems.Count > 0) {
				Service.Message.ShowErrorFormatted("${res:Codon.NoSubItems}", this.Name);
			}
			if (this.folder.StartsWith("data/"))
				this.folder = Path.Combine(Service.Property.DataDirectory, this.folder.Substring(5));
			ShareFolderRequestHandler r;
			r = new ShareFolderRequestHandler(this.Alias, this.CaseSensitive,
			                                  this.AliasIsRegex, this.Folder,
			                                  this.Browsable);
			r.Server = this.Server;
			return r;
		}
	}
	
	[CodonName("ShareFile")]
	public class ShareFileCodon : ShareCodon
	{
		[XmlMemberAttribute("file", IsRequired = true)]
		string file = null;
		
		[XmlMemberAttribute("contenttype", IsRequired = false)]
		string contenttype = null;
		
		public string ContentType {
			get { return contenttype; }
		}
		
		public string File {
			get { return file; }
		}
		
		/// <summary>
		/// Creates an item with the specified sub items. And the current
		/// Condition status for this item.
		/// </summary>
		public override object BuildItem(object owner, ArrayList subItems, ConditionCollection conditions)
		{
			if (subItems.Count > 0) {
				Service.Message.ShowErrorFormatted("${res:Codon.NoSubItems}", this.Name);
			}
			if (this.file.StartsWith("data/"))
				this.file = Path.Combine(Service.Property.DataDirectory, this.file.Substring(5));
			ShareFileRequestHandler r;
			r = new ShareFileRequestHandler(this.Alias, this.CaseSensitive,
			                                this.AliasIsRegex, this.File,
			                                this.ContentType);
			r.ExtraHeaders.Set("Server", this.Server);
			return r;
		}
	}
}
