// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.Reflection;
using System.Windows.Forms;

using ICSharpCode.Core.AddIns;
using ICSharpCode.Core.AddIns.Codons;
using ICSharpCode.Core.AddIns.Conditions;
using ICSharpCode.Core.Properties;
using DGNetwork.Gui;
using DGNetwork.Services;

namespace DGNetwork.AddIns.Codons
{
	[CodonName("TNAIcon")]
	public class TNAIconCodon : AbstractCodon
	{
		[XmlMemberAttribute("tooltip", IsRequired = true)]
		string tooltip     = null;
		
		[XmlMemberAttribute("icon", IsRequired = true)]
		string icon        = null;
		
		public string ToolTip {
			get {
				return tooltip;
			}
			set {
				tooltip = value;
			}
		}
		
		public string Icon {
			get {
				return icon;
			}
			set {
				icon = value;
			}
		}
		
		/// <summary>
		/// Creates an item with the specified sub items. And the current
		/// Condition status for this item.
		/// </summary>
		public override object BuildItem(object owner, ArrayList subItems, ConditionCollection conditions)
		{
			ITNAIcon icon = Service.TNA.CreateNewIcon();
			icon.ToolTip = this.ToolTip;
			if (subItems.Count > 0) {
				icon.ContextMenu = Service.Menu.CreateContextMenu(this, subItems);
			}
			try {
				icon.Icon = Service.Resource.GetIcon(Icon);
			} catch (ResourceNotFoundException ex) {
				Service.Message.ShowError(ex.Message);
			}
			return icon;
		}
	}
}
