// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.Xml;
using DGNetwork.Services;
using Grunwald.Gui.Settings;

namespace DGNetwork.Chat
{
	#region EditBoxSetting
	/// <summary>
	/// The setting that determines the used chat editBox.
	/// </summary>
	public class EditBoxSetting : ChooseOptionSetting
	{
		public EditBoxSetting() : base("ChatEditBox", "${res:Settings.Chat.EditBox}", "")
		{
		}
		
		protected override void SaveValue(XmlWriter writer)
		{
			// ignore this setting
		}
		
		public override void Load(XmlReader reader)
		{
			reader.Skip();
		}
		
		private void CreateOptions()
		{
			ArrayList editBoxes = Service.Chat.EditBoxes;
			string[] options = new string[editBoxes.Count];
			string[] texts = new string[editBoxes.Count];
			for (int i = 0; i < editBoxes.Count; ++i) {
				options[i] = ((IEditBoxFactory)editBoxes[i]).Name;
				texts[i] = ((IEditBoxFactory)editBoxes[i]).Title;
			}
			this.Options = options;
			this.Texts = texts;
			this.InEditValue = this.Value;
		}
		
		public override ISettingEditor CreateEditor(EditorFactory factory)
		{
			if (this.Options == null)
				CreateOptions();
			return base.CreateEditor(factory);
		}
		
		public override object Value {
			get {
				return Service.Chat.ActiveEditBox.Name;
			}
			set {
				string oldValue = Service.Chat.ActiveEditBox.Name;
				string newValue = (string)value;
				if (oldValue != newValue) {
					foreach (IEditBoxFactory editBox in Service.Chat.EditBoxes) {
						if (editBox.Name == newValue) {
							Service.Chat.ActiveEditBox = editBox;
							break;
						}
					}
					this.OnValueChanged(new SettingChangedEventArgs(this, oldValue, newValue));
				}
				if (newValue != (string)this.InEditValue)
					this.InEditValue = newValue;
			}
		}
	}
	#endregion
	
	#region DisplaySetting
	/// <summary>
	/// The setting that determines the used chat display.
	/// </summary>
	public class DisplaySetting : ChooseOptionSetting
	{
		public DisplaySetting() : base("ChatDisplay", "${res:Settings.Chat.Display}", "")
		{
		}
		
		protected override void SaveValue(XmlWriter writer)
		{
			// ignore this setting
		}
		
		public override void Load(XmlReader reader)
		{
			reader.Skip();
		}
		
		private void CreateOptions()
		{
			ArrayList displays = Service.Chat.Displays;
			string[] options = new string[displays.Count];
			string[] texts = new string[displays.Count];
			for (int i = 0; i < displays.Count; ++i) {
				options[i] = ((IDisplayFactory)displays[i]).Name;
				texts[i] = ((IDisplayFactory)displays[i]).Title;
			}
			this.Options = options;
			this.Texts = texts;
			this.InEditValue = this.Value;
		}
		
		public override ISettingEditor CreateEditor(EditorFactory factory)
		{
			if (this.Options == null)
				CreateOptions();
			return base.CreateEditor(factory);
		}
		
		public override object Value {
			get {
				return Service.Chat.ActiveDisplay.Name;
			}
			set {
				string oldValue = Service.Chat.ActiveDisplay.Name;
				string newValue = (string)value;
				if (oldValue != newValue) {
					foreach (IDisplayFactory display in Service.Chat.Displays) {
						if (display.Name == newValue) {
							Service.Chat.ActiveDisplay = display;
							break;
						}
					}
					this.OnValueChanged(new SettingChangedEventArgs(this, oldValue, newValue));
				}
				if (newValue != (string)this.InEditValue)
					this.InEditValue = newValue;
			}
		}
	}
	#endregion
}
