// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using DGNetwork.Services;
using ICSharpCode.Core.AddIns;
using ICSharpCode.Core.AddIns.Conditions;

namespace DGNetwork.Chat
{
	[Condition]
	public class EditBoxClassCondition : AbstractCondition
	{
		[XmlMemberAttribute("editboxclass", IsRequired = true)]
		string editboxclass = null;
		Type neededType;
		
		public override bool IsValid(object caller)
		{
			if (neededType == null) {
				neededType = Helper.GetType(editboxclass);
				if (neededType == null) {
					Service.Message.ShowMessageFormatted("${res:Global.ClassNotFound}", new string[] { editboxclass });
					neededType = GetType(); // prevent showing message again and again
				}
			}
			ChatPanel panel = (ChatPanel)caller;
			return neededType.IsInstanceOfType(panel.EditBoxPanel.EditBox);
		}
	}
	
	[Condition]
	public class DisplayClassCondition : AbstractCondition
	{
		[XmlMemberAttribute("displayclass", IsRequired = true)]
		string displayclass = null;
		Type neededType;
		
		public override bool IsValid(object caller)
		{
			if (neededType == null) {
				neededType = Helper.GetType(displayclass);
				if (neededType == null) {
					Service.Message.ShowMessageFormatted("${res:Global.ClassNotFound}", new string[] { displayclass });
					neededType = GetType(); // prevent showing message again and again
				}
			}
			ChatPanel panel = (ChatPanel)caller;
			return neededType.IsInstanceOfType(panel.DisplayPanel.Display);
		}
	}
}
