// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Drawing;
using System.Text;
using DGNetwork.Chat.Parser;

namespace DGNetwork.Chat
{
	/// <summary>
	/// The Converter is capable of converting between text, rich text and html.
	/// </summary>
	public class Converter
	{
		private Converter() {}
		
		// TODO: Write test fixture
		
		#region TextToRtf
		public static string TextToRtf(string text)
		{
			return text;
		}
		#endregion
		
		#region RtfToText
		public static string RtfToText(string rtf)
		{
			return rtf;
		}
		#endregion
		
		#region TextToHtml
		public static string TextToHtml(string text)
		{
			if (text == null)
				throw new ArgumentNullException("text");
			return HtmlHelper.EncodeText(text);
		}
		#endregion
		
		#region HtmlToText
		public static string HtmlToText(string html)
		{
			if (html == null)
				throw new ArgumentNullException("html");
			StringBuilder b = new StringBuilder();
			char c;
			for (int i = 0; i < html.Length; ++i) {
				c = html[i];
				switch (c) {
					case '<':
						TagInfo tag = HtmlHelper.ParseTag(html, i);
						if (tag == null) {
							b.Append("<");
						} else {
							i = tag.EndPos;
						}
						break;
					case '&':
						b.Append(HtmlHelper.ParseEntity(html, ref i));
						break;
					default:
						b.Append(c);
						break;
				}
			}
			return b.ToString();
		}
		#endregion
		
		#region HtmlToRtf
		public static string HtmlToRtf(string html)
		{
			return TextToRtf(HtmlToText(html));
		}
		#endregion
		
		#region RtfToHtml
		public static string RtfToHtml(string rtf)
		{
			return TextToHtml(RtfToText(rtf));
		}
		#endregion
		
		#region FontToHtml
		public static string FontToHtml(Font font, Color color)
		{
			StringBuilder b = new StringBuilder(FontToHtml(font.Name, font.Size, color));
			if (font.Bold)
				b.Append("<b>");
			if (font.Italic)
				b.Append("<i>");
			if (font.Strikeout)
				b.Append("<s>");
			if (font.Underline)
				b.Append("<u>");
			return b.ToString();
		}
		
		public static string FontToHtml(string name, float size, Color color)
		{
			return string.Format(System.Globalization.CultureInfo.InvariantCulture,
			                     //"<font face=\"{0}\" size=\"{1:f2}pt\" color=\"#{2:x2}{3:x2}{4:x2}\">",
			                     // "pt" können nicht im Font-Tag angegeben werden...
			                     "<font face=\"{0}\" color=\"#{2:x2}{3:x2}{4:x2}\">",
			                     name, size,
			                     color.R, color.G, color.B);
		}
		#endregion
	}
}
