// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Drawing;
using System.Windows.Forms;
using DGNetwork.Services;
using RTFDW = Grunwald.Gui.Controls.RearrangeTreeView.DataWrapper;

namespace DGNetwork.Chat
{
	public interface IDataObjectConverter
	{
		/// <summary>
		/// Gets the object out of the IDataObject.
		/// </summary>
		object GetObject(IDataObject d, object dropTarget);
		
		/// <summary>
		/// Gets the allowed effects when dropping the specified object.
		/// </summary>
		DragDropEffects GetEffects(IDataObject d, object dropTarget);
		
		/// <summary>
		/// Gets the allowed effects when dropping the specified object.
		/// </summary>
		DragDropEffects GetEffects(object o, object dropTarget);
		
		/// <summary>
		/// Gets the text representing the specified object in the specified DropTarget.
		/// </summary>
		string GetText(object o, object dropTarget);
	}
	
	internal sealed class StandardDataObjectConverter : IDataObjectConverter
	{
		/// <summary>
		/// Gets the object out of the IDataObject.
		/// </summary>
		public object GetObject(IDataObject d, object dropTarget)
		{
			RTFDW w = (RTFDW)d.GetData(typeof(RTFDW));
			if (w != null && w.GetOriginalNode())
				return w.Tag;
			if (d.GetDataPresent(DataFormats.Bitmap)) {
				Bitmap bmp = (Bitmap)d.GetData(DataFormats.Bitmap);
				if (bmp == null) {
					Service.Message.ShowError("${res:Chat.ImageIsNull}");
				}
				return bmp;
			}
			return null;
		}
		
		/// <summary>
		/// Gets the allowed effects when dropping the specified object.
		/// </summary>
		public DragDropEffects GetEffects(IDataObject d, object dropTarget)
		{
			if (d.GetDataPresent(typeof(RTFDW))) // RearrangeTreeView.DataWrapper
				return DragDropService.GetEffects(GetObject(d, dropTarget), dropTarget);
			else
				return DragDropEffects.None;
		}
		
		/// <summary>
		/// Gets the allowed effects when dropping the specified object.
		/// </summary>
		public DragDropEffects GetEffects(object o, object dropTarget)
		{
			return DragDropEffects.None;
		}
		
		/// <summary>
		/// Gets the text representing the specified object in the specified DropTarget.
		/// </summary>
		public string GetText(object o, object dropTarget)
		{
			return null;
		}
	}
}
