// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;

namespace DGNetwork.Chat
{
	/// <summary>
	/// The chat display is the class showing the list of chat entries.
	/// This is normally done by a browser.
	/// </summary>
	public interface IDisplayFactory
	{
		string Name { get; }
		string Title { get; }
		IDisplay Create();
	}
	
	public class DefaultDisplayFactory : IDisplayFactory
	{
		string name, title;
		
		public DefaultDisplayFactory()
		{
			name = "DefaultDisplay";
			title = null;
			if (CommandLine.Instance.HasFlag("test"))
				title = "Dummy display";
		}
		
		public DefaultDisplayFactory(string name, string title)
		{
			this.name = name;
			this.title = title;
		}
		
		public string Name {
			get {
				return name;
			}
			set {
				name = value;
			}
		}
		
		public string Title {
			get {
				return title;
			}
			set {
				title = value;
			}
		}
		
		public virtual IDisplay Create()
		{
			return new DummyDisplay();
		}
	}
}
