// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Drawing;
using System.Windows.Forms;
using DGNetwork.Services;

namespace DGNetwork.Chat
{
	public class DisplayPanel : UserControl, IHasRoom
	{
		IDisplay display;
		RoomUserView userView;
		
		public DisplayPanel()
		{
			userView = new RoomUserView();
			userView.Visible = false;
			userView.Width = 120;
			userView.Dock = DockStyle.Right;
			this.Controls.Add(userView);
			CreateDisplay();
			Service.Chat.ActiveDisplayChanged += new EventHandler(ActiveDisplayChanged);
		}
		
		private void CreateDisplay()
		{
			display = Service.Chat.ActiveDisplay.Create();
			display.RoomChanged += new EventHandler(DisplayRoomChanged);
			Control ctl = display.Control;
			ctl.Dock = DockStyle.Fill;
			this.Controls.Add(ctl);
			this.Controls.SetChildIndex(ctl, 0);
		}
		
		protected override void Dispose(bool disposing)
		{
			if (disposing) {
				Service.Chat.ActiveDisplayChanged -= new EventHandler(ActiveDisplayChanged);
				display.Dispose();
			}
			base.Dispose(disposing);
		}
		
		public event EventHandler DisplayChanged;
		
		private void ActiveDisplayChanged(object sender, EventArgs e)
		{
			IChatRoom oldRoom = this.Room;
			Control ctl = display.Control;
			this.Controls.Remove(ctl);
			ctl.Dispose();
			display.Dispose();
			CreateDisplay();
			if (DisplayChanged != null)
				DisplayChanged(this, e);
			roomEventActive = false;
			try {
				this.Room = oldRoom;
			} finally {
				roomEventActive = true;
			}
		}
		
		public IDisplay Display {
			get {
				return display;
			}
		}
		
		public IChatRoom Room {
			get {
				return display.Room;
			}
			set {
				display.Room = value;
			}
		}
		
		bool roomEventActive = true;
		
		private void DisplayRoomChanged(object sender, EventArgs e)
		{
			userView.Room = this.Room;
			if (roomEventActive)
				OnRoomChanged(e);
		}
		
		public event EventHandler RoomChanged;
		
		protected virtual void OnRoomChanged(EventArgs e)
		{
			if (RoomChanged != null) {
				RoomChanged(this, e);
			}
		}
	}
}
