// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.Windows.Forms;
using DGNetwork.Services;

namespace DGNetwork.Chat
{
	/// <summary>
	/// Service class for drag'n'drop operations into the editbox.
	/// </summary>
	public sealed class DragDropService
	{
		private DragDropService() {}
		
		static ArrayList converter;
		
		static DragDropService()
		{
			converter = Helper.BuildItems("/Chat/DataObjectConverter", null);
			converter.Add(new StandardDataObjectConverter());
		}
		
		public static object GetObject(IDataObject d, object dropTarget)
		{
			object o;
			foreach (IDataObjectConverter c in converter) {
				o = c.GetObject(d, dropTarget);
				if (o != null)
					return o;
			}
			return null;
		}
		
		public static DragDropEffects GetEffects(IDataObject d, object dropTarget)
		{
			if (d == null)
				throw new ArgumentNullException("d");
			foreach (IDataObjectConverter c in converter) {
				DragDropEffects e = c.GetEffects(d, dropTarget);
				if (e != DragDropEffects.None)
					return e;
			}
			return DragDropEffects.None;
		}
		
		public static DragDropEffects GetEffects(object o, object dropTarget)
		{
			if (o == null)
				return DragDropEffects.None;
			foreach (IDataObjectConverter c in converter) {
				DragDropEffects e = c.GetEffects(o, dropTarget);
				if (e != DragDropEffects.None)
					return e;
			}
			return DragDropEffects.None;
		}
		
		/// <summary>
		/// Converts a data object to a text.
		/// </summary>
		public static string GetText(IDataObject d, object dropTarget)
		{
			return GetText(GetObject(d, dropTarget), dropTarget);
		}
		
		/// <summary>
		/// Converts a data object to a text.
		/// </summary>
		public static string GetText(object o, object dropTarget)
		{
			if (o == null)
				return null;
			foreach (IDataObjectConverter c in converter) {
				string txt = c.GetText(o, dropTarget);
				if (txt != null)
					return txt;
			}
			return null;
		}
	}
}
