// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Drawing;
using System.Windows.Forms;

namespace DGNetwork.Chat
{
	public interface IEditBox : IHasRoom
	{
		Control Control { get; }
		
		//IChatRoom Room { get; set; }    // inherited from IHasRoom
		//event EventHandler RoomChanged; // inherited from IHasRoom
		
		bool CanSend { get; }
		event EventHandler CanSendChanged;
		
		/// <summary>
		/// This event is raised when the user triggers the send button via some key combination
		/// or similar action. It is not raised when the button is *clicked*.
		/// Normally, you should call <see cref="Send"/> in this event's handler.
		/// </summary>
		event EventHandler SendTriggered;
		
		IMessageSendResult Send();
		void Focus();
		
		string Rtf { get; set; }
		string Text { get; set; }
		string Html { get; set; }
		
		Font Font { get; set; }
		Color Color { get; set; }
		
		void InsertText(string text);
	}
	
	public abstract class AbstractEditBox : IEditBox
	{
		#region CanSend
		bool canSend = true;
		public event EventHandler CanSendChanged;
		
		public bool CanSend {
			get {
				return canSend;
			}
			set {
				if (canSend == value)
					return;
				canSend = value;
				OnCanSendChanged(EventArgs.Empty);
			}
		}
		
		protected virtual void OnCanSendChanged(EventArgs e)
		{
			if (CanSendChanged != null) {
				CanSendChanged(this, e);
			}
		}
		#endregion
		
		#region Room
		IChatRoom room;
		
		public IChatRoom Room {
			get {
				return room;
			}
			set {
				if (room != value) {
					room = value;
					OnRoomChanged(EventArgs.Empty);
				}
			}
		}
		
		public event EventHandler RoomChanged;
		
		protected virtual void OnRoomChanged(EventArgs e)
		{
			if (RoomChanged != null) {
				RoomChanged(this, e);
			}
		}
		#endregion
		
		public event EventHandler SendTriggered;
		
		protected virtual void OnSendTriggered(EventArgs e)
		{
			if (SendTriggered != null) {
				SendTriggered(this, e);
			}
		}
		
		public abstract Control Control { get; }
		public abstract IMessageSendResult Send();
		public abstract void Focus();
		
		public abstract string Rtf { get; set; }
		public abstract string Text { get; set; }
		public abstract string Html { get; set; }
		
		public abstract Font Font { get; set; }
		public abstract Color Color { get; set; }
		public abstract void InsertText(string text);
	}
}
