// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.ComponentModel;
using System.Reflection;
using System.Windows.Forms;

using ICSharpCode.Core.AddIns;
using ICSharpCode.Core.AddIns.Codons;
using ICSharpCode.Core.AddIns.Conditions;
using ICSharpCode.Core.Properties;

using DGNetwork.Gui;
using DGNetwork.Services;

namespace DGNetwork.Chat
{
	[CodonName("EditBox")]
	public class EditBoxCodon : AbstractCodon
	{
		[XmlMemberAttribute("title", IsRequired = true)]
		string title = null;
		
		public string Title {
			get {
				return title;
			}
		}
		
		/// <summary>
		/// Creates an item with the specified sub items. And the current
		/// Condition status for this item.
		/// </summary>
		public override object BuildItem(object owner, ArrayList subItems, ConditionCollection conditions)
		{
			if (subItems.Count > 0) {
				Service.Message.ShowErrorFormatted("${res:Codon.NoSubItems}", this.Name);
			}
			Type type = Helper.GetType(this.Class);
			if (type == null) {
				Service.Message.ShowErrorFormatted("${res:Global.ClassNotFound}", this.Class);
				return null;
			}
			if (typeof(IEditBox).IsAssignableFrom(type)) {
				return new DefaultEditBoxFactory(this.ID, this.Title, type);
			} else {
					Service.Message.ShowError("EditBox class must implement IEditBox.");
					return null;
			}
		}
	}
}
