// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.ComponentModel;
using System.Reflection;
using System.Windows.Forms;

using ICSharpCode.Core.AddIns;
using ICSharpCode.Core.AddIns.Codons;
using ICSharpCode.Core.AddIns.Conditions;
using ICSharpCode.Core.Properties;

using DGNetwork.Gui;
using DGNetwork.Services;

namespace DGNetwork.Chat
{
	public class MessageFilterChain : IMessageFilter
	{
		ArrayList filters = new ArrayList();
		
		/// <summary>
		/// Gets the list of filters in this chain.
		/// </summary>
		public ArrayList Filters {
			get {
				return filters;
			}
		}
		
		/// <summary>
		/// Filters the message. The method can change the message's properties.
		/// </summary>
		/// <param name="message">
		/// The message being filtered.
		/// </param>
		/// <param name="mode">
		/// The mode parameter determines the message mode(s) that will be used further
		/// so e.g. the chat code to html filter can ignore messages if they are
		/// going to a text only network.
		/// </param>
		public void Filter(Message message, MessageModes mode, object owner)
		{
			foreach (IMessageFilter filter in filters) {
				filter.Filter(message, mode, owner);
			}
		}
	}
	
	[CodonName("MessageFilterChain")]
	public class MessageFilterChainCodon : AbstractCodon
	{
		/// <summary>
		/// Creates an item with the specified sub items. And the current
		/// Condition status for this item.
		/// </summary>
		public override object BuildItem(object owner, ArrayList subItems, ConditionCollection conditions)
		{
			MessageFilterChain m = new MessageFilterChain();
			m.Filters.AddRange(subItems);
			return m;
		}
	}
}

