// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;

namespace DGNetwork.Chat
{
	public interface IMessageSendResult
	{
		MessageSendStatus Status { get; }
		void CallOnFinish(EventHandler method);
		Message Message { get; }
		string ResultText { get; }
	}
	
	public class DefaultMessageSendResult : IMessageSendResult
	{
		MessageSendStatus status = MessageSendStatus.Sending;
		
		public MessageSendStatus Status {
			get {
				return status;
			}
		}
		
		Message message;
		
		public Message Message {
			get {
				return message;
			}
		}
		
		string resultText;
		
		public string ResultText {
			get {
				return resultText;
			}
			set {
				resultText = value;
			}
		}
		
		public DefaultMessageSendResult(Message message)
		{
			this.message = message;
		}
		
		object lockObject = new object();
		
		private event EventHandler Finish;
		
		public void CallOnFinish(EventHandler method)
		{
			lock (lockObject) {
				if (status != MessageSendStatus.Sending)
					method(this, EventArgs.Empty);
				else
					Finish += method;
			}
		}
		
		public void SetResult(MessageSendStatus result)
		{
			lock (lockObject) {
				if (result == MessageSendStatus.Sending)
					throw new ArgumentException("Sending is not a valid result status.");
				if (status != MessageSendStatus.Sending)
					throw new ArgumentException("Status was already set, you can call SetResult only once!");
				status = result;
				if (Finish != null) {
					Finish(this, EventArgs.Empty);
				}
			}
		}
	}
	
	public enum MessageSendStatus
	{
		Sending,
		SendError,
		SendTimeout,
		TargetRejectedMessage,
		UnknownError,
		Success,
		CommandExecuted,
		CommandFailed
	}
}
