// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections.Specialized;
using System.Text;

namespace DGNetwork.Chat.Parser
{
	public class BlockTag : AbstractTag
	{
		public override TagType GetTagType(TagInfo tag, HtmlFilter filter)
		{
			TagType t = TagType.Block;
			foreach (string attr in this.RequiredAttributes) {
				if (attr == this.KeyAttribute) {
					if (GetAttribute(tag, attr) == null)
						t = TagType.Content;
				} else {
					if (tag.Attributes[attr] == null)
						return TagType.Invalid;
				}
			}
			return t;
		}
		
		protected string GetAttribute(TagInfo tag, string name)
		{
			string val = tag.Attributes[name];
			if (val == null && name == this.KeyAttribute)
				return tag.DirectAttribute;
			else
				return val;
		}
		
		public override void WriteTag(StringBuilder b, TagInfo tag, HtmlFilter filter)
		{
			b.Append('<');
			if (tag.IsEndTag)
				b.Append('/');
			b.Append(this.HtmlName);
			if (!tag.IsEndTag) {
				foreach (string attr in this.RequiredAttributes) {
					WriteAttribute(b, attr, GetAttribute(tag, attr));
				}
				foreach (string attr in this.OptionalAttributes) {
					WriteAttribute(b, attr, GetAttribute(tag, attr));
				}
			}
			b.Append('>');
		}
		
		public override void WriteContentTag(StringBuilder b, TagInfo tag, string content, HtmlFilter filter)
		{
			tag.DirectAttribute = content;
			WriteTag(b, tag, filter);
			tag.DirectAttribute = null;
			b.Append(HtmlHelper.EncodeEntities(content));
			TagInfo end = tag.CreateEndTag();
			WriteTag(b, end, filter);
		}
	}
}
