// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Text;
using DGNetwork.FileTransfer;

namespace DGNetwork.Chat.Parser.Display
{
	public class ImageTag : SafeSimpleTag
	{
		public override void WriteTag(StringBuilder b, TagInfo tag, HtmlFilter filter)
		{
			b.Append('<');
			b.Append(this.HtmlName);
			foreach (string attr in this.RequiredAttributes) {
				CopyAttribute(b, attr, tag, filter);
			}
			foreach (string attr in this.OptionalAttributes) {
				CopyAttribute(b, attr, tag, filter);
			}
			b.Append('>');
		}
		
		private void CopyAttribute(StringBuilder b, string attr, TagInfo tag, HtmlFilter filter)
		{
			string val = GetAttribute(tag, attr);
			if (val == null)
				return;
			if (attr == "src" && val.StartsWith("file:")) {
				FileData data = (FileData)filter.Message.ExtraData[val];
				if (data == null) {
					b.Append(">&lt;<b>");
					b.Append(HtmlHelper.EncodeEntities(val));
					b.Append("</b> not found in message</i>");
				} else {
					b.Append(" src=");
					ITempFileManager manager = filter.Owner as ITempFileManager;
					string filename = data.SaveToTemp(manager);
					if (manager != null)
						filename = manager.GetRelativeName(filename);
					string url;
					if (System.IO.Path.IsPathRooted(filename))
					    url = "file://" + filename.Replace('\\', '/');
					else
						url = filename.Replace('\\', '/');
					
					
					b.Append(HtmlHelper.EncodeAttribute(url));
					b.Append(" alt=");
					b.Append(HtmlHelper.EncodeAttribute(data.Name));
					b.Append(" title=");
					b.Append(HtmlHelper.EncodeAttribute(data.Name));
				}
			} else {
				WriteAttribute(b, attr, val);
			}
		}
	}
}
