// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Text;

namespace DGNetwork.Chat.Parser.Display
{
	public class SafeSimpleTag : SimpleTag
	{
		protected override void WriteAttribute(StringBuilder b, string name, string val)
		{
			if (name == "src" || name == "href" || name == "url")
				base.WriteAttribute(b, name, MakeUrlSafe(val));
			else
				base.WriteAttribute(b, name, val);
		}
		
		public static string MakeUrlSafe(string url)
		{
			if (url == null)
				return null;
			string[] goodProtocols = {"http://", "https://", "ftp://", "mailto:"};
			foreach (string prefix in goodProtocols) {
				if (url.StartsWith(prefix))
					return url;
			}
			return "http://" + url;
		}
	}
	
	public class SafeBlockTag : BlockTag
	{
		protected override void WriteAttribute(StringBuilder b, string name, string val)
		{
			if (name == "src" || name == "href" || name == "url")
				base.WriteAttribute(b, name, SafeSimpleTag.MakeUrlSafe(val));
			else
				base.WriteAttribute(b, name, val);
		}
	}
}
