// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;

using ICSharpCode.Core.AddIns;
using ICSharpCode.Core.AddIns.Codons;
using ICSharpCode.Core.AddIns.Conditions;

using DGNetwork.Services;

namespace DGNetwork.Chat.Parser
{
	[CodonName("HtmlFilter")]
	public class HtmlFilterCodon : AbstractCodon
	{
		[XmlMemberAttribute("extendedSyntax", IsRequired = false)]
		string extendedSyntax = null;
		
		public bool ExtendedSyntax {
			get {
				return extendedSyntax == "true";
			}
		}
		
		/// <summary>
		/// Creates an item with the specified sub items. And the current
		/// Condition status for this item.
		/// </summary>
		public override object BuildItem(object owner, ArrayList subItems, ConditionCollection conditions)
		{
			HtmlFilter filter = new HtmlFilter();
			filter.ExtendedSyntax = this.ExtendedSyntax;
			foreach (object o in subItems) {
				if (o is ITag) {
					ITag tag = (ITag)o;
					string name = tag.Name;
					if (name == null) {
						Service.Message.ShowError("The name of " + tag.GetType().FullName + " is null.");
					} else {
						filter.Tags.Add(name, tag);
					}
				} else if (o is IReplacement) {
					filter.Replacements.Add(o);
				} else {
					Service.Message.ShowErrorFormatted("${res:Codon.InvalidSubItem}",
					                                   this.Name,
					                                   o.GetType().FullName);
				}
			}
			return filter;
		}
	}
}
