// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;

using ICSharpCode.Core.AddIns;
using ICSharpCode.Core.AddIns.Codons;
using ICSharpCode.Core.AddIns.Conditions;

using DGNetwork.Services;

namespace DGNetwork.Chat.Parser
{
	[CodonName("Replacement")]
	public class ReplacementCodon : AbstractCodon
	{
		[XmlMemberAttribute("code", IsRequired = true)]
		string code = null;
		
		[XmlMemberAttribute("replacement", IsRequired = true)]
		string replacement = null;
		
		[XmlMemberAttribute("isRegex", IsRequired = false)]
		string isRegex = null;
		
		[XmlMemberAttribute("isHtml", IsRequired = false)]
		string isHtml = null;
		
		/// <summary>
		/// Creates an item with the specified sub items. And the current
		/// Condition status for this item.
		/// </summary>
		public override object BuildItem(object owner, ArrayList subItems, ConditionCollection conditions)
		{
			if (subItems.Count > 0) {
				Service.Message.ShowErrorFormatted("${res:Codon.NoSubItems}", this.Name);
			}
			string code = this.code;
			string replacement = this.replacement;
			if (isHtml != "true") {
				code = HtmlHelper.EncodeEntities(code);
				replacement = HtmlHelper.EncodeEntities(replacement);
			}
			return new DefaultReplacement(code, replacement, isRegex == "true");
		}
	}
}
