// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections.Specialized;
using System.Text;

namespace DGNetwork.Chat.Parser
{
	public interface ITag
	{
		string Name { get; }
		TagType GetTagType(TagInfo tag, HtmlFilter filter);
		void WriteTag(StringBuilder b, TagInfo tag, HtmlFilter filter);
		
		/// <summary>
		/// Writes a content tag.
		/// </summary>
		void WriteContentTag(StringBuilder b, TagInfo tag, string content, HtmlFilter filter);
	}
	
	public abstract class AbstractTag : ITag
	{
		string name;
		
		public string Name {
			get {
				return name;
			}
			set {
				name = value;
			}
		}
		
		string htmlName;
		
		public string HtmlName {
			get {
				if (htmlName == null)
					return this.Name;
				return htmlName;
			}
			set {
				htmlName = value;
			}
		}
		
		string keyAttribute;
		
		/// <summary>
		/// Gets/Sets the name the direct attribute [tagname=value] is translated into.
		/// null to deactivate
		/// </summary>
		public string KeyAttribute {
			get {
				return keyAttribute;
			}
			set {
				keyAttribute = value;
			}
		}
		
		StringCollection requiredAttributes = new StringCollection();
		StringCollection optionalAttributes = new StringCollection();
		
		public StringCollection OptionalAttributes {
			get {
				return optionalAttributes;
			}
		}
		
		public StringCollection RequiredAttributes {
			get {
				return requiredAttributes;
			}
		}
		
		public abstract TagType GetTagType(TagInfo tag, HtmlFilter filter);
		
		public abstract void WriteTag(StringBuilder b, TagInfo tag, HtmlFilter filter);
		
		/// <summary>
		/// Writes a content tag.
		/// </summary>
		public virtual void WriteContentTag(StringBuilder b, TagInfo tag, string content, HtmlFilter filter)
		{
			throw new NotImplementedException("This tag does not support content mode.");
		}
		
		protected virtual void WriteAttribute(StringBuilder b, string name, string val)
		{
			if (val == null)
				return;
			b.Append(' ');
			b.Append(name);
			b.Append('=');
			b.Append(HtmlHelper.EncodeAttribute(val));
		}
	}
}
