// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.Collections.Specialized;

using ICSharpCode.Core.AddIns;
using ICSharpCode.Core.AddIns.Codons;
using ICSharpCode.Core.AddIns.Conditions;

using DGNetwork.Services;

namespace DGNetwork.Chat.Parser
{
	[CodonName("HtmlFilterTag")]
	public class TagCodon : AbstractCodon
	{
		[XmlMemberAttribute("htmlName", IsRequired = false)]
		string htmlName = null;
		
		[XmlMemberAttribute("keyAttribute", IsRequired = false)]
		string keyAttribute = null;
		
		[XmlMemberArray("required", IsRequired = false)]
		string[] required = null;
		
		[XmlMemberArray("optional", IsRequired = false)]
		string[] optional = null;
		
		/// <summary>
		/// Creates an item with the specified sub items. And the current
		/// Condition status for this item.
		/// </summary>
		public override object BuildItem(object owner, ArrayList subItems, ConditionCollection conditions)
		{
			if (subItems.Count > 0) {
				Service.Message.ShowErrorFormatted("${res:Codon.NoSubItems}", this.Name);
			}
			object o;
			switch (this.Class) {
				case "BlockTag":
					o = new BlockTag();
					break;
				case "SimpleTag":
					o = new SimpleTag();
					break;
				case "ContentTag":
					o = new ContentTag();
					break;
				default:
					o = this.AddIn.CreateObject(this.Class);
					break;
			}
			if (o == null)
				return null;
			AbstractTag tag = o as AbstractTag;
			if (tag == null) {
				Service.Message.ShowError("The codon <HtmlFilterTag> can only be used with classes that " +
				                          "inherit from AbstractTag. Use the <Class> codon for other " +
				                          "tags instead.");
				return null;
			}
			tag.Name = this.ID;
			tag.HtmlName = htmlName;
			tag.KeyAttribute = keyAttribute;
			if (required != null)
				tag.RequiredAttributes.AddRange(required);
			if (optional != null)
				tag.OptionalAttributes.AddRange(optional);
			if (keyAttribute != null) {
				if (!tag.RequiredAttributes.Contains(keyAttribute) && !tag.OptionalAttributes.Contains(keyAttribute))
					tag.RequiredAttributes.Add(keyAttribute);
			}
			return tag;
		}
	}
}
