// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using DGNetwork.Services;

namespace DGNetwork.Chat
{
	public class PlainTextEditBox : AbstractEditBox
	{
		TextBox textBox;
		
		public PlainTextEditBox()
		{
			textBox = new TextBox();
			textBox.Multiline = true;
			textBox.ScrollBars = ScrollBars.Vertical;
			textBox.Font = ChatProperties.Font;
			textBox.ForeColor = ChatProperties.Color;
			textBox.TextChanged += new EventHandler(TextBoxTextChanged);
			textBox.AllowDrop = true;
			textBox.DragEnter += new DragEventHandler(TextBoxDrag);
			textBox.DragDrop  += new DragEventHandler(TextBoxDrop);
			textBox.KeyDown += new KeyEventHandler(TextBoxKeyDown);
			this.CanSend = false;
		}
		
		private void TextBoxKeyDown(object sender, KeyEventArgs e)
		{
			if (e.KeyCode == Keys.Enter) {
				if (e.Control || (!e.Shift && Service.Chat.SendOnEnter)) {
					e.Handled = true;
					OnSendTriggered(EventArgs.Empty);
				}
			}
		}
		
		private void TextBoxTextChanged(object sender, EventArgs e)
		{
			this.CanSend = textBox.Text.Trim().Length > 0;
		}
		
		#region EditBox implementation
		public override Control Control {
			get {
				return textBox;
			}
		}
		
		public override IMessageSendResult Send()
		{
			Message message = new Message(true);
			message.Room = this.Room;
			message.Text = this.Text;
			message.Html = Converter.FontToHtml(this.Font, this.Color) + this.Text;
			Service.Chat.SendFilter.Filter(message, MessageModes.Text | MessageModes.Html, this);
			IMessageSendResult result = this.Room.SendMessage(message);
			if (result != null)
				this.Text = "";
			return result;
		}
		
		public override void InsertText(string text)
		{
			if (text == null)
				return;
			textBox.SelectedText = text;
		}
		
		public override void Focus()
		{
			textBox.Focus();
		}
		
		public override string Rtf {
			get {
				return Converter.TextToRtf(this.Text);
			}
			set {
				this.Text = Converter.RtfToText(value);
			}
		}
		
		public override string Text {
			get {
				return textBox.Text;
			}
			set {
				textBox.Text = value;
			}
		}
		
		public override string Html {
			get {
				return Converter.TextToHtml(this.Text);
			}
			set {
				this.Text = Converter.HtmlToText(value);
			}
		}
		
		public override Font Font {
			get {
				return textBox.Font;
			}
			set {
				textBox.Font = value;
				ChatProperties.Font = value;
			}
		}
		
		public override Color Color {
			get {
				return textBox.ForeColor;
			}
			set {
				textBox.ForeColor = value;
				ChatProperties.Color = value;
			}
		}
		#endregion
		
		#region Drag'n'Drop
		private void TextBoxDrag(object sender, DragEventArgs e)
		{
			try {
				e.Effect = DragDropService.GetEffects(e.Data, this);
			} catch (Exception ex) {
				Service.Message.ShowError(ex);
			}
		}
		
		private void TextBoxDrop(object sender, DragEventArgs e)
		{
			try {
				InsertText(DragDropService.GetText(e.Data, this));
			} catch (Exception ex) {
				Service.Message.ShowError(ex);
			}
		}
		#endregion
	}
}
