// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;

namespace DGNetwork.Chat
{
	public interface IUser
	{
		string Name { get; }
		event EventHandler NameChanged;
		
		string Icon { get; }
		event EventHandler IconChanged;
		
		PrivateChatRoom PrivateChatRoom { get; }
		
		IMessageSendResult SendMessage(Message message);
	}
	
	public abstract class AbstractUser : IUser
	{
		#region Name
		string name;
		
		public string Name {
			get {
				return name;
			}
			set {
				if (name == value)
					return;
				name = value;
				OnNameChanged(EventArgs.Empty);
			}
		}
		
		public event EventHandler NameChanged;
		
		protected virtual void OnNameChanged(EventArgs e)
		{
			if (NameChanged != null) {
				NameChanged(this, e);
			}
		}
		#endregion
		
		#region Icon
		string icon;
		
		public string Icon {
			get {
				return icon;
			}
			set {
				if (icon == value)
					return;
				icon = value;
				OnIconChanged(EventArgs.Empty);
			}
		}
		
		public event EventHandler IconChanged;
		
		protected virtual void OnIconChanged(EventArgs e)
		{
			if (IconChanged != null) {
				IconChanged(this, e);
			}
		}
		#endregion
		
		PrivateChatRoom privateChat;
		
		public virtual PrivateChatRoom PrivateChatRoom {
			get {
				if (privateChat == null) {
					privateChat = new PrivateChatRoom(this);
				}
				return privateChat;
			}
		}
		
		public abstract IMessageSendResult SendMessage(Message message);
	}
}
