// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.Collections.Specialized;

namespace DGNetwork
{
	/// <summary>
	/// The CommandLine class is representing a number of command line parameters.
	/// </summary>
	public class CommandLine
	{
		public static CommandLine Instance;
		
		Hashtable hash = CollectionsUtil.CreateCaseInsensitiveHashtable(); // string -> string
		
		public string this [string parameter] {
			get {
				return (string)hash[parameter];
			}
			set {
				hash[parameter] = value;
			}
		}
		
		public void Remove(string parameter)
		{
			hash.Remove(parameter);
		}
		
		public bool HasFlag(string parameter)
		{
			return hash.ContainsKey(parameter);
		}
		
		public void SetFlag(string parameter)
		{
			hash[parameter] = null;
		}
		
		public CommandLine()
		{
		}
		
		public CommandLine(string[] args)
		{
			string lastKey = null;
			for (int i = 0; i < args.Length; i++) {
				string arg = args[i].Trim();
				if (arg.Length == 0)
					continue;
				if (arg[0] == '-' || arg[0] == '/') {
					if (lastKey != null) {
						hash[lastKey] = null;
						lastKey = null;
					}
					if (arg.StartsWith("--"))
						arg = arg.Substring(2);
					else
						arg = arg.Substring(1);
					int pos = arg.IndexOf(':');
					if (pos > 0) {
						string val = arg.Substring(pos + 1);
						arg = arg.Substring(0, pos);
						if (val.Length > 0)
							hash[arg] = val;
						else
							lastKey = arg;
					} else {
						if (arg.Length > 0)
							lastKey = arg;
					}
				} else {
					if (lastKey == null)
						continue;
					hash[lastKey] = arg;
					lastKey = null;
				}
			}
			if (lastKey != null)
				hash[lastKey] = null;
		}
	}
}
