// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using DGNetwork.Chat;
using DGNetwork.Gui;
using DGNetwork.Gui.Dialogs;
using DGNetwork.Services;
using ICSharpCode.Core.AddIns;
using ICSharpCode.Core.AddIns.Codons;

namespace DGNetwork.Commands
{
	public class InitializeTNA : AbstractCommand
	{
		ITNAIcon icon;
		
		public override void Run()
		{
			icon = Service.TNA.MainIcon;
			icon.DoubleClick += new EventHandler(TNAIconDoubleClick);
			icon.Visible = true;
			if (Service.Property.GetProperty("FirstStart", true)) {
				Service.Property.SetProperty("FirstStart", false);
				icon.ShowBalloon(BalloonIcon.Info, "${res:TNAIcon.TNAInformation}", "DG Network", 5000, null, null);
			}
			Service.Chat.StatusChanged += new StatusChangedEventHandler(ChatStatusChanged);
		}
		
		private void ChatStatusChanged(object sender, StatusChangedEventArgs e)
		{
			try {
				icon.Icon = Service.Resource.GetIcon("Icons.Status." + Service.Chat.Status);
			} catch (ResourceNotFoundException) {
				icon.Icon = System.Drawing.SystemIcons.Error;
			}
		}
		
		private void TNAIconDoubleClick(object sender, EventArgs e)
		{
			try {
				if (Service.Chat.OpenNotifiedRooms())
					return;
				Service.Window.ShowMainWindow();
			} catch (Exception ex) {
				Service.Message.ShowError(ex);
			}
		}
	}
	
	public class ShowWizards : AbstractCommand
	{
		public override void Run()
		{
			string addInTreePath = "/Wizards";
			ArrayList arr;
			try {
				arr = AddInTreeSingleton.AddInTree.GetTreeNode(addInTreePath).BuildChildItems(null);
			} catch (TreePathNotFoundException) {
				Service.Message.ShowErrorFormatted("${res:Global.TreePathNotFound}", addInTreePath);
				return;
			}
			WizardDialog dialog = new WizardDialog(arr);
			if (dialog.ActiveWizard != null) {
				DefaultMessageService.ReleaseBuffer();
				using (XmlForm f = new XmlForm("WizardDialog", dialog)) {
					f.Form.ShowDialog();
				}
				DefaultMessageService.ActivateBuffer();
			}
		}
	}
	
	public class StartOnline : AbstractCommand
	{
		public override void Run()
		{
			bool startOnline = Service.Property.GetProperty("ChatService.StartOnline", true);
			if (startOnline) {
				Service.Chat.Status = "Online";
			}
		}
	}
}
