// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using DGNetwork.AddIns.Codons;
using DGNetwork.Gui;
using DGNetwork.Services;
using ICSharpCode.Core.AddIns;
using ICSharpCode.Core.AddIns.Conditions;

namespace DGNetwork.Commands
{
	public interface IExecutable
	{
		bool CanExecute { get; }
		void Execute(ExecuteContext executeContext);
	}
	
	public class ExecuteContext
	{
		string menuItemID;
		
		public string MenuItemID {
			get {
				return menuItemID;
			}
		}
		
		public ExecuteContext(string menuItemID)
		{
			this.menuItemID = menuItemID;
		}
	}
	
	public class ExecuteOwnerCommand : AbstractMenuCommand
	{
		public override void Run()
		{
			object owner = this.Owner;
			IExecutable executable = (IExecutable)owner;
			executable.Execute(new ExecuteContext(this.MenuItemID));
		}
	}
	
	[Condition]
	public class CanExecuteCondition : AbstractCondition
	{
		[XmlMemberAttribute("canexecute", IsRequired = true)]
		string canexecute = null;
		
		public override bool IsValid(object caller)
		{
			IExecutable executable = (IExecutable)caller;
			if (canexecute == "true") {
				return executable.CanExecute;
			} else if (canexecute == "false") {
				return !executable.CanExecute;
			} else {
				Service.Message.ShowMessage("CanExecuteCondition must be 'true' or 'false'");
				return false;
			}
		}
	}
}
