// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.IO;
using System.Reflection;
using System.Windows.Forms;
using DGNetwork.AddIns.Codons;
using DGNetwork.Gui;
using DGNetwork.Gui.Dialogs;
using DGNetwork.Services;

namespace DGNetwork.Commands
{
	public class MenuShow : AbstractMenuCommand {
		public override void Run() {
			Service.Window.ShowMainWindow();
		}
	}
	
	public class FindFriend : AbstractMenuCommand {
		public override void Run() {
			XmlForm.ShowForm("SearchDialog", typeof(SearchDialogSink));
		}
	}
	
	public class MenuMonitor : AbstractMenuCommand {
		public override void Run() {
			XmlForm.ShowForm("Monitor", typeof(MonitorSink));
		}
	}
	
	public class MenuSettings : AbstractMenuCommand {
		public override void Run() {
			XmlForm.ShowForm("SettingsDialog", typeof(SettingsDialogSink));
		}
	}
	
	public class MenuTransfer : AbstractMenuCommand {
		public override void Run() {
			DGNetwork.FileTransfer.TransferService.ShowTransferForm();
		}
	}
	
	public class MenuAbout : AbstractMenuCommand {
		public override void Run() {
			XmlForm.ShowForm("AboutDialog", typeof(AboutDialogSink));
			XmlForm form = XmlForm.GetForm("AboutDialog");
			using (Stream data = Assembly.GetExecutingAssembly().GetManifestResourceStream("info.rtf")) {
				((RichTextBox)form["InfoRichTextBox"]).LoadFile(data, RichTextBoxStreamType.RichText);
			}
		}
	}
	
	public class MenuQuit : AbstractMenuCommand {
		public override void Run() {
			Application.Exit();
		}
	}
	
	public class SetStatus : AbstractMenuCommand {
		public override void Run() {
			Service.Chat.Status = this.MenuItemID;
		}
	}
}
