// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Windows.Forms;
using DGNetwork.AddIns.Codons;
using DGNetwork.Gui;
using DGNetwork.Gui.Dialogs;
using DGNetwork.Services;
using ICSharpCode.Core.Services;

namespace DGNetwork.Contacts
{
	public class CreateNewGroupCommand : AbstractMenuCommand
	{
		public override void Run()
		{
			string name = Service.Message.ShowInputBox("${res:Contacts.CreateNewGroupEnterName}", "${res:Contacts.CreateNewGroup}", "");
			if (name == null || name.Length == 0)
				return;
			DefaultContactList list = new DefaultContactList();
			list.Name = name;
			list.Icon = "Icons.OpenFolder";
			list.ContextMenuTreePath = "/Context/Contacts/DefaultGroup";
			((IContactList)this.Owner).Contacts.Add(list);
		}
	}
	
	public class RenameCommand : AbstractMenuCommand
	{
		public override void Run()
		{
			IContact contact = (IContact)this.Owner;
			string name = Service.Message.ShowInputBox("${res:Contacts.EnterNewName}", "${res:Contacts.Rename}", contact.Name);
			if (name == null || name.Length == 0)
				return;
			contact.Name = name;
		}
	}
	
	public class DeleteCommand : AbstractMenuCommand
	{
		public override void Run()
		{
			IContact contact = (IContact)this.Owner;
			Delete(Service.Contact.Contacts, contact);
		}
		
		private bool Delete(ContactCollection contacts, IContact contact)
		{
			foreach (IContact c in contacts) {
				if (c == contact) {
					contacts.Remove(contact);
					return true;
				} else if (c is IContactList) {
					if (Delete(((IContactList)c).Contacts, contact))
						return true;
				}
			}
			return false;
		}
	}
}
