// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;

namespace DGNetwork.Contacts
{
	/// <summary>
	/// A fixed contact is a contact in a FixedContactList.
	/// </summary>
	public class FixedContact : DefaultContact
	{
		FixedContactList parent;
		
		public FixedContact()
		{
		}
		
		public FixedContact(FixedContact fixedContact) : base(fixedContact)
		{
		}
		
		public FixedContactList Parent {
			get {
				return parent;
			}
		}
		
		/// <summary>
		/// Gets if this contact can be cloned.
		/// Special case: A fixed contact SHOULD not be cloned directly, therefore
		/// this method is returning false.
		/// The Clone() method may work nevertheless because it is used when the parent
		/// FixedContactList is cloned.
		/// </summary>
		public override bool CanClone {
			get {
				return false;
			}
		}
		
		/// <summary>
		/// Sets the parent of the FixedContact.
		/// This method is ONLY intended for use by FixedContactList.OnContactAdded
		/// </summary>
		internal void SetParent(FixedContactList newParent)
		{
			if (this.parent != null)
				throw new InvalidOperationException("parent was already set");
			this.parent = newParent;
		}
		
		/// <summary>
		/// Gets if the specified contact list can be a parent of this contact.
		/// </summary>
		public override bool IsValidParent(IContactList parent)
		{
			return parent == this.parent;
		}
	}
}
