// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;

namespace DGNetwork.Contacts
{
	/// <summary>
	/// A contact list that specifies its content on its own and cannot be modified by the user.
	/// </summary>
	public class FixedContactList : DefaultContactList
	{
		public FixedContactList()
		{
		}
		
		public FixedContactList(FixedContactList fixedContactList) : base(fixedContactList)
		{
		}
		
		public override bool CanClone {
			get {
				return true;
			}
		}
		
		protected override void CheckContactBeforeAdd(IContact contact)
		{
			FixedContact c = contact as FixedContact;
			if (c != null && c.Parent == null)
				c.SetParent(this);
			base.CheckContactBeforeAdd(contact);
		}
		
		public override bool IsValidChild(IContact contact)
		{
			if (contact is FixedContact)
				return ((FixedContact)contact).Parent == this;
			else
				return false;
		}
	}
}
