// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using DGNetwork.Services;
using Grunwald.Gui.Controls;
using Grunwald.Gui.Settings;
using Grunwald.P2PLib.DGNetwork;
using DGNetwork.AddIns.Codons;

namespace DGNetwork.DGN
{
	public class ForgePacketCommand : AbstractMenuCommand {
		public override void Run() {
			ForgePacket.ShowForm();
		}
	}
	
	public class ForgePacket : ReflectionSettingPage
	{
		static Form forgeForm;
		
		public static void ShowForm()
		{
			forgeForm = new Form();
			forgeForm.Text = "Forge DG Network packet";
			SettingsPanel set = new SettingsPanel();
			ForgePacket fp = new ForgePacket();
			set.Page = fp;
			set.Dock = DockStyle.Fill;
			forgeForm.Controls.Add(set);
			ControlRow row = new ControlRow();
			row.Direction = Direction.LeftToRight;
			row.ContentAlignment = ContentAlignment.MiddleRight;
			row.ArrangeActive = true;
			Button button = new AutoSizedButton("&Send");
			button.Click += new EventHandler(fp.Send);
			row.Controls.Add(button);
			row.Dock = DockStyle.Bottom;
			row.Height = button.Height + 8;
			forgeForm.Controls.Add(row);
			forgeForm.AcceptButton = button;
			forgeForm.FormBorderStyle = FormBorderStyle.FixedDialog;
			forgeForm.MinimizeBox = false;
			forgeForm.MaximizeBox = false;
			forgeForm.Show();
		}
		
		[Setting("source", "Source")]
		int source = GetSource();
		
		static int GetSource()
		{
			foreach (Network n in Service.DGN.Networks) {
				return n.UserID;
			}
			return 0;
		}
		
		[Setting("packetID", "Packet ID")]
		int packetId = Helper.Random.Next();
		
		[Setting("answerID", "Answer ID")]
		int answerId = Helper.Random.Next();
		
		[Setting("target", "Target")]
		string target = "";
		
		[Setting("to", "Destination")]
		string to = "";
		
		public void Send(object sender, EventArgs e)
		{
			this.Save();
			Packet p = new Packet();
			PacketHeader h = p.Header;
			h.PacketID = packetId;
			h.Source = source;
			string[] targets = to.Split(',');
			int[] dest = new int[targets.Length];
			for (int i = 0; i < dest.Length; ++i) {
				dest[i] = int.Parse(targets[i]);
			}
			h.Destination = dest;
			p.AnswerId = answerId;
			p.Target = target;
			foreach (Network n in Service.DGN.Networks) {
				SendResult res = n.Send(p);
				if (res != null) {
					res.CallOnAnswer(new AnswerEventHandler(GotAnswer));
					break;
				}
			}
			packetId += 1;
			this.Load();
		}
		
		private void GotAnswer(object sender, AnswerEventArgs e)
		{
			StringBuilder b = new StringBuilder();
			b.Append("Got answer from ");
			b.Append(e.Answer.FromUser);
			b.Append(":\n");
			b.Append(e.Answer);
			b.Append('\n');
			switch (e.Answer.ErrorCode) {
				case Answer.ErrorCodeSuccess:
					break;
				case Answer.ErrorCodeTargetNotFound:
					b.Append(" TARGET NOT FOUND\n");
					break;
				case Answer.ErrorCodeInvalidRequest:
					b.Append(" INVALID REQUEST\n");
					break;
				case Answer.ErrorCodeInternalError:
					b.Append(" INTERNAL ERROR\n");
					break;
				default:
					b.Append(" UNKNOWN ERROR (" + e.Answer.ErrorCode + ")\n");
					break;
			}
			b.Append("Values:\n");
			for (int i = 0; i < e.Answer.Values.Count; ++i) {
				b.AppendFormat("#{0} = {1}\n", i, FormatData(e.Answer.Values.GetValue(i)));
			}
			foreach (DictionaryEntry entry in e.Answer.NamedValues) {
				b.AppendFormat("{0} = {1}\n", entry.Key, FormatData((byte[])entry.Value));
			}
			MessageBox.Show(b.ToString());
		}
		
		static string FormatData(byte[] data)
		{
			if (data.Length > 1024)
				return string.Format("{0} byte", data.Length);
			for (int i = 0; i < data.Length; ++i) {
				if (data[i] < 32) {
					if (data.Length > 20)
						return string.Format("{0} byte", data.Length);
					else
						return Helper.ToHex(data);
				}
			}
			return Encoding.UTF8.GetString(data);
		}
	}
}
