// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.Windows.Forms;
using Grunwald.P2PLib.Network;
using Grunwald.P2PLib.DGNetwork;
using Grunwald.Gui.Controls;
using DGNetwork.Commands;
using DGNetwork.Gui.Dialogs;
using DGNetwork.Services;

namespace DGNetwork.DGN.Monitor
{
	public class ConnectionMonitorItem : DefaultMonitorItem, IExecutable
	{
		NetClient netClient;
		
		public ConnectionMonitorItem(NetClient netClient)
		{
			this.ContextMenuTreePath = "/Context/Monitor/Connection";
			this.netClient = netClient;
			this.ValueText = netClient.Partner.ToString();
			netClient.StatusChanged += new EventHandler(NetClientStatusChanged);
			netClient.Closed        += new ClosedEventHandler(NetClientClosed);
		}
		
		public bool CanExecute {
			get {
				return netClient.Status != NetClientStatus.Dead;
			}
		}
		
		public void Execute(ExecuteContext executeContext)
		{
			netClient.Close("Close by user");
		}
		
		public override void Remove()
		{
			netClient.StatusChanged -= new EventHandler(NetClientStatusChanged);
			netClient.Closed        -= new ClosedEventHandler(NetClientClosed);
			base.Remove();
		}
		
		private void NetClientStatusChanged(object sender, EventArgs e)
		{
			if (netClient.Status == NetClientStatus.Dead)
				return;
			this.ValueText = "${res:DGN.NetClientStatus." + netClient.Status.ToString() + "}";
		}
		
		private void NetClientClosed(object sender, ClosedEventArgs e)
		{
			if (e.ReasonSpecified) {
				this.ValueText = e.Message;
			} else {
				if (e.RemotelyClosed)
					this.ValueText = "${res:DGN.NetClientStatus.ClosedRemotely}";
				else
					this.ValueText = "${res:DGN.NetClientStatus.Closed}";
			}
		}
	}
}
