// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.Windows.Forms;
using Grunwald.P2PLib.Network;
using Grunwald.P2PLib.DGNetwork;
using Grunwald.Gui.Controls;
using DGNetwork.Commands;
using DGNetwork.Gui.Dialogs;
using DGNetwork.Services;

namespace DGNetwork.DGN.Monitor
{
	public class IPServerMonitorItem : DefaultMonitorItem, IExecutable
	{
		DgnIPServer server;
		Network network;
		
		internal NetworkMonitorItem reportBack;
		
		public IPServerMonitorItem(DgnIPServer server, Network network)
		{
			this.ContextMenuTreePath = "/Context/Monitor/IPServer";
			this.server = server;
			this.network = network;
			server.RefreshFinished += new EventHandler(Refreshed);
			Refreshed(null, null);
		}
		
		public override void Remove()
		{
			server.RefreshFinished -= new EventHandler(Refreshed);
			base.Remove();
		}
		
		public bool CanExecute {
			get {
				return server.RefreshNeeded || server.SecondsSinceLastRequest > 10;
			}
		}
		
		public void Execute(ExecuteContext executeContext)
		{
			network.ExecuteRefresh(server);
		}
		
		private void Refreshed(object sender, EventArgs e)
		{
			if (server.LastRefreshWasSuccessful) {
				this.FormatValue = server.UserID;
				this.ValueText = "${res:DGN.IPServer.OK}";
			} else if (server.LastError == DgnIPServerError.ServerError) {
				this.ValueText = server.LastServerError;
			} else {
				string err;
				if (server.LastError == DgnIPServerError.DetectorError)
					err = server.IPDetector.LastError.ToString();
				else
					err = server.LastError.ToString();
				this.ValueText = "${res:DGN.IPServer." + err + "}";
			}
			if (reportBack != null)
				reportBack.RefreshHostCache();
		}
	}
}
