// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.Windows.Forms;
using Grunwald.P2PLib.Network;
using Grunwald.P2PLib.DGNetwork;
using Grunwald.Gui.Controls;
using DGNetwork.Commands;
using DGNetwork.Gui.Dialogs;
using DGNetwork.Services;

namespace DGNetwork.DGN.Monitor
{
	public class NetworkMonitorItem : DefaultMonitorItem
	{
		Network network;
		DefaultMonitorItem connectionsItem = new DefaultMonitorItem();
		DefaultMonitorItem hostCacheItem   = new DefaultMonitorItem();
		ArrayList ipServerMonitors   = new ArrayList();
		ArrayList connectionMonitors = new ArrayList();
		
		public NetworkMonitorItem(Network network) {
			this.network = network;
			network.IPServersChanged   += new EventHandler(IPServersChanged);
			network.ConnectionsChanged += new EventHandler(ConnectionsChanged);
			network.ConnectedChanged   += new EventHandler(ConnectedChanged);
		}
		
		public override void Remove() {
			network.IPServersChanged   -= new EventHandler(IPServersChanged);
			network.ConnectionsChanged -= new EventHandler(ConnectionsChanged);
			network.ConnectedChanged   -= new EventHandler(ConnectedChanged);
			base.Remove();
		}
		
		public override void Initialize(TreeListNode node)
		{
			base.Initialize(node);
			connectionsItem.Icon = "Icons.TwoPCs";
			connectionsItem.Title = "${res:DGN.Connections}";
			connectionsItem.Initialize(new TreeListNode());
			node.Nodes.Add(connectionsItem.TreeNode);
			hostCacheItem.Icon = "Icons.TwoPCs";
			hostCacheItem.Title = "${res:DGN.HostCache}";
			hostCacheItem.Initialize(new TreeListNode());
			node.Nodes.Add(hostCacheItem.TreeNode);
			IPServersChanged(null, null);
			ConnectionsChanged(null, null);
			RefreshHostCache();
		}
		
		private void IPServersChanged(object sender, EventArgs e)
		{
			foreach (IPServerMonitorItem m in ipServerMonitors) {
				this.TreeNode.Nodes.Remove(m.TreeNode);
				m.Remove();
			}
			ipServerMonitors.Clear();
			int i = 0;
			foreach (DgnIPServer s in network.IpServer) {
				i += 1;
				IPServerMonitorItem m = new IPServerMonitorItem(s, network);
				m.reportBack = this;
				m.Icon = "Icons.HolyPC";
				m.Title = "IP Server " + i.ToString();
				m.Initialize(new TreeListNode());
				this.TreeNode.Nodes.Add(m.TreeNode);
				ipServerMonitors.Add(m);
			}
		}
		
		private void ConnectedChanged(object sender, EventArgs e)
		{
			int connCount = network.Connections.Count;
			if (network.Connected) {
				if (connCount > 0)
					this.ValueText = "${res:DGN.NetStatus.Connected}";
				else
					this.ValueText = "${res:DGN.NetStatus.NoConnection}";
			} else {
				this.ValueText = "${res:DGN.NetStatus.Offline}";
			}
		}
		
		private void ConnectionsChanged(object sender, EventArgs e)
		{
			foreach (ConnectionMonitorItem m in connectionMonitors) {
				m.Remove();
			}
			connectionsItem.TreeNode.Nodes.Clear();
			connectionMonitors.Clear();
			int connCount = network.Connections.Count;
			connectionsItem.FormatValue = connCount;
			if (connCount == 1)
				connectionsItem.ValueText = "${res:DGN.OneConnection}";
			else
				connectionsItem.ValueText = "${res:DGN.ConnectionCount}";
			ConnectedChanged(sender, e);
			foreach (NetClient c in network.Connections) {
				ConnectionMonitorItem m = new ConnectionMonitorItem(c);
				m.TitleFormatValue = c.Partner.UserID;
				m.Icon = c.Outgoing ? "Icons.ArrowUp" : "Icons.ArrowDown";
				m.Title = "${res:DGN.ConnectionTo}";
				m.Initialize(new TreeListNode());
				connectionsItem.TreeNode.Nodes.Add(m.TreeNode);
				connectionMonitors.Add(m);
			}
		}
		
		public void RefreshHostCache()
		{
			foreach (TreeListNode node in hostCacheItem.TreeNode.Nodes) {
				((IMonitorItem)node.Tag).Remove();
			}
			hostCacheItem.TreeNode.Nodes.Clear();
			hostCacheItem.FormatValue = network.HostCache.Count;
			if (network.HostCache.Count == 1)
				hostCacheItem.ValueText = "${res:DGN.OneHost}";
			else
				hostCacheItem.ValueText = "${res:DGN.HostCount}";
			foreach (AddressEntry e in network.HostCache) {
				HostEntryItem m = new HostEntryItem(network.HostCache, e, this);
				m.TitleFormatValue = ((UserAddress)e.Address).UserID;
				m.Title = "${res:DGN.HostTitle}";
				m.Initialize(new TreeListNode());
				m.ValueText = e.EndPoint.ToString();
				hostCacheItem.TreeNode.Nodes.Add(m.TreeNode);
			}
		}
	}
}
