// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using DGNetwork.Commands;
using DGNetwork.Gui.Dialogs;
using DGNetwork.Services;
using Grunwald.P2PLib.Server;

namespace DGNetwork.DGN.Monitor
{
	public class ServerAbilityItem : DefaultMonitorItem, IExecutable
	{
		ServerAbilityTester sat;
		
		public ServerAbilityItem()
		{
			this.Title = "${res:DGN.ServerAbility}";
			this.ContextMenuTreePath = "/Context/Monitor/ServerAbility";
			sat = Service.P2P.ServerAbilityTester;
			sat.TestStarted  += new EventHandler(OnTestStarted);
			sat.TestFinished += new EventHandler(OnTestFinished);
			SetText();
		}
		
		public override void Remove()
		{
			sat.TestStarted  -= new EventHandler(OnTestStarted);
			sat.TestFinished -= new EventHandler(OnTestFinished);
			base.Remove();
		}
		
		private void OnTestStarted(object sender, EventArgs e)
		{
			SetText();
		}
		
		private void OnTestFinished(object sender, EventArgs e)
		{
			SetText();
		}
		
		private void SetText()
		{
			if (sat.LastFinishedTest < sat.LastTestStart)
				this.FormatValue = Service.Parse("${res:DGN.ServerAbilityTestRunning}");
			else
				this.FormatValue = sat.LastFinishedTest;
			if (sat.LastTestStart == DateTime.MinValue) {
				this.ValueText = "${res:DGN.ServerAbilityNotTested}";
			} else if (sat.LastFinishedTest == DateTime.MinValue) {
				this.ValueText = "${res:DGN.ServerAbilityTestRunning}";
			} else if (sat.LastResult) {
				this.ValueText = "${res:DGN.ServerAbilityOK}";
			} else {
				this.ValueText = "${res:DGN.ServerAbilityFailed}";
			}
			if (sat.LastResult)
				this.Icon = "Icons.ServerAbilityOK";
			else
				this.Icon = "Icons.ServerAbilityFailed";
		}
		
		public bool CanExecute {
			get {
				return sat.LastFinishedTest >= sat.LastTestStart;
			}
		}
		
		public void Execute(ExecuteContext executeContext)
		{
			if (this.CanExecute)
				Service.P2P.RefreshServerAbility();
		}
	}
}
