// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.Collections.Specialized;
using Grunwald.Gui.Settings;

using DGNetwork.Settings;
using DGNetwork.Services;
using ICSharpCode.Core.Services;

namespace DGNetwork.DGN
{
	/// <summary>
	/// A setting page representing a dg network.
	/// </summary>
	public class NetworkPage : AbstractNetworkPage, IDisposable
	{
		string identifier = null;
		
		[Setting("identifier", "Identifier")]
		public string Identifier {
			get {
				return identifier;
			}
			set {
				if (identifier == value)
					return;
				if (Service.DGN.GetNetwork(value) != null) {
					Service.Message.ShowMessageFormatted("${res:DGN.IdentifierAlreadyUsed}", new string[] {value});
					return;
				}
				identifier = value;
				if (!Service.DGN.Initialized)
					return;
				if (network != null)
					DestroyNetwork();
				identifier = value;
				CreateNetwork();
			}
		}
		
		string ipServer1 = null;
		string ipServer2 = null;
		string ipServer3 = null;
		
		[Setting("ipServer", "IP-Server 1")]
		public string IpServer1 {
			get {
				return ipServer1;
			}
			set {
				ipServer1 = value;
				if (network != null)
					network.SetIPServers(IpServer1, IpServer2, IpServer3);
			}
		}
		
		[Setting("ipServer2", "IP-Server 2")]
		public string IpServer2 {
			get {
				return ipServer2;
			}
			set {
				ipServer2 = value;
				if (network != null)
					network.SetIPServers(IpServer1, IpServer2, IpServer3);
			}
		}
		
		[Setting("ipServer3", "IP-Server 3")]
		public string IpServer3 {
			get {
				return ipServer3;
			}
			set {
				ipServer3 = value;
				if (network != null)
					network.SetIPServers(IpServer1, IpServer2, IpServer3);
			}
		}
		
		public NetworkPage() {
			this.Title = "${res:Settings.DGNetworkPage.Title}";
		}
		
		public NetworkPage(NetworkPage original) : base(original)
		{
		}
		
		Network network;
		
		public override void CreateNetwork()
		{
			if (network != null)
				throw new InvalidOperationException();
			if (identifier == null)
				return;
			network = new Network(this);
			Service.DGN.RegisterNetwork(network);
		}
		
		public override void DestroyNetwork()
		{
			if (network == null)
				throw new InvalidOperationException();
			Service.DGN.RemoveNetwork(network);
			network = null;
		}
		
		public void Dispose()
		{
			if (network != null)
				DestroyNetwork();
		}
	}
}
