// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using Grunwald.P2PLib.DGNetwork;

namespace DGNetwork.DGN
{
	public delegate void PacketReceiver(Network n, PacketEventArgs e);
	
	public class PacketReceiverCollection : IEnumerable
	{
		Hashtable packetReceivers = new Hashtable();
		
		public void Add(string target, PacketReceiver receiver)
		{
			if (packetReceivers.ContainsKey(target))
				throw new ArgumentException("There is already a packet receiver for " + target);
			if (receiver == null)
				throw new ArgumentNullException("receiver");
			packetReceivers[target] = receiver;
		}
		
		public bool Call(Network network, PacketEventArgs e)
		{
			PacketReceiver receiver = (PacketReceiver)packetReceivers[e.Packet.Target];
			if (receiver != null) {
				receiver(network, e);
				return true;
			} else {
				return false;
			}
		}
		
		public PacketReceiver this[string target] {
			get {
				return (PacketReceiver)packetReceivers[target];
			}
			set {
				if (value == null)
					packetReceivers.Remove(target);
				else
					packetReceivers[target] = value;
			}
		}
		
		public void Remove(string target)
		{
			packetReceivers.Remove(target);
		}
		
		public IEnumerator GetEnumerator()
		{
			return packetReceivers.GetEnumerator();
		}
	}
}
