// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;

namespace DGNetwork.FileTransfer
{
	/// <summary>
	/// The structure can help splitting a dgn:-URL.
	/// </summary>
	public struct DGNURL
	{
		/// <summary>
		/// Splits the specified URL. This method checks the hash included in the url.
		/// </summary>
		public DGNURL(string url)
		{
			if (!url.StartsWith("dgn:"))
				throw new ArgumentException("url ist no dgn:-url", "url");
			int pos = url.IndexOf("::");
			if (pos <= 0)
				throw new Exception("Invalid URL");
			FileName = url.Substring(pos + 2);
			string[] parts = url.Substring(0, pos).Split(':');
			FileSize = long.Parse(parts[1]);
			UserID = int.Parse(parts[2]);
			UserName = parts[3];
			BlockSize = int.Parse(parts[4]);
			BlockHash = parts[5];
			Hash = parts[6];
			Identifier = parts[7];
			if (Hash != Helper.MD5(BlockHash).Substring(0, Hash.Length))
				throw new ArgumentException("Hash invalid");
		}
		
		/// <summary>
		/// The name of the tranferred file.
		/// </summary>
		public readonly string FileName;
		
		/// <summary>
		/// The identifier of the network.
		/// </summary>
		public readonly string Identifier;
		
		/// <summary>
		/// The size of the file.
		/// </summary>
		public readonly long FileSize;
		
		/// <summary>
		/// The user id of the user who has sent the file.
		/// </summary>
		public readonly int UserID;
		
		/// <summary>
		/// The nick name of the user who has sent the file.
		/// </summary>
		public readonly string UserName;
		
		/// <summary>
		/// The size of the hash block.
		/// </summary>
		public readonly int BlockSize;
		
		/// <summary>
		/// The hash over the blocks of the file.
		/// </summary>
		public readonly string BlockHash;
		
		/// <summary>
		/// A hash over the blockhash (a kind of hash over the whole file).
		/// </summary>
		public readonly string Hash;
	}
}
