// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.IO;
using System.Xml;
using ICSharpCode.Core.Properties;

namespace DGNetwork.FileTransfer
{
	public class FileData : IXmlConvertable
	{
		byte[] data;
		string name;
		
		public FileData(byte[] data, string name)
		{
			this.data = data;
			this.name = name;
		}
		
		public byte[] Data {
			get {
				return data;
			}
			set {
				data = value;
			}
		}
		
		public string Name {
			get {
				return name;
			}
			set {
				name = value;
			}
		}
		
		public object FromXmlElement(XmlElement element)
		{
			name = element.GetAttribute("name");
			data = Convert.FromBase64String(element.InnerText);
			return this;
		}
		
		public XmlElement ToXmlElement(XmlDocument doc)
		{
			XmlElement e = doc.CreateElement("FileData");
			e.SetAttribute("name", name);
			e.InnerText = Convert.ToBase64String(data);
			return e;
		}
		
		public string SaveToTemp(ITempFileManager manager)
		{
			if (manager == null)
				manager = new DefaultTempFileManager();
			string ext;
			if (Path.HasExtension(name)) {
				ext = Path.GetExtension(name);
				if (!System.Text.RegularExpressions.Regex.IsMatch(ext, @"\.\w+"))
					throw new ApplicationException("Possibly unsafe extension: " + ext);
			} else {
				ext = "";
			}
			string tempFileName = manager.GetTempFileName("file" + Helper.MD5(data) + ext);
			if (!File.Exists(tempFileName)) {
				using (FileStream fs = new FileStream(tempFileName, FileMode.Create, FileAccess.Write)) {
					fs.Write(data, 0, data.Length);
				}
			}
			return tempFileName;
		}
	}
}
