// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.IO;
using System.Text;
using DGNetwork.Chat;
using DGNetwork.Chat.Parser;
using DGNetwork.Services;
using DGNetwork.DGN;

namespace DGNetwork.FileTransfer
{
	public class FileTag : ITag
	{
		public string Name {
			get {
				return "file";
			}
		}
		
		public TagType GetTagType(TagInfo tag, HtmlFilter filter)
		{
			if (tag.Attributes["src"] != null) {
				return TagType.Simple;
			} else {
				if (tag.Attributes["length"] == null)
					return TagType.Invalid;
				if (tag.Attributes["hash"] == null)
					return TagType.Invalid;
				return TagType.Content;
			}
		}
		
		public void WriteTag(StringBuilder b, TagInfo tag, HtmlFilter filter)
		{
			// write the tag for a small file
			string val = tag.Attributes["src"];
			FileData data = (FileData)filter.Message.ExtraData[val];
			if (data == null) {
				b.Append("<b>");
				b.Append(HtmlHelper.EncodeEntities(val));
				b.Append("</b> not found in message</i>");
			} else {
				WriteTag(b, data.SaveToTemp(filter.Owner as ITempFileManager), data.Name);
			}
		}
		
		public void WriteContentTag(StringBuilder b, TagInfo tag, string content, HtmlFilter filter)
		{
			Message msg = filter.Message;
			Network network = msg.ExtraData["network"] as Network;
			if (network == null || msg.ExtraData["userid"] == null) {
				WriteAlert(b, "${res:FileTransfer.CantDownloadWithoutConnectInformation}", content);
			} else {
				StringBuilder url = new StringBuilder("dgn:");
				url.Append(tag.Attributes["length"]);
				url.Append(':');
				url.Append(msg.ExtraData["userid"]);
				url.Append(':');
				url.Append(filter.Message.Nickname);
				url.Append(':');
				url.Append(tag.Attributes["hash"]);
				url.Append(':');
				url.Append(network.Identifier);
				WriteTag(b, url.ToString(), content);
			}
		}
		
		public void WriteAlert(StringBuilder b, string alert, string filename)
		{
			b.AppendFormat("<span class=file onClick=\"alert('{0}')\">" +
			               "<img src=\"file://{1}\" width=14 height=16 alt={3} " +
			               "border=0>&nbsp;<u>{2}</u></span>",
			               ToJavascript(Service.Parse(alert)),
			               Path.Combine(Service.Property.DataDirectory, "file.gif").Replace('\\', '/'),
			               ToJavascript(filename),
			               HtmlHelper.EncodeAttribute(Service.Parse("${res:FileTransfer.AltImageText}")));
		}
		
		public void WriteTag(StringBuilder b, string fileURL, string filename)
		{
			b.AppendFormat("<span class=file onClick=\"cmd('/getfile {0}::{2}')\">" +
			               "<img src=\"file://{1}\" width=14 height=16 alt={3} " +
			               "border=0>&nbsp;<u>{2}</u></span>",
			               ToJavascript(Helper.EscapePart(fileURL)),
			               Path.Combine(Service.Property.DataDirectory, "file.gif").Replace('\\', '/'),
			               ToJavascript(Helper.EscapePart(filename)),
			               HtmlHelper.EncodeAttribute(Service.Parse("${res:FileTransfer.AltImageText}")));
		}
		
		private string ToJavascript(string text)
		{
			return HtmlHelper.EncodeEntities(text.Replace("\\", "\\\\").Replace("\"", "\\\"").Replace("'", "\\'"));
		}
	}
}
