// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.IO;
using System.Text;
using DGNetwork.Chat;
using DGNetwork.Chat.Parser;
using DGNetwork.Services;

namespace DGNetwork.FileTransfer
{
	public class SendFileTag : SimpleTag
	{
		/// <summary>
		/// Die maximale Größe von Dateien, die direkt verschickt werden können (256 KB) in Byte
		/// </summary>
		public const long MaxSmallFileSize = 256*1024;
		
		/// <summary>
		/// Die maximale Größe von Dateien, die an ein Chatpaket angehängt werden können. (32 KB)
		/// </summary>
		public const long MaxInPacketFileSize = 32*1024;
		
		public override void WriteTag(StringBuilder b, TagInfo tag, HtmlFilter filter)
		{
			string file = GetAttribute(tag, "file");
			file = TransferService.FromShortName(file);
			if (!File.Exists(file)) {
				Service.Message.ShowMessageFormatted("${res:FileTransfer.FileDoesNotExist}", new string[] { file });
			} else {
				string ext = Path.GetExtension(file).ToLower();
				FileInfo f = new FileInfo(file);
				if (f.Length > MaxSmallFileSize) {
					b.Append(SendFile.Send(file));
				} else {
					bool isImage = "*.gif;*.jpg;*.jp2;*.png;*.wmf;*.emf;*.jpeg;*.bmp;".IndexOf(ext + ";") >= 0;
					if (f.Length <= MaxInPacketFileSize) {
						byte[] data = new byte[f.Length];
						FileStream fs = f.Open(FileMode.Open, FileAccess.Read, FileShare.Read);
						if (fs.Read(data, 0, (int) f.Length) != f.Length) {
							Service.Message.ShowError("Sending small file: reading file returned wrong length");
							return;
						}
						fs.Close();
						string name = "file:" + Helper.Random.Next().ToString("X");
						filter.Message.ExtraData.Add(name, new FileData(data, Path.GetFileName(file)));
						//path = "attachment:" + path;
						if (isImage)
							b.Append("<img src=");
						else
							b.Append("<file src=");
						b.Append(HtmlHelper.EncodeAttribute(name));
						b.Append(">");
					} else {
						b.Append(SendFile.Send(file));
					}
				}
			}
		}
	}
}
