// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.IO;
using System.Net;
using Grunwald.P2PLib;
using Grunwald.P2PLib.Server;
using Grunwald.P2PLib.Server.Http;

namespace DGNetwork.FileTransfer
{
	/// <summary>
	/// Manages the upload of a file.
	/// </summary>
	public class SharedFileHandler : IRequestHandler
	{
		ArrayList uploads = new ArrayList();
		
		/// <summary>
		/// This list contains all currently active FileUploads uploading this file.
		/// </summary>
		public ArrayList Uploads {
			get {
				return uploads;
			}
		}
		
		#region Properties
		public SharedFileHandler(string filename, long fileSize, string hash, string tag)
		{
			this.filename = filename;
			this.fileSize = fileSize;
			this.hash = hash;
			this.tag = tag;
		}
		
		string hash, tag;
		long fileSize;
		
		public long FileSize {
			get {
				return fileSize;
			}
		}
		
		public string Hash {
			get {
				return hash;
			}
		}
		
		public string Tag {
			get {
				return tag;
			}
		}
		
		/// <summary>
		/// The name of the file.
		/// </summary>
		public string Name {
			get {
				return Path.GetFileName(filename);
			}
		}
		
		/// <summary>
		/// The full filename of the shared file.
		/// </summary>
		public string FullName {
			get {
				return filename;
			}
		}
		#endregion
		
		#region Send
		string filename;
		
		/// <summary>
		/// Sendet die angegebene lokale Datei an den <see cref="ResponseStream" />.
		/// </summary>
		public void Send(HttpRequest request)
		{
			FileUpload upload = new FileUpload(request);
			try {
				uploads.Add(upload);
				upload.Send(filename);
			} finally {
				uploads.Remove(upload);
			}
		}
		
		#endregion
		
		/// <summary>
		/// This method handles a file upload request.
		/// </summary>
		public void Handle(HttpRequest request)
		{
			FileUpload upload = new FileUpload(request);
			lock (uploads) {
				uploads.Add(upload);
			}
			upload.Send(filename);
			lock (uploads) {
				uploads.Remove(upload);
			}
		}
		
		public int GetPriority(HttpRequest request)
		{
			if (request.File.StartsWith("/FileTransfer/" + hash + "/"))
				return 1000;
			else
				return int.MinValue;
		}
	}
}
