// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;

namespace DGNetwork.FileTransfer
{
	/// <summary>
	/// Little structure to help measuring the speed of a download.
	/// Internal because nobody should really use it.
	/// </summary>
	internal struct Speed
	{
		long[] pos; // 4 Sekunden-Messung
		
		/// <summary>
		/// Initialisiert eine neue Speed-Struktur
		/// </summary>
		/// <param name="startPosition">Die Startposition des Downloads</param>
		public Speed(long startPosition)
		{
			pos = new long[4];
			SetPosition(startPosition);
		}
		
		/// <summary>
		/// Ermittelt die Anzahl der Bytes, die seit dem letzten Aufruf dazugekommen sind.
		/// </summary>
		public int GetSpeed(long newPosition)
		{
			int speed = (int) (newPosition - pos[0])/pos.Length;
			for (int i = 0; i < (pos.Length - 1); i++)
				pos[i] = pos[i + 1];
			pos[pos.Length - 1] = newPosition;
			return speed;
		}
		
		/// <summary>
		/// Setzt die Position aller Messeinträge, so dass die Geschwindigkeit wieder
		/// 0 KB/s ist
		/// </summary>
		public void SetPosition(long position)
		{
			for (int i = 0; i < pos.Length; i++)
				pos[i] = position;
		}
	}
}
