// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Windows.Forms;
using DGNetwork.Chat;
using DGNetwork.Chat.Parser;
using DGNetwork.AddIns.Codons;
using DGNetwork.Gui;
using DGNetwork.Gui.Dialogs;
using DGNetwork.Services;

namespace DGNetwork.FileTransfer
{
	public class SendImageCommand : AbstractMenuCommand
	{
		public override void Run()
		{
			SendFileCommand.Execute((ChatPanel)this.Owner,
			                        "${res:FileTransfer.InsertImage}",
			                        "${res:Global.AllImages}|*.gif;*.jpg;*.jp2;*.png;*.wmf;*.emf;*.jpeg;*.bmp|${res:Global.AllFiles}|*.*",
			                        Environment.GetFolderPath(Environment.SpecialFolder.MyPictures));
		}
	}
	
	public class SendFileCommand : AbstractMenuCommand
	{
		public override void Run()
		{
			Execute((ChatPanel)this.Owner,
			        "${res:FileTransfer.InsertFile}",
			        "${res:Global.AllFiles}|*.*",
			        Environment.GetFolderPath(Environment.SpecialFolder.Personal));
		}
		
		public static void Execute(ChatPanel panel, string title, string filter, string initialDirectory)
		{
			using (OpenFileDialog d = new OpenFileDialog()) {
				d.CheckFileExists = true;
				d.CheckPathExists = true;
				d.DereferenceLinks = true;
				d.ShowHelp = false;
				d.ShowReadOnly = false;
				d.Title = Service.Parse(title);
				d.ValidateNames = true;
				d.Multiselect = false;
				d.Filter = Service.Parse(filter);
				d.InitialDirectory = initialDirectory;
				if (d.ShowDialog() == DialogResult.OK) {
					string file = TransferService.ToShortName(d.FileName);
					panel.EditBoxPanel.EditBox.InsertText("<sendfile=" + HtmlHelper.EncodeAttribute(file) + ">");
				}
			}
		}
	}
}
