// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.IO;
using System.Windows.Forms;
using DGNetwork.Services;

namespace DGNetwork.FileTransfer
{
	public class TransferService
	{
		// This service is simply a static class... and maybe the other services
		// will become static classes in the future, too.
		
		#region Downloads (Load + Save)
		static ArrayList downloads = new ArrayList();
		
		public static ArrayList Downloads {
			get {
				return downloads;
			}
		}
		
		/*
		private static void LoadDownloads(AdvUser owner)
		{
			try {
				ConfigItem cfg = FileUploadHandler.cfg.Node("downloads");
				foreach (ConfigItem node in cfg.GetSubNodes()) {
					if (File.Exists(node["save"] + ".part")) {
						Downloads.Add(new Download(owner, node["url"], node["save"]));
					}
				}
			} catch (Exception ex) {
				UI.UserInterface.DisplayError(ex);
			}
		}
		
		private static void SaveDownloads()
		{
			ConfigItem cfg = FileUploadHandler.cfg.Node("downloads");
			cfg.RemoveAll();
			lock (cfg) {
				int i = 0;
				foreach (Download dl in Downloads) {
					ConfigItem node = cfg.Node("f" + i.ToString());
					node["url"] = dl.URL;
					node["save"] = dl.SaveAs;
					i++;
				}
			}
		}
		 */
		#endregion
		
		#region StartDownload
		private static string GetFileName(string defaultname)
		{
			using (SaveFileDialog dlg = new SaveFileDialog()) {
				dlg.AddExtension = true;
				dlg.CheckPathExists = true;
				dlg.DefaultExt = Path.GetExtension(defaultname);
				dlg.FileName = defaultname;
				dlg.Filter = Service.Parse("${res:Global.AllFiles}|*.*");
				dlg.Title = Service.Parse("${res:Global.SaveFile}");
				dlg.ValidateNames = true;
				if (dlg.ShowDialog() == DialogResult.OK)
					return dlg.FileName;
				else
					return null;
			}
		}
		
		private static string URLToFileName(string url)
		{
			int pos = url.IndexOf("::");
			if (pos < 0)
				throw new ArgumentException("Invalid URL");
			else
				return url.Substring(pos + 2);
		}
		
		/// <summary>
		/// Erzeugt einen neuen Download, der die angegebene URL herunterldt.
		/// </summary>
		public static Download StartDownload(string url)
		{
			foreach (Download d in downloads) {
				if (d.URL == url) {
					ShowTransferForm();
					return null;
				}
			}
			string file = URLToFileName(url);
			string saveas = GetFileName(file);
			if (saveas == null)
				return null;
			if (File.Exists(saveas)) {
				try {
					File.Delete(saveas);
				} catch (Exception) {
					Service.Message.ShowMessage("${res:FileTransfer.CantOverwrite}");
					return null;
				}
			}
			if (File.Exists(saveas + ".part")) {
				Service.Message.ShowMessage("${res:FileTransfer.AlreadyDownloadingToThisFile}");
				return null;
			}
			if (!url.StartsWith("dgn:")) {
				int pos = url.IndexOf("::");
				file = url.Substring(0, pos);
				try {
					File.Copy(file, saveas, true);
				} catch (Exception ex) {
					Service.Message.ShowError(ex.Message);
				}
				return null;
			} else {
				File.Create(saveas + ".part").Close();
				Download dl = new Download(url, saveas);
				downloads.Add(dl);
				// SaveDownloads();
				ShowTransferForm();
				dl.Start();
				return dl;
			}
		}
		#endregion
		
		#region ShowTransferForm
		public static void ShowTransferForm()
		{
			DGNetwork.Gui.XmlForm.ShowForm("TransferForm", typeof(TransferDialogSink));
		}
		#endregion
		
		#region Shared Files
		static ArrayList sharedFiles = new ArrayList();
		static ArrayList roSharedFiles = ArrayList.ReadOnly(sharedFiles);
		
		public static ArrayList SharedFiles {
			get {
				return roSharedFiles;
			}
		}
		
		public static void ShareFile(string filename, long size, string shortHash, string tag)
		{
			SharedFileHandler sharedFile = new SharedFileHandler(filename, size, shortHash, tag);
			sharedFiles.Add(sharedFile);
			Service.P2P.Server.RequestHandlers.Add(sharedFile);
		}
		#endregion
		
		#region Short Names
		public static string ToShortName(string fullName)
		{
			string mymusic = Environment.GetFolderPath(Environment.SpecialFolder.MyMusic).ToLower() + "\\";
			string mypic = Environment.GetFolderPath(Environment.SpecialFolder.MyPictures).ToLower() + "\\";
			string desktop = Environment.GetFolderPath(Environment.SpecialFolder.DesktopDirectory).ToLower() + "\\";
			string temp = Helper.GetTempPath();
			string fn = fullName.ToLower();
			if (fn.StartsWith(mymusic))
				return "music:" + fullName.Substring(mymusic.Length);
			if (fn.StartsWith(mypic))
				return "pictures:" + fullName.Substring(mypic.Length);
			if (fn.StartsWith(desktop))
				return "desktop:" + fullName.Substring(desktop.Length);
			if (fn.StartsWith(temp))
				return "temp:" + fullName.Substring(temp.Length);
			return fullName;
		}
		
		public static string FromShortName(string shortName)
		{
			if (shortName.StartsWith("music:"))
				return (Environment.GetFolderPath(Environment.SpecialFolder.MyMusic) + "\\"
				        + shortName.Substring(6));
			if (shortName.StartsWith("pictures:"))
				return (Environment.GetFolderPath(Environment.SpecialFolder.MyPictures) + "\\"
				        + shortName.Substring(9));
			if (shortName.StartsWith("desktop:"))
				return (Environment.GetFolderPath(Environment.SpecialFolder.Desktop) + "\\"
				        + shortName.Substring(8));
			if (shortName.StartsWith("temp:"))
				return (Helper.GetTempPath() + shortName.Substring(5));
			return shortName;
		}
		#endregion
	}
}
