// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Drawing;
using System.Windows.Forms;
using Grunwald.Gui.Controls;
using DGNetwork.Services;
using DGNetwork.Gui.Menu;
using DGNetwork.AddIns.Codons;
using ICSharpCode.Core.AddIns;
using ICSharpCode.Core.AddIns.Codons;

namespace DGNetwork.Gui
{
	/// <summary>
	/// Description of ButtonRow.
	/// </summary>
	public class ButtonRow : ControlRow
	{
		internal ButtonRow(IAddInTreeNode node, object owner)
		{
			this.Direction = Direction.TopDown;
			this.ContentAlignment = ContentAlignment.TopLeft;
			this.AutoSize = true;
			this.AutoScroll = false;
			this.FullWidth = true;
			this.ArrangeActive = true;
			foreach (IAddInTreeNode n in node.GetSubnodesAsSortedArray()) {
				if (n.Codon == null) continue;
				if (n.Codon is MenuItemCodon) {
					MenuItemCodon m = (MenuItemCodon)n.Codon;
					if (m.Label == "-") {
						Control ctl = new Control();
						ctl.BackColor = ctl.ForeColor;
						ctl.Height = 2;
						this.Controls.Add(ctl);
					} else {
						this.Controls.Add(new ButtonRowButton(m));
					}
				} else {
					Service.Message.ShowErrorFormatted("${res:Codon.InvalidSubItem}", "ButtonRow", n.GetType().FullName);
				}
			}
		}
		
		private class ButtonRowButton : Button
		{
			string text;
			ICommand command;
			
			public ButtonRowButton(MenuItemCodon m)
			{
				this.FlatStyle = FlatStyle.System;
				text = m.Label;
				Service.Resource.Reloaded += new EventHandler(UpdateText);
				UpdateText(null, null);
				if (m.Class != null) {
					object o = m.AddIn.CreateObject(m.Class);
					if (o != null) {
						ICommand c = o as ICommand;
						if (c == null)
							Service.Message.ShowError("MenuItem in ButtonRow has invalid class associated.");
						command = c;
					}
				} else {
					if (m.Link != null) {
						command = new LinkCommand(m.Link);
					} else {
						Service.Message.ShowError("MenuItem in ButtonRow must have a class or link specified.");
					}
				}
				if (command == null)
					this.Enabled = false;
			}
			
			protected override void Dispose(bool disposing)
			{
				if (disposing)
					Service.Resource.Reloaded -= new EventHandler(UpdateText);
				base.Dispose(disposing);
			}
			
			private void UpdateText(object sender, EventArgs e)
			{
				this.Text = Service.Parse(text);
			}
			
			protected override void OnClick(EventArgs e)
			{
				try {
					command.Run();
				} catch (Exception ex) {
					Service.Message.ShowError(ex);
				} finally {
					base.OnClick(e);
				}
			}
		}
	}
}

