// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Drawing;
using System.Windows.Forms;
using DGNetwork.Chat;
using DGNetwork.Contacts;
using DGNetwork.Services;

namespace DGNetwork.Gui
{
	public class CombinedMainWindowBuilder : AbstractMainWindowBuilder
	{
		public CombinedMainWindowBuilder() : base("Combined", "${res:Settings.Chat.MainWindow.Combined}")
		{
		}
		
		/// <summary>
		/// Gets if the main windows build by this window builder can be used as chat windows.
		/// For example, this property is used in the click event handler of the balloon
		/// 'message received' to determine if a chat window or the main window should be opened.
		/// </summary>
		public override bool UseAsChatWindow {
			get {
				return true;
			}
		}
		
		public override Form CreateForm()
		{
			ChatForm frm = new ChatForm(null);
			frm.StartPosition = FormStartPosition.CenterScreen;
			Service.Window.ManageWindowSize("CombinedMainForm", frm);
			return frm;
		}
		
		public override void AddContent(MainWindowCreationEventArgs e)
		{
			ChatForm frm = (ChatForm)e.Form;
			Panel content = CreateContent(e);
			Splitter splitter = new Splitter();
			splitter.Dock = DockStyle.Right;
			frm.Controls.Add(splitter);
			content.Dock = DockStyle.Right;
			frm.Controls.Add(content);
			content.Width = 125;
			Service.Chat.RegisterChatForm(frm);
			if (e.ContactTreeView != null)
				e.ContactTreeView.RoomOwner = frm;
		}
	}
}
