// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Windows.Forms;
using Grunwald.Gui.Controls;
using Grunwald.Gui.Settings;
using DGNetwork.Services;
using DGNetwork.Gui;

namespace DGNetwork.Gui.Dialogs
{
	public class SettingsDialogSink
	{
		Form form = null;
		Button applyButton = null;
		HelpingSettingsPanel panel = null;
		SettingTreeView tree = null;
		Timer timer;
		bool dirty;
		
		private void FormLoad(object sender, EventArgs e) {
			timer = new Timer();
			timer.Interval = 100;
			timer.Tick += new EventHandler(TimerTick);
			timer.Enabled = true;
			tree.RootPage = Service.Setting.RootPage;
			panel.Page = tree.RootPage;
			Service.Resource.Reloaded += new EventHandler(ResourceReloaded);
		}
		
		private void FormClosed(object sender, EventArgs e) {
			Service.Resource.Reloaded -= new EventHandler(ResourceReloaded);
			timer.Dispose();
			if (dirty) Service.Setting.RootPage.Load();
		}
		
		private void ResourceReloaded(object sender, EventArgs e) {
			panel.Page = null; // reset settings panel because it can't be retranslated
			setPage = true;
		}
		
		private void OKClick(object sender, EventArgs e) {
			if (dirty) Apply();
			form.Close();
		}
		
		private void ApplyClick(object sender, EventArgs e) {
			applyButton.Enabled = false;
			dirty = false;
			Apply();
		}
		
		private void Apply() {
			Service.Setting.RootPage.Save();
			Service.Property.SaveProperties();
		}
		
		public void EditSetting(ISettingPage page) {
			tree.SelectedPage = page;
		}
		
		bool setPage = false;
		DateTime lastSet;
		
		private void TreeSelectedPageChanged(object sender, System.EventArgs e) {
			if (panel.Page == tree.SelectedPage) return;
			panel.Page = null;
			lastSet = DateTime.Now;
			setPage = true;
		}
		
		private void PanelSettingChanged(object sender, SettingChangedEventArgs e) {
			applyButton.Enabled = true;
			dirty = true;
		}
		
		private void TimerTick(object sender, EventArgs e) {
			if (setPage && DateTime.Now.Subtract(lastSet).TotalMilliseconds > 60) {
				setPage = false;
				panel.Page = tree.SelectedPage;
			}
		}
	}
}
