// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.Windows.Forms;
using DGNetwork.Services;

namespace DGNetwork.Gui.Dialogs
{
	public class WizardDialog : IWizardContainer
	{
		private Form form = null;
		private Label titleLabel = null;
		private Control contentPanel = null;
		private Button nextButton = null;
		private Button backButton = null;
		
		#region IWizardContainer properties
		string title;
		public string Title {
			get {
				return title;
			}
			set {
				title = value;
				if (titleLabel != null)
					titleLabel.Text = Service.Parse(title);
			}
		}
		
		Control control;
		public Control Control {
			get {
				return control;
			}
			set {
				control = value;
				if (contentPanel != null) {
					contentPanel.Controls.Clear();
					contentPanel.Controls.Add(value);
				}
			}
		}
		
		public void SetControl(Control ctl) {
			Control old = control;
			this.Control = ctl;
			if (old != null)
				old.Dispose();
		}
		
		bool nextEnabled;
		public bool NextEnabled {
			get {
				return nextEnabled;
			}
			set {
				nextEnabled = value;
				if (nextButton != null)
					nextButton.Enabled = value;
			}
		}
		string nextText;
		public string NextText {
			get {
				return nextText;
			}
			set {
				nextText = value;
				if (nextButton != null)
					nextButton.Text = Service.Parse(value);
			}
		}
		
		bool backEnabled;
		public bool BackEnabled {
			get {
				return backEnabled;
			}
			set {
				backEnabled = value;
				if (backButton != null)
					backButton.Enabled = value;
			}
		}
		string backText;
		public string BackText {
			get {
				return backText;
			}
			set {
				backText = value;
				if (backButton != null)
					backButton.Text = Service.Parse(value);
			}
		}
		
		public void Invoke(Delegate method, params object[] args) {
			form.Invoke(method, args);
		}
		#endregion
		
		IWizard activeWizard;
		IWizard[] wizards;
		int num;
		bool formLoaded = false;
		
		public IWizard ActiveWizard {
			get {
				return activeWizard;
			}
		}
		
		public WizardDialog(params IWizard[] wizards) {
			this.wizards = wizards;
			num = -1;
			NextWizard();
		}
		
		public WizardDialog(ArrayList wizards) {
			this.wizards = (IWizard[])wizards.ToArray(typeof(IWizard));
			num = -1;
			NextWizard();
		}
		
		public void NextWizard() {
			num += 1;
			if (num >= wizards.Length) {
				activeWizard = null;
				if (formLoaded)
					form.Close();
				return;
			}
			activeWizard = wizards[num];
			try {
				activeWizard.ShowOn(this);
			} catch (Exception ex) {
				if (Helper.IsCritical(ex))
					throw;
				Service.Message.ShowError(ex);
			}
		}
		
		private void BackClick(object sender, EventArgs e) {
			activeWizard.Back();
		}
		
		private void NextClick(object sender, EventArgs e) {
			activeWizard.Next();
		}
		
		private void FormLoad(object sender, EventArgs e) {
			form.AcceptButton = nextButton;
			formLoaded = true;
			this.Title = this.Title;
			this.Control = this.Control;
			this.BackEnabled = this.BackEnabled;
			this.BackText = this.BackText;
			this.NextEnabled = this.NextEnabled;
			this.NextText = this.NextText;
		}
	}
}
