// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.Drawing;
using System.Windows.Forms;
using DGNetwork.Services;
using ICSharpCode.Core.Services;

namespace DGNetwork.Gui
{
	/// <summary>
	/// The ImageListManager manages the icons in an imagelist.
	/// </summary>
	public class ImageListManager : IDisposable
	{
		ImageList imageList;
		Hashtable hash = new Hashtable(); // string -> int
		
		/// <summary>Gets the imagelist this manager was created for.</summary>
		public ImageList ImageList {
			get {
				return imageList;
			}
		}
		
		/// <summary>
		/// Creates a new ImageListManager
		/// </summary>
		public ImageListManager() {
			this.imageList = new ImageList();
			imageList.ColorDepth = ColorDepth.Depth32Bit;
		}
		
		/// <summary>
		/// Creates a new ImageListManager
		/// </summary>
		public ImageListManager(ImageList imageList) {
			if (imageList.ColorDepth != ColorDepth.Depth32Bit)
				throw new ArgumentException("the specified imageList is not a 32-bit imagelist");
			this.imageList = imageList;
		}
		
		/// <summary>
		/// Disposes the embedded image list.
		/// </summary>
		public void Dispose()
		{
			imageList.Dispose();
		}
		
		/// <summary>
		/// Gets the image index of the specified image.
		/// </summary>
		public int GetIndex(string name)
		{
			return this[name];
		}
		
		/// <summary>
		/// Gets the image index of the specified image.
		/// </summary>
		public int this[string name] {
			get {
				if (name == null || name.Length == 0)
					return -1;
				object o = hash[name];
				if (o != null)
					return (int)o;
				try {
					Bitmap bmp = Service.Resource.GetBitmap(name);
					int res = imageList.Images.Count;
					hash[name] = res;
					imageList.Images.Add(bmp);
					return res;
				} catch (ResourceNotFoundException) {
					hash[name] = -1;
					Service.Message.ShowError("Image " + name + " not found");
					return -1;
				}
			}
		}
	}
}
