// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.Xml;
using DGNetwork.Services;
using Grunwald.Gui.Settings;

namespace DGNetwork.Gui
{
	#region MainWindowSetting
	/// <summary>
	/// The setting that determines the used chat MainWindow.
	/// </summary>
	public class MainWindowSetting : ChooseOptionSetting
	{
		public MainWindowSetting() : base("MainWindowBuilder", "${res:Settings.Chat.MainWindow}", "")
		{
		}
		
		protected override void SaveValue(XmlWriter writer)
		{
			// ignore this setting
		}
		
		public override void Load(XmlReader reader)
		{
			reader.Skip();
		}
		
		private void CreateOptions()
		{
			ArrayList builders = Service.Window.MainWindowBuilders;
			string[] options = new string[builders.Count];
			string[] texts = new string[builders.Count];
			for (int i = 0; i < builders.Count; ++i) {
				options[i] = ((IMainWindowBuilder)builders[i]).Name;
				texts[i] = ((IMainWindowBuilder)builders[i]).Title;
			}
			this.Options = options;
			this.Texts = texts;
			this.InEditValue = this.Value;
		}
		
		public override ISettingEditor CreateEditor(EditorFactory factory)
		{
			if (this.Options == null)
				CreateOptions();
			return base.CreateEditor(factory);
		}
		
		public override object Value {
			get {
				return Service.Window.ActiveMainWindowBuilder.Name;
			}
			set {
				string oldValue = Service.Window.ActiveMainWindowBuilder.Name;
				string newValue = (string)value;
				if (oldValue != newValue) {
					foreach (IMainWindowBuilder builder in Service.Window.MainWindowBuilders) {
						if (builder.Name == newValue) {
							Service.Window.ActiveMainWindowBuilder = builder;
							break;
						}
					}
					this.OnValueChanged(new SettingChangedEventArgs(this, oldValue, newValue));
				}
				if (newValue != (string)this.InEditValue)
					this.InEditValue = newValue;
			}
		}
	}
	#endregion
}
