// <file>
//     <copyright see="prj:///doc/copyright.txt"/>
//     <license see="prj:///doc/license.txt"/>
//     <owner name="Mike Krüger" email="mike@icsharpcode.net"/>
//     <version value="$version"/>
// </file>

using System;
using System.Collections;
using System.Drawing;
using System.Diagnostics;
using System.Drawing.Text;
using System.Drawing.Imaging;
using System.Windows.Forms;

using DGNetwork.Services;
using DGNetwork.AddIns.Codons;
using ICSharpCode.Core.AddIns.Conditions;

namespace DGNetwork.Gui.Menu
{
	public interface IStatusUpdate
	{
		void UpdateStatus();
	}
	public class SdMenu : CommandBarMenu, IStatusUpdate
	{
		ConditionCollection conditionCollection;
		object caller;
		string localizedText = String.Empty;
		public ArrayList SubItems = new ArrayList();
		
		public SdMenu(ConditionCollection conditionCollection, object caller, string text) : base(Service.StringParser.Parse(text))
		{
			this.conditionCollection = conditionCollection;
			this.caller              = caller;
			this.localizedText       = text;
		}
		
		protected override void OnDropDown(System.EventArgs e)
		{
			base.OnDropDown(e);
			foreach (object o in Items) {
				if (o is IStatusUpdate) {
					((IStatusUpdate)o).UpdateStatus();
				}
			}
		}
		
		string dynamicImage;
		
		/// <summary>
		/// Gets/Sets the dynamic image name to use. A dynamic image is updated whenever
		/// the menu is shown. When this property is set to null, the normal Image is used.
		/// </summary>
		public string DynamicImage {
			get {
				return dynamicImage;
			}
			set {
				dynamicImage = value;
			}
		}
		
		public virtual void UpdateStatus()
		{
			if (dynamicImage != null) {
				try {
					this.Image = Service.Resource.GetBitmap(dynamicImage);
				} catch (ResourceNotFoundException ex) {
					Service.Message.ShowError(ex.Message);
				}
			}
			if (conditionCollection != null) {
				ConditionFailedAction failedAction = conditionCollection.GetCurrentConditionFailedAction(caller);
				this.IsEnabled = failedAction != ConditionFailedAction.Disable;
				this.IsVisible = failedAction != ConditionFailedAction.Exclude;
			}
			
			if (IsVisible) {
				Items.Clear();
				foreach (object item in SubItems) {
					if (item is CommandBarItem) {
						if (item is IStatusUpdate) {
							((IStatusUpdate)item).UpdateStatus();
						}
						Items.Add((CommandBarItem)item);
					} else {
						ISubmenuBuilder submenuBuilder = (ISubmenuBuilder)item;
						Items.AddRange(submenuBuilder.BuildSubmenu(conditionCollection, caller));
					}
				}
			}
			Text = Service.StringParser.Parse(localizedText);
		}
	}
}
