// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using Grunwald.Gui.Components;
using DGNetwork.Services;

namespace DGNetwork.Gui
{
	public interface ITNAIcon : IDisposable
	{
		Icon Icon { get; set; }
		string ToolTip { get; set; }
		bool Visible { get; set; }
		ContextMenu ContextMenu { get; set; }
		
		event EventHandler Click;
		event EventHandler Disposed;
		event EventHandler DoubleClick;
		event MouseEventHandler MouseDown;
		event MouseEventHandler MouseMove;
		event MouseEventHandler MouseUp;
		
		void ShowBalloon(BalloonIcon icon, string text, string title, int timeout,
		                 EventHandler balloonClick, EventHandler balloonTimeout);
	}
	
	public enum BalloonIcon
	{
		None,
		Info,
		Warning,
		Error
	}
	
	internal class NotifyIconWrapper : ITNAIcon
	{
		BalloonNotifyIcon icon;
		
		public NotifyIconWrapper() {
			icon = new BalloonNotifyIcon();
			icon.BalloonClick += new EventHandler(IconBalloonClick);
			icon.BalloonTimeout += new EventHandler(IconBalloonTimeout);
		}
		
		public string ToolTip {
			get { return icon.Text; }
			set { icon.Text = value; }
		}
		
		public Icon Icon {
			get { return icon.Icon; }
			set { icon.Icon = value; }
		}
		
		public bool Visible {
			get { return icon.Visible; }
			set { icon.Visible = value; }
		}
		
		public ContextMenu ContextMenu {
			get { return icon.ContextMenu; }
			set { icon.ContextMenu = value; }
		}
		
		public void Dispose()
		{
			icon.Dispose();
		}
		
		public event EventHandler Click {
			add    { icon.Click += value; }
			remove { icon.Click -= value; }
		}
		
		public event EventHandler Disposed {
			add    { icon.Disposed += value; }
			remove { icon.Disposed -= value; }
		}
		
		public event EventHandler DoubleClick {
			add    { icon.DoubleClick += value; }
			remove { icon.DoubleClick -= value; }
		}
		
		public event MouseEventHandler MouseDown {
			add    { icon.MouseDown += value; }
			remove { icon.MouseDown -= value; }
		}
		
		public event MouseEventHandler MouseMove {
			add    { icon.MouseMove += value; }
			remove { icon.MouseMove -= value; }
		}
		
		public event MouseEventHandler MouseUp {
			add    { icon.MouseUp += value; }
			remove { icon.MouseUp -= value; }
		}
		
		EventHandler balloonClick, balloonTimeout;
		
		public void ShowBalloon(BalloonIcon balloonIcon, string text, string title, int timeout,
		                        EventHandler balloonClick, EventHandler balloonTimeout)
		{
			text = Service.StringParser.Parse(text);
			title = Service.StringParser.Parse(title);
			this.balloonClick = balloonClick;
			this.balloonTimeout = balloonTimeout;
			switch(balloonIcon) {
				case BalloonIcon.Info:
					icon.ShowBalloon(BalloonNotifyIcon.EBalloonIcon.Info, text, title, timeout);
					break;
				case BalloonIcon.Warning:
					icon.ShowBalloon(BalloonNotifyIcon.EBalloonIcon.Info, text, title, timeout);
					break;
				case BalloonIcon.Error:
					icon.ShowBalloon(BalloonNotifyIcon.EBalloonIcon.Info, text, title, timeout);
					break;
				default:
					icon.ShowBalloon(BalloonNotifyIcon.EBalloonIcon.None, text, title, timeout);
					break;
			}
		}
		
		private void IconBalloonClick(object sender, EventArgs e)
		{
			if (balloonClick != null)
				balloonClick(this, e);
		}
		
		private void IconBalloonTimeout(object sender, EventArgs e)
		{
			if (balloonTimeout != null)
				balloonTimeout(this, e);
		}
	}
}
