// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.Drawing;
using System.Windows.Forms;

using DGNetwork.Services;
using ICSharpCode.Core.AddIns;
using ICSharpCode.Core.AddIns.Codons;
using DGNetwork.AddIns.Codons;
using DGNetwork.Gui;
using DGNetwork.Gui.Menu;
using ICSharpCode.Core.Services;

namespace DGNetwork.Gui
{
	public class ToolBarService
	{
		public static CommandBar CreateToolBar(string path, object owner)
		{
			CommandBar bar = new CommandBar(CommandBarStyle.ToolBar);
			foreach (object obj in Helper.BuildItems(path, owner)) {
				if (obj is ToolBarItemCodon) {
					ToolBarItemCodon childCodon = (ToolBarItemCodon)obj;
					CommandBarItem item = null;
					
					if (childCodon.ToolTip == "-") {
						item = new CommandBarSeparator();
					} else {
						item = new SdMenuCommand(childCodon.Conditions, owner, childCodon.ToolTip);
						item.Image = Service.Resource.GetBitmap(childCodon.Icon);
						ICommand command = (ICommand)childCodon.AddIn.CreateObject(childCodon.Class);
						if (command != null)
							command.Owner = owner;
						((SdMenuCommand)item).Command = command;
						if (childCodon.Shortcut != null) {
							try {
								foreach (string key in childCodon.Shortcut) {
									((SdMenuCommand)item).Shortcut |= (Keys)Enum.Parse(typeof(Keys), key.Trim());
								}
							} catch (Exception) {
								((SdMenuCommand)item).Shortcut = Keys.None;
							}
						}
					}
					bar.Items.Add(item);
				} else {
					Service.Message.ShowErrorFormatted("${res:Codon.InvalidSubItem}",
					                                   "ToolBar",
					                                   obj.GetType().FullName);
				}
			}
			return bar;
		}
		
		private void ToolBarButtonClick(object sender, EventArgs e)
		{
			if (sender is CommandBarItem) {
				((ICommand)((CommandBarItem)sender).Tag).Run();
			}
		}
	}
}
