// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.Drawing;
using System.Windows.Forms;
using Grunwald.Gui.Controls;
using DGNetwork.Services;

namespace DGNetwork.Gui
{
	public class TreeElementView : UserControl
	{
		RearrangeTreeView treeView = new RearrangeTreeView();
		TreeElement root = new TreeElement();
		
		public TreeElementCollection Elements {
			get {
				return root.Children;
			}
		}
		
		public TreeElementView() {
			TreeView tv = treeView;
			tv.AfterExpand += new TreeViewEventHandler(TreeViewAfterExpand);
			tv.AfterExpand += new TreeViewEventHandler(TreeViewAfterCollapse);
			root.ChildAdded   += new TreeElementEventHandler(ChildAdded);
			root.ChildRemoved += new TreeElementEventHandler(ChildRemoved);
			tv.Dock = DockStyle.Fill;
			this.Controls.Add(tv);
		}
		
		#region Child management
		Hashtable nodeHash = new Hashtable(); // TreeElement -> TreeNode
		
		private TreeNode AddChild(TreeElement e) {
			TreeNode node = new TreeNode(Service.StringParser.Parse(e.Text));
			nodeHash[e] = node;
			e.ChildAdded   += new TreeElementEventHandler(ChildAdded);
			e.ChildRemoved += new TreeElementEventHandler(ChildRemoved);
			e.TextChanged  += new EventHandler(ChildTextChanged);
			foreach (TreeElement sub in e.Children) {
				node.Nodes.Add(AddChild(sub));
			}
			return node;
		}
		
		private void ReleaseMonitor(TreeElement e) {
			nodeHash[e] = null;
			e.ChildAdded   -= new TreeElementEventHandler(ChildAdded);
			e.ChildRemoved -= new TreeElementEventHandler(ChildRemoved);
			e.TextChanged  -= new EventHandler(ChildTextChanged);
			foreach (TreeElement sub in e.Children) {
				ReleaseMonitor(sub);
			}
		}
		
		private void ChildAdded(object sender, TreeElementEventArgs e) {
			if (sender == root) {
				treeView.Nodes.Insert(e.Index, AddChild(e.Element));
			} else {
				object node = nodeHash[(TreeElement)sender];
				if (node == null)
					throw new Exception("TreeNode not found");
				AddChild(e.Element);
				
			}
		}
		
		private void ChildRemoved(object sender, TreeElementEventArgs e) {
			TreeNode treeNode = (TreeNode)nodeHash[e.Element];
			// treeNode can already be removed if it was deleted by the tree view.
			if (treeNode.Parent != null)
				treeNode.Remove();
			ReleaseMonitor(e.Element);
		}
		
		private void ChildTextChanged(object sender, EventArgs e) {
			TreeElement n = (TreeElement)sender;
			((TreeNode)nodeHash[n]).Text = Service.StringParser.Parse(n.Text);
		}
		#endregion
		
		#region TreeView Event Handler
		private void TreeViewAfterExpand(object sender, TreeViewEventArgs e) {
			((TreeWrapperTag)e.Node.Tag).Element.Expanded = true;
		}
		
		private void TreeViewAfterCollapse(object sender, TreeViewEventArgs e) {
			((TreeWrapperTag)e.Node.Tag).Element.Expanded = false;
		}
		#endregion
		
		#region MultiSelect
		public bool MultiSelect {
			get {
				return false;
				//return treeView.MultiSelect;
			}
			set {
				if (value)
					throw new NotSupportedException("Multiselect is not supported");
				/*if (treeView.MultiSelect == value) return;
				treeView.MultiSelect = value;
				OnMultiSelectChanged(EventArgs.Empty);*/
			}
		}
		
		public event EventHandler MultiSelectChanged;
		
		protected virtual void OnMultiSelectChanged(EventArgs e) {
			if (MultiSelectChanged != null) {
				MultiSelectChanged(this, e);
			}
		}
		#endregion
		
		#region Control property forwarding
		protected override void OnBackColorChanged(EventArgs e) {
			treeView.BackColor = this.BackColor;
			base.OnBackColorChanged(e);
		}
		protected override void OnBackgroundImageChanged(EventArgs e) {
			treeView.BackgroundImage = this.BackgroundImage;
			base.OnBackgroundImageChanged(e);
		}
		protected override void OnFontChanged(EventArgs e) {
			treeView.Font = this.Font;
			base.OnFontChanged(e);
		}
		protected override void OnForeColorChanged(EventArgs e) {
			treeView.ForeColor = this.ForeColor;
			base.OnForeColorChanged(e);
		}
		#endregion
	}
}
