// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Drawing;
using System.Windows.Forms;
using Grunwald.Gui.Controls;

namespace DGNetwork.Gui
{
	/// <summary>
	/// A wrapper for easier changing the underlying ListElementView.
	/// </summary>
	public class TreeWrapperTag : ITreeTag
	{
		#region ITreeTag implementation
		ITreeTag parent;
		TreeElement element;
		
		/// <summary>
		/// Creates a new PageTreeTag object for the specified SettingPage
		/// </summary>
		public TreeWrapperTag(ITreeTag parent, TreeElement element)
		{
			if (element == null)
				throw new ArgumentNullException("element");
			this.element = element;
			this.parent = parent;
		}
		
		/// <summary>
		/// Gets the parent page of the wrapped page.
		/// </summary>
		public ITreeTag Parent {
			get {
				return parent;
			}
			set {
				parent = value;
			}
		}
		
		public TreeElement Element {
			get {
				return element;
			}
		}
		
		/// <summary>
		/// Gets if this node can be a root node.
		/// </summary>
		public bool CanBeRoot {
			get {
				return true;
			}
		}
		
		/// <summary>
		/// Gets if this node can be dragged around.
		/// </summary>
		public bool CanDrag {
			get {
				if (parent == null) return false;
				return true;
			}
		}
		
		/// <summary>
		/// Gets if this tree node can accept the specified node as a child.
		/// </summary>
		public bool AcceptChild(ITreeTag node, int index)
		{
			return true;
		}
		
		/// <summary>
		/// Adds a child to this tree node.
		/// </summary>
		public void AddChild(ITreeTag node, int index)
		{
			
		}
		
		/// <summary>
		/// Moves a child inside this tree node.
		/// </summary>
		public void MoveChild(ITreeTag node, int index)
		{
			
		}
		
		/// <summary>
		/// Removes a child from this tree node.
		/// </summary>
		public void RemoveChild(ITreeTag node)
		{
			
		}
		
		/// <summary>
		/// Gets a copy of this object.
		/// </summary>
		public object Clone()
		{
			return new TreeWrapperTag(null, element.Clone());
		}
		
		/// <summary>
		/// Gets if the object can be cloned.
		/// </summary>
		public bool CanClone {
			get {
				return true;
			}
		}
		
		/// <summary>
		/// Gets if this node can be delete using the
		/// <see cref="RearrangeTreeView.Delete">Delete method</see>.
		/// </summary>
		public bool CanDelete {
			get {
				if (parent == null)
					return false;
				return true;
			}
		}
		
		/// <summary>
		/// Deletes this tree node.<br/>
		/// This method is responsible for calling <see cref="RemoveChild"/> on its
		/// parent!
		/// </summary>
		public void Delete()
		{
			if (!CanDelete)
				throw new InvalidOperationException("delete not allowed");
			parent.RemoveChild(this);
		}
		#endregion
	}
}
