// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Windows.Forms;

namespace DGNetwork.Gui
{
	/// <summary>
	/// Basic interface for wizards that can be used in a
	/// <see cref="IWizardContainer">WizardContainer</see>.
	/// </summary>
	public interface IWizard
	{
		/// <summary>
		/// Tells the wizard to display itself by setting the properties on the
		/// IWizardContainer.<br/>
		/// If the wizard wishes not to be displayed, it should call IWizardContainer.NextWizard()
		/// instead of displaying it's control in this method.
		/// </summary>
		void ShowOn(IWizardContainer container);
		
		/// <summary>
		/// Is called when the next button on the container is pressed and this wizard is
		/// the active wizard.<br/>
		/// Normally this method should call IWizardContainer.NextWizard() or display the next
		/// page in a multipage wizard.
		/// </summary>
		void Next();
		
		/// <summary>
		/// Is called when the back button on the container is pressed and this wizard is
		/// active. The back button is only used for multipage wizards.
		/// </summary>
		void Back();
	}
	
	public abstract class AbstractWizard : IWizard
	{
		IWizardContainer container;
		
		public IWizardContainer Container {
			get {
				return container;
			}
		}
		
		public virtual void ShowOn(IWizardContainer container) {
			this.container = container;
			container.BackEnabled = false;
			container.BackText = "${res:Global.Back}";
			container.NextEnabled = true;
			container.NextText = "${res:Global.Next}";
		}
		
		public abstract void Next();
		
		public virtual void Back() { }
	}
}
