// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Windows.Forms;

namespace DGNetwork.Gui
{
	/// <summary>
	/// Interface for classes that can contain wizards.
	/// The wizard container has to implement the back and next buttons.
	/// </summary>
	public interface IWizardContainer
	{
		/// <summary>
		/// Gets/Sets the title of the wizard window.
		/// </summary>
		string Title { get; set; }
		
		/// <summary>
		/// Changes the currently displayed control to the specified control.
		/// If you want to dispose the old control, you should call <see cref="SetControl"/>
		/// instead.
		/// </summary>
		Control Control { get; set; }
		
		/// <summary>
		/// Advances to the next wizard that is waiting to greet the user ;-)
		/// </summary>
		void NextWizard();
		
		/// <summary>
		/// Changes the currently displayed control to the specified control.
		/// The control displayed before is disposed.
		/// If you do not wish to dispose the old control, you should use the set-accessor
		/// of the Control property.
		/// </summary>
		void SetControl(Control ctl);
		
		/// <summary>
		/// Gets/Sets the enabled value of the next button.
		/// </summary>
		bool NextEnabled { get; set; }
		
		/// <summary>
		/// Gets/Sets the text of the next button.
		/// </summary>
		string NextText { get; set; }
		
		/// <summary>
		/// Gets/Sets the enabled value of the back button.
		/// </summary>
		bool BackEnabled { get; set; }
		
		/// <summary>
		/// Gets/Sets the text of the back button.
		/// </summary>
		string BackText { get; set; }
		
		void Invoke(Delegate method, params object[] args);
	}
}
