// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.IO;
using System.Windows.Forms;
using System.Xml;

using DGNetwork.Gui.MycroXaml;
using DGNetwork.Services;

namespace DGNetwork.Gui
{
	public class XmlControl : IDisposable
	{
		public XmlControl(XmlDocument doc) : this(doc, null) { }
		
		public XmlControl(string dialogName) : this(dialogName, null) { }
		
		public XmlControl(XmlDocument doc, object eventSink) {
			Load(doc, eventSink);
		}
		
		public XmlControl(string dialogName, object eventSink) {
			XmlDocument doc = new XmlDocument();
			try {
				using(Stream s = Service.Resource.GetResourceStream(dialogName + ".xml", "DGNetwork.Gui.Dialogs")) {
					doc.Load(s);
				}
			} catch (XmlException ex) {
				throw new XmlException("Error parsing " + dialogName + ": " + ex.Message);
			}
			Load(doc, eventSink);
		}
		
		Parser parser = new Parser();
		Control control;
		object eventSink;
		
		private void Load(XmlDocument doc, object eventSink)
		{
			parser.xmlControl = this;
			this.eventSink = eventSink;
			control = (Control)parser.Load(doc, eventSink);
			control.Disposed += new EventHandler(ControlDisposed);
			Service.Resource.Reloaded += new EventHandler(ResourcesReloaded);
		}
		
		private void ResourcesReloaded(object sender, EventArgs e) {
			parser.UpdateTexts();
		}
		
		private void ControlDisposed(object sender, EventArgs e) {
			Service.Resource.Reloaded -= new EventHandler(ResourcesReloaded);
		}
		
		public Control Control {
			get {
				return control;
			}
		}
		
		public object EventSink {
			get {
				return eventSink;
			}
		}
		
		public object this [string name] {
			get {
				return parser.GetInstance(name);
			}
		}
		
		public void Dispose() {
			control.Dispose();
		}
	}
}
