// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.IO;
using System.Windows.Forms;
using System.Xml;

using DGNetwork.Gui.MycroXaml;
using DGNetwork.Services;

namespace DGNetwork.Gui
{
	public class XmlForm : XmlControl
	{
		static Hashtable forms = new Hashtable();
		
		/// <summary>
		/// Gets the form with the specified name. Returns null if the form does not exist.
		/// </summary>
		public static XmlForm GetForm(string name)
		{
			XmlForm xml = (XmlForm)forms[name];
			if (xml == null || xml.Form.IsDisposed)
				return null;
			else
				return xml;
		}
		
		/// <summary>
		/// Shows the specified form. If it is already visible, the current
		/// instance is brought to front.
		/// </summary>
		public static void ShowForm(string name)
		{
			XmlForm xml = (XmlForm)forms[name];
			if (xml == null || xml.Form.IsDisposed) {
				xml = new XmlForm(name);
				forms[name] = xml;
				xml.Form.Show();
			} else {
				xml.Form.Focus();
			}
		}
		
		/// <summary>
		/// Shows the specified form. If it is already visible, the current
		/// instance is focussed.
		/// </summary>
		public static void ShowForm(string name, Type sinkType)
		{
			XmlForm xml = (XmlForm)forms[name];
			if (xml == null || xml.Form.IsDisposed) {
				xml = new XmlForm(name, Activator.CreateInstance(sinkType));
				forms[name] = xml;
				Service.Window.ManageWindowSize("Xml." + name, xml.Form);
				xml.Form.Show();
			} else {
				xml.Form.Activate();
			}
		}
		
		public XmlForm(XmlDocument doc) : base(doc) { }
		
		public XmlForm(string dialogName) : base(dialogName) { }
		
		public XmlForm(XmlDocument doc, object eventSink) : base(doc, eventSink) { }
		
		public XmlForm(string dialogName, object eventSink) : base(dialogName, eventSink) { }
		
		public Form Form {
			get {
				return (Form)base.Control;
			}
		}
	}
}
