// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.ComponentModel;
using System.Reflection;

using ICSharpCode.Core.AddIns;
using ICSharpCode.Core.AddIns.Codons;
using ICSharpCode.Core.AddIns.Conditions;
using ICSharpCode.Core.Properties;
using ICSharpCode.Core.Services;
using DGNetwork.Services;

namespace DGNetwork.Search
{
	[CodonName("SearchType")]
	public class SearchTypeCodon : AbstractCodon
	{
		[XmlMemberAttribute("title", IsRequired = false)]
		string title = null;
		
		[XmlMemberAttribute("icon", IsRequired = false)]
		string icon = null;
		
		/// <summary>
		/// Creates an item with the specified sub items. And the current
		/// Condition status for this item.
		/// </summary>
		public override object BuildItem(object owner, ArrayList subItems, ConditionCollection conditions)
		{
			ISearchType s;
			if (this.Class == null) {
				if (title == null) {
					Service.Message.ShowError("title attribute missing on SearchType-codon");
					return null;
				}
				s = new DefaultSearchType(icon, title);
			} else {
				s = (ISearchType)this.AddIn.CreateObject(this.Class);
				if (s == null)
					return null;
			}
			foreach (object o in subItems) {
				if (o is IManager) {
					s.Managers.Add(o);
				} else if (o is ICriteria) {
					s.Criterias.Add((ICriteria)o);
				} else {
					Service.Message.ShowErrorFormatted("${res:Codon.InvalidSubItem}",
					                                   this.Name,
					                                   o.GetType().FullName);
				}
			}
			return s;
		}
	}
	
	[CodonName("SearchCriteria")]
	public class CriteriaCodon : AbstractCodon
	{
		[XmlMemberAttribute("title", IsRequired = true)]
		string title = null;
		
		[XmlMemberAttribute("type", IsRequired = true)]
		string type = null;
		
		[XmlMemberAttribute("value", IsRequired = false)]
		string defaultvalue = null;
		
		/// <summary>
		/// Creates an item with the specified sub items. And the current
		/// Condition status for this item.
		/// </summary>
		public override object BuildItem(object owner, ArrayList subItems, ConditionCollection conditions)
		{
			if (subItems.Count > 0) {
				Service.Message.ShowErrorFormatted("${res:Codon.NoSubItems}", this.Name);
			}
			Type type = Helper.GetType(this.type);
			if (type == null) {
				Service.Message.ShowErrorFormatted("${res:Global.ClassNotFound}", this.type);
				return null;
			}
			if (this.defaultvalue != null) {
				TypeConverter tc = TypeDescriptor.GetConverter(type);
				if (tc == null) {
					Service.Message.ShowErrorFormatted("${res:Global.CantDeserializeType}", this.type);
					return null;
				}
				object val = tc.ConvertFromInvariantString(defaultvalue);
				return new DefaultCriteria(this.ID, title, val);
			} else {
				return new DefaultCriteria(this.ID, title, type);
			}
		}
	}
}
