// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Xml;
using Grunwald.Gui.Settings;

namespace DGNetwork.Search
{
	public interface ICriteria : ISetting
	{
		bool Enabled { get; set; }
		ISettingEditor Editor { get; }
	}
	
	public class DefaultCriteria : DefaultSetting, ICriteria
	{
		object val;
		Type valueType;
		
		public DefaultCriteria(string name, string displayName, object defaultValue)
			: base(name, displayName, "", defaultValue)
		{
			if (defaultValue == null)
				throw new ArgumentNullException("defaultValue");
			this.val = defaultValue;
			this.valueType = defaultValue.GetType();
		}
		
		public DefaultCriteria(string name, string displayName, Type valueType)
			: base(name, displayName, "", null)
		{
			if (valueType == null)
				throw new ArgumentNullException("valueType");
			this.valueType = valueType;
		}
		
		public override object Value {
			get {
				return this.val;
			}
			set {
				object oldValue = this.val;
				if (Object.Equals(oldValue, value))
					return;
				this.val = value;
				this.OnValueChanged(new SettingChangedEventArgs(this, oldValue, value));
				if (!Object.Equals(value, this.InEditValue))
					this.InEditValue = value;
			}
		}
		
		public Type ValueType {
			get {
				return valueType;
			}
		}
		
		ISettingEditor editor;
		
		public ISettingEditor Editor {
			get {
				return editor;
			}
		}
		
		bool enabled = true;
		
		public bool Enabled {
			get {
				return enabled;
			}
			set {
				enabled = value;
				if (editor != null)
					editor.Enabled = value;
			}
		}
		
		public override ISettingEditor CreateEditor(EditorFactory factory) {
			if (factory == null)
				throw new ArgumentNullException("factory");
			editor = factory.GetEditor(this, this.ValueType);
			editor.Enabled = enabled;
			return editor;
		}
		
		protected override void SaveValue(XmlWriter writer) {
			throw new NotSupportedException();
		}
		
		public override void Load(XmlReader reader) {
			throw new NotSupportedException();
		}
	}
}
