// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;

namespace DGNetwork.Search
{
	public interface IResult : DGNetwork.Gui.Menu.IItemWithContextMenu
	{
		string Icon { get; }
		string Name { get; }
		ArrayList Columns { get; }
		
		event EventHandler ResultChanged;
		
		
		void DoubleClick();
		void AddToList();
		bool CanAddToList { get; }
	}
	
	#region ResultColumn
	public class ResultColumn
	{
		string title;
		string value;
		
		public string Title {
			get {
				return title;
			}
			set {
				title = value;
			}
		}
		
		public string Value {
			get {
				return this.value;
			}
			set {
				this.value = value;
			}
		}
		
		public ResultColumn() {
		}
		
		public ResultColumn(string title, string value) {
			this.title = title;
			this.value = value;
		}
	}
	#endregion
	
	#region DefaultResult
	public class DefaultResult : IResult
	{
		string icon;
		
		public string Icon {
			get {
				return icon;
			}
			set {
				icon = value;
			}
		}
		
		string name;
		
		public string Name {
			get {
				return name;
			}
			set {
				name = value;
			}
		}
		
		object DGNetwork.Gui.Menu.IItemWithContextMenu.Owner {
			get {
				return this;
			}
		}
		
		string contextMenuTreePath;
		
		public string ContextMenuTreePath {
			get {
				return contextMenuTreePath;
			}
			set {
				contextMenuTreePath = value;
			}
		}
		
		ArrayList columns = new ArrayList();
		
		public ArrayList Columns {
			get {
				return columns;
			}
		}
		
		public event EventHandler ResultChanged;
		
		protected virtual void OnResultChanged(EventArgs e)
		{
			if (ResultChanged != null) {
				ResultChanged(this, e);
			}
		}
		
		public virtual void DoubleClick()
		{
		}
		
		public virtual void AddToList()
		{
		}
		
		bool canAddToList;
		
		public bool CanAddToList {
			get {
				return canAddToList;
			}
			set {
				canAddToList = value;
			}
		}
	}
	#endregion
	
	public class ErrorResult : DefaultResult
	{
		Exception ex;
		
		public ErrorResult()
		{
			this.Icon = "Icons.Delete"; // TODO: Include real error icon
		}
		
		public ErrorResult(string source, Exception ex) : this()
		{
			this.ex = ex;
			this.Name = source;
			this.Columns.Add(new ResultColumn("Exception", ex.GetType().Name + ": " + ex.Message));
		}
		
		public override void DoubleClick()
		{
			if (ex != null)
				Grunwald.Gui.ErrorReporter.ShowErrorReport(ex);
		}
	}
	
	#region ResultEventHandler / ResultEventArgs
	/// <summary>
	/// Event handler for ResultAdded event.
	/// </summary>
	public delegate void ResultEventHandler(object sender, ResultEventArgs e);
	
	/// <summary>
	/// Event args class for ResultAdded event.
	/// </summary>
	public class ResultEventArgs : EventArgs
	{
		IResult result;
		
		/// <summary>
		/// Gets the result.
		/// </summary>
		public IResult Result {
			get {
				return result;
			}
		}
		
		/// <summary>
		/// Creates a new instance of the <see cref="ResultEventArgs"/> class.
		/// </summary>
		/// <param name="result">The result.</param>
		public ResultEventArgs(IResult result) {
			this.result = result;
		}
	}
	#endregion
}
