// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Windows.Forms;
using Grunwald.Gui.Settings;
using Grunwald.Gui.Settings.Editors;
using DGNetwork.Services;

namespace DGNetwork.Search
{
	public class SearchSettingPage : DefaultSettingPage
	{
		ISetting[] settings;
		ISearchType searchType;
		ISearch search;
		ButtonEditor button;
		
		public Button Button {
			get {
				return (Button)button.Control;
			}
		}
		
		public SearchSettingPage(ISearchType searchType)
		{
			settings = new ISetting[searchType.Criterias.Count + 1];
			this.searchType = searchType;
			for (int i = 0; i < settings.Length - 1; i++) {
				settings[i] = searchType.Criterias[i];
			}
			FixedEditorSetting fes = new FixedEditorSetting("startsearchbutton", "${res:SearchDialog.StartSearch}");
			button = new ButtonEditor(fes);
			button.ButtonClick += new EventHandler(SearchButtonClick);
			button.UseFullWidth = true;
			fes.Editor = button;
			settings[settings.Length - 1] = fes;
		}
		
		private void SearchButtonClick(object sender, EventArgs e)
		{
			try {
				if (search == null) {
					if (searchType.Managers.Count == 0) {
						Service.Message.ShowError("${res:SearchDialog.NoManagerFound}");
						return;
					}
					if (SearchStarted != null)
						SearchStarted(this, EventArgs.Empty);
					this.Save(); // save edited criteria from InEditValue to Value
					search = searchType.Search();
					search.Aborted  += new EventHandler(OnSearchEnded);
					search.Finished += new EventHandler(OnSearchEnded);
					search.ResultAdded += new ResultEventHandler(OnResultAdded);
					button.ButtonText = "${res:SearchDialog.Abort}";
					lock (search.Results) {
						foreach (IResult result in search.Results) {
							RaiseResultAdded(this, new ResultEventArgs(result));
						}
					}
					if (search.IsFinished)
						OnSearchEnded();
				} else {
					search.Abort();
				}
			} catch (Exception ex) {
				Service.Message.ShowError(ex);
			}
		}
		
		public event EventHandler SearchStarted;
		public event EventHandler SearchEnded;
		public event ResultEventHandler ResultAdded;
		
		private void OnSearchEnded(object sender, EventArgs e)
		{
			button.Control.BeginInvoke(new MethodInvoker(OnSearchEnded));
		}
		
		private void OnResultAdded(object sender, ResultEventArgs e)
		{
			button.Control.BeginInvoke(new ResultEventHandler(RaiseResultAdded),
			                           new object[] {this, e});
		}
		
		private void RaiseResultAdded(object sender, ResultEventArgs e)
		{
			try {
				if (ResultAdded != null)
					ResultAdded(sender, e);
			} catch (Exception ex) {
				Service.Message.ShowError(ex);
			}
		}
		
		private void OnSearchEnded()
		{
			try {
				if (SearchEnded != null)
					SearchEnded(this, EventArgs.Empty);
				search = null;
				button.ButtonText = "${res:SearchDialog.StartSearch}";
			} catch (Exception ex) {
				Service.Message.ShowError(ex);
			}
		}
		
		public ISearch Search {
			get {
				return search;
			}
		}
		
		public override ISetting[] Settings {
			get {
				return settings;
			}
		}
		
		public override object Clone()
		{
			throw new NotImplementedException();
		}
	}
}
