// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;

namespace DGNetwork.Search
{
	/// <summary>
	/// Represents a type of search.
	/// Example: search for user, search for group
	/// </summary>
	public interface ISearchType
	{
		// WHIDBEY GENERIC: here are ArrayLists!
		
		string Icon { get; }
		string Name { get; }
		
		CriteriaCollection Criterias { get; }
		ArrayList Managers { get; }
		
		ISearch Search();
	}
	
	public class DefaultSearchType : ISearchType
	{
		string icon;
		string name;
		CriteriaCollection criterias = new CriteriaCollection();
		ArrayList managers = new ArrayList();
		
		public DefaultSearchType(string icon, string name)
		{
			this.icon = icon;
			this.name = name;
		}
		
		#region Getter
		public CriteriaCollection Criterias {
			get {
				return criterias;
			}
		}
		public string Icon {
			get {
				return icon;
			}
		}
		public ArrayList Managers {
			get {
				return managers;
			}
		}
		public string Name {
			get {
				return name;
			}
		}
		#endregion
		
		public virtual ISearch Search()
		{
			MultiSearch search = new MultiSearch();
			foreach (IManager manager in managers) {
				ISearch s = manager.BeginSearch(this);
				if (s != null) {
					search.AddSearch(s);
				}
			}
			search.FinishIfClientsAreFinished();
			return search;
		}
	}
}
