// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Text;
using System.Windows.Forms;
using Grunwald.Gui.Controls;
using ICSharpCode.Core.Services;

namespace DGNetwork.Services
{
	public class BrowserService : AbstractService
	{
		public void OpenWindow(string url)
		{
			if (!url.StartsWith("http://") && !url.StartsWith("https://"))
				url = "http://" + url;
			StartProcess(url, "${res:BrowserService.CannotStartBrowser}");
		}
		
		private void StartProcess(string cmd, string errorText)
		{
			try {
				System.Diagnostics.Process.Start(cmd);
			} catch (Exception ex) {
				if (Helper.IsCritical(ex)) throw;
				Service.Message.ShowErrorFormatted(errorText, ex.Message, ex.ToString());
			}
		}
		
		public void WriteMail(string address)
		{
			StartProcess("mailto:" + Helper.Escape(address), "${res:BrowserService.CannotStartMailClient}");
		}
		
		public void WriteMail(string address, string subject)
		{
			StartProcess(String.Format("mailto:{0}?subject={1}",
			                           Helper.EscapePart(address),
			                           Helper.EscapePart(subject)),
			             "${res:BrowserService.CannotStartMailClient}");
		}
		
		public void WriteMail(string address, string subject, string body)
		{
			StartProcess(String.Format("mailto:{0}?subject={1}&body={2}",
			                           Helper.EscapePart(address),
			                           Helper.EscapePart(subject),
			                           Helper.EscapePart(body)),
			             "${res:BrowserService.CannotStartMailClient}");
		}
	}
}
