// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Reflection;
using ICSharpCode.Core.Services;

namespace DGNetwork.Services
{
	/// <summary>
	/// Class with static getters to get the most common services.
	/// </summary>
	public sealed class Service
	{
		private Service() { }
		
		static IService Get(Type t) {
			IService service = ServiceManager.Services.GetService(t);
			if (service == null)
				throw new BaseException("Service not found: " + t.FullName);
			return service;
		}
		
		public static DGNetwork.DGN.DGNService DGN {
			get {
				return (DGNetwork.DGN.DGNService)Get(typeof(DGNetwork.DGN.DGNService));
			}
		}
		
		public static WindowService Window {
			get {
				return (WindowService)Get(typeof(WindowService));
			}
		}
		
		public static P2PService P2P {
			get {
				return (P2PService)Get(typeof(P2PService));
			}
		}
		
		public static DGNetwork.Chat.ChatService Chat {
			get {
				return (DGNetwork.Chat.ChatService)Get(typeof(DGNetwork.Chat.ChatService));
			}
		}
		
		public static SettingService Setting {
			get {
				return (SettingService)Get(typeof(SettingService));
			}
		}
		
		public static TNAService TNA {
			get {
				return (TNAService)Get(typeof(TNAService));
			}
		}
		
		public static IResourceService Resource {
			get {
				return (IResourceService)Get(typeof(IResourceService));
			}
		}
		
		public static IMessageService Message {
			get {
				return (IMessageService)Get(typeof(IMessageService));
			}
		}
		
		public static BrowserService Browser {
			get {
				return (BrowserService)Get(typeof(BrowserService));
			}
		}
		
		public static MenuService Menu {
			get {
				return (MenuService)Get(typeof(MenuService));
			}
		}
		
		public static PropertyService Property {
			get {
				return (PropertyService)Get(typeof(PropertyService));
			}
		}
		
		public static FileUtilityService FileUtility {
			get {
				return (FileUtilityService)Get(typeof(FileUtilityService));
			}
		}
		
		static StringParserService stringParser;
		
		public static StringParserService StringParser {
			get {
				if (stringParser == null)
					stringParser = (StringParserService)Get(typeof(StringParserService));
				return stringParser;
			}
		}
		
		public static DGNetwork.Contacts.ContactService Contact {
			get {
				return (DGNetwork.Contacts.ContactService)Get(typeof(DGNetwork.Contacts.ContactService));
			}
		}
		
		public static TimerService Timer {
			get {
				return (TimerService)Get(typeof(TimerService));
			}
		}
		
		public static string Parse(string text)
		{
			return StringParser.Parse(text);
		}
		
		public static string Parse(string text, string name)
		{
			return Parse(text, "name", name);
		}
		
		public static string Parse(string text, string propertyName, string propertyValue)
		{
			return Parse(text, new string[,] {{propertyName, propertyValue}});
		}
		
		public static string Parse(string text, string[,] properties)
		{
			return StringParser.Parse(text, properties);
		}
	}
}
