// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.IO;
using System.Collections;
using System.Xml;

using Grunwald.Gui.Settings;

using ICSharpCode.Core.AddIns;
using ICSharpCode.Core.AddIns.Codons;
using ICSharpCode.Core.Properties;
using DGNetwork.Gui;
using DGNetwork.Gui.Dialogs;
using DGNetwork.Settings;

using ICSharpCode.Core.Services;

namespace DGNetwork.Services
{
	public class SettingService : AbstractService
	{
		ISettingPage rootPage;
		
		public ISettingPage RootPage {
			get {
				return rootPage;
			}
		}
		
		const string rootPagePath = "/Settings/RootPage";
		
		protected override void OnInitialize(EventArgs e)
		{
			rootPage = CreatePage(rootPagePath);
			SettingSerializer s = new SettingSerializer(rootPage);
			if (s != Service.Property.GetProperty("Settings", s))
				throw new BaseException("Loading settings failed!");
			base.OnInitialize(e);
		}
		
		/*
		/// <summary>
		/// Tells the setting service to write all settings to disk.
		/// </summary>
		public void SaveConfig()
		{
			using (Stream s = new FileStream(configFile, FileMode.Create, FileAccess.Write, FileShare.None)) {
				XmlTextWriter writer = new XmlTextWriter(s, System.Text.Encoding.UTF8);
				writer.Formatting = Formatting.Indented;
				writer.WriteStartDocument();
				writer.WriteStartElement("settings");
				rootPage.Save(writer);
				writer.WriteEndElement();
				writer.WriteEndDocument();
				writer.Close();
			}
		}
		 */
		
		public ISettingPage CreatePage(string addInTreePath)
		{
			IAddInTreeNode node = AddInTreeSingleton.AddInTree.GetTreeNode(addInTreePath);
			ArrayList subItems = node.BuildChildItems(this);
			object o = node.Codon.BuildItem(null, subItems, node.ConditionCollection);
			return (ISettingPage)o;
		}
		
		public void EditSetting(ISettingPage page)
		{
			XmlForm.ShowForm("SettingsDialog", typeof(SettingsDialogSink));
			SettingsDialogSink sink = (SettingsDialogSink)XmlForm.GetForm("SettingsDialog").EventSink;
			sink.EditSetting(page);
		}
		
		/// <summary>
		/// Gets the page with the specified path relative to the root page.
		/// </summary>
		public ISettingPage GetPage(string path)
		{
			string[] s = path.Split('/');
			ISettingPage page = this.RootPage;
			IAddInTreeNode node = AddInTreeSingleton.AddInTree.GetTreeNode(rootPagePath);
			foreach (string t in s) {
				node = (IAddInTreeNode)node.ChildNodes[t];
				if (node == null)
					return null;
				SettingPageCodon codon = (SettingPageCodon)node.Codon;
				page = page.GetPage(codon.Title);
				if (page == null)
					throw new BaseException("could not find page with title " + codon.Title);
			}
			return page;
		}
		
		private class SettingSerializer : IXmlConvertable
		{
			ISettingPage page;
			
			public SettingSerializer(ISettingPage page)
			{
				this.page = page;
			}
			
			public object FromXmlElement(XmlElement element)
			{
				StringWriter sw = new StringWriter();
				element.WriteTo(new XmlTextWriter(sw));
				XmlTextReader r = new XmlTextReader(new StringReader(sw.ToString()));
				r.Read();
				page.Load(r);
				return this;
			}
			
			public XmlElement ToXmlElement(XmlDocument doc)
			{
				StringWriter sw = new StringWriter();
				page.Save(new XmlTextWriter(sw));
				return (XmlElement)doc.ReadNode(new XmlTextReader(new StringReader(sw.ToString())));
			}
		}
	}
}
