// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.Windows.Forms;
using ICSharpCode.Core.AddIns;
using DGNetwork.AddIns.Codons;
using DGNetwork.Gui;
using ICSharpCode.Core.Services;

namespace DGNetwork.Services
{
	public class TNAService : AbstractService
	{
		/// <summary>
		/// Creates a new TNAIcon. Warning: Don't forget to call Dispose() on it when
		/// you are finished!
		/// </summary>
		public virtual ITNAIcon CreateNewIcon()
		{
			return new NotifyIconWrapper();
		}
		
		Hashtable icons = new Hashtable();
		
		protected override void OnUnload(EventArgs e)
		{
			foreach (ITNAIcon icon in icons.Values) {
				icon.Dispose();
			}
			base.OnUnload(e);
		}
		
		public ITNAIcon GetIcon(string addInTreePath)
		{
			ITNAIcon icon = (ITNAIcon)icons[addInTreePath];
			if (icon == null) {
				icon = CreateIcon(addInTreePath);
				icons[addInTreePath] = icon;
			}
			return icon;
		}
		
		public ITNAIcon MainIcon {
			get {
				return GetIcon("/TNAIcon/MainIcon");
			}
		}
		
		protected virtual ITNAIcon CreateIcon(string addInTreePath)
		{
			return (ITNAIcon)Helper.BuildItem(addInTreePath, this);
		}
	}
}
