// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Text;
using System.Windows.Forms;
using ICSharpCode.Core.Services;

namespace DGNetwork.Services
{
	/// <summary>
	/// Contains a timer ticking every second in the main thread.
	/// </summary>
	public class TimerService : AbstractService
	{
		Timer timer;
		int ticks = 0;
		
		/// <summary>
		/// Gets the number of timer ticks raised since the start of DG Network.
		/// This is not equal to the number of seconds since application start
		/// because the timer could "lag" when executions take longer than one second
		/// (remember it is running on the main thread).
		/// </summary>
		public int Ticks {
			get {
				return ticks;
			}
		}
		
		public override void InitializeService()
		{
			timer = new Timer();
			timer.Tick += new EventHandler(TimerTick);
			timer.Interval = 1000;
			timer.Start();
			base.InitializeService();
		}
		
		public override void UnloadService()
		{
			timer.Dispose();
			base.UnloadService();
		}
		
		private void TimerTick(object sender, EventArgs e)
		{
			ticks += 1;
			try {
				OnTick(e);
			} catch (Exception ex) {
				Service.Message.ShowError(ex);
			}
		}
		
		public event EventHandler Tick;
		
		protected virtual void OnTick(EventArgs e)
		{
			if (Tick != null) {
				Tick(this, e);
			}
		}
	}
}
