// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.Reflection;
using Grunwald.Gui.Settings;
using DGNetwork.Services;

namespace DGNetwork.Settings
{
	public class ExtensibleSettingPage : DefaultSettingPage
	{
		ISetting[] settings;
		Type subpageType;
		
		public ExtensibleSettingPage(ArrayList settingList, ArrayList subPages, SettingPageCodon codon)
			: this(settingList, subPages)
		{
			this.DynamicPage = false;
			if (codon.SubpageType != null) {
				this.subpageType = Helper.GetType(codon.SubpageType);
				this.AllowAddNew = true;
				this.AllowDelete = true;
				this.AllowMove = true;
			}
		}
		
		/// <summary>
		/// Needed as workaround. Don't ask why, better use your time to rewrite the settings
		/// system.
		/// </summary>
		public override bool CanLoadWithOtherTitle {
			get {
				return false;
			}
		}
		
		public override bool CanAddPage(ISettingPage page, int index)
		{
			if (subpageType == null)
				return false;
			return subpageType.IsInstanceOfType(page);
		}
		
		public ExtensibleSettingPage(ArrayList settingList, ArrayList subPages)
		{
			settings = (ISetting[])settingList.ToArray(typeof(ISetting));
			foreach (ISetting setting in settings) {
				setting.ValueChanged += new SettingChangedEventHandler(SettingValueChanged);
			}
			for (int i = 0; i < subPages.Count; i++) {
				this.AddPage((ISettingPage)subPages[i], i);
			}
		}
		
		private void SettingValueChanged(object sender, SettingChangedEventArgs e)
		{
			this.OnSettingChanged(e);
		}
		
		/// <summary>
		/// Gets the settings of this object as array.
		/// </summary>
		public override ISetting[] Settings {
			get { return settings; }
		}
		
		#region Clone/Create
		/// <summary>
		/// Always throws a NotSupportedException - ExtensibleSettingPages can't be cloned.
		/// </summary>
		public override object Clone()
		{
			throw new NotSupportedException();
		}
		
		/// <summary>
		/// This method is used by Load(XmlReader) and must be overridden if this page
		/// can have dynamic sub pages.
		/// </summary>
		protected override ISettingPage CreatePage(string typeName)
		{
			foreach(Assembly assembly in AppDomain.CurrentDomain.GetAssemblies()) {
				Type type = assembly.GetType(typeName);
				if (type == null)
					continue;
				
				ConstructorInfo c = type.GetConstructor(Type.EmptyTypes);
				if (c == null)
					throw new SettingLoadException("The type '" + typeName + "' has no parameterless constructor.");
				return (ISettingPage)c.Invoke(null);
			}
			throw new SettingLoadException("Could not find page type '" + typeName + "'.");
		}
		#endregion
	}
}
