// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.ComponentModel;
using System.Reflection;
using System.Windows.Forms;

using Grunwald.Gui.Settings;

using ICSharpCode.Core.AddIns;
using ICSharpCode.Core.AddIns.Codons;
using ICSharpCode.Core.AddIns.Conditions;
using ICSharpCode.Core.Properties;

using DGNetwork.Gui;
using DGNetwork.Services;

namespace DGNetwork.Settings
{
	[CodonName("Setting")]
	public class SettingCodon : AbstractCodon
	{
		[XmlMemberAttribute("title", IsRequired = true)]
		string title = null;
		
		[XmlMemberAttribute("property", IsRequired = false)]
		string property = null;
		
		[XmlMemberAttribute("description", IsRequired = false)]
		string description = null;
		
		[XmlMemberAttribute("type", IsRequired = true)]
		string type = null;
		
		[XmlMemberAttribute("defaultvalue", IsRequired = false)]
		string defaultvalue = null;
		
		/// <summary>
		/// Creates an item with the specified sub items. And the current
		/// Condition status for this item.
		/// </summary>
		public override object BuildItem(object owner, ArrayList subItems, ConditionCollection conditions)
		{
			if (description == null)
				description = "";
			if (subItems.Count > 0) {
				Service.Message.ShowErrorFormatted("${res:Codon.NoSubItems}", this.Name);
			}
			Type type = Helper.GetType(this.type);
			if (type == null) {
				Service.Message.ShowErrorFormatted("${res:Global.ClassNotFound}", this.type);
				return null;
			}
			if (typeof(ISettingEditor).IsAssignableFrom(type)) {
				SpecialEditorSetting ses = new SpecialEditorSetting(this.ID, title, description);
				ses.EditorType = type;
				return ses;
			} else {
				if (property == null) {
					Service.Message.ShowError("Property attribute is needed if type does not implement ISettingEditor");
					return null;
				}
			}
			if (defaultvalue == null) {
				return new PropertySetting(this.ID, title, description, property, type);
			} else {
				TypeConverter tc = TypeDescriptor.GetConverter(type);
				if (tc == null) {
					Service.Message.ShowErrorFormatted("${res:Global.CantDeserializeType}", this.type);
					return null;
				}
				object val = tc.ConvertFromInvariantString(defaultvalue);
				return new PropertySetting(this.ID, title, description, property, val);
			}
		}
	}
}
