// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.Reflection;
using System.Windows.Forms;

using Grunwald.Gui.Settings;

using ICSharpCode.Core.Properties;
using ICSharpCode.Core.AddIns;
using ICSharpCode.Core.AddIns.Codons;
using ICSharpCode.Core.AddIns.Conditions;
using DGNetwork.AddIns;
using DGNetwork.AddIns.Codons;
using DGNetwork.Gui;
using DGNetwork.Services;

namespace DGNetwork.Settings
{
	[CodonName("SettingPage")]
	public class SettingPageCodon : AbstractCodon
	{
		[XmlMemberAttribute("title", IsRequired = true)]
		string title = null;
		
		[XmlMemberAttribute("icon", IsRequired = false)]
		string icon = null;
		
		[XmlMemberAttribute("subpagetype", IsRequired = false)]
		string subpagetype = null;
		
		public string Title {
			get {
				return title;
			}
			set {
				title = value;
			}
		}
		
		public string Icon {
			get {
				return icon;
			}
			set {
				icon = value;
			}
		}
		
		public string SubpageType {
			get {
				return subpagetype;
			}
			set {
				subpagetype = value;
			}
		}
		
		/// <summary>
		/// Creates an item with the specified sub items. And the current
		/// Condition status for this item.
		/// </summary>
		public override object BuildItem(object owner, ArrayList subItems, ConditionCollection conditions)
		{
			ISettingPage page;
			if (this.Class == null) {
				ArrayList settings = new ArrayList();
				ArrayList subPages = new ArrayList();
				// sort sub items
				foreach (object o in subItems) {
					if (o is ISetting) {
						settings.Add(o);
					} else if (o is ISettingPage) {
						subPages.Add(o);
					} else {
						Service.Message.ShowErrorFormatted("${res:Codon.InvalidSubItem}",
						                                   this.Name,
						                                   o.GetType().FullName);
					}
				}
				page = new ExtensibleSettingPage(settings, subPages, this);
			} else {
				if (subItems.Count != 0) {
					Service.Message.ShowErrorFormatted("${res:Codon.NoSubItemsWithAttribute}", this.Name, "class");
				}
				page = (ISettingPage)this.AddIn.CreateObject(this.Class);
				if (page == null) {
					// object could not be created, create dummy page
					page = new ExtensibleSettingPage(new ArrayList(), new ArrayList());
				}
			}
			page.Title = this.Title;
			if (this.Icon != null) {
				try {
					page.Image = Service.Resource.GetBitmap(this.Icon);
				} catch (ResourceNotFoundException ex) {
					Service.Message.ShowError(ex.Message);
				}
			}
			return page;
		}
	}
}
