// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.IO;
using System.Reflection;
using System.Xml;
using Microsoft.Win32;
using Grunwald.Gui.Settings;

namespace DGNetwork.Settings
{
	public class StartWithWindowsSetting : ISetting
	{
		public StartWithWindowsSetting()
		{
			inEditValue = this.Value;
		}
		
		public string Name {
			get {
				return "StartWithWindows";
			}
		}
		
		public string DisplayName {
			get {
				return "${res:Settings.RootPage.StartWithWindows}";
			}
		}
		
		public string Description {
			get {
				return "${res:Settings.RootPage.StartWithWindowsDescription}";
			}
		}
		
		object inEditValue;
		
		public object InEditValue {
			get { return inEditValue; }
			set {
				object oldValue = inEditValue;
				if (Object.Equals(oldValue, value))
					return;
				inEditValue = value;
				if (InEditValueChanged != null)
					InEditValueChanged(this, new SettingChangedEventArgs(this, oldValue, value));
			}
		}
		
		public event SettingChangedEventHandler InEditValueChanged;
		public event SettingChangedEventHandler ValueChanged;
		
		const string autorunKey = @"Software\Microsoft\Windows\CurrentVersion\Run";
		
		public object Value {
			get {
				string dgn = Path.GetFullPath(Assembly.GetEntryAssembly().Location).ToLower();
				RegistryKey key = Registry.CurrentUser.OpenSubKey(autorunKey);
				if (key == null)
					return false;
				string val = (string)key.GetValue("DG Network");
				key.Close();
				if (val == null)
					return false;
				else
					return val.ToLower().StartsWith(dgn);
			}
			set {
				bool newValue = (bool)value;
				bool oldValue = (bool)this.Value;
				if (newValue == oldValue)
					return;
				RegistryKey key = Registry.CurrentUser.CreateSubKey(autorunKey);
				// CODE DUPLICATION: REG LOCATION ALSO SET IN NSIS SETUP!
				if (newValue) {
					key.SetValue("DG Network", "\"" + Path.GetFullPath(Assembly.GetEntryAssembly().Location) + "\" /autostart");
				} else {
					key.DeleteValue("DG Network", false);
				}
				if (ValueChanged != null)
					ValueChanged(this, new SettingChangedEventArgs(this, oldValue, value));
			}
		}
		
		public ISettingEditor CreateEditor(EditorFactory factory)
		{
			return factory.GetEditor(this, typeof(bool));
		}
		
		public void Save(XmlWriter writer)
		{
			// don't need to save because the value is in the registry
		}
		
		public void Load(XmlReader reader)
		{
			// don't need to load because the value is in the registry
			reader.Skip();
		}
	}
}
