// <file>
//     <copyright see="prj:///doc/copyright.txt"/>
//     <license see="prj:///doc/license.txt"/>
//     <owner name="Mike Krüger" email="mike@icsharpcode.net"/>
//     <version value="$version"/>
// </file>

using System.Xml;

namespace ICSharpCode.Core.Properties
{
	/// <summary>
	/// If you want define own, complex options you can implement this interface
	/// and save it in the main Option class, your class will be saved as xml in
	/// the global properties.
	/// Use your class like any other property. (the conversion will be transparent)
	/// </summary>
	public interface IXmlConvertable
	{
		/// <summary>
		/// Converts a <see cref="XmlElement"/> to an <see cref="IXmlConvertable"/> object.
		/// </summary>
		/// <returns>
		/// A new <see cref="IXmlConvertable"/> object 
		/// </returns>
		object FromXmlElement(XmlElement element);
		
		/// <summary>
		/// Converts the <see cref="IXmlConvertable"/> object to a <see cref="XmlElement"/>.
		/// </summary>
		/// <returns>
		/// A new <see cref="XmlElement"/> object which represents the state
		/// of the <see cref="IXmlConvertable"/> object.
		/// </returns>
		XmlElement ToXmlElement(XmlDocument doc);
	}
}
