// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using DGNetwork.Chat;
using DGNetwork.Chat.Parser;
using NUnit.Framework;

namespace DGNetwork.Tests.Chat.Tests
{
	[TestFixture]
	public class ParserTest
	{
		HtmlFilter filter;
		
		[TestFixtureSetUp]
		public void Init()
		{
			filter = new HtmlFilter();
			BlockTag b = new BlockTag();
			b.Name = "b";
			BlockTag a = new BlockTag();
			a.Name = "a";
			a.RequiredAttributes.Add("href");
			filter.Tags.Add("a", a);
			filter.Tags.Add("b", b);
			filter.ExtendedSyntax = false;
			filter.Replacements.Add(new DefaultReplacement(":<", ":-("));
			filter.Replacements.Add(new DefaultReplacement(";(", "smilie"));
			filter.Replacements.Add(new DefaultReplacement(@"\b(?:http:\/\/|www\.)(?:[^\s\)\]}:\>""'!?\.]|\.\w)+", @"<a href=""$0"">$0</a>", true));
			filter.Replacements.Add(new DefaultReplacement("://", "sm_b"));
			filter.Replacements.Add(new DefaultReplacement(":/", "sm_a"));
			filter.Replacements.Add(new DefaultReplacement(":///", "sm_c"));
		}
		
		[Test]
		public void Replacement()
		{
			Filter("Daniel:<b>Text", "Daniel:<b>Text</b>");
			Filter("Daniel:<c>Text", "Daniel:-(c&gt;Text");
			Filter("Daniel:<b>Text :< </b> :<", "Daniel:<b>Text :-( </b> :-(");
		}
		
		[Test]
		public void ReplacementAndWhitespace()
		{
			Filter(":<", ":-(");
			Filter("  (", "(");
			Filter("&nbsp;&nbsp;(", "&nbsp;&nbsp;(");
		}
		
		[Test]
		public void ReplacementAndNewLine()
		{
			Filter("a\nb", "a b");
			Filter("a\r\n\r\nb", "a b");
		}
		
		const string url = "www.danielgrunwald.de";
		const string link = "<a href=\"www.danielgrunwald.de\">www.danielgrunwald.de</a>";
		const string httpurl = "http://www.danielgrunwald.de";
		const string httplink = "<a href=\"http://www.danielgrunwald.de\">http://www.danielgrunwald.de</a>";
		
		[Test]
		public void URLFiltering()
		{
			Filter(url, link);
			Filter("Homepage " + url + "!", "Homepage " + link + "!");
			Filter("I already know " + url + ".", "I already know " + link + ".");
		}
		
		[Test]
		public void URLDoubleFiltering()
		{
			DoubleFilter(url, link);
			DoubleFilter(httpurl, httplink);
		}
		
		private void Filter(string input, string expectedOutput)
		{
			Assert.AreEqual(expectedOutput, filter.Process(input), input);
		}
		
		private void DoubleFilter(string input, string expectedOutput)
		{
			Assert.AreEqual(expectedOutput, filter.Process(filter.Process(input)), input);
		}
	}
}
