using System;
using System.ComponentModel;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace Grunwald.Gui.Components
{
	public sealed class BalloonNotifyIcon : System.ComponentModel.Component
	{
		private class MessageHandler : Form
		{
			[DllImport("User32", CharSet=CharSet.Auto)]
			private static extern Int32 RegisterWindowMessage(string lpString);
			
			private const int WM_USER = 1024;
			private const int WM_USER_TRAY = WM_USER + 1;
			private const int WM_MOUSEMOVE = 512;
			private const int WM_LBUTTONDOWN = 513;
			private const int WM_LBUTTONUP = 514;
			private const int WM_LBUTTONDBLCLK = 515;
			private const int WM_RBUTTONDOWN = 516;
			private const int WM_RBUTTONUP = 517;
			private const int WM_RBUTTONDBLCLK = 518;
			private const int WM_MBUTTONDOWN = 519;
			private const int WM_MBUTTONUP = 520;
			private const int WM_MBUTTONDBLCLK = 521;
			private const Int32 NIN_BALLOONSHOW = 1026;
			private const Int32 NIN_BALLOONHIDE = 1027;
			private const Int32 NIN_BALLOONTIMEOUT = 1028;
			private const Int32 NIN_BALLOONUSERCLICK = 1029;
			
			public event EventHandler Reload;
			public event EventHandler BalloonShow;
			public event EventHandler BalloonHide;
			public event EventHandler BalloonTimeout;
			public event EventHandler BalloonClick;
			private Int32 WM_TASKBARCREATED = RegisterWindowMessage("TaskbarCreated");
			
			public MessageHandler()
			{
				ShowInTaskbar = false;
				StartPosition = FormStartPosition.Manual;
				this.FormBorderStyle = FormBorderStyle.FixedToolWindow;
				Size = new Size(100, 100);
				Location = new Point(-500, -500);
				Show();
			}
			
			protected override void WndProc(ref System.Windows.Forms.Message m)
			{
				if (m.Msg == WM_USER_TRAY) {
					if (m.LParam.ToInt32() == WM_LBUTTONDBLCLK || m.LParam.ToInt32() == WM_RBUTTONDBLCLK || m.LParam.ToInt32() == WM_MBUTTONDBLCLK) {
						base.OnDoubleClick(new MouseEventArgs(MouseButtons, 0, MousePosition.X, MousePosition.Y, 0));
					} else if (m.LParam.ToInt32() == WM_LBUTTONDOWN || m.LParam.ToInt32() == WM_RBUTTONDOWN || m.LParam.ToInt32() == WM_MBUTTONDOWN) {
						base.OnMouseDown(new MouseEventArgs(MouseButtons, 0, MousePosition.X, MousePosition.Y, 0));
					} else if (m.LParam.ToInt32() == WM_MOUSEMOVE) {
						base.OnMouseMove(new MouseEventArgs(MouseButtons, 0, MousePosition.X, MousePosition.Y, 0));
					} else if (m.LParam.ToInt32() == WM_LBUTTONUP) {
						base.OnMouseUp(new MouseEventArgs(MouseButtons.Left, 0, MousePosition.X, MousePosition.Y, 0));
						base.OnClick(new MouseEventArgs(MouseButtons.Left, 0, MousePosition.X, MousePosition.Y, 0));
					} else if (m.LParam.ToInt32() == WM_RBUTTONUP) {
						base.OnMouseUp(new MouseEventArgs(MouseButtons.Right, 0, MousePosition.X, MousePosition.Y, 0));
						base.OnClick(new MouseEventArgs(MouseButtons.Right, 0, MousePosition.X, MousePosition.Y, 0));
					} else if (m.LParam.ToInt32() == WM_MBUTTONUP) {
						base.OnMouseUp(new MouseEventArgs(MouseButtons.Middle, 0, MousePosition.X, MousePosition.Y, 0));
						base.OnClick(new MouseEventArgs(MouseButtons.Middle, 0, MousePosition.X, MousePosition.Y, 0));
					} else if (m.LParam.ToInt32() == NIN_BALLOONSHOW) {
						if (BalloonShow != null) {
							BalloonShow(this, EventArgs.Empty);
						}
					} else if (m.LParam.ToInt32() == NIN_BALLOONHIDE) {
						if (BalloonHide != null) {
							BalloonHide(this, EventArgs.Empty);
						}
					} else if (m.LParam.ToInt32() == NIN_BALLOONTIMEOUT) {
						if (BalloonTimeout != null) {
							BalloonTimeout(this, EventArgs.Empty);
						}
					} else if (m.LParam.ToInt32() == NIN_BALLOONUSERCLICK) {
						if (BalloonClick != null) {
							BalloonClick(this, EventArgs.Empty);
						}
					}
				} else if (m.Msg == WM_TASKBARCREATED) {
					if (Reload != null) {
						Reload(this, EventArgs.Empty);
					}
				} else {
				}
				base.WndProc(ref m);
			}
		}
		
		public BalloonNotifyIcon(System.ComponentModel.IContainer Container) : this()
		{
			Container.Add(this);
			Initialize();
		}
		
		public BalloonNotifyIcon()
		{
			InitializeComponent();
			Initialize();
		}
		
		protected override void Dispose(bool disposing)
		{
			if (disposing) {
				Visible = false;
				if (!((components == null))) {
					components.Dispose();
				}
			}
			base.Dispose(disposing);
		}
		private System.ComponentModel.IContainer components;
		
		[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Auto)]
		private struct NOTIFYICONDATA
		{
			public Int32 cbSize;
			public IntPtr hwnd;
			public Int32 uID;
			public Int32 uFlags;
			public Int32 uCallbackMessage;
			public IntPtr hIcon;
			[MarshalAs(UnmanagedType.ByValTStr, SizeConst=128)]
			public string szTip;
			public Int32 dwState;
			public Int32 dwStateMask;
			[MarshalAs(UnmanagedType.ByValTStr, SizeConst=256)]
			public string szInfo;
			public Int32 uVersion;
			[MarshalAs(UnmanagedType.ByValTStr, SizeConst=64)]
			public string szInfoTitle;
			public Int32 dwInfoFlags;
		}
		
		[DllImport("Shell32", CharSet=CharSet.Auto)]
		private static extern bool Shell_NotifyIcon(int dwMessage, ref NOTIFYICONDATA lpData);
		
		private const Int32 NIF_MESSAGE = 1;
		private const Int32 NIF_ICON = 2;
		private const Int32 NIF_STATE = 8;
		private const Int32 NIF_INFO = 16;
		private const Int32 NIF_TIP = 4;
		private const Int32 NIM_ADD = 0;
		private const Int32 NIM_MODIFY = 1;
		private const Int32 NIM_DELETE = 2;
		private const Int32 NIM_SETVERSION = 4;
		private const Int32 NOTIFYICON_VERSION = 5;
		private const int WM_USER = 1024;
		private const int WM_USER_TRAY = WM_USER + 1;
		public enum EBalloonIcon
		{
			None = 0,
			Error = 3,
			Info = 1,
			Warning = 2
		}
		private NOTIFYICONDATA NID;
		private MessageHandler Messages = new MessageHandler();
		private bool m_VisibleBeforeBalloon;
		private bool m_Visible;
		private System.Drawing.Icon m_Icon;
		private ContextMenu m_ContextMenu;
		[DescriptionAttribute("Occurs when the user clicks the icon in the status area."), System.ComponentModel.CategoryAttribute("Action")]
		public event EventHandler Click;
		[DescriptionAttribute("Occurs when the user double-clicks the icon in the status notification area of the taskbar."), System.ComponentModel.CategoryAttribute("Action")]
		public event EventHandler DoubleClick;
		[DescriptionAttribute("Occurs when the user presses the mouse button while the pointer is over the icon in the status notification area of the taskbar."), CategoryAttribute("Mouse")]
		public event MouseEventHandler MouseDown;
		[DescriptionAttribute("Occurs when the user moves the mouse while the pointer is over the icon in the status notification area of the taskbar."), CategoryAttribute("Mouse")]
		public event MouseEventHandler MouseMove;
		[DescriptionAttribute("Occurs when the user releases the mouse button while the pointer is over the icon in the status notification area of the taskbar."), CategoryAttribute("Mouse")]
		public event MouseEventHandler MouseUp;
		[DescriptionAttribute("Occurs when the balloon is shown (balloons are queued)."), System.ComponentModel.CategoryAttribute("Behavior")]
		public event EventHandler BalloonShow {
			add    { Messages.BalloonShow += value; }
			remove { Messages.BalloonShow -= value; }
		}
		
		[DescriptionAttribute("Occurs when the balloon disappearsfor example, when the icon is deleted. This message is not sent if the balloon is dismissed because of a timeout or a mouse click."), System.ComponentModel.CategoryAttribute("Behavior")]
		public event EventHandler BalloonHide {
			add    { Messages.BalloonHide += value; }
			remove { Messages.BalloonHide -= value; }
		}
		
		[DescriptionAttribute("Occurs when the balloon is dismissed because of a timeout."), System.ComponentModel.CategoryAttribute("Behavior")]
		public event EventHandler BalloonTimeout;
		[DescriptionAttribute("Occurs when the balloon is dismissed because of a mouse click."), System.ComponentModel.CategoryAttribute("Action")]
		public event EventHandler BalloonClick;
		
		private void Initialize()
		{
			NID.hwnd = Messages.Handle;
			NID.cbSize = Marshal.SizeOf(typeof(NOTIFYICONDATA));
			NID.uFlags = NIF_ICON | NIF_TIP | NIF_MESSAGE;
			NID.uCallbackMessage = WM_USER_TRAY;
			NID.uVersion = NOTIFYICON_VERSION;
			NID.szTip = "";
			NID.uID = 0;
		}
		
		[Description("The icon to display in the system tray."), CategoryAttribute("Appearance"), System.ComponentModel.DefaultValueAttribute("")]
		public System.Drawing.Icon Icon {
			get {
				return m_Icon;
			}
			set {
				m_Icon = value;
				NID.uFlags = NID.uFlags | NIF_ICON;
				NID.hIcon = Icon.Handle;
				if (Visible) {
					Shell_NotifyIcon(NIM_MODIFY, ref NID);
				}
			}
		}
		
		[Description("The text that will be displayed when the mouse hovers over the icon."), CategoryAttribute("Appearance")]
		public string Text {
			get {
				return NID.szTip;
			}
			set {
				NID.szTip = value;
				if (Visible) {
					NID.uFlags = NID.uFlags | NIF_TIP;
					Shell_NotifyIcon(NIM_MODIFY, ref NID);
				}
			}
		}
		
		[Description("The pop-up menu to show when the user right-clicks the icon."), CategoryAttribute("Behavior"), System.ComponentModel.DefaultValueAttribute("")]
		public System.Windows.Forms.ContextMenu ContextMenu {
			get {
				return m_ContextMenu;
			}
			set {
				m_ContextMenu = value;
			}
		}
		
		[Description("Determines whether the control is visible or hidden."), CategoryAttribute("Behavior"), System.ComponentModel.DefaultValueAttribute(false)]
		public bool Visible {
			get {
				return m_Visible;
			}
			set {
				m_Visible = value;
				if (!(DesignMode)) {
					if (m_Visible) {
						Shell_NotifyIcon(NIM_ADD, ref NID);
					} else {
						Shell_NotifyIcon(NIM_DELETE, ref NID);
					}
				}
			}
		}
		
		public void ShowBalloon(EBalloonIcon Icon, string Text, string Title, int Timeout)
		{
			m_VisibleBeforeBalloon = m_Visible;
			NID.uFlags = NID.uFlags | NIF_INFO;
			NID.uVersion = Timeout;
			NID.szInfo = Text;
			NID.szInfoTitle = Title;
			NID.dwInfoFlags = Convert.ToInt32(Icon);
			if (!(Visible)) {
				Visible = true;
			} else {
				Shell_NotifyIcon(NIM_MODIFY, ref NID);
			}
			NID.uFlags = NID.uFlags & ~(NIF_INFO);
		}
		
		[System.Diagnostics.DebuggerStepThrough()]
		private void InitializeComponent()
		{
			components = new System.ComponentModel.Container();
			Messages.Click += new EventHandler(Messages_Click);
			Messages.DoubleClick += new EventHandler(Messages_DoubleClick);
			Messages.MouseDown += new MouseEventHandler(Messages_MouseDown);
			Messages.MouseMove += new MouseEventHandler(Messages_MouseMove);
			Messages.MouseUp += new MouseEventHandler(Messages_MouseUp);
			Messages.Reload += new EventHandler(Messages_Reload);
			Messages.BalloonTimeout += new EventHandler(Messages_BalloonTimeout);
			Messages.BalloonClick += new EventHandler(Messages_BalloonClick);
		}
		
		private void Messages_Click(object sender, EventArgs e)
		{
			if (Click != null) {
				Click(this, e);
			}
		}
		
		private void Messages_DoubleClick(object sender, EventArgs e)
		{
			if (DoubleClick != null) {
				DoubleClick(this, e);
			}
		}
		
		private void Messages_MouseDown(object sender, MouseEventArgs e)
		{
			if (MouseDown != null) {
				MouseDown(this, e);
			}
		}
		
		private void Messages_MouseMove(object sender, MouseEventArgs e)
		{
			if (MouseMove != null) {
				MouseMove(this, e);
			}
		}
		
		private void Messages_MouseUp(object sender, MouseEventArgs e)
		{
			if (MouseUp != null) {
				MouseUp(this, e);
			}
			if (e.Button == MouseButtons.Right) {
				Messages.Activate();
				Point Position = new Point(Cursor.Position.X - Messages.PointToScreen(new Point(0, 0)).X, Cursor.Position.Y - Messages.PointToScreen(new Point(0, 0)).Y);
				if (!(m_ContextMenu == null)) {
					m_ContextMenu.Show(Messages, Position);
				}
			}
		}
		
		private void Messages_Reload(object sender, EventArgs e)
		{
			if (Visible) {
				Visible = true;
			}
		}
		
		private void Messages_BalloonTimeout(object sender, EventArgs e)
		{
			if (!(m_VisibleBeforeBalloon)) {
				Visible = false;
			}
			if (BalloonTimeout != null) {
				BalloonTimeout(this, e);
			}
		}
		
		private void Messages_BalloonClick(object sender, EventArgs e)
		{
			if (!(m_VisibleBeforeBalloon)) {
				Visible = false;
			}
			if (BalloonClick != null) {
				BalloonClick(this, e);
			}
		}
	}
}
