// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Windows.Forms;

namespace Grunwald.Gui.Controls
{
	/// <summary>
	/// Formular used to report errors.
	/// </summary>
	public class ErrorReporterForm : System.Windows.Forms.Form
	{
		private Grunwald.Gui.Controls.ControlRow buttonRow;
		private System.Windows.Forms.TextBox errorDetails;
		private System.Windows.Forms.Label errorDescriptionLabel;
		private System.Windows.Forms.TextBox reportTextBox;
		private Grunwald.Gui.Controls.DockedControlColumn controlColumn;
		
		/// <summary>
		/// Creates a new ErrorReporterForm
		/// </summary>
		public ErrorReporterForm()
		{
			InitializeComponent();
			this.DialogResult = DialogResult.Cancel;
			controlColumn[reportTextBox].MinSize = 50;
			controlColumn[errorDetails].MinSize = 50;
			controlColumn.Refresh(); // apply new MinSize values
			buttonRow.ArrangeActive = true;
		}
		
		#region Properties
		/// <summary>
		/// Gets/Sets the positioning data of the splitters.
		/// </summary>
		public byte[] SplitterData {
			get { return controlColumn.Factors; }
			set {
				Check();
				controlColumn.Factors = value;
			}
		}
		
		/// <summary>
		/// Gets the list of buttons.
		/// </summary>
		public Control.ControlCollection Buttons {
			get { return buttonRow.Controls; }
		}
		
		/// <summary>
		/// Gets/Sets the error details.
		/// </summary>
		public string ErrorDetailsText {
			get { return errorDetails.Text; }
			set {
				Check();
				errorDetails.Text = value;
			}
		}
		
		/// <summary>
		/// Gets/Sets the error description.
		/// </summary>
		public string ErrorDescriptionText {
			get { return errorDescriptionLabel.Text; }
			set {
				Check();
				errorDescriptionLabel.Text = value;
			}
		}
		
		/// <summary>
		/// Gets/Sets the report text entered by the user.
		/// </summary>
		public string ReportText {
			get { return reportTextBox.Text; }
			set {
				Check();
				reportTextBox.Text = value;
			}
		}
		
		/// <summary>
		/// Gets/Sets if the error details are visible.
		/// </summary>
		public bool ErrorDetailsVisible {
			get { return errorDetails.Visible; }
			set {
				Check();
				errorDetails.Visible = value;
			}
		}
		
		/// <summary>
		/// Gets/Sets if the error description is visible.
		/// </summary>
		public bool ErrorDescriptionVisible {
			get { return errorDescriptionLabel.Visible; }
			set {
				Check();
				errorDescriptionLabel.Visible = value;
			}
		}
		
		/// <summary>
		/// Gets/Sets the report text entered by the user.
		/// </summary>
		public bool ReportTextVisible {
			get { return reportTextBox.Visible; }
			set {
				Check();
				reportTextBox.Visible = value;
			}
		}
		#endregion
		
		private void Check()
		{
			if (base.IsDisposed) throw new ObjectDisposedException("ErrorReporterForm");
			if (base.InvokeRequired) throw new InvokeRequiredException();
		}
		
		#region Windows Forms Designer generated code
		/// <summary>
		/// This method is required for Windows Forms designer support.
		/// Do not change the method contents inside the source code editor. The Forms designer might
		/// not be able to load this method if it was changed manually.
		/// </summary>
		private void InitializeComponent() {
			this.controlColumn = new Grunwald.Gui.Controls.DockedControlColumn();
			this.reportTextBox = new System.Windows.Forms.TextBox();
			this.errorDescriptionLabel = new System.Windows.Forms.Label();
			this.errorDetails = new System.Windows.Forms.TextBox();
			this.buttonRow = new Grunwald.Gui.Controls.ControlRow();
			this.controlColumn.SuspendLayout();
			this.SuspendLayout();
			//
			// controlColumn
			//
			this.controlColumn.AddSplitter = true;
			this.controlColumn.Controls.Add(this.reportTextBox);
			this.controlColumn.Controls.Add(this.errorDetails);
			this.controlColumn.Dock = System.Windows.Forms.DockStyle.Fill;
			this.controlColumn.Location = new System.Drawing.Point(8, 72);
			this.controlColumn.Name = "controlColumn";
			this.controlColumn.Size = new System.Drawing.Size(352, 230);
			this.controlColumn.TabIndex = 1;
			//
			// reportTextBox
			//
			this.reportTextBox.Dock = System.Windows.Forms.DockStyle.Bottom;
			this.reportTextBox.Location = new System.Drawing.Point(0, 0);
			this.reportTextBox.Multiline = true;
			this.reportTextBox.Name = "reportTextBox";
			this.reportTextBox.ScrollBars = System.Windows.Forms.ScrollBars.Vertical;
			this.reportTextBox.Size = new System.Drawing.Size(352, 134);
			this.reportTextBox.TabIndex = 0;
			this.reportTextBox.Text = "";
			//
			// errorDescriptionLabel
			//
			this.errorDescriptionLabel.Dock = System.Windows.Forms.DockStyle.Top;
			this.errorDescriptionLabel.Location = new System.Drawing.Point(8, 8);
			this.errorDescriptionLabel.Name = "errorDescriptionLabel";
			this.errorDescriptionLabel.Size = new System.Drawing.Size(352, 64);
			this.errorDescriptionLabel.TabIndex = 2;
			this.errorDescriptionLabel.Text = "label";
			//
			// errorDetails
			//
			this.errorDetails.BackColor = System.Drawing.SystemColors.Window;
			this.errorDetails.Dock = System.Windows.Forms.DockStyle.Bottom;
			this.errorDetails.Location = new System.Drawing.Point(0, 134);
			this.errorDetails.Multiline = true;
			this.errorDetails.Name = "errorDetails";
			this.errorDetails.ReadOnly = true;
			this.errorDetails.ScrollBars = System.Windows.Forms.ScrollBars.Vertical;
			this.errorDetails.Size = new System.Drawing.Size(352, 96);
			this.errorDetails.TabIndex = 1;
			this.errorDetails.Text = "";
			//
			// buttonRow
			//
			this.buttonRow.AutoScroll = true;
			this.buttonRow.ContentAlignment = System.Drawing.ContentAlignment.MiddleRight;
			this.buttonRow.Direction = Grunwald.Gui.Controls.Direction.LeftToRight;
			this.buttonRow.Dock = System.Windows.Forms.DockStyle.Bottom;
			this.buttonRow.Location = new System.Drawing.Point(8, 302);
			this.buttonRow.Name = "buttonRow";
			this.buttonRow.Size = new System.Drawing.Size(352, 40);
			this.buttonRow.TabIndex = 0;
			//
			// ErrorReporterForm
			//
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 14);
			this.ClientSize = new System.Drawing.Size(468, 342);
			this.Controls.Add(this.controlColumn);
			this.Controls.Add(this.buttonRow);
			this.Controls.Add(this.errorDescriptionLabel);
			this.DockPadding.Left = 8;
			this.DockPadding.Right = 8;
			this.DockPadding.Top = 8;
			this.MinimumSize = new System.Drawing.Size(344, 320);
			this.StartPosition = FormStartPosition.CenterScreen;
			this.Name = "ErrorReporterForm";
			this.Text = "ErrorReporter";
			this.controlColumn.ResumeLayout(false);
			this.ResumeLayout(false);
		}
		#endregion
	}
}
