// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Drawing;
using System.Windows.Forms;

namespace Grunwald.Gui.Controls
{
	/// <summary>
	/// Extended version of the rich text box that supports changing of non selected
	/// text.
	/// </summary>
	public class ExtendedRichTextBox : RichTextBox
	{
		bool inUpdate = false;
		IntPtr eventMask;
		int selStart, selLength, scrollX, scrollY;
		
		/// <summary>
		/// Begins with a series of rich text box updates, this stops drawing and sending
		/// events.
		/// </summary>
		public void BeginUpdate()
		{
			if (inUpdate) return;
			inUpdate = true;
			
			this.SuspendLayout();
			Win32.SendMessage(this.Handle, Win32.WM_SETREDRAW, 0, IntPtr.Zero);
			eventMask = Win32.SendMessage(this.Handle, Win32.EM_GETEVENTMASK, 0, IntPtr.Zero);
			selStart = this.SelectionStart;
			selLength = this.SelectionLength;
			scrollX = Win32.GetScrollPos(this.Handle, Win32.SBS_HORZ);
			scrollY = Win32.GetScrollPos(this.Handle, Win32.SBS_VERT);
		}
		
		/// <summary>
		/// Ends with the series of updates.
		/// </summary>
		public void EndUpdate()
		{
			if (!inUpdate) return;
			inUpdate = false;
			Win32.SendMessage(this.Handle, Win32.EM_SETEVENTMASK, 0, eventMask);
			this.Select(selStart, selLength);
			Win32.PostMessageA(this.Handle, Win32.WM_HSCROLL,
			                   Win32.SB_THUMBPOSITION + 0x10000 * scrollX, 0);
			Win32.PostMessageA(this.Handle, Win32.WM_VSCROLL,
			                   Win32.SB_THUMBPOSITION + 0x10000 * scrollY, 0);
			Win32.SendMessage(this.Handle, Win32.WM_SETREDRAW, 1, IntPtr.Zero);
			this.ResumeLayout();
			this.Invalidate();
		}
	}
}
