// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Drawing;
using System.Windows.Forms;

namespace Grunwald.Gui.Settings.Editors
{
	/// <summary>
	/// 
	/// </summary>
	public class BoolEditor : SingleControlEditor
	{
		/// <summary>
		/// Creates a new TextBoxBasedEditor.
		/// </summary>
		public BoolEditor(ISetting setting) : base(setting, new CheckBox())
		{
			checkBox = (CheckBox)base.Control;
			checkBox.FlatStyle = FlatStyle.System;
		}
		
		CheckBox checkBox;
		
		/// <summary>
		/// Creates the controls needed for editing the <see cref="ISetting"/> and
		/// adds them to the specified parent control.
		/// </summary>
		public override void CreateControls(Rectangle bounds, EditorProperties properties)
		{
			checkBox.Text = properties.Translator.Translate(this.Setting.DisplayName);
			base.CreateControls(bounds, properties);
			checkBox.CheckedChanged += new EventHandler(CheckedChanged);
		}
		
		private void CheckedChanged(object sender, EventArgs e)
		{
			Setting.InEditValue = checkBox.Checked;
		}
		
		/// <summary>
		/// Loads the edit value into the editor.
		/// </summary>
		protected override void LoadEditValue()
		{
			checkBox.Checked = (bool)Setting.InEditValue;
		}
		
		/// <summary>
		/// Gets the <see cref="LabelMode"/> this setting editor uses.<br/>
		/// If not overridden, this property always returns
		/// <see cref="LabelMode">LabelMode.NoLabel</see>.
		/// </summary>
		public override LabelMode LabelMode {
			get { return LabelMode.NoLabel; }
		}
	}
}
