// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Windows.Forms;

namespace Grunwald.Gui.Settings.Editors
{
	/// <summary>
	/// Some properties the editors need to access the panel.
	/// </summary>
	public class EditorProperties
	{
		ErrorProvider errorProvider;
		Translator translator;
		Control parent;
		
		/// <summary>
		/// Creates a new EditorProperties object with the specified properties.
		/// </summary>
		/// <param name="errorProvider">The ErrorProvider used to display errors.</param>
		/// <param name="translator">The Translator used to translate the error messages.</param>
		/// <param name="parent">The parent control where the own controls should be added to.</param>
		/// <exception cref="ArgumentNullException">Occurs if any of the parameters is null.</exception>
		public EditorProperties(ErrorProvider errorProvider, Translator translator, Control parent)
		{
			if (errorProvider == null)
				throw new ArgumentNullException("errorProvider");
			if (translator == null)
				throw new ArgumentNullException("translator");
			if (parent == null)
				throw new ArgumentNullException("parent");
			this.errorProvider = errorProvider;
			this.translator = translator;
			this.parent = parent;
		}
		
		/// <summary>
		/// The ErrorProvider used to display errors.
		/// </summary>
		public ErrorProvider ErrorProvider {
			get { return errorProvider; }
		}
		
		/// <summary>
		/// The Translator used to translate the error messages.
		/// </summary>
		public Translator Translator {
			get { return translator; }
		}
		
		/// <summary>
		/// The parent control where the own controls should be added to.
		/// </summary>
		public Control Parent {
			get { return parent; }
		}
	}
}
