// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.IO;
using System.Drawing;
using System.Windows.Forms;
using Grunwald.Gui.Controls;

namespace Grunwald.Gui.Settings.Editors
{
	/// <summary>
	/// Description of StringEditor.
	/// </summary>
	public class FileNameEditor : StringEditor
	{
		string chooseButtonText = "...";
		Button btn;
		
		/// <summary>
		/// Creates a new FileNameEditor.
		/// </summary>
		public FileNameEditor(ISetting setting) : base(setting)
		{
		}
		
		/// <summary>
		/// Gets the size the edit field needs.
		/// </summary>
		public override Size GetSize()
		{
			Size baseSize = base.GetSize();
			return new Size(baseSize.Width + 16, Math.Max(baseSize.Height, 22));
		}
		
		/// <summary>
		/// Creates the controls needed for editing the <see cref="ISetting"/> and
		/// adds them to the specified parent control.
		/// </summary>
		public override void CreateControls(Rectangle bounds, EditorProperties properties)
		{
			btn = new AutoSizedButton(properties.Translator.Translate(chooseButtonText));
			RightSpace += btn.Width + 4;
			base.CreateControls(bounds, properties);
			Control ctl = base.Control;
			btn.Location = new Point(ctl.Right + 4, ctl.Top - 1);
			btn.Anchor = AnchorStyles.Top | AnchorStyles.Right;
			btn.Click += new EventHandler(ButtonClick);
			RegisterForOnActivated(btn);
			properties.Parent.Controls.Add(btn);
		}
		
		private void ButtonClick(object sender, EventArgs e)
		{
			OnChooseClick(e);
		}
		
		/// <summary>
		/// Occurs when the "choose..." button is clicked.
		/// </summary>
		public event EventHandler ChooseClick;
		
		/// <summary>
		/// Raises the <see cref="ChooseClick"/> event.
		/// </summary>
		protected virtual void OnChooseClick(EventArgs e)
		{
			if (ChooseClick != null) {
				ChooseClick(this, e);
			}
		}
		
		/// <summary>
		/// Saves the edited value to the setting.
		/// </summary>
		protected override void SaveEditValue()
		{
			base.SaveEditValue();
			string text = TextBox.Text;
			string error = null;
			if (!File.Exists(text)) {
				error = "File not found.";
			}
			SetError(error);
		}
	}
}
