// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Drawing;
using System.Windows.Forms;

namespace Grunwald.Gui.Settings.Editors
{
	/// <summary>
	/// Description of StringEditor.
	/// </summary>
	public class IntEditor : TextBoxBasedEditor
	{
		/// <summary>
		/// Creates a new StringEditor.
		/// </summary>
		public IntEditor(ISetting setting) : base(setting)
		{
		}
		
		/// <summary>
		/// Loads the edit value into the editor.
		/// </summary>
		protected override void LoadEditValue()
		{
			if (saving) return;
			object val = Setting.InEditValue;
			if (val == null) return;
			if (!(val is int))
				throw new Exception("value is " + val.GetType().FullName + "  //  " + val.ToString());
			string text = ((int)val).ToString();
			if (TextBox.Text != text) TextBox.Text = text;
		}
		
		bool saving = false;
		
		/// <summary>
		/// Saves the edited value to the setting.
		/// </summary>
		protected override void SaveEditValue()
		{
			saving = true;
			try {
				int v;
				try {
					v = int.Parse(TextBox.Text);
				} catch (FormatException ex) {
					SetError(ex.Message);
					return;
				} catch (OverflowException ex) {
					SetError(ex.Message);
					return;
				}
				try {
					Setting.InEditValue = v;
				} catch (SettingEditException ex) {
					SetError(ex.Message);
					return;
				}
				SetError(null);
			} finally {
				saving = false;
			}
		}
	}
}
