// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.Drawing;
using System.Windows.Forms;
using System.Xml;
using Grunwald.Gui.Controls;
using Grunwald.Gui.Settings;
using Grunwald.Gui.Settings.Editors;

namespace Grunwald.Gui.Settings.Editors
{
	/// <summary>
	/// Setting editor that can be used to display a label.
	/// Useful in combination with <see cref="FixedEditorSetting"/> or <see cref="SpecialEditorSetting"/>.
	/// </summary>
	public class LabelEditor : SingleControlEditor
	{
		Label lbl;
		
		/// <summary>
		/// Creates a new LabelEditor for the specified setting.
		/// </summary>
		public LabelEditor(ISetting setting) : base(setting, new Label())
		{
			lbl = (Label)Control;
			lbl.Height = 32;
			text = setting.DisplayName;
		}
		
		Translator translator;
		
		string text;
		
		/// <summary>
		/// Gets/Sets the text of the label. By default, the DisplayName of the setting is used.
		/// </summary>
		public string LabelText {
			get {
				return text;
			}
			set {
				text = value;
				if (translator != null)
					lbl.Text = translator.Translate(value);
			}
		}
		
		/// <summary>
		/// Do nothing because a label has no value.
		/// </summary>
		protected override void LoadEditValue() { }
		
		/// <summary>
		/// Creates the controls needed for editing the <see cref="ISetting"/> and
		/// adds them to the specified parent control.
		/// </summary>
		public override void CreateControls(Rectangle bounds, EditorProperties properties)
		{
			translator = properties.Translator;
			lbl.Text = translator.Translate(text);
			bounds.Height = lbl.Height;
			base.CreateControls(bounds, properties);
		}
		
		/// <summary>
		/// Gets the label mode of the label.
		/// This method always returns <see cref="LabelMode">LabelMode.NoLabel</see>.
		/// </summary>
		public override LabelMode LabelMode {
			get {
				return LabelMode.NoLabel;
			}
		}
	}
}
