// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Drawing;
using System.Windows.Forms;

namespace Grunwald.Gui.Settings.Editors
{
	/// <summary>
	/// Description of StringEditor.
	/// </summary>
	public class StringEditor : TextBoxBasedEditor
	{
		/// <summary>
		/// Creates a new StringEditor.
		/// </summary>
		public StringEditor(ISetting setting) : base(setting)
		{
		}
		
		/// <summary>
		/// Loads the edit value into the editor.
		/// </summary>
		protected override void LoadEditValue()
		{
			string text = (string)Setting.InEditValue;
			if (TextBox.Text != text) TextBox.Text = text;
		}
		
		/// <summary>
		/// Saves the edited value to the setting.
		/// </summary>
		protected override void SaveEditValue()
		{
			try {
				Setting.InEditValue = TextBox.Text;
			} catch (SettingEditException ex) {
				SetError(ex.Message);
				return;
			}
			SetError(null);
		}
	}
}
