// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Drawing;
using System.Windows.Forms;

namespace Grunwald.Gui.Settings.Editors
{
	/// <summary>
	/// Description of StringEditor.
	/// </summary>
	public abstract class TextBoxBasedEditor : SingleControlEditor
	{
		/// <summary>
		/// Creates a new TextBoxBasedEditor.
		/// </summary>
		protected TextBoxBasedEditor(ISetting setting) : base(setting, new TextBox())
		{
			textBox = (TextBox)base.Control;
		}
		
		TextBox textBox;
		
		/// <summary>
		/// Gets the TextBox used for editing the string.
		/// </summary>
		public TextBox TextBox {
			get { return textBox; }
		}
		
		/// <summary>
		/// Creates the controls needed for editing the <see cref="ISetting"/> and
		/// adds them to the specified parent control.
		/// </summary>
		public override void CreateControls(Rectangle bounds, EditorProperties properties)
		{
			base.CreateControls(bounds, properties);
			textBox.TextChanged += new EventHandler(TextChanged);
		}
		
		private void TextChanged(object sender, EventArgs e)
		{
			SaveEditValue();
		}
		
		/// <summary>
		/// Saves the edited value to the setting.
		/// </summary>
		protected abstract void SaveEditValue();
	}
}
