// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Reflection;

namespace Grunwald.Gui.Settings
{
	/// <summary>
	/// Description of SettingAttribute.
	/// </summary>
	[AttributeUsage(AttributeTargets.Field | AttributeTargets.Property,
	                Inherited=true, AllowMultiple = false)]
	public sealed class SettingAttribute : Attribute
	{
		string name;
		string displayName;
		string description;
		
		#region Constructors
		/// <summary>
		/// Creates a new SettingAttribute.
		/// </summary>
		/// <param name="name">The name of the setting (used for storing the setting).</param>
		/// <param name="displayName">The displayed name of this setting. This string is passed to the <see cref="Translator"/>.</param>
		/// <param name="description">The description of the setting. This string is passed to the <see cref="Translator"/>.</param>
		public SettingAttribute(string name, string displayName, string description)
		{
			DefaultSetting.CheckName(name);
			this.name = name;
			this.displayName = displayName;
			this.description = description;
		}
		
		/// <summary>
		/// Creates a new SettingAttribute.
		/// </summary>
		/// <param name="name">The name of the setting (used for storing the setting).</param>
		/// <param name="displayName">The displayed name of this setting. This string is passed to the <see cref="Translator"/>.</param>
		/// <param name="description">The description of the setting. This string is passed to the <see cref="Translator"/>.</param>
		/// /// <param name="v">The description of the setting. This string is passed to the <see cref="Translator"/>.</param>
		public SettingAttribute(string name, string displayName, string description, ISettingValidator v)
		{
			DefaultSetting.CheckName(name);
			this.name = name;
			this.displayName = displayName;
			this.description = description;
		}
		
		/// <summary>
		/// Creates a new SettingAttribute.
		/// </summary>
		/// <param name="name">The name of the setting (used for storing the setting).</param>
		/// <param name="displayName">The displayed name of this setting. This string is passed to the <see cref="Translator"/>.</param>
		public SettingAttribute(string name, string displayName)
		{
			DefaultSetting.CheckName(name);
			this.name = name;
			this.displayName = displayName;
		}
		
		/// <summary>
		/// Creates a new SettingAttribute.
		/// </summary>
		/// <param name="name">The name of the setting (used for storing the setting).</param>
		public SettingAttribute(string name)
		{
			DefaultSetting.CheckName(name);
			this.name = name;
		}
		#endregion
		
		#region Getters
		/// <summary>
		/// The name of the setting (used for storing the setting).
		/// </summary>
		public string Name {
			get { return name; }
		}
		
		/// <summary>
		/// The displayed name of this setting. This string is passed to the <see cref="Translator"/>.
		/// </summary>
		public string DisplayName {
			get { return displayName; }
		}
		
		/// <summary>
		/// The description of the setting. This string is passed to the <see cref="Translator"/>.
		/// </summary>
		public string Description {
			get { return description; }
		}
		#endregion
	}
}
