// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;

namespace Grunwald.Gui
{
	/// <summary>
	/// This class is capable of translating strings, for example loading parts of the
	/// specified string from resources etc...
	/// </summary>
	public class Translator
	{
		static Translator defaultTranslator = new Translator();
		
		/// <summary>
		/// Gets/Sets the default translator to use.
		/// </summary>
		public static Translator DefaultTranslator {
			get { return defaultTranslator; }
			set {
				if (value == null)
					defaultTranslator = new Translator();
				else
					defaultTranslator = value;
			}
		}
		
		/// <summary>
		/// Translates the specified string. If not overridden, this just returns the specified
		/// string.
		/// </summary>
		public virtual string Translate(string text)
		{
			return text;
		}
		
		/// <summary>
		/// Gets the text for the default OK button.
		/// </summary>
		public virtual string OK {
			get {
				return "OK";
			}
		}
		
		/// <summary>
		/// Gets the text for the cancel button.
		/// </summary>
		public virtual string Cancel {
			get {
				return "Cancel";
			}
		}
		
		/// <summary>
		/// Tranlates format and returns the result of <see cref="String.Format">String.Format</see>.
		/// </summary>
		public string Format(string format, object arg0)
		{
			return String.Format(Translate(format), arg0);
		}
		
		/// <summary>
		/// Tranlates format and returns the result of <see cref="String.Format">String.Format</see>.
		/// </summary>
		public string Format(string format, object arg0, object arg1)
		{
			return String.Format(Translate(format), arg0, arg1);
		}
		
		/// <summary>
		/// Tranlates format and returns the result of <see cref="String.Format">String.Format</see>.
		/// </summary>
		public string Format(string format, object arg0, object arg1, object arg2)
		{
			return String.Format(Translate(format), arg0, arg1, arg2);
		}
		
		/// <summary>
		/// Tranlates format and returns the result of <see cref="String.Format">String.Format</see>.
		/// </summary>
		public string Format(string format, params object[] args)
		{
			return String.Format(Translate(format), args);
		}
		
		/// <summary>
		/// Tranlates format and returns the result of <see cref="String.Format">String.Format</see>.
		/// </summary>
		public string Format(IFormatProvider formatProvider, string format, params object[] args)
		{
			return String.Format(formatProvider, Translate(format), args);
		}
		
		/// <summary>
		/// Occurs whenever the language has changed and everything needs to be re-translated.
		/// </summary>
		public event EventHandler Changed;
		
		/// <summary>
		/// Raises the <see cref="Changed">Changed event</see>.
		/// </summary>
		protected virtual void OnChanged(EventArgs e)
		{
			if (Changed != null) {
				Changed(this, e);
			}
		}
	}
}
