// P2PLib - Copyright (C) 2004-2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;

namespace Grunwald.P2PLib.DGNetwork
{
	/// <summary>
	/// Contains an address to an element in the DG network.
	/// </summary>
	public abstract class DgnAddress : P2PAddress
	{
		int userID;
		
		/// <summary>
		/// Gets the user ID this address is pointing to.
		/// </summary>
		public int UserID {
			get { return userID; }
		}
		
		/// <summary>
		/// Gets the network this address points to.
		/// </summary>
		public new DgnP2PNetwork Network {
			get { return base.Network as DgnP2PNetwork; }
		}
		
		Identity identity;
		/// <summary>
		/// Gets/Sets the Identity of the target user.
		/// </summary>
		public Identity Identity {
			get { return identity; }
			set { identity = value; }
		}
		
		
		/// <summary>
		/// Creates a new address pointing to the specified user.
		/// </summary>
		protected DgnAddress(DgnP2PNetwork network, int userID, AddressTarget target)
			: base (network, target)
		{
			this.userID = userID;
		}
		
		/// <summary>
		/// Gets the string representation of this DGNAddress.
		/// </summary>
		public override string ToString()
		{
			return String.Format("[{3}: Network = {0}, UserID = {2}, Target = {1}]",
			                     Network,
			                     Target,
			                     UserID,
			                     GetType().Name);
		}
	}
}
