// P2PLib - Copyright (C) 2004-2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;

namespace Grunwald.P2PLib.DGNetwork
{
	#region NetClient.Received
	/// <summary>
	/// Is used for the <see cref="NetClient.Received">NetClient.Received-event</see>.
	/// </summary>
	public delegate void ReceivedEventHandler(object sender, ReceivedEventArgs e);
	
	/// <summary>
	/// The arguments of the NetClient.Received-Event
	/// </summary>
	public class ReceivedEventArgs : EventArgs
	{
		byte[] data;
		
		/// <summary>
		/// The data that was received.
		/// </summary>
		public byte[] Data {
			get { return data; }
		}
		
		DataBlockType dataBlockType;
		
		/// <summary>
		/// The type of this packet.
		/// </summary>
		public DataBlockType DataBlockType {
			get { return dataBlockType; }
		}
		
		bool handled = false;
		
		/// <summary>
		/// Gets/Sets if this event was handled.
		/// If nobody handles the event, the NetClient closes the connection with an
		/// "Invalid Packet"-Message.
		/// </summary>
		public bool Handled {
			get { return handled; }
			set { handled = value; }
		}
		
		/// <summary>
		/// Created a new instance of ReceivedEventArgs representing the specified packet.
		/// </summary>
		public ReceivedEventArgs(DataBlockType dataBlockType, byte[] data)
		{
			this.data = data;
			this.dataBlockType = dataBlockType;
		}
	}
	#endregion
	
	#region P2PNetwork.Receive
	/// <summary>
	/// Is used for the <see cref="DgnP2PNetwork.Receive">DgnP2PNetwork.Receive-event</see>.
	/// </summary>
	public delegate void PacketEventHandler(object sender, PacketEventArgs e);
	
	/// <summary>
	/// Is used for the <see cref="DgnP2PNetwork.Receive">DgnP2PNetwork.Receive-event</see>.
	/// </summary>
	public class PacketEventArgs : EventArgs
	{
		Packet packet;
		
		/// <summary>
		/// Gets the packet that was received.
		/// </summary>
		public Packet Packet {
			get { return packet; }
		}
		
		Answer answer;
		
		/// <summary>
		/// Gets/Sets the answer to the packet.
		/// The answer object should be used to set the return value.
		/// </summary>
		public Answer Answer {
			get {
				return answer;
			}
			set {
				answer = value;
			}
		}
		
		/// <summary>
		/// Created a new instance of PacketEventArgs representing the specified packet.
		/// </summary>
		public PacketEventArgs(Packet packet, Answer answer)
		{
			this.packet = packet;
			this.answer = answer;
		}
	}
	#endregion
	
	#region Answer.Received
	/// <summary>
	/// Is used when a <see cref="SendResult"/> receives an answer.
	/// </summary>
	public delegate void AnswerEventHandler(object sender, AnswerEventArgs e);
	
	/// <summary>
	/// Is used when a <see cref="SendResult"/> receives an answer.
	/// </summary>
	public class AnswerEventArgs : EventArgs
	{
		Answer answer;
		
		/// <summary>
		/// Gets the answer.
		/// </summary>
		public Answer Answer {
			get {
				return answer;
			}
		}
		
		/// <summary>
		/// Created a new instance of AnswerEventArgs representing the specified answer.
		/// </summary>
		public AnswerEventArgs(Answer answer)
		{
			this.answer = answer;
		}
	}
	#endregion
}
