// P2PLib - Copyright (C) 2004-2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.Globalization;
using System.Text;

namespace Grunwald.P2PLib.DGNetwork
{
	/// <summary>
	/// A table containing <see cref="RoutingEntry">routing entries</see>.
	/// This table is used to calculate the route/next station on the route for a packet.
	/// </summary>
	public class RoutingTable
	{
		Hashtable h = new Hashtable(); // GENERIC : id -> entry
		
		/// <summary>
		/// Gets the RoutingEntry of the specified user.
		/// </summary>
		public RoutingEntry this [int uid] {
			get {
				return (RoutingEntry)h[uid];
			}
			set {
				if (value == null)
					h.Remove(uid);
				else
					h[uid] = value;
			}
		}
		
		// WHIDBEY GENERIC: List<entry>
		ArrayList changeLog = new ArrayList();
		
		/// <summary>
		/// Gets the changelog array list used to store the entries
		/// that were changed and need to be resent.
		/// </summary>
		public ArrayList ChangeLog {
			get {
				return changeLog;
			}
		}
		
		NetClient netClient;
		/// <summary>
		/// Gets the NetClient of this RoutingTable.
		/// </summary>
		public NetClient NetClient {
			get {
				return netClient;
			}
		}
		
		/// <summary>
		/// Creates a new RoutingTable.
		/// </summary>
		public RoutingTable(NetClient netClient)
		{
			this.netClient = netClient;
		}
		
		/// <summary>
		/// Converts the content of this routing table into a multiline string.
		/// </summary>
		public override string ToString() {
			StringBuilder b = new StringBuilder("- RoutingTable -\n");
			b.Append("  Changelog: ");
			foreach(object o in ChangeLog)
				b.AppendFormat("{0}, ", o);
			b.Append("\n  Entries:\n");
			foreach(RoutingEntry e in h.Values)
				b.AppendFormat("    {0}\n", e);
			return b.ToString();
		}
	}
}
