// P2PLib - Copyright (C) 2004-2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;

namespace Grunwald.P2PLib.DGNetwork
{
	/// <summary>
	/// An address pointing to a dg network user.
	/// </summary>
	public class UserAddress : DgnAddress
	{
		/// <summary>
		/// Creates a new address pointing to the specified user.
		/// </summary>
		public UserAddress(DgnP2PNetwork network, int userID)
			:base(network, userID, AddressTarget.User)
		{
		}
		
		/// <summary>
		/// Creates a new address pointing to the specified user.
		/// </summary>
		public UserAddress(DgnP2PNetwork network, int userID, Identity identity)
			:base(network, userID, AddressTarget.User)
		{
			base.Identity = identity;
		}
		
		/// <summary>
		/// Determines whether the specified UserAddress is equal to the current UserAddress.
		/// </summary>
		/// <returns>
		/// Returns true if both addresses are equal, returns false if the addresses are
		/// different or the object is not an address.
		/// </returns>
		public override bool Equals(object obj)
		{
			if (obj == this) return true;
			UserAddress a = obj as UserAddress;
			if (a == null) return false;
			if (a.Network == this.Network && a.Target == this.Target && a.UserID == this.UserID)
				return true;
			else
				return false;
		}
		
		/// <summary>
		/// Serves as a hash function for the object.
		/// </summary>
		public override int GetHashCode()
		{
			unchecked {
				return base.GetHashCode() * 29 + this.UserID;
			}
		}
		
		/// <summary>
		/// Gets the string representation of this UserAddress.
		/// </summary>
		public override string ToString()
		{
			return String.Format("{0}@{1}", UserID, Network.Identifier);
		}
	}
}
