// P2PLib - Copyright (C) 2004-2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.Text;
using System.Threading;
using NUnit.Framework;
using Grunwald.P2PLib.Network;
using Grunwald.P2PLib.Server;
using Grunwald.P2PLib.Server.Http;
using Grunwald.P2PLib.DGNetwork;


namespace Grunwald.P2PLib.Tests.DGNetwork
{
	//[TestFixture]
	public class RoutingTest
	{
		Exception ex;
		int running;
		
		[Test]
		public void PeerRoutingStrategy()
		{
			running = 0;
			PeerRoutingStrategy[] strat = new PeerRoutingStrategy[6];
			DgnP2PNetwork[] network = new DgnP2PNetwork[6];
			for (int i = 0; i < 6; ++i) {
				ReceivingUser usr = new ReceivingUser(new Identity(512));
				network[i] = new DgnP2PNetwork("test_id", usr);
				usr.Addresses.Add(new UserAddress(network[i], i + 100, usr.Identity));
				strat[i] = (PeerRoutingStrategy)network[i].RoutingStrategy;
			}
			//   4-------3
			//   |       |
			//   0-------2
			//    \     /
			//     \-1-/
			//       |
			//       5
			ArrayList netClients = new ArrayList();
			int step = 1000;
			netClients.Add(BuildNetClient(network[0], network[1]));
			Thread.Sleep(step);
			netClients.Add(BuildNetClient(network[1], network[2]));
			Thread.Sleep(step);
			netClients.Add(BuildNetClient(network[2], network[3]));
			Thread.Sleep(step);
			netClients.Add(BuildNetClient(network[3], network[4]));
			Thread.Sleep(step);
			netClients.Add(BuildNetClient(network[4], network[0]));
			Thread.Sleep(step);
			netClients.Add(BuildNetClient(network[0], network[2]));
			Thread.Sleep(step);
			NetClient lastNC = BuildNetClient(network[1], network[5]);
			Thread.Sleep(step);
			// TODO: check initial tables
			//CheckMainTable(strat,
			/*for (int j = 0; j < 5; ++j) {
				for (int i = 0; i < 6; ++i) {
					foreach (NetClient c in network[i].Connections) {
						strat[i].SendRoutingTable(c, false);
						Thread.Sleep(25);
					}
				}
			}*/
			
			for (int i = 0; i < 6; ++i) {
				Console.WriteLine();
				Console.WriteLine("Table of {0}", i + 100);
				Console.WriteLine(strat[i]);
			}
			
			lastNC.Close();
			// TODO: check routing tables directly after closing
			/*for (int i = 0; i < 6; ++i) {
				Console.WriteLine();
				Console.WriteLine("Table of {0}", i + 100);
				Console.WriteLine(strat[i]);
			}*/
			
			foreach (NetClient nc in netClients) {
				nc.Close();
			}
			Thread.Sleep(100);
			if (ex != null)
				throw new Exception("Thread got exception", ex);
			if (running > 0) {
				Assert.Fail("Threads did not stop.");
			}
		}
		
		private NetClient BuildNetClient(DgnP2PNetwork net1, DgnP2PNetwork net2)
		{
			LocalConnection c1, c2;
			LocalConnection.Create(out c1, out c2);
			DgnRequestHandler handler = new DgnRequestHandler();
			handler.ReceivingUsers.Add(net2.ReceivingUsers[0]);
			UserAddress target = (UserAddress)net2.ReceivingUsers[0].Addresses[0];
			NetClient nc = net1.BuildConnection(c1, net1.ReceivingUsers[0], target.UserID);
			HttpRequest request = new HttpRequest(c2, HttpLikeServer.GetRequest(c2));
			RequestManager rm = new RequestManager(request, handler, this);
			Thread thread;
			thread = new Thread(new ThreadStart(rm.Run));
			thread.IsBackground = true;
			thread.Start();
			nc.ReceiveServerResponse();
			if (nc.Status != NetClientStatus.ReadyForLoop)
				throw new Exception("Not ready for loop");
			ConnectManager cm = new ConnectManager(this, nc);
			thread = new Thread(new ThreadStart(cm.Run));
			thread.IsBackground = true;
			thread.Start();
			return nc;
		}
		
		private class ConnectManager
		{
			RoutingTest rt;
			NetClient nc;
			
			public ConnectManager(RoutingTest rt, NetClient nc)
			{
				this.rt = rt;
				this.nc = nc;
				lock (rt) {
					rt.running += 1;
				}
			}
			
			public void Run()
			{
				try {
					nc.ReceiveLoop();
				} catch (Exception ex) {
					Console.WriteLine(ex.ToString());
					rt.ex = ex;
				} finally {
					lock (rt) {
						rt.running -= 1;
					}
				}
			}
		}
		
		private class RequestManager
		{
			HttpRequest request;
			RoutingTest rt;
			DgnRequestHandler handler;
			
			public RequestManager(HttpRequest request, DgnRequestHandler handler, RoutingTest rt)
			{
				this.request = request;
				this.rt = rt;
				this.handler = handler;
				lock (rt) {
					rt.running += 1;
				}
			}
			
			public void Run()
			{
				try {
					handler.Handle(request);
				} catch (Exception ex) {
					Console.WriteLine(ex.ToString());
					rt.ex = ex;
				} finally {
					lock (rt) {
						rt.running -= 1;
					}
				}
			}
		}
	}
}
