// P2PLib - Copyright (C) 2004-2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Text;
using NUnit.Framework;

namespace Grunwald.P2PLib.Tests
{
	[TestFixture]
	public class HelperTest
	{
		public void TestEscape()
		{
			string testString = "http://username:passwort@www.host-name.example/path/" +
				"to file%21.ext?field=value&field2=value2#anchor";
			string correct = "http://username:passwort@www.host-name.example/path/" +
				"to%20file%2521.ext?field=value&field2=value2%23anchor";
			string res = Helper.Escape(testString);
			Assert.AreEqual(correct, res, "Error in Helper.Escape");
			res = Helper.Unescape(res);
			Assert.AreEqual(testString, res, "Error in Helper.Unescape");
			
			correct = "http%3a%2f%2fusername%3apasswort%40www.host-name.example%2f" +
				"path%2fto%20file%2521.ext%3ffield%3dvalue%26field2%3dvalue2%23anchor";
			res = Helper.EscapePart(testString);
			Assert.AreEqual(correct, res, "Error in Helper.EscapePart");
			res = Helper.Unescape(res);
			Assert.AreEqual(testString, res, "Error in Helper.Unescape (after EscapePart)");
		}
		
		public void TestEscapePlus()
		{
			string testString = "a/b+c";
			Assert.AreEqual("a/b%2bc", Helper.Escape(testString), "Escape +");
			Assert.AreEqual("a%2fb%2bc", Helper.EscapePart(testString), "EscapePart +");
		}
		
		public void TestEscapeWithEncoding()
		{
			Encoding iso = Encoding.GetEncoding("ISO8859-1");
			Assert.AreEqual("www.%c3%b6ko.de", Helper.Escape("www.öko.de"), "Default (UTF-8)");
			Assert.AreEqual("www.%f6ko.de", Helper.Escape("www.öko.de", iso), "ISO8859-1");
		}
		
		public void TestUnescapeWithEncoding()
		{
			Encoding iso = Encoding.GetEncoding("ISO8859-1");
			Assert.AreEqual("www.öko.de", Helper.Unescape("www.%C3%b6ko.de"), "Default (UTF-8)");
			Assert.AreEqual("www.öko.de", Helper.Unescape("www.%F6ko.de", iso), "ISO8859-1");
		}
	}
}
