// P2PLib - Copyright (C) 2004-2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Net;
using Grunwald.P2PLib.Server;
using Grunwald.P2PLib.Server.IPDetectors;
using log4net;
using NUnit.Framework;

namespace Grunwald.P2PLib.Tests.Server
{
	[TestFixture]
	public class IPDetectorTest
	{
		static readonly ILog log = LogManager.GetLogger(typeof (IPDetectorTest));
		
		#region Test if detectors work
		
		[Test]
		public void XmlBasedTest()
		{
			DoTest(new XmlBasedDetector(MainClass.TestDataDir + "showmyip.com.xml", "/ip_address/ip"));
		}
		
		[Test]
		public void TextBasedTest()
		{
			DoTest(new TextBasedDetector(MainClass.TestDataDir + "showmyip.com.txt"));
			DoTest(new TextBasedDetector(MainClass.TestDataDir + "WhatIsMyIP.com.html"));
		}
		
		void DoTest(IPDetector detector)
		{
			if (!detector.Refresh())
				Assert.Fail(detector.ToString() + " failed.");
			Assert.AreEqual(IPAddress.Parse("80.134.62.133"), detector.IPAddress, detector.ToString() + " returned wrong IP.");
		}
		
		#endregion
		
		#region Test proper behavior in error cases
		
		[Test]
		public void FailingDetector()
		{
			IPDetector d;
			// test TextBasedDetector with invalid URL
			d = new TextBasedDetector("http://0.0.0.0/");
			if (d.Refresh())
				Assert.Fail("Refresh (1) should not be succesfull");
			// test XmlBasedDetector with invalid URL
			d = new XmlBasedDetector("http://0.0.0.0/", "/");
			if (d.Refresh())
				Assert.Fail("Refresh (2) should not be succesfull");
			// test XmlBasedDetector with XPath pointing to non-existant element
			d = new XmlBasedDetector(MainClass.TestDataDir + "showmyip.com.xml", "/ip_adress/ip");
			if (d.Refresh())
				Assert.Fail("Refresh (3) should not be succesfull");
		}
		
		#endregion
		
		#region Test MultisourceDetector
		
		#endregion
	}
}
