// P2PLib - Copyright (C) 2004-2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.IO;
using System.Text;
using log4net;
using NUnit.Framework;
using Grunwald.P2PLib.Server;

namespace Grunwald.P2PLib.Tests.Server
{
	[TestFixture]
	public class PartnerStreamTest
	{
		[Test]
		public void NormalTest()
		{
			PartnerStream s1, s2;
			PartnerStream.Create(out s1, out s2);
			byte[] data = Encoding.UTF8.GetBytes("Hello ");
			s1.Write(data, 0, data.Length);
			data = Encoding.UTF8.GetBytes("World!");
			s1.Write(data, 0, data.Length);
			s1.Flush();
			byte[] data2 = new byte[100];
			int count = s2.Read(data2, 0, 100);
			string text = Encoding.UTF8.GetString(data2, 0, count);
			Assert.AreEqual("Hello World!", text, "PartnerStream does not work");
			s1.Close();
			s2.Close();
		}
		
		[Test]
		public void CloseTest()
		{
			PartnerStream s1, s2;
			PartnerStream.Create(out s1, out s2);
			byte[] data = Encoding.UTF8.GetBytes("Hello ");
			s1.Write(data, 0, data.Length);
			data = Encoding.UTF8.GetBytes("World!");
			s1.Write(data, 0, data.Length);
			s1.Close();
			if (s1.CanRead)
				Assert.Fail("After closing an stream reading is not allowed!");
			if (s1.CanWrite)
				Assert.Fail("After closing an stream writing is not allowed!");
			if (!s2.CanRead)
				Assert.Fail("After closing an stream the other stream should still be allowed to read!");
			if (s1.CanWrite)
				Assert.Fail("After closing an stream the other stream should not be allowed to write!");
			byte[] data2 = new byte[100];
			int count = s2.Read(data2, 0, 100);
			string text = Encoding.UTF8.GetString(data2, 0, count);
			Assert.AreEqual("Hello World!", text, "PartnerStream closing does not work");
			s2.Close();
			if (s2.CanRead)
				Assert.Fail("After closing the other stream it should not be allowed to read!");
		}
	}
}
