// P2PLib - Copyright (C) 2004-2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Net;
using System.Net.Sockets;

namespace Grunwald.P2PLib
{
	/// <summary>
	/// Class used for creating new TCP Connections.
	/// </summary>
	public class ConnectionFactory
	{
		static ConnectionFactory defaultFactory = new ConnectionFactory();
		
		/// <summary>
		/// Gets/Sets the default factory new networks use.
		/// </summary>
		public static ConnectionFactory DefaultFactory {
			get {
				return defaultFactory;
			}
			set {
				if (value == null) throw new ArgumentNullException();
				defaultFactory = value;
			}
		}
		
		/// <summary>
		/// Builds a connection and returns it as IConnection.
		/// Returns null if the endPoint has an unknown type.
		/// </summary>
		public virtual IConnection BuildConnection(EndPoint endPoint)
		{
			if (endPoint == null) throw new ArgumentNullException("endPoint");
			if (endPoint is IPEndPoint) {
				Socket s;
				try {
					s = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
					s.Connect(endPoint);
					if (!s.Connected) throw new BuildConnectionException("Could not connect.");
				} catch (SocketException ex) {
					throw new BuildConnectionException("Could not connect.", ex);
				}
				return new TcpConnection(s);
			} else {
				return null;
			}
		}
	}
}
