// P2PLib - Copyright (C) 2004-2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.IO;
using System.Net;

namespace Grunwald.P2PLib
{
	/// <summary>
	/// Interface for connections, e.g. the TcpConnection.
	/// </summary>
	public interface IConnection
	{
		/// <summary>
		/// The stream for accessing the connection.
		/// </summary>
		Stream Stream {get;}
		
		/// <summary>
		/// The reader for reading data from the connection.
		/// </summary>
		StreamReader Reader {get;}
		
		/// <summary>
		/// The writer for sending data on the connection.
		/// </summary>
		StreamWriter Writer {get;}
		
		/// <summary>
		/// Returns if the connection is still alive
		/// </summary>
		bool Connected {get;}
		
		/// <summary>
		/// Closes the connection
		/// </summary>
		void Close();
		
		/// <summary>
		/// Gets the remote end point this connection is connected with.
		/// Can be null if the end point is unknown.
		/// </summary>
		EndPoint RemoteEndPoint {get;}
	}
}
