// P2PLib - Copyright (C) 2004-2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Net;

namespace Grunwald.P2PLib.Network
{
	/// <summary>
	/// An entry in an <see cref="AddressCollection"/>.
	/// </summary>
	public class AddressEntry
	{
		EndPoint endPoint;
		Identity identity = null;
		P2PAddress address = null;
		int tries = 0;
		
		/// <summary>
		/// Gets/Sets the number of tries to connect.
		/// </summary>
		public int Tries {
			get {
				return tries;
			}
			set {
				tries = value;
			}
		}
		
		/// <summary>
		/// Creates a new AddressEntry.
		/// </summary>
		public AddressEntry(Identity identity, EndPoint endPoint)
		{
			EndPoint = endPoint;
			Identity = identity;
		}
		
		/// <summary>
		/// Creates a new AddressEntry.
		/// </summary>
		public AddressEntry(P2PAddress address, EndPoint endPoint)
		{
			EndPoint = endPoint;
			Address = address;
		}
		
		/// <summary>
		/// Gets/Sets the <see cref="P2PAddress"/> of this entry.
		/// Can return null if no unique identity is assigned.
		/// but Identity and Address cannot be both null.
		/// </summary>
		public P2PAddress Address {
			get { return address; }
			set {
				if (Identity == null && value == null)
					throw new ArgumentNullException("address");
				address = value;
			}
		}
		
		/// <summary>
		/// Gets/Sets the <see cref="Identity"/> of this entry.
		/// Can return null if no unique identity is assigned,
		/// but Identity and Address cannot be both null.
		/// </summary>
		public Identity Identity {
			get { return identity; }
			set {
				if (Address == null && value == null)
					throw new ArgumentNullException("identity");
				identity = value;
			}
		}
		
		/// <summary>The EndPoint of this entry.</summary>
		public EndPoint EndPoint {
			get { return endPoint; }
			set {
				if (value == null)
					throw new ArgumentNullException("endPoint");
				endPoint = value;
			}
		}
		
		/// <summary>Gibt die String-Darstellung dieses Eintrags zurck.</summary>
		public override string ToString()
		{
			return string.Format(System.Globalization.CultureInfo.InvariantCulture,
			                     "[{3}: Identity={0} Address={1} EndPoint={2}]",
			                     Identity, Address, EndPoint, GetType().FullName);
		}
	}
}
