// P2PLib - Copyright (C) 2004-2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;

namespace Grunwald.P2PLib.Network
{
	/// <summary>
	/// Used for the <see cref="Connection.Closed">Connection.Closed event</see>.
	/// </summary>
	public delegate void ClosedEventHandler(object sender, ClosedEventArgs e);
	
	/// <summary>
	/// Used for the <see cref="Connection.Closed">Connection.Closed event</see>.
	/// </summary>
	public class ClosedEventArgs
	{
		bool remotelyClosed;
		bool reasonSpecified;
		string message;
		
		/// <summary>
		/// Gets if this connection has been closed remotely (<b>true</b>) or
		/// by calling close on this connection (<b>false</b>).
		/// </summary>
		public bool RemotelyClosed {
			get {
				return remotelyClosed;
			}
		}
		
		/// <summary>
		/// Gets if the a reason was specified for closing the connection.
		/// </summary>
		public bool ReasonSpecified {
			get {
				return reasonSpecified;
			}
		}
		
		/// <summary>
		/// Gets the reason for closing, if specified, or the message why
		/// the connection was lost.
		/// </summary>
		public string Message {
			get {
				return message;
			}
		}
		
		/// <summary>
		/// Creates a new ClosedEventArgs object.
		/// </summary>
		public ClosedEventArgs(bool remotelyClosed, bool reasonSpecified, string message)
		{
			this.remotelyClosed = remotelyClosed;
			this.reasonSpecified = reasonSpecified;
			this.message = message;
		}
	}
}
