// P2PLib - Copyright (C) 2004-2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Globalization;

namespace Grunwald.P2PLib.Network
{
	/// <summary>
	/// Abstract class for a connection with another peer.
	/// </summary>
	public abstract class Connection
	{
		// TODO: Refactor this to an interface.
		
		/// <summary>
		/// The Path the network requests go to. This is "/system/network/";
		/// </summary>
		public const string RequestPath = "/system/network/";
		
		P2PAddress partner;
		
		/// <summary>
		/// Gets the address of the partner of this connection.
		/// </summary>
		public P2PAddress Partner {
			get { return partner; }
		}
		
		/// <summary>
		/// Gets if this NetClient is still connected with a socket.
		/// </summary>
		public abstract bool Connected { get; }
		
		/// <summary>
		/// Gets if this connection is ready.
		/// </summary>
		public abstract bool Ready { get; }
		
		/// <summary>
		/// Closes the connection without specifying a reason.
		/// </summary>
		public abstract void Close();
		
		/// <summary>
		/// Closes the connection with specifying a reason.
		/// </summary>
		public abstract void Close(string message);
		
		/// <summary>
		/// Occurs after the connection has been closed.
		/// </summary>
		public event ClosedEventHandler Closed;
		
		/// <summary>
		/// Raises the <see cref="Closed"/> event.
		/// </summary>
		protected virtual void OnClosed(ClosedEventArgs e) {
			if (Closed != null) {
				Closed(this, e);
			}
		}
		
		/// <summary>
		/// Creates a new Connection with the given partner.
		/// </summary>
		protected Connection(P2PAddress partner)
		{
			if (partner == null)
				throw new ArgumentNullException("partner");
			if (partner.Target != AddressTarget.User)
				throw new ArgumentException("Address must point to an user.", "partner");
			this.partner = partner;
		}
		
		/// <summary>
		/// Gets the string representation of this connection.
		/// </summary>
		public override string ToString()
		{
			return String.Format("[{0}: Partner = {1}]",
			                     GetType().FullName, partner);
		}
	}
}
