// P2PLib - Copyright (C) 2004-2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;

namespace Grunwald.P2PLib.Network
{
	/// <summary>
	/// Interface for a class that can buffer it's output.
	/// </summary>
	public interface IBufferedOutput
	{
		/// <summary>
		/// <para>Increases the count of send buffers by one.</para>
		/// <para>This makes the send method redirect it's output into the buffer
		/// instread of the stream.</para>
		/// <para>Use this method to send multiple data blocks in one TCP packet.
		/// When you are finished sending the blocks, call EndBuffer to send
		/// them all at once.</para>
		/// <para>If you fail to call EndBuffer (e.g. exception), this NetClient
		/// won't ever send any data again but cache everything!</para>
		/// </summary>
		void StartBuffer();
		
		/// <summary>
		/// <para>Decreaes the count of send buffers by one.</para>
		/// <para>If the count reaches zero, the current buffer content is flushed
		/// to the network stream.</para>
		/// </summary>
		void EndBuffer();
		
		/// <summary>
		/// Flushes the current buffer content without closing it.
		/// </summary>
		void FlushBuffer();
	}
}
