// P2PLib - Copyright (C) 2004-2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;

namespace Grunwald.P2PLib.Network
{
	/// <summary>
	/// A user on this computer receiving packages from one or more networks.
	/// </summary>
	public class ReceivingUser
	{
		Identity identity;
		
		/// <summary>
		/// Gets the identity of this ReceivingUser.
		/// </summary>
		public Identity Identity {
			get { return identity; }
		}
		
		P2PAddressCollection addresses = new P2PAddressCollection();
		
		/// <summary>
		/// Gets the collection containing the addresses this ReceivingUser has.
		/// </summary>
		public P2PAddressCollection Addresses {
			get { return addresses; }
		}
		
				/// <summary>
		/// Is raised when a receiving user is added to the collection.
		/// </summary>
		public event P2PAddressEventHandler AddressAdded {
			add    { addresses.AddressAdded += value; }
			remove { addresses.AddressAdded -= value; }
		}
		
		/// <summary>
		/// Is raised when a receiving user is removed from the collection.
		/// </summary>
		public event P2PAddressEventHandler AddressRemoved {
			add    { addresses.AddressRemoved += value; }
			remove { addresses.AddressRemoved -= value; }
		}
		
		/// <summary>
		/// Creates a new ReceivingUser with the specified identity.
		/// </summary>
		public ReceivingUser(Identity identity)
		{
			this.identity = identity;
		}
		
		/// <summary>
		/// Gets the string representation of this ReceivingUser.
		/// </summary>
		public override string ToString()
		{
			return "[" + GetType().FullName + ": " + identity.PublicKey + "]";
		}
	}
	
	/// <summary>
	/// Delegate for the ReceivingUserAdded/ReceivingUserRemoved events.
	/// </summary>
	public delegate void ReceivingUserEventHandler(object sender, ReceivingUserEventArgs e);
	
	/// <summary>
	/// Event args for the ReceivingUserAdded/ReceivingUserRemoved events.
	/// </summary>
	public class ReceivingUserEventArgs : EventArgs
	{
		ReceivingUser user;
		
		/// <summary>
		/// The affected ReceivingUser.
		/// </summary>
		public ReceivingUser User {
			get {
				return user;
			}
		}
		
		/// <summary>
		/// Creates a new ReceivingUserEventArgs object.
		/// </summary>
		/// <param name="user">The affected ReceivingUser.</param>
		public ReceivingUserEventArgs(ReceivingUser user)
		{
			this.user = user;
		}
	}
}
