// P2PLib - Copyright (C) 2004-2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Globalization;

namespace Grunwald.P2PLib.Server.Http
{
	/// <summary>
	/// Processes requests to <see cref="ServerAbilityTester.PingPath"/>
	/// </summary>
	public class PingRequestHandler : IRequestHandler
	{
		/// <summary>
		/// Creates a new PingRequestHandler
		/// </summary>
		public PingRequestHandler()
		{
			serverID = Helper.GetRandomInt();
		}
		
		int serverID;
		/// <summary>
		/// Gets the id of this server (is randomly created when the PingRequestHandler is
		/// created). This is used for checking if the <see cref="ServerAbilityTester"/>
		/// has connected with the correct server.
		/// </summary>
		public int ServerId {
			get { return serverID; }
		}
		
		
		/// <summary>
		/// Handles the given connection
		/// </summary>
		public void Handle(HttpRequest request)
		{
			HttpResponse response = new HttpResponse(request);
			string txt = "[[P2PLib:" + ServerId.ToString("d", CultureInfo.InvariantCulture) + "]]";
			response.ContentLength = txt.Length;
			response.Server = "P2PLib PingRequestHandler";
			response.SendHeaders(request.Connection.Writer, false);
			request.Connection.Writer.Write(txt);
			request.Connection.Writer.Flush();
		}
		
		/// <summary>
		/// Gets the priority this ConnectionHandler has for the given request.
		/// The ConnectionHandler that returns the highest priority will handle the
		/// request. The PingRequestHandler has a priority of 20000 for requests to
		/// <see cref="ServerAbilityTester.PingPath"/>
		/// </summary>
		public int GetPriority(HttpRequest request)
		{
			if (request.File == ServerAbilityTester.PingPath)
				return 20000;
			else
				return Int32.MinValue;
		}
	}
}
