// P2PLib - Copyright (C) 2004-2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.IO;
using System.Net;
using System.Text;
using System.Text.RegularExpressions;

namespace Grunwald.P2PLib.Server.Http
{
	/// <summary>
	/// 
	/// </summary>
	public class ShareFileRequestHandler : IRequestHandler
	{
		string alias;
		string file;
		string contentType;
		bool caseSensitive;
		bool regex;
		
		WebHeaderCollection extraHeaders = null;
		
		/// <summary>
		/// Collection that can be used to specify additional headers to the output.
		/// </summary>
		public WebHeaderCollection ExtraHeaders {
			get {
				if (extraHeaders == null)
					extraHeaders = new WebHeaderCollection();
				return extraHeaders;
			}
		}
		
		/// <summary>
		/// 
		/// </summary>
		/// <param name="alias"></param>
		/// <param name="caseSensitive"></param>
		/// <param name="regex"></param>
		/// <param name="file"></param>
		/// <param name="contentType"></param>
		public ShareFileRequestHandler(string alias, bool caseSensitive, bool regex, string file, string contentType) {
			this.alias = alias;
			this.caseSensitive = caseSensitive;
			this.contentType = contentType;
			this.file = file;
			this.regex = regex;
		}
		
		///<summary></summary>
		public virtual void Handle(HttpRequest request) {
			FileUpload upload = new FileUpload(request);
			upload.Headers.Add(extraHeaders);
			if (contentType != null)
				upload.Headers["Content-Type"] = contentType;
			upload.Send(file);
		}
		
		/// <summary>
		/// 
		/// </summary>
		public virtual int GetPriority(HttpRequest request) {
			if (regex) {
				// TODO: Regex mode
				throw new Exception("Implement regex mode");
			} else {
				if (caseSensitive) {
					if (request.File == alias)
						return 100;
					else
						return int.MinValue;
				} else {
					if (request.File.ToLower() == alias.ToLower())
						return 100;
					else
						return int.MinValue;
				}
			}
		}
	}
}
