// P2PLib - Copyright (C) 2004-2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.IO;
using System.Net;
using System.Text;
using System.Text.RegularExpressions;

namespace Grunwald.P2PLib.Server.Http
{
	/// <summary>
	/// 
	/// </summary>
	public class ShareFolderRequestHandler : IRequestHandler
	{
		string server = "P2PLib Server";
		
		/// <summary>The name to use as server header.</summary>
		public string Server {
			get { return server; }
			set { server = value; }
		}
		
		string alias;
		string aliasPath;
		string folder;
		bool caseSensitive;
		bool regex;
		bool browsable;
		
		/// <summary>
		/// 
		/// </summary>
		/// <param name="alias"></param>
		/// <param name="caseSensitive"></param>
		/// <param name="regex"></param>
		/// <param name="folder"></param>
		/// <param name="browsable"></param>
		public ShareFolderRequestHandler(string alias, bool caseSensitive, bool regex, string folder, bool browsable) {
			if (alias.EndsWith("/")) {
				this.aliasPath = alias;
				this.alias = alias.Substring(0, alias.Length - 1);
			} else {
				this.alias = alias;
				this.aliasPath = alias + "/";
			}
			this.caseSensitive = caseSensitive;
			this.browsable = browsable;
			folder = folder.Replace('\\', '/');
			if (folder.EndsWith("/"))
				this.folder = folder;
			else
				this.folder = folder + '/';
			this.regex = regex;
		}
		
		/// <summary>
		/// 
		/// </summary>
		public void Handle(HttpRequest request) {
			if (!Directory.Exists(folder))
				throw new HttpException(HttpStatusCode.NotFound);
			HttpResponse response = null;
			WebHeaderCollection header = new WebHeaderCollection();
			header.Set("Server", server);
			if (regex) {
				throw new Exception("Implement regex mode");
			} else {
				if (request.File.Length <= alias.Length) {
					// redirect to real folder
					response = new HttpResponse(request);
					response.StatusCode = HttpStatusCode.Moved;
					response.MergeHeaders(header);
					response.Headers.Set("Location", aliasPath);
					response.SendHeaders(request.Connection.Writer, true);
				} else {
					string relPath = request.File.Substring(aliasPath.Length);
					// note: HttpRequest takes care that using /.. is illegal in URLs
					string localPath = Path.Combine(folder, relPath);
					if (localPath.EndsWith("/")) {
						FileUpload upload = new FileUpload(request);
						upload.Headers.Add(header);
						if (File.Exists(localPath + "index.htm")) {
							upload.Send(localPath + "index.htm");
						} else if (File.Exists(localPath + "index.html")) {
							upload.Send(localPath + "index.html");
						} else {
							if (browsable) {
								// TODO: browse folder
							} else {
								throw new HttpException(HttpStatusCode.Forbidden, "Directory Listing Denied");
							}
						}
					} else {
						if (File.Exists(localPath)) {
							FileUpload upload = new FileUpload(request);
							upload.Headers.Add(header);
							upload.Send(localPath);
						} else if (Directory.Exists(localPath)) {
							response = new HttpResponse(request);
							response.StatusCode = HttpStatusCode.Moved;
							response.MergeHeaders(header);
							response.Headers.Set("Location", request.File + "/");
							response.SendHeaders(request.Connection.Writer, true);
						} else {
							throw new HttpException(HttpStatusCode.NotFound);
						}
					}
				}
			}
			if (response != null && !response.KeepAlive)
				request.Connection.Close();
		}
		
		/// <summary>
		/// 
		/// </summary>
		public int GetPriority(HttpRequest request) {
			if (regex) {
				// TODO: Regex mode
				throw new Exception("Implement regex mode");
			} else {
				if (caseSensitive) {
					if (request.File == alias)
						return 50;
					else if (request.File.StartsWith(aliasPath))
						return TestExist(request.File) ? 50 : int.MinValue;
					else
						return int.MinValue;
				} else {
					string low = request.File.ToLower();
					if (low == alias.ToLower())
						return 50;
					else if (low.StartsWith(aliasPath.ToLower()))
						return TestExist(request.File) ? 50 : int.MinValue;
					else
						return int.MinValue;
				}
			}
		}
		
		bool TestExist(string file)
		{
			string relPath = file.Substring(aliasPath.Length);
			string localPath = Path.Combine(folder, relPath);
			if (localPath.EndsWith("/"))
				return Directory.Exists(localPath.Substring(0, localPath.Length - 1));
			else
				return File.Exists(localPath) || Directory.Exists(localPath);
		}
	}
}
