// P2PLib - Copyright (C) 2004-2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Net;

namespace Grunwald.P2PLib.Server.IPDetectors
{
	/// <summary>
	/// The MultisourceDetector has a collection of other IPDetectors.
	/// On each refresh this class chooses one detector and passes the call to
	/// the other detector.
	/// </summary>
	public class MultisourceDetector : IPDetector
	{
		IPDetectorCollection ipDetectors = new IPDetectorCollection();
		/// <summary>
		/// The collection of child-detectors to use.
		/// </summary>
		public IPDetectorCollection IPDetectors {
			get { return ipDetectors; }
		}
		
		#region Refresh
		
		int nextDetector = 0;
		
		/// <summary>
		/// Re-Detects the IP without setting the time properties.
		/// </summary>
		/// <returns>Returns true if the refresh was successfull.</returns>
		protected override bool DoRefresh()
		{
			if (lastDetector != null) {
				// reuse successfull detectors
				if (lastDetector.LastRefreshWasSuccessful)
					return lastDetector.Refresh();
			}
			if (nextDetector >= IPDetectors.Count)
				nextDetector = 0;
			IPDetector d = IPDetectors[nextDetector++];
			bool success = d.Refresh();
			if (success)
				lastDetector = d;
			return success;
		}
		
		#endregion
		
		#region IP-Address
		IPDetector lastDetector;
		
		/// <summary>
		/// Gets the detector that was most recently successfull
		/// </summary>
		public IPDetector LastDetector {
			get { return lastDetector; }
		}
		
		/// <summary>
		/// Gets the IPAddress that was detected last.
		/// </summary>
		public override IPAddress IPAddress {
			get { return lastDetector.IPAddress; }
		}
		
		#endregion
	}
}
