// P2PLib - Copyright (C) 2004-2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Globalization;
using System.Xml;

namespace Grunwald.P2PLib.Server.IPDetectors
{
	/// <summary>
	/// A address to a string value inside a xml document.
	/// </summary>
	public class XmlAddress
	{
		string xmlPath;
		
		/// <summary>
		/// Gets/Sets the XPath in the xml document to the node that contains the ip.
		/// </summary>
		public string XmlPath {
			get { return xmlPath; }
			set {
				if (value == null) throw new ArgumentNullException();
				xmlPath = value;
			}
		}
		
		string attribute;
		
		/// <summary>
		/// Gets/Sets the attribute in the xml node defined by <see cref="XmlPath"/>
		/// </summary>
		public string Attribute {
			get { return attribute; }
			set { attribute = value; }
		}
		
		/// <summary>
		/// Creates a new XML Address pointing to the value of an node.
		/// </summary>
		public XmlAddress(string xmlPath)
		{
			XmlPath = xmlPath;
		}
		
		/// <summary>
		/// Creates a new XML Address pointing to the attribute of an node.
		/// </summary>
		public XmlAddress(string xmlPath, string attribute)
		{
			XmlPath = xmlPath;
			Attribute = attribute;
		}
		
		/// <summary>
		/// Gets the value in the xml document.
		/// </summary>
		public string GetValue(XmlDocument document)
		{
			if (document == null)
				throw new ArgumentNullException();
			XmlNode node = document.SelectSingleNode(XmlPath);
			if (node == null)
				return null;
			if (node.NodeType != XmlNodeType.Element)
				return null;
			XmlElement e = (XmlElement) node;
			if (Attribute == null || Attribute.Length == 0)
				return e.InnerText;
			else
				return e.GetAttribute(Attribute);
		}
		
		/// <summary>
		/// Gets the string representation of this XmlAddress.
		/// </summary>
		public override string ToString()
		{
			return String.Format(CultureInfo.InvariantCulture,
			                     "[XmlAddress: XmlPath = {0}, Attribute = {1}]",
			                     XmlPath,
			                     Attribute);
		}
	}
}
