// P2PLib - Copyright (C) 2004-2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.IO;
using System.Windows.Forms;
using System.Text;
using log4net.Config;
using Grunwald.Gui;
using Grunwald.Gui.Settings;
using Grunwald.Gui.Controls;
using Grunwald.P2PLib;
using Grunwald.P2PLib.Server;
using Grunwald.P2PLib.Server.Http;
using Grunwald.P2PLib.Network;
using Grunwald.P2PLib.DGNetwork;

namespace DefaultNamespace
{
	/// <summary>
	/// Description of MainForm.
	/// </summary>
	public class MainForm : System.Windows.Forms.Form
	{
		private System.ComponentModel.IContainer components;
		private Grunwald.Gui.Controls.AutoSizedButton connectButton;
		private Grunwald.Gui.Controls.AutoSizedButton disconnectButton;
		private Grunwald.Gui.Controls.DockedControlColumn dockedControlColumn1;
		private Grunwald.Gui.Controls.AutoSizedButton closeButton;
		private Grunwald.Gui.Controls.ControlRow controlRow;
		private System.Windows.Forms.TextBox textBox;
		private System.Windows.Forms.Timer timer1;
		private Grunwald.Gui.Controls.AutoSizedButton sendButton;
		private Grunwald.Gui.Controls.AutoSizedButton newInstanceButton;
		
		public MainForm()
		{
			//
			// The InitializeComponent() call is required for Windows Forms designer support.
			//
			InitializeComponent();
			
			string[] texte = new string[] {"NOOP", "ROUT", "CLRT", "QUIT", "SEND", "ANSW"};
			foreach(string text in texte) {
				string txt = NetworkBitConverter.ToUInt32(Encoding.ASCII.GetBytes(text),0).ToString("x08");
				Console.WriteLine("{0} : 0x{1}", text, txt);
			}
			controlRow.ActivateArrange();
			server.RequestHandlers.Add(requestHandler);
			try {
				server.Port = 81;
				server.Start();
			} catch (System.Net.Sockets.SocketException) {
				server.Port = new Random().Next(48000, 60000);
				server.Start();
			}
			
			user1 = new ReceivingUser(ident1);
			requestHandler.ReceivingUsers.Add(user1);
			net = new DgnP2PNetwork("dgn", user1);
			ipServer = new DgnIPServer("http://www.danielgrunwald.de/network/ipserver.php", server);
			ipServer.ReceivingUser = user1;
			net.IPServers.Add(ipServer);
			net.Receive += new PacketEventHandler(PacketReceived);
		}
		
		DgnIPServer ipServer;
		DgnRequestHandler requestHandler = new DgnRequestHandler();
		Identity ident1 = new Identity(512); // 512 bit encryption
		
		HttpServer server = new HttpServer();
		ReceivingUser user1;
		
		private void PacketReceived(object sender, PacketEventArgs e)
		{
			PacketHeader header = e.Packet.Header;
			Console.WriteLine("Received packet {0} from user {1}", header.PacketID, header.Source);
		}
		
		[STAThread]
		public static void Main(string[] args)
		{
			try {
				FileInfo file = new FileInfo("./Log4Net.Config.xml");
				if (!file.Exists)
					file = new FileInfo("./../SampleClient/Log4Net.Config.xml");
				if (!file.Exists)
					file = new FileInfo("./../../SampleClient/Log4Net.Config.xml");
				if (!file.Exists)
					throw new FileNotFoundException("Log4Net.Config.xml not found", "Log4Net.Config.xml");
				XmlConfigurator.Configure(file);
				Application.ThreadException += new System.Threading.ThreadExceptionEventHandler(ThreadException);
				try {
					Application.Run(new MainForm());
				} catch (Exception ex) {
					ErrorReporter e = new ErrorReporter();
					e.ReportError(ex);
				}
			} catch (Exception ex) {
				MessageBox.Show(ex.ToString());
			}
		}
		
		static void ThreadException(object sender, System.Threading.ThreadExceptionEventArgs e)
		{
			try {
				ErrorReporter er = new ErrorReporter();
				er.ReportError(e.Exception);
			} catch (Exception ex) {
				MessageBox.Show(ex.ToString());
			}
		}
		
		#region Windows Forms Designer generated code
		/// <summary>
		/// This method is required for Windows Forms designer support.
		/// Do not change the method contents inside the source code editor. The Forms designer might
		/// not be able to load this method if it was changed manually.
		/// </summary>
		private void InitializeComponent() {
			this.components = new System.ComponentModel.Container();
			this.newInstanceButton = new Grunwald.Gui.Controls.AutoSizedButton();
			this.sendButton = new Grunwald.Gui.Controls.AutoSizedButton();
			this.timer1 = new System.Windows.Forms.Timer(this.components);
			this.textBox = new System.Windows.Forms.TextBox();
			this.controlRow = new Grunwald.Gui.Controls.ControlRow();
			this.closeButton = new Grunwald.Gui.Controls.AutoSizedButton();
			this.dockedControlColumn1 = new Grunwald.Gui.Controls.DockedControlColumn();
			this.disconnectButton = new Grunwald.Gui.Controls.AutoSizedButton();
			this.connectButton = new Grunwald.Gui.Controls.AutoSizedButton();
			this.controlRow.SuspendLayout();
			this.dockedControlColumn1.SuspendLayout();
			this.SuspendLayout();
			// 
			// newInstanceButton
			// 
			this.newInstanceButton.AutoSize = true;
			this.newInstanceButton.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.newInstanceButton.Location = new System.Drawing.Point(8, 8);
			this.newInstanceButton.Name = "newInstanceButton";
			this.newInstanceButton.Size = new System.Drawing.Size(80, 24);
			this.newInstanceButton.TabIndex = 3;
			this.newInstanceButton.Text = "new instance";
			this.newInstanceButton.Click += new System.EventHandler(this.NewInstanceButtonClick);
			// 
			// sendButton
			// 
			this.sendButton.AutoSize = false;
			this.sendButton.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.sendButton.Location = new System.Drawing.Point(96, 8);
			this.sendButton.Name = "sendButton";
			this.sendButton.Size = new System.Drawing.Size(48, 24);
			this.sendButton.TabIndex = 4;
			this.sendButton.Text = "Send";
			this.sendButton.Click += new System.EventHandler(this.SendButtonClick);
			// 
			// timer1
			// 
			this.timer1.Enabled = true;
			this.timer1.Tick += new System.EventHandler(this.Timer1Tick);
			// 
			// textBox
			// 
			this.textBox.Dock = System.Windows.Forms.DockStyle.Bottom;
			this.textBox.Location = new System.Drawing.Point(0, 174);
			this.textBox.Multiline = true;
			this.textBox.Name = "textBox";
			this.textBox.ScrollBars = System.Windows.Forms.ScrollBars.Both;
			this.textBox.Size = new System.Drawing.Size(416, 168);
			this.textBox.TabIndex = 0;
			this.textBox.Text = "";
			// 
			// controlRow
			// 
			this.controlRow.AutoScroll = true;
			this.controlRow.ContentAlignment = System.Drawing.ContentAlignment.MiddleRight;
			this.controlRow.Controls.Add(this.sendButton);
			this.controlRow.Controls.Add(this.newInstanceButton);
			this.controlRow.Controls.Add(this.connectButton);
			this.controlRow.Controls.Add(this.disconnectButton);
			this.controlRow.Controls.Add(this.closeButton);
			this.controlRow.Direction = Grunwald.Gui.Controls.Direction.LeftToRight;
			this.controlRow.Dock = System.Windows.Forms.DockStyle.Bottom;
			this.controlRow.Location = new System.Drawing.Point(0, 342);
			this.controlRow.Name = "controlRow";
			this.controlRow.Size = new System.Drawing.Size(416, 40);
			this.controlRow.TabIndex = 1;
			// 
			// closeButton
			// 
			this.closeButton.AutoSize = true;
			this.closeButton.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.closeButton.Location = new System.Drawing.Point(328, 8);
			this.closeButton.Name = "closeButton";
			this.closeButton.Size = new System.Drawing.Size(75, 24);
			this.closeButton.TabIndex = 0;
			this.closeButton.Text = "Close";
			this.closeButton.Click += new System.EventHandler(this.CloseButtonClick);
			// 
			// dockedControlColumn1
			// 
			this.dockedControlColumn1.Controls.Add(this.textBox);
			this.dockedControlColumn1.Dock = System.Windows.Forms.DockStyle.Fill;
			this.dockedControlColumn1.Location = new System.Drawing.Point(0, 0);
			this.dockedControlColumn1.Name = "dockedControlColumn1";
			this.dockedControlColumn1.Size = new System.Drawing.Size(416, 342);
			this.dockedControlColumn1.TabIndex = 2;
			// 
			// disconnectButton
			// 
			this.disconnectButton.AutoSize = true;
			this.disconnectButton.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.disconnectButton.Location = new System.Drawing.Point(248, 8);
			this.disconnectButton.Name = "disconnectButton";
			this.disconnectButton.Size = new System.Drawing.Size(75, 24);
			this.disconnectButton.TabIndex = 1;
			this.disconnectButton.Text = "Disconnect";
			this.disconnectButton.Click += new System.EventHandler(this.DisconnectButtonClick);
			// 
			// connectButton
			// 
			this.connectButton.AutoSize = true;
			this.connectButton.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.connectButton.Location = new System.Drawing.Point(152, 8);
			this.connectButton.Name = "connectButton";
			this.connectButton.Size = new System.Drawing.Size(88, 24);
			this.connectButton.TabIndex = 2;
			this.connectButton.Text = "connectButton";
			this.connectButton.Click += new System.EventHandler(this.ConnectButtonClick);
			// 
			// MainForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(416, 382);
			this.Controls.Add(this.dockedControlColumn1);
			this.Controls.Add(this.controlRow);
			this.Name = "MainForm";
			this.Text = "SampleClient";
			this.controlRow.ResumeLayout(false);
			this.dockedControlColumn1.ResumeLayout(false);
			this.ResumeLayout(false);
		}
		#endregion
		
		DgnP2PNetwork net;
		
		void CloseButtonClick(object sender, System.EventArgs e)
		{
			Close();
		}
		
		void DisconnectButtonClick(object sender, System.EventArgs e)
		{
			net.Disconnect();
		}
		
		int tick = 0;
		void Timer1Tick(object sender, System.EventArgs e)
		{
			if (net == null) {
				textBox.Text = "net == null";
				return;
			}
			string txt = net.RoutingStrategy.ToString();
			txt += "\n\nConnections:\n";
			foreach(Connection c in net.Connections) {
				txt += c.ToString() + "\n";
			}
			textBox.Text = txt.Replace("\n", "\r\n");
			tick += 1;
			if (ipServer.UserID > 0)
				this.Text = "SampleClient - #" + ipServer.UserID;
			if ((tick % 15) == 0) {
				MethodInvoker d = new MethodInvoker(net.Work);
				d.BeginInvoke(null, null);
			}
		}
		
		void ConnectButtonClick(object sender, System.EventArgs e)
		{
			net.Connect();
		}
		
		void NewInstanceButtonClick(object sender, System.EventArgs e)
		{
			System.Diagnostics.Process.Start("SampleClient.exe");
		}
		
		string to;
		
		void SendButtonClick(object sender, System.EventArgs e)
		{
			to = InputBox.Show("Send packet to:", "Send packet", to);
			if (to == null || to.Length == 0)
				return;
			string[] targets = to.Split(',');
			int[] dest = new int[targets.Length];
			for (int i = 0; i < dest.Length; ++i) {
				dest[i] = int.Parse(targets[i]);
			}
			Packet p = new Packet();
			p.Header.Destination = dest;
			p.Header.Source = net.UserID;
			net.SendPacket(p);
		}
	}
}
