// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Net;
using System.Windows.Forms;
using Grunwald.Gui.Components;

namespace TCPSniffer
{
	public class ConnectionView : System.Windows.Forms.UserControl, IPacketListener
	{
		private System.ComponentModel.IContainer components;
		private System.Windows.Forms.Panel middlePanel;
		private System.Windows.Forms.Panel bottomPanel;
		private System.Windows.Forms.ColumnHeader columnRemoteEP;
		private System.Windows.Forms.ListView connectionList;
		private System.Windows.Forms.Panel innerBottomPanel;
		private System.Windows.Forms.ColumnHeader columnTime;
		private System.Windows.Forms.Timer timer;
		private System.Windows.Forms.ColumnHeader columnStatus;
		private Grunwald.Gui.Controls.ExtendedRichTextBox textBox;
		private System.Windows.Forms.ColumnHeader columnLocalEP;
		private System.Windows.Forms.Splitter splitter;
		private System.Windows.Forms.ColumnHeader columnPackets;
		
		#region Analyzers
		#if WithoutDGN
		#if WithoutOscar
		ConnectionAnalyzer[] analyzers = { };
		#else
		ConnectionAnalyzer[] analyzers = { new OscarAnalyzer() };
		#endif
		#else
		#if WithoutOscar
		ConnectionAnalyzer[] analyzers = { new DGNetworkAnalyzer() };
		#else
		ConnectionAnalyzer[] analyzers = { new DGNetworkAnalyzer(), new OscarAnalyzer() };
		#endif
		#endif
		#endregion
		
		#region Constructor
		public ConnectionView()
		{
			//
			// The InitializeComponent() call is required for Windows Forms designer support.
			//
			InitializeComponent();
			
			if (!MainForm.Is2000)
				connectionList.SmallImageList = ProcessCache.Instance.ImageList;
			Corrections.MakeProportional(connectionList);
			Font f = new Font("Courier New", 8.25F, FontStyle.Regular);
			textBox.Font = f;
			foreach (Control analyzer in analyzers) {
				analyzer.Dock = DockStyle.Fill;
				innerBottomPanel.Controls.Add(analyzer);
			}
			ClearBottom();
		}
		#endregion
		
		#region Windows Forms Designer generated code
		/// <summary>
		/// This method is required for Windows Forms designer support.
		/// Do not change the method contents inside the source code editor. The Forms designer might
		/// not be able to load this method if it was changed manually.
		/// </summary>
		private void InitializeComponent() {
			this.components = new System.ComponentModel.Container();
			this.columnPackets = new System.Windows.Forms.ColumnHeader();
			this.splitter = new System.Windows.Forms.Splitter();
			this.columnLocalEP = new System.Windows.Forms.ColumnHeader();
			this.textBox = new Grunwald.Gui.Controls.ExtendedRichTextBox();
			this.columnStatus = new System.Windows.Forms.ColumnHeader();
			this.timer = new System.Windows.Forms.Timer(this.components);
			this.columnTime = new System.Windows.Forms.ColumnHeader();
			this.innerBottomPanel = new System.Windows.Forms.Panel();
			this.connectionList = new System.Windows.Forms.ListView();
			this.columnRemoteEP = new System.Windows.Forms.ColumnHeader();
			this.bottomPanel = new System.Windows.Forms.Panel();
			this.middlePanel = new System.Windows.Forms.Panel();
			this.innerBottomPanel.SuspendLayout();
			this.bottomPanel.SuspendLayout();
			this.SuspendLayout();
			// 
			// columnPackets
			// 
			this.columnPackets.Text = "# packets";
			this.columnPackets.TextAlign = System.Windows.Forms.HorizontalAlignment.Right;
			this.columnPackets.Width = 58;
			// 
			// splitter
			// 
			this.splitter.Dock = System.Windows.Forms.DockStyle.Top;
			this.splitter.Location = new System.Drawing.Point(0, 128);
			this.splitter.MinExtra = 85;
			this.splitter.MinSize = 60;
			this.splitter.Name = "splitter";
			this.splitter.Size = new System.Drawing.Size(504, 4);
			this.splitter.TabIndex = 1;
			this.splitter.TabStop = false;
			// 
			// columnLocalEP
			// 
			this.columnLocalEP.Text = "End Point 1";
			this.columnLocalEP.Width = 125;
			// 
			// textBox
			// 
			this.textBox.DetectUrls = false;
			this.textBox.Dock = System.Windows.Forms.DockStyle.Fill;
			this.textBox.Location = new System.Drawing.Point(0, 0);
			this.textBox.Name = "textBox";
			this.textBox.ReadOnly = true;
			this.textBox.ScrollBars = System.Windows.Forms.RichTextBoxScrollBars.ForcedBoth;
			this.textBox.Size = new System.Drawing.Size(504, 220);
			this.textBox.TabIndex = 3;
			this.textBox.Text = "leftBox";
			this.textBox.WordWrap = false;
			// 
			// columnStatus
			// 
			this.columnStatus.Text = "Status";
			this.columnStatus.Width = 80;
			// 
			// timer
			// 
			this.timer.Tick += new System.EventHandler(this.TimerTick);
			// 
			// columnTime
			// 
			this.columnTime.Text = "Time created";
			this.columnTime.Width = 100;
			// 
			// innerBottomPanel
			// 
			this.innerBottomPanel.Controls.Add(this.textBox);
			this.innerBottomPanel.Dock = System.Windows.Forms.DockStyle.Fill;
			this.innerBottomPanel.Location = new System.Drawing.Point(0, 24);
			this.innerBottomPanel.Name = "innerBottomPanel";
			this.innerBottomPanel.Size = new System.Drawing.Size(504, 220);
			this.innerBottomPanel.TabIndex = 1;
			// 
			// connectionList
			// 
			this.connectionList.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
			                                     	this.columnTime,
			                                     	this.columnLocalEP,
			                                     	this.columnRemoteEP,
			                                     	this.columnStatus,
			                                     	this.columnPackets});
			this.connectionList.Dock = System.Windows.Forms.DockStyle.Top;
			this.connectionList.FullRowSelect = true;
			this.connectionList.HeaderStyle = System.Windows.Forms.ColumnHeaderStyle.Nonclickable;
			this.connectionList.HideSelection = false;
			this.connectionList.Location = new System.Drawing.Point(0, 0);
			this.connectionList.MultiSelect = false;
			this.connectionList.Name = "connectionList";
			this.connectionList.Size = new System.Drawing.Size(504, 128);
			this.connectionList.TabIndex = 0;
			this.connectionList.View = System.Windows.Forms.View.Details;
			this.connectionList.SelectedIndexChanged += new System.EventHandler(this.ConnectionListSelectedIndexChanged);
			// 
			// columnRemoteEP
			// 
			this.columnRemoteEP.Text = "End Point 2";
			this.columnRemoteEP.Width = 125;
			// 
			// bottomPanel
			// 
			this.bottomPanel.Controls.Add(this.innerBottomPanel);
			this.bottomPanel.Controls.Add(this.middlePanel);
			this.bottomPanel.Dock = System.Windows.Forms.DockStyle.Fill;
			this.bottomPanel.Location = new System.Drawing.Point(0, 132);
			this.bottomPanel.Name = "bottomPanel";
			this.bottomPanel.Size = new System.Drawing.Size(504, 244);
			this.bottomPanel.TabIndex = 2;
			this.bottomPanel.Visible = false;
			// 
			// middlePanel
			// 
			this.middlePanel.Dock = System.Windows.Forms.DockStyle.Top;
			this.middlePanel.Location = new System.Drawing.Point(0, 0);
			this.middlePanel.Name = "middlePanel";
			this.middlePanel.Size = new System.Drawing.Size(504, 24);
			this.middlePanel.TabIndex = 0;
			// 
			// ConnectionView
			// 
			this.Controls.Add(this.bottomPanel);
			this.Controls.Add(this.splitter);
			this.Controls.Add(this.connectionList);
			this.Name = "ConnectionView";
			this.Size = new System.Drawing.Size(504, 376);
			this.Load += new System.EventHandler(this.ConnectionViewLoad);
			this.innerBottomPanel.ResumeLayout(false);
			this.bottomPanel.ResumeLayout(false);
			this.ResumeLayout(false);
		}
		#endregion
		
		#region Clear/Receive
		ArrayList connections = new ArrayList();
		
		public void Clear()
		{
			connectionList.Items.Clear();
			connections.Clear();
			bottomPanel.Visible = false;
			ClearBottom();
		}
		
		private void ClearBottom()
		{
			textBox.Text = "";
			foreach (ConnectionAnalyzer analyzer in analyzers) {
				analyzer.Visible = false;
			}
			selectedConnection = null;
		}
		
		public void Receive(IPPacket p)
		{
			TcpPacket tcp = p as TcpPacket;
			if (tcp == null) return;
			foreach (Connection c in connections) {
				if (c.MatchesPacket(tcp)) {
					c.Receive(tcp);
					if (selectedConnection == c) {
						if (c.Analyzer != null)
							c.Analyzer.AnalyzePacket(tcp);
						else
							c.Append(tcp, textBox, false);
					}
					return;
				}
			}
			// unknown packet came in:
			// make new connection
			ListViewItem item = new ListViewItem(new string[] {
			                                     	tcp.Time.ToString(),
			                                     	"ep1",
			                                     	"ep2",
			                                     	"Unknown",
			                                     	"0"
			                                     }, tcp.ImageIndex);
			Connection con = new Connection(tcp, item);
			ResolveTag r1 = null;
			ResolveTag r2 = null;
			string source = DnsCache.Instance.GetName(p.SourceAddress);
			if (source == null) {
				source = p.SourceAddress.ToString();
				r1 = new ResolveTag(true, con);
			}
			if (p.SourcePort >= 0)
				source += ":" + p.SourcePort.ToString();
			string destination = DnsCache.Instance.GetName(p.DestinationAddress);
			if (destination == null) {
				destination = p.DestinationAddress.ToString();
				r2 = new ResolveTag(false, con);
			}
			if (p.DestinationPort >= 0)
				destination += ":" + p.DestinationPort.ToString();
			item.SubItems[1].Text = source;
			item.SubItems[2].Text = destination;
			connections.Add(con);
			con.Receive(tcp);
			connectionList.Items.Add(item);
			if (r1 != null)
				DnsCache.Instance.BeginResolve(p.SourceAddress, new ResolveCallback(Resolved), r1);
			if (r2 != null)
				DnsCache.Instance.BeginResolve(p.DestinationAddress, new ResolveCallback(Resolved), r2);
		}
		#endregion
		
		#region Dns Resolving
		private class ResolveTag
		{
			public bool Source;
			public Connection Connection;
			
			public ResolveTag(bool source, Connection connection)
			{
				this.Connection = connection;
				this.Source = source;
			}
		}
		
		private void Resolved(object tag, System.Net.IPAddress address, string name)
		{
			if (InvokeRequired) {
				Invoke(new ResolveCallback(Resolved),
				       new object[] { tag, address, name });
				return;
			}
			ResolveTag r = (ResolveTag)tag;
			ListViewItem item = r.Connection.Item;
			if (item.ListView == null)
				return;
			if (r.Source) {
				item.SubItems[1].Text = name + ":" + r.Connection.SourcePort.ToString();
			} else {
				item.SubItems[2].Text = name + ":" + r.Connection.DestinationPort.ToString();
			}
		}
		#endregion
		
		#region Timer
		int tick;
		private void TimerTick(object sender, System.EventArgs e)
		{
			if (++tick == 100) tick = 0;
			if (tick % 10 == 0) {
				foreach (Connection c in connections) {
					if (c.Status == Status.Active &&
					    DateTime.Now.Subtract(c.LastPacket.Time).TotalSeconds > 15)
						c.Status = Status.Inactive;
				}
			}
		}
		
		private void ConnectionViewLoad(object sender, System.EventArgs e)
		{
			timer.Enabled = true;
		}
		#endregion
		
		Connection selectedConnection;
		
		private void ConnectionListSelectedIndexChanged(object sender, EventArgs e)
		{
			ClearBottom();
			if (connectionList.SelectedIndices.Count == 0) {
				bottomPanel.Visible = false;
				return;
			}
			Connection c = (Connection)connections[connectionList.SelectedIndices[0]];
			selectedConnection = c;
			bool foundAnalyzer = false;
			if (c.Analyzer != null) {
				foundAnalyzer = true;
				c.Analyzer.Visible = true;
				c.Analyzer.BeginAnalyze(c);
			} else {
				foreach (ConnectionAnalyzer analyzer in analyzers) {
					if (analyzer.CanAnalyze(c)) {
						c.Analyzer = analyzer;
						foundAnalyzer = true;
						analyzer.Visible = true;
						analyzer.BeginAnalyze(c);
						break;
					}
				}
			}
			if (!foundAnalyzer) {
				foreach(TcpPacket p in c.Packets) {
					c.Append(p, textBox, true);
				}
			}
			textBox.Visible = !foundAnalyzer;
			if (!bottomPanel.Visible)
				bottomPanel.Visible = true;
		}
	}
	
	public enum Status {
		Unknown,
		Connecting,
		Active,
		Inactive,
		Closed,
		Cancelled
	}
}
