/*
 *   Mentalis.org Packet Monitor
 *
 *     Copyright © 2003, The KPD-Team
 *     All rights reserved.
 *     http://www.mentalis.org/
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *
 *     - Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *
 *     - Neither the name of the KPD-Team, nor the names of its contributors
 *        may be used to endorse or promote products derived from this
 *        software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 *   THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 *   INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 *   OF THE POSSIBILITY OF SUCH DAMAGE.
 */

using System;

namespace TCPSniffer
{
	/// <summary>
	/// The Network Control precedence designation is intended to be used within
	/// a network only. The actual use and control of that designation is up to
	/// each network. The Internetwork Control designation is intended for use by
	/// gateway control originators only. If the actual use of these precedence
	/// designations is of concern to a particular network, it is the responsibility
	/// of that network to control the access to, and use of, those precedence
	/// designations.
	/// </summary>
	public enum Precedence
	{
		Routine = 0,
		Priority = 1,
		Immediate = 2,
		Flash = 3,
		FlashOverride = 4,
		CRITICECP = 5,
		InternetworkControl = 6,
		NetworkControl = 7
	}
	
	/// <summary>
	/// The use of the Delay, Throughput, and Reliability indications may increase
	/// the cost (in some sense) of the service. In many networks better performance
	/// for one of these parameters is coupled with worse performance on another.
	/// </summary>
	public enum Delay
	{
		NormalDelay = 0,
		LowDelay = 1
	}
	
	/// <summary>
	/// The use of the Delay, Throughput, and Reliability indications may increase the
	/// cost (in some sense) of the service. In many networks better performance for
	/// one of these parameters is coupled with worse performance on another.
	/// </summary>
	public enum Throughput
	{
		NormalThroughput = 0,
		HighThroughput = 1
	}
	
	/// <summary>
	/// The use of the Delay, Throughput, and Reliability indications may increase the
	/// cost (in some sense) of the service. In many networks better performance for
	/// one of these parameters is coupled with worse performance on another.
	/// </summary>
	public enum Reliability
	{
		NormalReliability = 0,
		HighReliability = 1
	}
	
	/// <summary>
	/// This field indicates the next level protocol used in the data portion of the
	/// internet datagram.
	/// </summary>
	public enum Protocol
	{
		Tcp = 6,
		Udp = 17,
		Icmp = 1,
		Ggp = 3,
		Idp = 22,
		Igmp = 2,
		IP = 4,
		ND = 77,
		Pup = 12,
	}
}
