// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.ComponentModel;
using System.Windows.Forms;
using Grunwald.Gui.Controls;
using Grunwald.Gui.Settings;
using System.Xml;
using System.IO;

namespace TCPSniffer
{
	public class FilterEditor : System.Windows.Forms.UserControl
	{
		private System.ComponentModel.IContainer components;
		private System.Windows.Forms.ImageList imageList;
		private Grunwald.Gui.Settings.SettingTreeView treeView;
		private System.Windows.Forms.Timer timer;
		private System.Windows.Forms.Panel topPanel;
		private Grunwald.Gui.Controls.ControlRow controlRow;
		private Grunwald.Gui.Settings.HelpingSettingsPanel settingsPanel;
		
		public FilterEditor()
		{
			//
			// The InitializeComponent() call is required for Windows Forms designer support.
			//
			InitializeComponent();
			
			settingsPanel.EditorFactory = new SnifferEditorFactory();
			controlRow.ArrangeActive = true;
		}
		
		private void AddFilter(string text, Filter filter) {
			NewFilterIcon nfi = new NewFilterIcon(text);
			nfi.Image = filter.Image;
			nfi.Filter = filter;
			controlRow.Controls.Add(nfi);
		}
		
		#region Windows Forms Designer generated code
		/// <summary>
		/// This method is required for Windows Forms designer support.
		/// Do not change the method contents inside the source code editor. The Forms designer might
		/// not be able to load this method if it was changed manually.
		/// </summary>
		private void InitializeComponent() {
			this.components = new System.ComponentModel.Container();
			this.settingsPanel = new Grunwald.Gui.Settings.HelpingSettingsPanel();
			this.controlRow = new Grunwald.Gui.Controls.ControlRow();
			this.topPanel = new System.Windows.Forms.Panel();
			this.timer = new System.Windows.Forms.Timer(this.components);
			this.treeView = new Grunwald.Gui.Settings.SettingTreeView();
			this.imageList = new System.Windows.Forms.ImageList(this.components);
			this.topPanel.SuspendLayout();
			this.SuspendLayout();
			// 
			// settingsPanel
			// 
			this.settingsPanel.DirectEdit = true;
			this.settingsPanel.Dock = System.Windows.Forms.DockStyle.Right;
			this.settingsPanel.Location = new System.Drawing.Point(160, 32);
			this.settingsPanel.Name = "settingsPanel";
			this.settingsPanel.Size = new System.Drawing.Size(176, 296);
			this.settingsPanel.TabIndex = 3;
			// 
			// controlRow
			// 
			this.controlRow.AutoScroll = true;
			this.controlRow.ContentAlignment = System.Drawing.ContentAlignment.MiddleLeft;
			this.controlRow.Direction = Grunwald.Gui.Controls.Direction.LeftToRight;
			this.controlRow.Dock = System.Windows.Forms.DockStyle.Fill;
			this.controlRow.Location = new System.Drawing.Point(0, 0);
			this.controlRow.Name = "controlRow";
			this.controlRow.Size = new System.Drawing.Size(336, 32);
			this.controlRow.TabIndex = 0;
			// 
			// topPanel
			// 
			this.topPanel.Controls.Add(this.controlRow);
			this.topPanel.Dock = System.Windows.Forms.DockStyle.Top;
			this.topPanel.Location = new System.Drawing.Point(0, 0);
			this.topPanel.Name = "topPanel";
			this.topPanel.Size = new System.Drawing.Size(336, 32);
			this.topPanel.TabIndex = 0;
			// 
			// timer
			// 
			this.timer.Tick += new System.EventHandler(this.TimerTick);
			// 
			// treeView
			// 
			this.treeView.AllowDrop = true;
			this.treeView.CanDelete = true;
			this.treeView.Dock = System.Windows.Forms.DockStyle.Fill;
			this.treeView.HideSelection = false;
			this.treeView.ImageList = this.imageList;
			this.treeView.Indent = 22;
			this.treeView.ItemHeight = 19;
			this.treeView.Location = new System.Drawing.Point(0, 32);
			this.treeView.Name = "treeView";
			this.treeView.ShowRootLines = false;
			this.treeView.Size = new System.Drawing.Size(160, 296);
			this.treeView.TabIndex = 4;
			this.treeView.SelectedPageChanged += new System.EventHandler(this.TreeViewSelectedPageChanged);
			// 
			// imageList
			// 
			this.imageList.ImageSize = new System.Drawing.Size(16, 16);
			this.imageList.TransparentColor = System.Drawing.Color.Transparent;
			// 
			// FilterEditor
			// 
			this.Controls.Add(this.treeView);
			this.Controls.Add(this.settingsPanel);
			this.Controls.Add(this.topPanel);
			this.Name = "FilterEditor";
			this.Size = new System.Drawing.Size(336, 328);
			this.Load += new System.EventHandler(this.FilterEditorLoad);
			this.topPanel.ResumeLayout(false);
			this.ResumeLayout(false);
		}
		#endregion
		
		void FilterEditorLoad(object sender, System.EventArgs e)
		{
			try {
				AddFilter("Logical Filter", new LogicalFilter());
				AddFilter("IP Filter", new IPFilter());
				if (!MainForm.Is2000)
					AddFilter("Application", new ApplicationFilter());
				AddFilter("Protocol Filter", new ProtocolFilter());
				treeView.RootPage = Filter.RootFilter;
				treeView.ExpandAll();
				timer.Enabled = true;
			} catch (Exception ex) {
				Grunwald.Gui.ErrorReporter.ShowErrorReport(ex);
			}
		}
		
		bool setPage = false;
		DateTime lastSet;
		
		void TreeViewSelectedPageChanged(object sender, System.EventArgs e)
		{
			if (settingsPanel.Page == treeView.SelectedPage) return;
			settingsPanel.Page = null;
			lastSet = DateTime.Now;
			setPage = true;
		}
		
		void TimerTick(object sender, System.EventArgs e)
		{
			if (setPage && DateTime.Now.Subtract(lastSet).TotalMilliseconds > 60) {
				setPage = false;
				settingsPanel.Page = treeView.SelectedPage;
			}
		}
		
		public void ClearRightSide()
		{
			settingsPanel.Page = null;
			setPage = true;
		}
		
		public Grunwald.Gui.Settings.SettingTreeView TreeView {
			get {
				return treeView;
			}
		}
	}
}
