/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

using System;
using System.Runtime.InteropServices;

namespace TCPSniffer
{
	internal class iphelper
	{
		public const UInt32 StatisticsEx_dwFamily_AF_INET=2; // UInt32ernetwork: UDP, TCP, etc.
		public const UInt32 StatisticsEx_dwFamily_AF_INET6=23; // UInt32ernetwork Version 6
		
		#region GetProcessHeap
		[ DllImport( "kernel32" ,SetLastError=true)]
		private static extern IntPtr GetProcessHeap();
		#endregion
		
		#region GetUdpTable
		[DllImport("iphlpapi.dll",SetLastError=true)]
		private static extern
			UInt32 GetUdpTable(
			                   byte[] pUdpTable,//PCMIB_UDPTABLE pUdpTable,
			                   ref UInt32 pdwSize,
			                   bool bOrder
			                  );
		public static CMIB_UDPTABLE GetUdpTable()
		{
			UInt32 error_code=0;
			return iphelper.GetUdpTable(true,ref error_code);
		}
		public static CMIB_UDPTABLE GetUdpTable(bool b_verbose,ref UInt32 error_code)
		{
			UInt32 size=CMIB_UDPROW.size*100+4;// allow memory for 100 udprow (+4 for dwNumEntries)
			byte[] buffer=new byte[size];
			error_code=GetUdpTable(buffer,ref size,true);
			if (error_code==122)//not enougth memory
			{
				buffer=new byte[size];
				error_code=GetUdpTable(buffer,ref size,true);
			}
			if (error_code==0)// no error
			{
				CMIB_UDPTABLE udptable=new CMIB_UDPTABLE();
				udptable.decode(buffer);
				return udptable;
			}
			// else
			if (b_verbose)
			{
				string str_msg=API_error.GetAPIErrorMessageDescription(error_code);
				System.Windows.Forms.MessageBox.Show( str_msg,"Error",
				                                     System.Windows.Forms.MessageBoxButtons.OK,
				                                     System.Windows.Forms.MessageBoxIcon.Error);
			}
			return null;
		}
		#endregion
		
		#region GetUdpExTable
		[DllImport("iphlpapi.dll",SetLastError=true)]
		private extern static
			UInt32 AllocateAndGetUdpExTableFromStack(
			                                         ref IntPtr pUdpTable,
			                                         bool bOrder,
			                                         IntPtr heap,
			                                         UInt32 zero,
			                                         UInt32 flags
			                                        );
		public static CMIB_UDPEXTABLE GetUdpExTable()
		{
			UInt32 error_code=0;
			return iphelper.GetUdpExTable(true,ref error_code);
		}
		public static CMIB_UDPEXTABLE GetUdpExTable(bool b_verbose,ref UInt32 error_code)
		{
			// allocate a dump memory space in order to retrieve nb of connexion
			int BufferSize = 100*CMIB_UDPEXROW.size_ex+4;//NumEntries*CMIB_UDPEXROW.size_ex+4
			IntPtr lpTable = Marshal.AllocHGlobal(BufferSize);
			//getting infos
			error_code= AllocateAndGetUdpExTableFromStack(ref lpTable, true, GetProcessHeap(),0,2);
			if (error_code!=0)
			{
				if (b_verbose)
				{
					string str_msg=API_error.GetAPIErrorMessageDescription(error_code);
					System.Windows.Forms.MessageBox.Show( str_msg,"Error",
					                                     System.Windows.Forms.MessageBoxButtons.OK,
					                                     System.Windows.Forms.MessageBoxIcon.Error);
				}
				return null;
			}
			//get the number of entries in the table
			int NumEntries= (int)Marshal.ReadIntPtr(lpTable);
			int real_buffer_size=NumEntries*CMIB_UDPEXROW.size_ex+4;
			// check if memory was enougth (needed buffer size: NumEntries*MIB_UDPEXROW.size_ex +4 (for dwNumEntries))
			if (BufferSize<real_buffer_size)
			{// if it wasn't the case call function another time
				
				// free the buffer
				Marshal.FreeHGlobal(lpTable);
				// get the needed buffer size: NumEntries*MIB_UDPEXROW.size_ex +4 (for dwNumEntries)
				BufferSize = real_buffer_size;
				// Allocate memory
				lpTable = Marshal.AllocHGlobal(BufferSize);
				error_code=AllocateAndGetUdpExTableFromStack(ref lpTable,true,GetProcessHeap(),0,2);
				if (error_code!=0)
				{
					if (b_verbose)
					{
						string str_msg=API_error.GetAPIErrorMessageDescription(error_code);
						System.Windows.Forms.MessageBox.Show( str_msg,"Error",
						                                     System.Windows.Forms.MessageBoxButtons.OK,
						                                     System.Windows.Forms.MessageBoxIcon.Error);
					}
					return null;
				}
			}
			BufferSize=real_buffer_size;
			byte[] array=new byte[BufferSize];
			Marshal.Copy(lpTable,array,0,BufferSize);
			// free the buffer
			Marshal.FreeHGlobal(lpTable);
			
			CMIB_UDPEXTABLE muet = new CMIB_UDPEXTABLE();
			muet.decode(array);
			return muet;
		}
		
		#endregion
		
		#region GetTcpTable
		[DllImport("iphlpapi.dll",SetLastError=true)]
		private static extern
			UInt32 GetTcpTable(
			                   byte[] pTcpTable,//MIB_TCPTABLE pTcpTable,
			                   ref UInt32 pdwSize,
			                   bool bOrder
			                  );
		public static CMIB_TCPTABLE GetTcpTable()
		{
			UInt32 error_code=0;
			return iphelper.GetTcpTable(true,ref error_code);
		}
		public static CMIB_TCPTABLE GetTcpTable(bool b_verbose,ref UInt32 error_code)
		{
			UInt32 size=CMIB_TCPROW.size*300+4;// allow memory for 300 tcprow (+4 for dwNumEntries)
			byte[] buffer=new byte[size];
			error_code=GetTcpTable(buffer,ref size,true);
			if (error_code==122)//not enougth memory
			{
				buffer=new byte[size];
				error_code=GetTcpTable(buffer,ref size,true);
			}
			if (error_code==0)// no error
			{
				CMIB_TCPTABLE tcptable=new CMIB_TCPTABLE();
				tcptable.decode(buffer);
				return tcptable;
			}
			// else
			if (b_verbose)
			{
				string str_msg=API_error.GetAPIErrorMessageDescription(error_code);
				System.Windows.Forms.MessageBox.Show( str_msg,"Error",
				                                     System.Windows.Forms.MessageBoxButtons.OK,
				                                     System.Windows.Forms.MessageBoxIcon.Error);
			}
			return null;
		}
		#endregion
		
		#region GetTcpExTable
		[DllImport("iphlpapi.dll",SetLastError=true)]
		private extern static
			UInt32 AllocateAndGetTcpExTableFromStack(
			                                         ref IntPtr pTcpTable,
			                                         bool bOrder,
			                                         IntPtr heap,
			                                         UInt32 zero,
			                                         UInt32 flags
			                                        );
		public static CMIB_TCPEXTABLE GetTcpExTable()
		{
			UInt32 error_code=0;
			return iphelper.GetTcpExTable(true,ref error_code);
		}
		public static CMIB_TCPEXTABLE GetTcpExTable(bool b_verbose,ref UInt32 error_code)
		{
			// allocate a dump memory space in order to retrieve nb of connexion
			int BufferSize = 300*CMIB_TCPEXROW.size_ex+4;//NumEntries*CMIB_TCPEXROW.size_ex+4
			IntPtr lpTable = Marshal.AllocHGlobal(BufferSize);
			//getting infos
			error_code= AllocateAndGetTcpExTableFromStack(ref lpTable, true, GetProcessHeap(),0,2);
			if (error_code!=0)
			{
				if (b_verbose)
				{
					string str_msg=API_error.GetAPIErrorMessageDescription(error_code);
					System.Windows.Forms.MessageBox.Show( str_msg,"Error",
					                                     System.Windows.Forms.MessageBoxButtons.OK,
					                                     System.Windows.Forms.MessageBoxIcon.Error);
				}
				return null;
			}
			//get the number of entries in the table
			int NumEntries= (int)Marshal.ReadIntPtr(lpTable);
			int real_buffer_size=NumEntries*CMIB_TCPEXROW.size_ex+4;
			// check if memory was enougth (needed buffer size: NumEntries*MIB_UDPEXROW.size_ex +4 (for dwNumEntries))
			if (BufferSize<real_buffer_size)
			{
				// free the buffer
				Marshal.FreeHGlobal(lpTable);
				
				// get the needed buffer size: NumEntries*MIB_TCPEXROW.size_ex +4 (for dwNumEntries)
				BufferSize = real_buffer_size;
				// Allocate memory
				lpTable = Marshal.AllocHGlobal(BufferSize);
				
				error_code=AllocateAndGetTcpExTableFromStack(ref lpTable,true,GetProcessHeap(),0,2);
				if (error_code!=0)
				{
					if (b_verbose)
					{
						string str_msg=API_error.GetAPIErrorMessageDescription(error_code);
						System.Windows.Forms.MessageBox.Show( str_msg,"Error",
						                                     System.Windows.Forms.MessageBoxButtons.OK,
						                                     System.Windows.Forms.MessageBoxIcon.Error);
					}
					return null;
				}
			}
			BufferSize=real_buffer_size;
			byte[] array=new byte[BufferSize];
			Marshal.Copy(lpTable,array,0,BufferSize);
			// free the buffer
			Marshal.FreeHGlobal(lpTable);
			CMIB_TCPEXTABLE mtet = new CMIB_TCPEXTABLE();
			mtet.decode(array);
			return mtet;
		}
		#endregion
		
		//MSDN:"Currently, the only state to which a TCP connection can be set is MIB_TCP_STATE_DELETE_TCB"
		//return 65 - User has no sufficient privilege to execute this API successfully
		//return 87 - Specified port is not in state to be closed down.
		[DllImport("iphlpapi.dll",SetLastError=true)]
		public extern static
			UInt32 SetTcpEntry(
			                   ref MIB_TCPROW pTcpRow
			                  );
	}
	
	#region GetUdpTable class
	internal class CMIB_UDPTABLE
	{
		public UInt32 dwNumEntries=0;
		public CMIB_UDPROW[] table;
		public bool decode(byte[] array)
		{
			byte[] pb_tmp=new byte[CMIB_UDPROW.size];
			if (array==null)
				return false;
			if (array.Length<4)
				return false;
			this.dwNumEntries=System.BitConverter.ToUInt32(array,0);
			if (array.Length<this.dwNumEntries)
				return false;
			this.table=new CMIB_UDPROW[this.dwNumEntries];
			for (UInt32 cpt=0;cpt<this.dwNumEntries;cpt++)
			{
				System.Array.Copy(array,(int)(cpt*CMIB_UDPROW.size+4),pb_tmp,0,CMIB_UDPROW.size);
				this.table[cpt]=new CMIB_UDPROW();
				if (!this.table[cpt].decode(pb_tmp))
					return false;
			}
			return true;
		}
	}
	
	internal class CMIB_UDPROW
	{
		public const byte size=8;
		// in networkhost order
		public UInt32 dwLocalAddr=0;
		// in networkhost order
		public UInt32 dwLocalPort=0;
		public bool decode(byte[] array)
		{
			if (array==null)
				return false;
			if (array.Length<CMIB_UDPROW.size)
				return false;
			this.dwLocalAddr=System.BitConverter.ToUInt32(array,0);
			this.dwLocalPort=System.BitConverter.ToUInt32(array,4);
			return true;
		}
		/// <summary>
		/// get ip in standard notation
		/// </summary>
		/// <returns></returns>
		public string get_ip()
		{
			System.Net.IPAddress ipa=new System.Net.IPAddress(this.dwLocalAddr);
			string ret=ipa.ToString();
			return ret;
		}
		public UInt32 get_port()
		{
			UInt32 ret=((this.dwLocalPort>>8)&0xff) +((this.dwLocalPort&0xff)<<8);
			return ret;
		}
	}
	#endregion
	
	#region GetUdpExTable class
	internal class CMIB_UDPEXTABLE
	{
		public UInt32 dwNumEntries=0;
		public CMIB_UDPEXROW[] table;
		public bool decode(byte[] array)
		{
			byte[] pb_tmp=new byte[CMIB_UDPEXROW.size_ex];
			if (array==null)
				return false;
			if (array.Length<4)
				return false;
			this.dwNumEntries=System.BitConverter.ToUInt32(array,0);
			if (array.Length<this.dwNumEntries)
				return false;
			this.table=new CMIB_UDPEXROW[this.dwNumEntries];
			for (UInt32 cpt=0;cpt<this.dwNumEntries;cpt++)
			{
				System.Array.Copy(array,(int)(cpt*CMIB_UDPEXROW.size_ex+4),pb_tmp,0,CMIB_UDPEXROW.size_ex);
				this.table[cpt]=new CMIB_UDPEXROW();
				if (!this.table[cpt].decode_ex(pb_tmp))
					return false;
			}
			return true;
		}
	}
	
	internal class CMIB_UDPEXROW:CMIB_UDPROW
	{
		public const byte size_ex=CMIB_UDPROW.size+4;
		public int dwProcessId=0;
		public bool decode_ex(byte[] array)
		{
			if (!this.decode(array))
				return false;
			// array is not null (see decode())
			if (array.Length<CMIB_UDPEXROW.size_ex)
				return false;
			this.dwProcessId = BitConverter.ToInt32(array,CMIB_UDPROW.size);
			return true;
		}
	}
	
	#endregion
	
	#region GetTcpTable
	internal class CMIB_TCPTABLE
	{
		public UInt32 dwNumEntries=0;
		public CMIB_TCPROW[] table=null;
		public bool decode(byte[] array)
		{
			byte[] pb_tmp=new byte[CMIB_TCPROW.size];
			if (array==null)
				return false;
			if (array.Length<4)
				return false;
			this.dwNumEntries=System.BitConverter.ToUInt32(array,0);
			if (array.Length<this.dwNumEntries)
				return false;
			this.table=new CMIB_TCPROW[this.dwNumEntries];
			for (UInt32 cpt=0;cpt<this.dwNumEntries;cpt++)
			{
				System.Array.Copy(array,(int)(cpt*CMIB_TCPROW.size+4),pb_tmp,0,CMIB_TCPROW.size);
				this.table[cpt]=new CMIB_TCPROW();
				if (!this.table[cpt].decode(pb_tmp))
					return false;
			}
			return true;
		}
	}
	[StructLayout(LayoutKind.Sequential)]
	internal struct MIB_TCPROW
	{
		public UInt32 dwState;
		public UInt32 dwLocalAddr;
		public UInt32 dwLocalPort;
		public UInt32 dwRemoteAddr;
		public UInt32 dwRemotePort;
	}
	internal class CMIB_TCPROW
	{
		public const byte MIB_TCP_STATE_CLOSED = 1;
		public const byte MIB_TCP_STATE_LISTEN = 2;
		public const byte MIB_TCP_STATE_SYN_SENT = 3;
		public const byte MIB_TCP_STATE_SYN_RCVD = 4;
		public const byte MIB_TCP_STATE_ESTAB = 5;
		public const byte MIB_TCP_STATE_FIN_WAIT1 = 6;
		public const byte MIB_TCP_STATE_FIN_WAIT2 = 7;
		public const byte MIB_TCP_STATE_CLOSE_WAIT = 8;
		public const byte MIB_TCP_STATE_CLOSING = 9;
		public const byte MIB_TCP_STATE_LAST_ACK = 10;
		public const byte MIB_TCP_STATE_TIME_WAIT = 11;
		public const byte MIB_TCP_STATE_DELETE_TCB = 12;
		
		public const byte size=5*4;
		public UInt32 dwState=0;
		// in networkhost order
		public UInt32 dwLocalAddr=0;
		// in networkhost order
		public UInt32 dwLocalPort=0;
		// in networkhost order
		public UInt32 dwRemoteAddr=0;
		// in networkhost order
		public UInt32 dwRemotePort=0;
		public bool decode(byte[] array)
		{
			if (array==null)
				return false;
			if (array.Length<CMIB_TCPROW.size)
				return false;
			this.dwState=System.BitConverter.ToUInt32(array,0);
			this.dwLocalAddr=System.BitConverter.ToUInt32(array,4);
			this.dwLocalPort=System.BitConverter.ToUInt32(array,8);
			this.dwRemoteAddr=System.BitConverter.ToUInt32(array,12);
			if (this.dwState==CMIB_TCPROW.MIB_TCP_STATE_LISTEN)
				this.dwRemotePort=0;
			else
				this.dwRemotePort=System.BitConverter.ToUInt32(array,16);
			return true;
		}
		public System.Net.IPAddress get_localip()
		{
			return new System.Net.IPAddress(this.dwLocalAddr);
		}
		public System.Net.IPAddress get_remoteip()
		{
			return new System.Net.IPAddress(this.dwRemoteAddr);
		}
		public UInt32 get_local_port()
		{
			UInt32 ret=((this.dwLocalPort>>8)&0xff) +((this.dwLocalPort&0xff)<<8);
			return ret;
		}
		public UInt32 get_remote_port()
		{
			UInt32 ret=((this.dwRemotePort>>8)&0xff) +((this.dwRemotePort&0xff)<<8);
			return ret;
		}
		public string get_state()
		{
			string ret="Unknown";
			switch (this.dwState)
			{
					case MIB_TCP_STATE_CLOSED: ret = "CLOSED"; break;
					case MIB_TCP_STATE_LISTEN: ret = "LISTENING"; break;
					case MIB_TCP_STATE_SYN_SENT: ret = "SYN_SENT"; break;
					case MIB_TCP_STATE_SYN_RCVD: ret = "SYN_RCVD"; break;
					case MIB_TCP_STATE_ESTAB: ret = "ESTABLISHED"; break;
					case MIB_TCP_STATE_FIN_WAIT1: ret = "FIN_WAIT1"; break;
					case MIB_TCP_STATE_FIN_WAIT2: ret = "FIN_WAIT2"; break;
					case MIB_TCP_STATE_CLOSE_WAIT: ret = "CLOSE_WAIT"; break;
					case MIB_TCP_STATE_CLOSING: ret = "CLOSING"; break;
					case MIB_TCP_STATE_LAST_ACK: ret = "LAST_ACK"; break;
					case MIB_TCP_STATE_TIME_WAIT: ret = "TIME_WAIT"; break;
					case MIB_TCP_STATE_DELETE_TCB: ret = "DELETE_TCB"; break;
			}
			return ret;
		}
		/// <summary>
		/// used for SetTcpEntry
		/// </summary>
		/// <returns></returns>
		public MIB_TCPROW get_MIB_TCPROW_struct()
		{
			MIB_TCPROW ret=new MIB_TCPROW();
			ret.dwLocalAddr=this.dwLocalAddr;
			ret.dwLocalPort=this.dwLocalPort;
			ret.dwRemoteAddr=this.dwRemoteAddr;
			ret.dwRemotePort=this.dwRemotePort;
			ret.dwState=this.dwState;
			return ret;
		}
	}
	#endregion
	
	#region GetTcpExTable class
	
	internal class CMIB_TCPEXTABLE
	{
		public UInt32 dwNumEntries=0;
		public CMIB_TCPEXROW[] table;
		public bool decode(byte[] array)
		{
			byte[] pb_tmp=new byte[CMIB_TCPEXROW.size_ex];
			if (array==null)
				return false;
			if (array.Length<4)
				return false;
			this.dwNumEntries = BitConverter.ToUInt32(array,0);
			if (array.Length<this.dwNumEntries)
				return false;
			this.table=new CMIB_TCPEXROW[this.dwNumEntries];
			for (UInt32 cpt=0;cpt<this.dwNumEntries;cpt++)
			{
				System.Array.Copy(array,(int)(cpt*CMIB_TCPEXROW.size_ex+4),pb_tmp,0,CMIB_TCPEXROW.size_ex);
				this.table[cpt]=new CMIB_TCPEXROW();
				if (!this.table[cpt].decode_ex(pb_tmp))
					return false;
			}
			return true;
		}
	}
	
	internal class CMIB_TCPEXROW:CMIB_TCPROW
	{
		public const byte size_ex=CMIB_TCPROW.size+4;
		public int dwProcessId=0;
		public bool decode_ex(byte[] array)
		{
			if (!this.decode(array))
				return false;
			// array is not null (see decode())
			if (array.Length<CMIB_TCPEXROW.size_ex)
				return false;
			this.dwProcessId = BitConverter.ToInt32(array,CMIB_TCPEXROW.size);
			return true;
		}
	}
	#endregion
	
	/// <summary>
	/// Summary description for API_error.
	/// </summary>
	internal class API_error
	{
		private const int FORMAT_MESSAGE_FROM_SYSTEM  = 0x00001000;
		[ DllImport( "kernel32" ,SetLastError=true)]
		private static extern UInt32 FormatMessage(UInt32 flags, IntPtr source, UInt32 messageId,
		                                           UInt32 languageId, System.Text.StringBuilder buffer, UInt32 size, IntPtr arguments );
		
		
		public static string GetAPIErrorMessageDescription(UInt32 ApiErrNumber )
		{
			System.Text.StringBuilder sError = new System.Text.StringBuilder(512);
			UInt32 lErrorMessageLength;
			lErrorMessageLength = FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM,(IntPtr)0, ApiErrNumber, 0, sError, (UInt32)sError.Capacity, (IntPtr)0) ;
			
			if(lErrorMessageLength > 0)
			{
				string strgError = sError.ToString();
				strgError=strgError.Substring(0,strgError.Length-2);
				return strgError+" ("+ApiErrNumber.ToString()+")";
			}
			return "none";
			
		}
		[DllImport("kernel32",EntryPoint="GetLastError",SetLastError=true)]
		public static extern uint GetLastError();
	}
}
