// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Drawing;
using System.ComponentModel;
using System.Windows.Forms;
using Grunwald.Gui.Controls;
using Grunwald.Gui.Settings;

namespace TCPSniffer
{
	public class NewFilterIcon : System.Windows.Forms.UserControl
	{
		private System.Windows.Forms.Label label;
		private System.Windows.Forms.PictureBox pictureBox;
		
		public NewFilterIcon()
		{
			InitializeComponent();
		}
		
		public NewFilterIcon(string text) : this()
		{
			label.Text = text;
			using (Graphics g = CreateGraphics()) {
				SizeF size = g.MeasureString(text, label.Font);
				this.Width = (int)size.Width + 1 + label.Left;
			}
		}
		
		public Image Image {
			get { return pictureBox.Image; }
			set { pictureBox.Image = value; }
		}
		
		[Browsable(true),
		 Description("The text shown on the icon")]
		public string Caption {
			get { return label.Text; }
			set { label.Text = value; }
		}
		
		#region Windows Forms Designer generated code
		/// <summary>
		/// This method is required for Windows Forms designer support.
		/// Do not change the method contents inside the source code editor. The Forms designer might
		/// not be able to load this method if it was changed manually.
		/// </summary>
		private void InitializeComponent() {
			this.pictureBox = new System.Windows.Forms.PictureBox();
			this.label = new System.Windows.Forms.Label();
			this.SuspendLayout();
			// 
			// pictureBox
			// 
			this.pictureBox.Location = new System.Drawing.Point(4, 4);
			this.pictureBox.Name = "pictureBox";
			this.pictureBox.Size = new System.Drawing.Size(16, 16);
			this.pictureBox.TabIndex = 1;
			this.pictureBox.TabStop = false;
			this.pictureBox.MouseDown += new System.Windows.Forms.MouseEventHandler(this.NewFilterIconMouseDown);
			// 
			// label
			// 
			this.label.Location = new System.Drawing.Point(24, 4);
			this.label.Name = "label";
			this.label.Size = new System.Drawing.Size(80, 16);
			this.label.TabIndex = 0;
			this.label.Text = "Text";
			this.label.MouseDown += new System.Windows.Forms.MouseEventHandler(this.NewFilterIconMouseDown);
			// 
			// NewFilterIcon
			// 
			this.Controls.Add(this.pictureBox);
			this.Controls.Add(this.label);
			this.Name = "NewFilterIcon";
			this.Size = new System.Drawing.Size(104, 24);
			this.MouseDown += new System.Windows.Forms.MouseEventHandler(this.NewFilterIconMouseDown);
			this.ResumeLayout(false);
		}
		#endregion
		
		TreeNode node = new TreeNode();
		Filter filter;
		
		[Browsable(false)]
		public Filter Filter {
			get { return filter; }
			set {
				if (filter == value) return;
				filter = value;
				if (value == null) {
					node.Tag = null;
				} else {
					node.Tag = new PageTreeTag(value, null);
					node.Text = value.Title;
				}
			}
		}
		
		void NewFilterIconMouseDown(object sender, MouseEventArgs e)
		{
			RearrangeTreeView.StartDrag(this, node, DragDropEffects.Copy);
		}
	}
}
