// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using Grunwald.Gui.Components;

namespace TCPSniffer
{
	public class TcpTable : System.Windows.Forms.UserControl
	{
		private System.Windows.Forms.CheckBox showListening;
		private Grunwald.Gui.Controls.AutoSizedButton refreshButton;
		private System.Windows.Forms.CheckBox showUDP;
		private System.Windows.Forms.Panel panel;
		private System.Windows.Forms.ListView list;
		
		public TcpTable()
		{
			if (!DesignMode && MainForm.Is2000) {
				Label lbl = new Label();
				lbl.Dock = DockStyle.Fill;
				lbl.Text = "The extended IP table is only available on Windows XP or higher.";
				this.Controls.Add(lbl);
				return;
			}
			//
			// The InitializeComponent() call is required for Windows Forms designer support.
			//
			InitializeComponent();
			
			list.SmallImageList = ProcessCache.Instance.ImageList;
			Corrections.MakeProportional(list);
			SetColumns("Process Name", "Process Id", "Local Address", "Remote Address", "State", "Process path");
			list.Columns[1].Width /= 2;
			list.Columns[5].Width += list.Columns[1].Width;
		}
		
		private void SetColumns(params string[] titles)
		{
			int size = list.Width - 10;
			size /= titles.Length;
			for (int i = 0; i < titles.Length; i++) {
				list.Columns.Add(titles[i], size, HorizontalAlignment.Left);
			}
		}
		
		#region Windows Forms Designer generated code
		/// <summary>
		/// This method is required for Windows Forms designer support.
		/// Do not change the method contents inside the source code editor. The Forms designer might
		/// not be able to load this method if it was changed manually.
		/// </summary>
		private void InitializeComponent() {
			this.list = new System.Windows.Forms.ListView();
			this.panel = new System.Windows.Forms.Panel();
			this.showUDP = new System.Windows.Forms.CheckBox();
			this.refreshButton = new Grunwald.Gui.Controls.AutoSizedButton();
			this.showListening = new System.Windows.Forms.CheckBox();
			this.panel.SuspendLayout();
			this.SuspendLayout();
			// 
			// list
			// 
			this.list.Dock = System.Windows.Forms.DockStyle.Fill;
			this.list.FullRowSelect = true;
			this.list.GridLines = true;
			this.list.Location = new System.Drawing.Point(0, 32);
			this.list.Name = "list";
			this.list.Size = new System.Drawing.Size(292, 234);
			this.list.TabIndex = 2;
			this.list.View = System.Windows.Forms.View.Details;
			// 
			// panel
			// 
			this.panel.Controls.Add(this.showUDP);
			this.panel.Controls.Add(this.showListening);
			this.panel.Controls.Add(this.refreshButton);
			this.panel.Dock = System.Windows.Forms.DockStyle.Top;
			this.panel.Location = new System.Drawing.Point(0, 0);
			this.panel.Name = "panel";
			this.panel.Size = new System.Drawing.Size(292, 32);
			this.panel.TabIndex = 0;
			// 
			// showUDP
			// 
			this.showUDP.Checked = true;
			this.showUDP.CheckState = System.Windows.Forms.CheckState.Checked;
			this.showUDP.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.showUDP.Location = new System.Drawing.Point(112, 8);
			this.showUDP.Name = "showUDP";
			this.showUDP.Size = new System.Drawing.Size(80, 16);
			this.showUDP.TabIndex = 2;
			this.showUDP.Text = "Show UDP";
			this.showUDP.CheckedChanged += new System.EventHandler(this.RefreshButtonClick);
			// 
			// refreshButton
			// 
			this.refreshButton.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.refreshButton.Location = new System.Drawing.Point(208, 4);
			this.refreshButton.Name = "refreshButton";
			this.refreshButton.Size = new System.Drawing.Size(75, 24);
			this.refreshButton.TabIndex = 0;
			this.refreshButton.Text = "Refresh";
			this.refreshButton.Click += new System.EventHandler(this.RefreshButtonClick);
			// 
			// showListening
			// 
			this.showListening.Checked = true;
			this.showListening.CheckState = System.Windows.Forms.CheckState.Checked;
			this.showListening.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.showListening.Location = new System.Drawing.Point(8, 8);
			this.showListening.Name = "showListening";
			this.showListening.Size = new System.Drawing.Size(104, 16);
			this.showListening.TabIndex = 1;
			this.showListening.Text = "Show Listening";
			this.showListening.CheckedChanged += new System.EventHandler(this.RefreshButtonClick);
			// 
			// TcpTable
			// 
			this.Controls.Add(this.list);
			this.Controls.Add(this.panel);
			this.Name = "TcpTable";
			this.Size = new System.Drawing.Size(292, 266);
			this.Load += new System.EventHandler(this.TcpTableLoad);
			this.panel.ResumeLayout(false);
			this.ResumeLayout(false);
		}
		#endregion
		
		void RefreshButtonClick(object sender, System.EventArgs e)
		{
			CMIB_TCPEXTABLE table = iphelper.GetTcpExTable();
			CMIB_UDPEXTABLE table2 = null;
			if (showUDP.Checked)
				table2 = iphelper.GetUdpExTable();
			list.BeginUpdate();
			list.Items.Clear();
			ArrayList rows = new ArrayList();
			if (table != null) {
				for (int i = 0; i < table.dwNumEntries; i++) {
					CMIB_TCPEXROW entry = table.table[i];
					if (!showListening.Checked && entry.dwState == CMIB_TCPROW.MIB_TCP_STATE_LISTEN)
						continue;
					int time = Environment.TickCount;
					ProcessInfo p = ProcessCache.Instance.GetProcessInfo(entry.dwProcessId);
					rows.Add(new string[] {
					         	p.Name,
					         	entry.dwProcessId.ToString(),
					         	entry.get_localip() + ":" + entry.get_local_port().ToString(),
					         	entry.get_remoteip() + ":" + entry.get_remote_port().ToString(),
					         	entry.get_state(),
					         	p.Filename
					         });
				}
			}
			if (table2 != null) {
				for (int i = 0; i < table2.dwNumEntries; i++) {
					CMIB_UDPEXROW entry = table2.table[i];
					int time = Environment.TickCount;
					ProcessInfo p = ProcessCache.Instance.GetProcessInfo(entry.dwProcessId);
					rows.Add(new string[] {
					         	p.Name,
					         	entry.dwProcessId.ToString(),
					         	entry.get_ip() + ":" + entry.get_port().ToString(),
					         	"",
					         	"",
					         	p.Filename
					         });
				}
			}
			rows.Sort(new NameComparer());
			foreach(string[] row in rows) {
				list.Items.Add(new ListViewItem(row, ProcessCache.Instance.GetIcon(row[5])));
			}
			list.EndUpdate();
		}
		
		private class NameComparer : IComparer
		{
			public int Compare(object x, object y)
			{
				string[] a = (string[])x;
				string[] b = (string[])y;
				int res;
				res = a[0].CompareTo(b[0]);
				if (res != 0) return res;
				res = int.Parse(a[1]).CompareTo(int.Parse(b[1]));
				if (res != 0) return res;
				return a[2].CompareTo(b[2]);
			}
		}
		
		void TcpTableLoad(object sender, System.EventArgs e)
		{
			refreshButton.PerformClick();
		}
	}
}
