// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Net;
using System.Windows.Forms;

namespace TCPSniffer
{
	public class UdpPacket : IPPacket
	{
		int pos;
		
		public UdpPacket(IPPacket basePacket) : base(basePacket.Raw, basePacket.Time)
		{
			pos = base.HeaderLength;
		}
		
		public UdpPacket(byte[] raw) : base(raw)
		{
			pos = base.HeaderLength;
		}
		
		public int UdpLength {
			get {
				return Raw[pos + 4] * 256 + Raw[pos + 5];
			}
		}
		
		public int UdpChecksum {
			get {
				return Raw[pos + 6] * 256 + Raw[pos + 7];
			}
		}
		
		public override void CreateInfoNodes(TreeNodeCollection c, TreeNodeFactory f)
		{
			base.CreateInfoNodes(c, f);
			TreeNode node = f.Create("UDP header", pos, 8);
			c.Add(node);
			TreeNodeCollection a = node.Nodes;
			string txt;
			txt = Format("Source port", this.SourcePort, 4);
			a.Add(f.Create(txt, pos, 2));
			txt = Format("Destination port", this.DestinationPort, 4);
			a.Add(f.Create(txt, pos + 2, 2));
			txt = Format("Length", this.UdpLength, 4);
			a.Add(f.Create(txt, pos + 4, 2));
			txt = Format("Checksum", this.UdpChecksum, 4);
			a.Add(f.Create(txt, pos + 6, 2));
		}
		
		public override int ImageIndex {
			get {
				if (MainForm.Is2000)
					return base.ImageIndex;
				ProcessInfo p = this.Process;
				if (p != null)
					return ProcessCache.Instance.GetIcon(p.Filename);
				else
					return -1;
			}
		}
		
		ProcessInfo process;
		bool gotProcess = false;
		public ProcessInfo Process {
			get {
				if (gotProcess) return process;
				gotProcess = true;
				if (MainForm.Is2000)
					return null;
				CMIB_UDPEXTABLE table = iphelper.GetUdpExTable();
				int sourcePort = this.SourcePort;
				int destPort = this.DestinationPort;
				for (int i = 0; i < table.dwNumEntries; i++) {
					CMIB_UDPEXROW row = table.table[i];
					uint port = row.get_port();
					if (port == sourcePort || port == destPort) {
						process = ProcessCache.Instance.GetProcessInfo(row.dwProcessId);
						return process;
					}
				}
				return null;
			}
		}
	}
}
