// Copyright (C) 2005  Daniel Grunwald
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// For more information read the file "LICENSE.txt".

using System;
using System.Diagnostics;
using System.IO;
using System.Xml;

class MainClass
{
	public static void Main(string[] args)
	{
		if (args.Length < 1 || args.Length > 2) {
			Console.WriteLine("SVNStatistic FROM:TO");
			Console.WriteLine("SVNStatistic FROM:TO startlinecount");
			return;
		}
		try {
			if (args.Length == 1)
				new MainClass(args[0], "0");
			else
				new MainClass(args[0], args[1]);
		} catch (Exception e) {
			Console.WriteLine(e);
		}
	}
	
	string SVN(string parameters) {
		ProcessStartInfo info = new ProcessStartInfo("svn", parameters);
		// if (parameters.StartsWith("log")) {
		info.CreateNoWindow = true;
		info.UseShellExecute = false;
		info.RedirectStandardOutput = true;
		info.RedirectStandardError = true;
		// }
		info.WorkingDirectory = Path.GetFullPath("../../..");
		//Console.WriteLine("{0}>{1} {2}", info.WorkingDirectory, info.FileName, info.Arguments);
		Process p = Process.Start(info);
		string data = p.StandardOutput.ReadToEnd();
		/*if (!p.WaitForExit(30000))
			throw new Exception("Process timeout");*/
		if (p.ExitCode != 0)
			throw new Exception(p.StandardError.ReadToEnd() + "\n\n\n" + p.StandardOutput.ReadToEnd());
		return data;
	}
	
	XmlDocument SVNXML(string parameters) {
		string text = SVN(parameters + " --xml");
		XmlDocument doc = new XmlDocument();
		doc.LoadXml(text);
		return doc;
	}
	
	TextReader SVNDiff(int fromrev, int torev) {
		if (!Directory.Exists("patchtemp/"))
			Directory.CreateDirectory("patchtemp/");
		string filename = String.Format("patchtemp/dgn{0}to{1}.patch", fromrev, torev);
		if (!File.Exists(filename)) {
			string data = SVN(String.Format("diff -r {0}:{1}", fromrev, torev));
			using (StreamWriter w = new StreamWriter(filename)) {
				w.Write(data);
			}
			return new StringReader(data);
		} else {
			return new StreamReader(filename);
		}
	}
	
	public MainClass(string arg, string arg2) {
		XmlDocument doc = SVNXML("log -r " + arg);
		Console.WriteLine("Revision;Date;LineCount;AddedLines;RemovedLines;ChangedLines");
		XmlElement log = doc.DocumentElement;
		int linecount = int.Parse(arg2);
		foreach(XmlElement logentry in log) {
			int rev = int.Parse(logentry.GetAttribute("revision"));
			string author = logentry["author"].InnerText;
			if (author.ToLower().Trim() == "importoperator")
				continue;
			if (rev == 21) continue; // move Grunwald.Gui
			DateTime date = DateTime.Parse(logentry["date"].InnerText);
			Console.Write("{0};{1}; ", rev, date);
			int changedLines = 0;
			int addedLines = 0;
			int removedLines = 0;
			using (TextReader r = SVNDiff(rev - 1, rev)) {
				string line;
				int mode = 0;
				int minusRow = 0;
				while ((line = r.ReadLine()) != null) {
					if (line.Length == 0)
						continue;
					char b = line[0];
					if (b == '-') {
						if (mode != 0)
							minusRow = 0;
						mode = 0;
						minusRow += 1;
						removedLines += 1;
					} else if (b == '+') {
						mode = 1;
						if (minusRow > 0) {
							minusRow -= 1;
							changedLines += 1;
						} else {
							addedLines += 1;
						}
					} else {
						minusRow = 0;
					}
				}
				removedLines -= changedLines;
			}
			linecount += addedLines - removedLines;
			Console.WriteLine("{0}; {1}; {2}; {3}", linecount,
			                  addedLines, removedLines, changedLines);
		}
	}
}
