; DG Network installation script. Based on example2.nsi

;--------------------------------
!define DOT_MAJOR 1
!define DOT_MINOR 1

!include "MUI.nsh"

; The name of the installer
Name "DG Network"

CRCCheck on
XPStyle on

BrandingText " 2004-2005 Daniel Grunwald"

; The file to write
OutFile "DG Network Setup.exe"

; The default installation directory
InstallDir "$PROGRAMFILES\DG Network"

; Registry key to check for directory (so if you install again, it will 
; overwrite the old one automatically)
InstallDirRegKey HKLM "Software\Grunwald\DG Network" "Install_Dir"

Function .onInit
    Call IsDotNetInstalled
FunctionEnd

; Usage
; Define in your script two constants:
;   DOT_MAJOR "(Major framework version)"
;   DOT_MINOR "{Minor frameword version)"
; 
; Call IsDotNetInstalled
; This function will abort the installation if the required version 
; or higher version of the .NETFramework is not installed.  Place it in
; either your .onInit function or your first install section before 
; other code.
Function IsDotNetInstalled

  StrCpy $0 "0"
  StrCpy $1 "SOFTWARE\Microsoft\.NETFramework" ;registry entry to look in.
  StrCpy $2 0

  StartEnum:
    ;Enumerate the versions installed.
    EnumRegKey $3 HKLM "$1\policy" $2
    
    ;If we don't find any versions installed, it's not here.
    StrCmp $3 "" noDotNet notEmpty
    
    ;We found something.
    notEmpty:
      ;Find out if the RegKey starts with 'v'.  
      ;If it doesn't, goto the next key.
      StrCpy $4 $3 1 0
      StrCmp $4 "v" +1 goNext
      StrCpy $4 $3 1 1
      
      ;It starts with 'v'.  Now check to see how the installed major version
      ;relates to our required major version.
      ;If it's equal check the minor version, if it's greater, 
      ;we found a good RegKey.
      IntCmp $4 ${DOT_MAJOR} +1 goNext yesDotNetReg
      ;Check the minor version.  If it's equal or greater to our requested 
      ;version then we're good.
      StrCpy $4 $3 1 3
      IntCmp $4 ${DOT_MINOR} yesDotNetReg goNext yesDotNetReg

    goNext:
      ;Go to the next RegKey.
      IntOp $2 $2 + 1
      goto StartEnum

  yesDotNetReg:
    ;Now that we've found a good RegKey, let's make sure it's actually
    ;installed by getting the install path and checking to see if the 
    ;mscorlib.dll exists.
    EnumRegValue $2 HKLM "$1\policy\$3" 0
    ;$2 should equal whatever comes after the major and minor versions 
    ;(ie, v1.1.4322)
    StrCmp $2 "" noDotNet
    ReadRegStr $4 HKLM $1 "InstallRoot"
    ;Hopefully the install root isn't empty.
    StrCmp $4 "" noDotNet
    ;build the actuall directory path to mscorlib.dll.
    StrCpy $4 "$4$3.$2\mscorlib.dll"
    IfFileExists $4 yesDotNet noDotNet

  noDotNet:
    ;Nope, something went wrong along the way.  Looks like the 
    ;proper .NETFramework isn't installed.  
    ExecShell Open "http://www.microsoft.com/downloads/details.aspx?FamilyID=262d25e3-f589-4842-8157-034d1e7cf3a3&DisplayLang=de" SW_SHOWNORMAL
    MessageBox MB_OK "DG Network bentigt das Microsoft .NET Framework v${DOT_MAJOR}.${DOT_MINOR}. Bitte starten Sie das Setup erneut, sobald Sie das .NET Framework installiert haben."
    Abort

  yesDotNet:
    ;Everything checks out.  Go on with the rest of the installation.
    
FunctionEnd

;--------------------------------
; Page settings

!define MUI_FINISHPAGE_RUN "DG Network.exe"
!define MUI_FINISHPAGE_RUN_PARAMETERS "/setup /show"
!define MUI_FINISHPAGE_LINK "Klicken Sie hier, um die Online Hilfe anzuzeigen."
!define MUI_FINISHPAGE_LINK_LOCATION "http://www.danielgrunwald.de/network/help/"
!define MUI_FINISHPAGE_NOREBOOTSUPPORT

;--------------------------------

; Pages

  !insertmacro MUI_PAGE_COMPONENTS
  !insertmacro MUI_PAGE_DIRECTORY
  !insertmacro MUI_PAGE_INSTFILES
  !insertmacro MUI_PAGE_FINISH

  !insertmacro MUI_UNPAGE_CONFIRM
  !insertmacro MUI_UNPAGE_INSTFILES
  !insertmacro MUI_UNPAGE_FINISH

!insertmacro MUI_LANGUAGE "German"

;--------------------------------

; The stuff to install
Section "DG Network (required)" SecDGN

  SectionIn RO
  
  ; Set output path to the installation directory.
  SetOutPath $INSTDIR
  
  ; Put file there
  File "..\..\bin\Release\DG Network.exe"
  File "..\..\bin\Release\DG Network.exe.config"
  File "..\..\bin\Release\DG Network.exe.manifest"
  File "..\..\bin\Release\Base.dll"
  File "..\..\bin\Release\Core.dll"
  File "..\..\bin\Release\log4net.dll"
  File "..\..\bin\Release\Grunwald.Gui.dll"
  File "..\..\bin\Release\Grunwald.P2PLib.dll"
  File "..\..\bin\Release\Grunwald.P2PLib.DGNetwork.dll"
  File /r "..\..\bin\Release\data"
  
  SetOutPath "$INSTDIR\Addins"
  File "..\..\bin\Release\Addins\Core.addin"
  File /r "..\..\bin\Release\Addins\Discovery"
  File /r "..\..\bin\Release\Addins\History"
  File /r "..\..\bin\Release\Addins\AutoUpdate"
  SetOutPath $INSTDIR
  
  ; Write the installation path into the registry
  WriteRegStr HKLM "SOFTWARE\Grunwald\DG Network" "Install_Dir" "$INSTDIR"
  
  ; Write "start with windows" entry
  ; Code duplication: reg location also set in StartWithWindowsSetting.cs
  WriteRegStr HKCU "Software\Microsoft\Windows\CurrentVersion\Run" "DG Network" '"$INSTDIR\DG Network.exe" /autostart'
  
  ; Write the uninstall keys for Windows
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\DG Network" "DisplayName" "DG Network"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\DG Network" "UninstallString" '"$INSTDIR\uninstall.exe"'
  WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\DG Network" "NoModify" 1
  WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\DG Network" "NoRepair" 1
  WriteUninstaller "uninstall.exe"
  CreateShortCut "$SMPROGRAMS\DG Network.lnk" "$INSTDIR\DG Network.exe" "/show" "$INSTDIR\DG Network.exe" 0
SectionEnd

Section "Internet Explorer display" SecIE
  SetOutPath "$INSTDIR\Addins\Browser"
  File /r "..\..\bin\Release\Addins\Browser\MSIE"
SectionEnd

Section "Mozilla display" SecMozilla
  SetOutPath "$INSTDIR\Addins\Browser"
  File /r "..\..\bin\Release\Addins\Browser\Mozilla"
SectionEnd

Section "ICQ plugin" SecICQ
  SetOutPath "$INSTDIR\Addins"
  File /r "..\..\bin\Release\Addins\ICQ"
SectionEnd

Section "IRC plugin" SecIRC
  SetOutPath "$INSTDIR\Addins"
  File /r "..\..\bin\Release\Addins\IRC"
SectionEnd

; Optional section (can be disabled by the user)
Section "Desktop-Verknpfung" SecShortcut
  SetOutPath $INSTDIR
  CreateShortCut "$DESKTOP\DG Network.lnk" "$INSTDIR\DG Network.exe" "/show" "$INSTDIR\DG Network.exe" 0
SectionEnd

;--------------------------------
;Descriptions

  ;Language strings
  LangString DESC_SecDGN ${LANG_GERMAN} "Das DG Network Hauptprogramm. Muss installiert werden."
  LangString DESC_SecIE ${LANG_GERMAN} "Plugin, um die empfangenen Nachrichten mit dem Internet Explorer anzuzeigen."
  LangString DESC_SecMozilla ${LANG_GERMAN} "Plugin, um die empfangenen Nachrichten mit dem Mozilla Control anzuzeigen."
  LangString DESC_SecICQ ${LANG_GERMAN} "Plugin fr das ICQ-Protokoll."
  LangString DESC_SecIRC ${LANG_GERMAN} "Plugin fr das IRC-Protokoll."
  LangString DESC_SecShortcut ${LANG_GERMAN} "Erstellt eine Verknpfung zu DG Network auf dem Desktop."

  ;Assign language strings to sections
  !insertmacro MUI_FUNCTION_DESCRIPTION_BEGIN
    !insertmacro MUI_DESCRIPTION_TEXT ${SecDGN} $(DESC_SecDGN)
    !insertmacro MUI_DESCRIPTION_TEXT ${SecIE} $(DESC_SecIE)
    !insertmacro MUI_DESCRIPTION_TEXT ${SecMozilla} $(DESC_SecMozilla)
    !insertmacro MUI_DESCRIPTION_TEXT ${SecICQ} $(DESC_SecICQ)
    !insertmacro MUI_DESCRIPTION_TEXT ${SecIRC} $(DESC_SecIRC)
    !insertmacro MUI_DESCRIPTION_TEXT ${SecShortcut} $(DESC_SecShortcut)
  !insertmacro MUI_FUNCTION_DESCRIPTION_END


;--------------------------------

; Uninstaller

Section "Uninstall"
  retry:
  FindProcDLL::FindProc "DG Network.exe"
  StrCmp $R0 "1" 0 next
  MessageBox MB_RETRYCANCEL "DG Network luft im Moment. Sie mssen DG Network beenden, bevor Sie es deinstallieren knnen!" IDRETRY retry IDCANCEL 0
  Quit
  next:
  
  
  ; Remove registry keys
  DeleteRegKey HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\DG Network"
  DeleteRegKey HKLM "SOFTWARE\Grunwald\DG Network"
  DeleteRegValue HKCU "Software\Microsoft\Windows\CurrentVersion\Run" "DG Network"

  ; Remove files and uninstaller
  Delete "$INSTDIR\DG Network.exe"
  Delete "$INSTDIR\DG Network.exe"
  Delete "$INSTDIR\DG Network.exe.config"
  Delete "$INSTDIR\DG Network.exe.manifest"
  Delete "$INSTDIR\Base.dll"
  Delete "$INSTDIR\Core.dll"
  Delete "$INSTDIR\log4net.dll"
  Delete "$INSTDIR\Grunwald.Gui.dll"
  Delete "$INSTDIR\Grunwald.P2PLib.dll"
  Delete "$INSTDIR\Grunwald.P2PLib.DGNetwork.dll"
  RMDir /r "$INSTDIR\data"

  Delete $INSTDIR\uninstall.exe
  ; Remove shortcuts, if any
  Delete "$SMPROGRAMS\DG Network.lnk"
  Delete "$DESKTOP\DG Network.lnk"

  Delete "$INSTDIR\Addins\Core.addin"
  RMDir /r "$INSTDIR\Addins\Discovery"
  RMDir /r "$INSTDIR\Addins\Browser\MSIE"
  RMDir /r "$INSTDIR\Addins\Browser\Mozilla"
  RMDir "$INSTDIR\Addins\Browser"
  RMDir /r "$INSTDIR\Addins\ICQ"
  RMDir /r "$INSTDIR\Addins\IRC"
  RMDir /r "$INSTDIR\Addins\AutoUpdate"
  RMDir /r "$INSTDIR\Addins\History"
  RMDir "$INSTDIR\Addins"
  
  ; Remove directories used
  RMDir "$INSTDIR"

SectionEnd